   !&E   ***************   include dyn2dy_mtm   **********************************
   !&E
   !&E ** Purpose : calcul des coefficients de la matrice tridiagonale
   !&E              du systeme lineaire et du second membre relatifs
   !&E              a l equation de conservation de la quantite de mouvement
   !&E              pour le calcul en colonne. (etape de prediction dans
   !&E              l algorithme de predicteur-correcteur)
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : dyn2dyx.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : xeu,u,v,ustar,vstar,igdv,hy,htx,hty,
   !&E                     dx_f,dx_v,dy,dy_v,fv,me,mo,why,
   !&E                     tfivsr,vis2d,tosy,pa,potgene,buoy2d
   !&E
   !&E ** Modified variables : y2dv,cv,bv,av
   !&E
   !&E ** Reference :
   !&E                T. Duhaut, M. Honnorat, L. Debreu, 2008 : Rapport
   !&E                contrat PREVIMER - Ref : 06/2 210 290 (pages 100-125)
   !&E
   !&E ** History :
   !&E       !  2007-09 (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, T. Duhaut, M. Honnorat) time scheme
   !&E       !  2009-03  (V. Garnier) modification of udvsdx term (centered scheme)
   !&E       !  2009-03  (T. Duhaut V. Garnier) QUICK and QUICKEST scheme (with masks)
   !&E       !  2009-07  (N. Bruneau F. Dumas) coupling with swan
   !&E       !  2010-07  (L. Debreu, F. Dumas) rewritting of ADI
   !&E       !  2011-03  (F. Dumas) bottom stress parametrisation + remove case 2 and 3
   !&E       !  2011-10  (L. Pineau, F. Dumas, V. garnier) correction bottom stress parametrisation
   !&E        !  2011-11  (F. Vandermeirsch) correct spatial extension on which we can estimate quick and quickest scheme
   !&E
   !&E-------------------------------------------------------------------------

     gdt=grav*dt

     IF (j >= jb(i) .and. j <= jh(i)-1) THEN

   ! cas d un point v sans eau de part ni d autre
   ! drying v-point : no water on the right and left

      IF (igdv(i,j) == 4) THEN

        av(i,j)=0.0_rlg
        bv(i,j)=1.0_rlg
        cv(i,j)=0.0_rlg
        y2dv(i,j)=0.0_rlg

   ! cas d un point u avec de l eau de part et d autre
   ! u-point surrounded by water

      ELSE IF (igdv(i,j) == 1) THEN

        u_vtmp2=0.25_rsh*(ustar(i,j)+ustar(i,j+1)+ustar(i-1,j)+ustar(i-1,j+1))

        ! sea surface pressure gradient : implicit part
        av(i,j)=-alphan*0.5_rlg*gdt/dy_v(i,j)                ! j-1
        cv(i,j)= alphan*0.5_rlg*gdt/dy_v(i,j)                ! j

#ifdef key_dyn_adv_linear

        ! time evolution (v at n+1)
        bv(i,j)=1.0_rlg

        ! rhs : explicit terms
        y2dv(i,j)=                                       &
                  ! time evolution (v at n)
                  v(i,j)                                 &
                  ! coriolis
                  -fv(i,j)*u_vtmp2*halfdt                &
                  ! sea surface pressure gradient
                  -0.5_rlg*gdt/dy_v(i,j)*alphap*( xeu(i,j+1)-xeu(i,j) )  &
                  ! viscosity (along y)
                  +halfdt*( vis2d(i,j+1)*(v(i,j+1)-v(i,j  ))/dy(i,j+1)   &
                           -vis2d(i,j  )*(v(i,j  )-v(i,j-1))/dy(i,j  ) ) &
                           /dy_v(i,j)
#else

        ! bottom stress
#ifdef key_no_bottom_stress
        ft=0.0_rsh
#elif key_linear_bottom_stress
       !ft=frot_lin/htx(i,j)
       ft=halfdt*frot_lin/MAX(hty(i,j),hminfrot)
#else

#ifdef key_bottom_stress_wave
        ft = halfdt * 2.0_rsh/pi* Orb_v(i,j) * CfH(i,j) / max(htx(i,j),hminfrot)
#else
        vit=sqrt(u_vtmp(i,j)**2+v(i,j)**2)
        factft=1.0_rsh+vit-botstressmax+abs(vit-botstressmax)
        ft=factft*0.5_rsh*dt*vit/hty(i,j)*MIN((0.4_rsh/LOG(MAX(min(hty(i,j),hmaxfrot),hminfrot)/(exp1*z0b(i,j))))**2,0.1_rsh)
#endif

#endif

#if ! defined key_dyn_adv_none
     ! non-linear terms (udu/dy and vdu/dy) : explicit
#ifdef key_dyn_adv_quickest
        vc=vstar(i,j)
        vp=0.5_rsh*(vc+abs(vc))
        vm=0.5_rsh*(vc-abs(vc))
        up=0.5_rsh*(u_vtmp2+abs(u_vtmp2))
        um=0.5_rsh*(u_vtmp2-abs(u_vtmp2))
        IF (i >= imin+3 .AND. i <= imax-2 .AND. j <= jmax-2 .AND. j >= jmin+2) THEN
          vdvdy=( (vp+vm)*(0.5_rsh*(v(i,j+1)-v(i,j-1))                                       &
                 -0.5_rsh*(vp+vm)*(v(i,j+1)+v(i,j-1)-2.0_rsh*v(i,j)) )                       &
                 +vp/6.0_rsh*(1-vp*vp)*(v(i,j-2)*why(i,j-1)-3.0_rsh*v(i,j-1)+3.0_rsh*v(i,j  )-v(i,j+1)) &
                 +vm/6.0_rsh*(1-vm*vm)*(v(i,j-1)-3.0_rsh*v(i,j  )+3.0_rsh*v(i,j+1)-v(i,j+2)*why(i,j+1)) )/dy(i,j)
          vdvdy2=( (vp+vm)*(0.5_rsh*(vstar(i,j+1)-vstar(i,j-1))                                               &
                  -0.5_rsh*(vp+vm)*(vstar(i,j+1)+vstar(i,j-1)-2.0_rsh*vstar(i,j)) )                           &
                  +vp/6.0_rsh*(1-vp*vp)*(vstar(i,j-2)*why(i,j-1)-3.0_rsh*vstar(i,j-1)+3.0_rsh*vstar(i,j  )-vstar(i,j+1)) &
                  +vm/6.0_rsh*(1-vm*vm)*(vstar(i,j-1)-3.0_rsh*vstar(i,j  )+3.0_rsh*vstar(i,j+1)-vstar(i,j+2)*why(i,j+1)) )/dy(i,j)
          vdvsdy = (alphan*vdvdy2 + alphap*vdvdy)
          udvdx=( (up+um)*(0.5_rsh*(v(i+1,j)-v(i-1,j))                                       & 
                 -0.5_rsh*(up+um)*(v(i+1,j)+v(i-1,j)-2.0_rsh*v(i,j)) )                       &
                 +up/6.0_rsh*(1-up*up)*(v(i-2,j)*mo(i-1,j)*mo(i,j)-3.0_rsh*v(i-1,j)+3.0_rsh*v(i  ,j)-v(i+1,j)) &
                 +um/6.0_rsh*(1-um*um)*(v(i-1,j)-3.0_rsh*v(i  ,j)+3.0_rsh*v(i+1,j)-v(i+2,j)*me(i+1,j)*me(i,j)) )/dx_v(i,j)
          udvdx2=( (up+um)*(0.5_rsh*(vstar(i+1,j)-vstar(i-1,j))                                               & 
                  -0.5_rsh*(up+um)*(vstar(i+1,j)+vstar(i-1,j)-2.0_rsh*vstar(i,j)) )                           &
                  +up/6.0_rsh*(1-up*up)*(vstar(i-2,j)*mo(i-1,j)*mo(i,j)-3.0_rsh*vstar(i-1,j)+3.0_rsh*vstar(i  ,j)-vstar(i+1,j)) &
                  +um/6.0_rsh*(1-um*um)*(vstar(i-1,j)-3.0_rsh*vstar(i  ,j)+3.0_rsh*vstar(i+1,j)-vstar(i+2,j)*me(i+1,j)*me(i,j)) ) &
                 /dx_v(i,j)
          udvsdx = (alphan*udvdx2 + alphap*udvdx)
        ELSE
   ! upwind scheme
          vdvdy  = vp*(v(i,j)-v(i,j-1))/dy(i,j)+vm*(v(i,j+1)-v(i,j))/dy(i,j+1)
          vdvdy2 = vp*(vstar(i,j)-vstar(i,j-1))/dy(i,j)+vm*(vstar(i,j+1)-vstar(i,j))/dy(i,j+1)
          vdvsdy = (alphan*vdvdy2 + alphap*vdvdy)
          udvdx  = up*(v(i,j)-v(i-1,j))/dx_f(i-1,j)+um*(v(i+1,j)-v(i,j))/dx_f(i,j)
          udvdx2 = up*(vstar(i,j)-vstar(i-1,j))/dx_f(i-1,j)+um*(vstar(i+1,j)-vstar(i,j))/dx_f(i,j)
          udvsdx = (alphan*udvdx2 + alphap*udvdx)
        END IF
#elif key_dyn_adv_quick
        vc=vstar(i,j)
        vp=0.5_rsh*(vc+abs(vc))
        vm=0.5_rsh*(vc-abs(vc))
        up=0.5_rsh*(u_vtmp2+abs(u_vtmp2))
        um=0.5_rsh*(u_vtmp2-abs(u_vtmp2))
        IF (i >= imin+3.AND.i <= imax-2.AND.j <= jmax-2.AND.j >= jmin+2) THEN
          vdvdy=(vp*v(i,j-2)*why(i,j-1)+(-7.0_rsh*vp-3.0_rsh*vm)*v(i,j-1)+  &
                 (3.0_rsh*vp-3.0_rsh*vm)*v(i,j)+(3.0_rsh*vp+7.0_rsh*vm)*v(i,j+1)-vm*v(i,j+2)*why(i,j+1))/(8.0_rsh*dy_v(i,j)) 
          vdvdy2=(vp*vstar(i,j-2)*why(i,j-1)+(-7.0_rsh*vp-3.0_rsh*vm)*vstar(i,j-1)+  &
                  (3.0_rsh*vp-3.0_rsh*vm)*vstar(i,j)+(3.0_rsh*vp+7.0_rsh*vm)*vstar(i,j+1)-vm*vstar(i,j+2)*why(i,j+1)) &
                 /(8.0_rsh*dy_v(i,j)) 
          vdvsdy = (alphan*vdvdy2 + alphap*vdvdy)
          udvdx=(up*v(i-2,j)*mo(i-1,j)*mo(i,j)+(-7.0_rsh*up-3.0_rsh*um)*v(i-1,j)+  &
                 (3.0_rsh*up-3.0_rsh*um)*v(i,j)+(3.0_rsh*up+7.0_rsh*um)*v(i+1,j)-um*v(i+2,j)*me(i+1,j)*me(i,j))/(8.0_rsh*dx_v(i,j))
          udvdx2=(up*vstar(i-2,j)*mo(i-1,j)*mo(i,j)+(-7.0_rsh*up-3.0_rsh*um)*vstar(i-1,j)+  &
                  (3.0_rsh*up-3.0_rsh*um)*vstar(i,j)+(3.0_rsh*up+7.0_rsh*um)*vstar(i+1,j)-um*vstar(i+2,j)*me(i+1,j)*me(i,j)) &
                 /(8.0_rsh*dx_v(i,j))
          udvsdx = (alphan*udvdx2 + alphap*udvdx)
        ELSE
   ! upwind scheme
          vdvdy  = vp*(v(i,j)-v(i,j-1))/dy(i,j)+vm*(v(i,j+1)-v(i,j))/dy(i,j+1)
          vdvdy2 = vp*(vstar(i,j)-vstar(i,j-1))/dy(i,j)+vm*(vstar(i,j+1)-vstar(i,j))/dy(i,j+1)
          vdvsdy = (alphan*vdvdy2 + alphap*vdvdy)
          udvdx  = up*(v(i,j)-v(i-1,j))/dx_f(i-1,j)+um*(v(i+1,j)-v(i,j))/dx_f(i,j)
          udvdx2 = up*(vstar(i,j)-vstar(i-1,j))/dx_f(i-1,j)+um*(vstar(i+1,j)-vstar(i,j))/dx_f(i,j)
          udvsdx = (alphan*udvdx2 + alphap*udvdx)
        END IF
#else
   ! default : centered scheme
        vdvsdy=vstar(i,j)*                                                &
                 ( alphan*(vstar(i,j+1)-vstar(i,j-1))/(dy(i,j+1)+dy(i,j)) &
                  +alphap*(v(i,j+1)-v(i,j-1))/(dy(i,j+1)+dy(i,j)) )
        udvsdx=0.5_rsh*u_vtmp2*(                                          &
               mo(i,j)*(  alphan*(vstar(i,j)-vstar(i-1,j))                &
                        + alphap*(v(i,j)-v(i-1,j)) )/dx_f(i-1,j)          &
              +me(i,j)*(  alphan*(vstar(i+1,j)-vstar(i,j))                &
                        + alphap*(v(i+1,j)-v(i,j)) )/dx_f(i,j)  )
#endif
#endif

        bv(i,j)=1.0_rlg/(                                &
                ! time evolution (v at n+1)
                1.0_rlg                                  &
                ! bottom stress : implicit part
                +ft*cfroi)

        ! rhs : explicit terms
        y2dv(i,j)=                                       &
                  ! time evolution (v at n)
                  v(i,j)                                 &
                  ! bottom stress (v at n)
                  -ft*cfroe*v(i,j)                       &
                  ! coriolis
                  -fv(i,j)*u_vtmp2*halfdt                &
                  ! sea surface pressure gradient
                  -0.5_rlg*gdt/dy_v(i,j)*alphap*( xeu(i,j+1)-xeu(i,j) )  &
#if ! defined key_dyn_adv_none
                  ! NL advection : u*u*tan(phi)/R (spherical coordinates)
                  -halfdt*tfivsr(i,j)*u_vtmp2**2         &
                  ! non-linear (vdv/dy) : explicit
                  -halfdt*vdvsdy                         &
                  ! non-linear (udv/dx) : explicit
                  -halfdt*udvsdx                         &
#endif
                  ! viscosity (along y)
                  +halfdt*( vis2d(i,j+1)*(v(i,j+1)-v(i,j  ))/dy(i,j+1)   &
                           -vis2d(i,j  )*(v(i,j  )-v(i,j-1))/dy(i,j  ) ) &
                           /dy_v(i,j)                                    &
                  ! viscosity (along x)
                  +halfdt*( vis2d(i  ,j)*me(i,j)*(v(i+1,j)-v(i  ,j))/dx_f(i  ,j)   &
                           -vis2d(i-1,j)*mo(i,j)*(v(i  ,j)-v(i-1,j))/dx_f(i-1,j) ) &
                           /dx_v(i,j)
#endif


        ! wind stress and atmospherical pressure gradient
        y2dv(i,j)=y2dv(i,j)                              &

                  ! wind stress in y-direction : explicit
                  +halfdt*tosy(i,j)/max(hty(i,j),hminfrot)/rhoref &

                  ! atmospherical pressure gradient : explicit
                  -halfdt*(pa(i,j+1)-pa(i,j))/(dy_v(i,j)*rhoref)

        ! generator potential : explicit
        IF (l_genpot) &
          y2dv(i,j)=y2dv(i,j)+halfdt/dy_v(i,j)*(potgene(i,j+1)-potgene(i,j))

        ! internal pressure gradient : explicit
        IF ( nbfl/=0 .and. icon==2 ) &
          y2dv(i,j)=y2dv(i,j)+halfdt/dy_v(i,j)*hty(i,j)*(buoy2d(i,j+1)-buoy2d(i,j))

#ifdef key_waveforcing
        IF (l_wave_brgm_phillips) THEN
          y2dv(i,j) = y2dv(i,j) - halfdt *( temp_radv(i,j) / rhoref + v  (i,j) * (Qy(i,j+1)-Qy(i,j))/dy_v(i,j) &
                                         +  2.0_rsh*Qy_v(i,j) * (v(i,j+1)-v(i,j-1))/(dy(i,j)+dy(i,j+1)) )/ max(hty(i,j),hminfrot)
          IF ( htx(i-1,j)+htx(i-1,j+1) >= 0.0_rsh )  &
            y2dv(i,j) = y2dv(i,j) - halfdt * ( Qy_v(i,j) * (u_vtmp(i,j )-u_vtmp(i-1,j)) &
                                             + Qx_v(i,j) * (v (i,j  )-v   (i-1,j)) &
                                             + u_vtmp(i,j) * (Qy_v(i,j)-Qy_v(i-1,j))  )/(2.0_rsh*dx_u(i-1,j)*max(hty(i,j),hminfrot))
          IF ( htx(i  ,j)+htx(i  ,j+1) >= 0.0_rsh ) &
            y2dv(i,j) = y2dv(i,j) - halfdt * (  Qy_v(i,j) * (u_vtmp (i+1,j)-u_vtmp (i,j)) &
                                           +  Qx_v(i,j) * (v   (i+1,j)-v   (i,j)) &
                                           +  u_vtmp(i,j) * (Qy_v(i+1,j)-Qy_v(i,j)) )/(2.0_rsh*dx_u(i,j)*max(hty(i,j),hminfrot))

        ELSE IF (l_wave_brgm_smith) THEN
          y2dv(i,j) = y2dv(i,j) - halfdt * (JSMITH(i,j+1)-JSMITH(i,j)) /dy_v(i,j)
          y2dv(i,j) = y2dv(i,j) &
                      + halfdt * (k_y(i,j)+k_y(i,j+1))/2.0_rsh * (Dsmith(i,j)+Dsmith(i,j+1))/2.0_rsh/ max(hty(i,j),hminfrot)/rhoref
          y2dv(i,j) = y2dv(i,j) + halfdt * Qx_v(i,j) * (u_vtmp(i,j+1)-u_vtmp(i,j-1))/(dy(i,j)+dy(i,j+1)) / max(hty(i,j),hminfrot)
          IF((htx(i-1,j)+htx(i-1,j+1)) >= 0.0_rsh) &
            y2dv(i,j) = y2dv(i,j) - halfdt * Qx_v(i,j) * (v  (i,j)-v  (i-1,j))/(2.0_rsh*dx_u(i-1,j)) / max(hty(i,j),hminfrot)
          IF((htx(i,j)+htx(i,j+1)) >= 0.0_rsh) &
            y2dv(i,j) = y2dv(i,j) - halfdt * Qx_v(i,j) * (v  (i+1,j)-v  (i,j))/(2.0_rsh*dx_u(i,j )) / max(hty(i,j),hminfrot)
        ELSE
          y2dv(i,j) = y2dv(i,j) - halfdt * temp_radv(i,j) / max(hty(i,j),hminfrot)/rhoref
          y2dv(i,j) = y2dv(i,j) + ft*Qy_u(i,j)/max(hty(i,j),hminfrot)
        END IF
#endif 

      END IF  ! ends test on mask for wetting and drying
     END IF    ! fin de test sur l emprise de la  colonne

   !&E   ***************   end include dyn2dy_mtm   ******************************

