  SUBROUTINE dyn3d_dt(uz,       &
#if defined key_lans_turb || defined key_leray_turb
                      uzs,      &
#endif
#if defined key_lans_turb || defined key_leray_turb
                      vzs,      &
#endif
                      vz)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE dyn3d_dt  ***
   !&E
   !&E ** Purpose : adaptative computation of the time-step by examining
   !&E              by examining the surface current number.
   !&E
   !&E ** Description : The time-step increase is applied after a lap of tobs,
   !&E                  if during this observation period the stability criterion
   !&E                  has been exceeded.
   !&E
   !&E ** Called by : stepu
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Used ij-arrays : uz, vz, dy_v, htx, hty
   !&E
   !&E ** Modified variables : dt, cflumaxmax, cflvmaxmax, tdebobs, tobs
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2004-12-29
   !&E       !  2007-09    (V. Garnier) homogeneneity of dx dy...
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters,ONLY : limin,limax,ljmin,kmax,ljminp1,ljmaxm1,rsh,rlg, &
                         liminm2,limaxp2,ljminm2,ljmaxp2,epsilon,        &
                         jshift_obc_south,jshift_obc_north,jmax
   USE comvars2d, ONLY : ig,id,dx_u,dy_v,htx,hty
   USE comvars2d, ONLY : cflumaxmax,cflvmaxmax,cflcrt ,cdate
   USE comvars2d, ONLY : dt,halfdt,tobs,dtmin,t,tdebobs,dtmax,iscreen
#if defined key_agrif
   USE comvars2d, ONLY : dtmaxi
#endif
   USE_MPI toolmpi, ONLY : FINDMAX_MPI,MASTER

   IMPLICIT NONE

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(in) :: uz,vz
#if defined key_lans_turb || defined key_leray_turb
   REAL(KIND=rsh),DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(in) :: uzs,vzs
#endif

   !! * Local declarations
   INTEGER        :: i,j,iumax,jumax,ivmax,jvmax
#if defined key_lans_turb || defined key_leray_turb
   INTEGER        :: iusmax,jusmax,ivsmax,jvsmax
#endif
   INTEGER        :: liumx,ljumx,livmx,ljvmx
#if defined key_lans_turb || defined key_leray_turb
   INTEGER        :: liusmx,ljusmx,livsmx,ljvsmx
#endif
   REAL(KIND=rsh) :: flumx,flvmx
#if defined key_lans_turb || defined key_leray_turb
   REAL(KIND=rsh) :: flusmx,flvsmx
#endif
   REAL(KIND=rsh) :: uzmax,htxmax,dxumax,vzmax,htymax,dyvmax
#if defined key_lans_turb || defined key_leray_turb
   REAL(KIND=rsh) :: uzsmax,vzsmax
#endif
   REAL(KIND=rsh) :: cflumax,cflvmax,cflmax,cflu,cflv
#if defined key_lans_turb || defined key_leray_turb
   REAL(KIND=rsh) :: cflusmax,cflvsmax,cflus,cflvs
#endif
   REAL_AGRIF(KIND=rlg) :: dtmaxp

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*,'enter DYN3DDT'

   ! on examine le nb de courant max en surface
   !-------------------------------------------

   cflumax=-100.0_rsh
   cflvmax=-100.0_rsh
#if defined key_lans_turb || defined key_leray_turb
   cflusmax=-100.0_rsh
   cflvsmax=-100.0_rsh
#endif

   iumax=limin
   jumax=ljmin
   ivmax=limin
   jvmax=ljmin
#if defined key_lans_turb || defined key_leray_turb
   iusmax=limin
   jusmax=ljmin
   ivsmax=limin
   jvsmax=ljmin
#endif

!$OMP PARALLEL DEFAULT(NONE) &
!$OMP& PRIVATE(i,j,liumx,ljumx,livmx,ljvmx,flumx,flvmx,cflu,cflv) &
!$OMP& SHARED(cflumax,cflvmax,uz,vz,dt,dx_u,iumax,jumax,ivmax,jvmax,ig,id,dy_v, &
#ifdef key_agrif
!$OMP&        jmax,jshift_obc_south,jshift_obc_north, &
#endif
!$OMP&        ljminp1,ljmaxm1,ljmin,limin,limax)

   flumx = cflumax
   flvmx = cflvmax
#if defined key_lans_turb || defined key_leray_turb
   flusmx = cflusmax
   flvsmx = cflvsmax
#endif

#ifdef key_MPI_2D
!$OMP do SCHEDULE(runtime)
#else
!$OMP do SCHEDULE(static)
#endif

   !DO j=ljminp1,ljmaxm1
   DO j=MAX0(ljminp1,jshift_obc_south+1), MIN0(ljmaxm1,jmax-1-jshift_obc_north)
     DO i=MAX0(limin,ig(j)),MIN0(limax,id(j)-1)

       cflu=ABS(uz(kmax,i,j))*dt/dx_u(i,j)
       IF(cflu > flumx) THEN
         flumx=cflu
         liumx=i
         ljumx=j
       ENDIF

#if defined key_lans_turb || defined key_leray_turb
       cflus=ABS(uzs(kmax,i,j))*dt/dx_u(i,j)
       IF(cflus > flusmx) THEN
         flusmx=cflus
         liusmx=i
         ljusmx=j
       ENDIF
#endif

       cflv=ABS(vz(kmax,i,j))*dt/dy_v(i,j)
       IF(cflv > flvmx) THEN
         flvmx=cflv
         livmx=i
         ljvmx=j
       ENDIF

#if defined key_lans_turb || defined key_leray_turb
       cflvs=ABS(vzs(kmax,i,j))*dt/dy_v(i,j)
       IF(cflvs > flvsmx) THEN
         flvsmx=cflvs
         livsmx=i
         ljvsmx=j
       ENDIF
#endif

     ENDDO
   ENDDO
!$OMP end do nowait

!$OMP CRITICAL
   IF(flumx.GT.cflumax) THEN
     cflumax = flumx
     iumax = liumx
     jumax = ljumx
   ENDIF

#if defined key_lans_turb || defined key_leray_turb
   IF(flusmx.GT.cflusmax) THEN
     cflusmax = flusmx
     iusmax = liusmx
     jusmax = ljusmx
   ENDIF
#endif

   IF(flvmx.GT.cflvmax) THEN
     cflvmax = flvmx
     ivmax = livmx
     jvmax = ljvmx
   ENDIF

#if defined key_lans_turb || defined key_leray_turb
   IF(flvsmx.GT.cflvsmax) THEN
     cflvsmax = flvsmx
     ivsmax = livsmx
     jvsmax = ljvsmx
   ENDIF
#endif

!$OMP END CRITICAL
!$OMP END PARALLEL
   uzmax=uz(kmax,iumax,jumax)
   htxmax=htx(iumax,jumax)
   dxumax=dx_u(iumax,jumax)
   vzmax=vz(kmax,ivmax,jvmax)
   htymax=hty(ivmax,jvmax)
   dyvmax=dy_v(ivmax,jvmax)
#if defined key_lans_turb || defined key_leray_turb
   uzsmax=uz(kmax,iumax,jumax)
   vzsmax=vz(kmax,ivmax,jvmax)
#endif

   CALL_MPI FINDMAX_MPI(cflumax,iumax,jumax,uzmax,htxmax,dxumax)
   CALL_MPI FINDMAX_MPI(cflvmax,ivmax,jvmax,vzmax,htymax,dyvmax)
#if defined key_lans_turb || defined key_leray_turb
   CALL_MPI FINDMAX_MPI(cflusmax,iusmax,jusmax,uzsmax,htxmax,dxumax)
   CALL_MPI FINDMAX_MPI(cflvsmax,ivsmax,jvsmax,vzsmax,htymax,dyvmax)
#endif

   IF_MPI (MASTER) THEN 
   WRITE(iscreen,*) 'DATE : ',cdate
   WRITE(iscreen,*) 'dt=',dt
!   WRITE(iscreen,*) 'cflumax=',cflumax
!   WRITE(iscreen,*) 'cflvmax=',cflvmax
   WRITE(iscreen,*) 'max en u = ',uzmax,iumax,jumax,htxmax
   WRITE(iscreen,*) 'max en v = ',vzmax,ivmax,jvmax,htymax
#if defined key_lans_turb || defined key_leray_turb
   WRITE(iscreen,*) 'max en us = ',uzsmax,iusmax,jusmax,htxmax
   WRITE(iscreen,*) 'max en vs = ',vzsmax,ivsmax,jvsmax,htymax
#endif

   ENDIF_MPI

   IF(MAX(cflumax,cflvmax) > cflcrt) THEN
     IF_MPI (MASTER) THEN 
     WRITE(iscreen,*) 'ALERTE ADS!!!!!!!!!!!!!'
     ENDIF_MPI
     IF_AGRIF (agrif_root()) THEN
       dt=MIN(0.83_rsh*cflcrt*dyvmax/MAX(REAL(ABS(vzmax),rlg),epsilon), &
              0.83_rsh*cflcrt*dxumax/MAX(REAL(ABS(uzmax),rlg),epsilon))
       dt=MAX(dt,dtmin)
       halfdt=dt*0.5_rlg
       IF_MPI (MASTER) THEN 
       WRITE(iscreen,*) 'HIIIIIIIII!!!! nouveau dt=',dt
       ENDIF_MPI
     ELSE_AGRIF
       INST_AGRIF dtmaxp=MAX(MIN(0.83_rsh*cflcrt*dyvmax/MAX(REAL(ABS(vzmax),rlg),epsilon), &
       INST_AGRIF                0.83_rsh*cflcrt*dxumax/MAX(REAL(ABS(uzmax),rlg),epsilon)),dtmin)
! Set the next time step of the parent grid
       CALL_AGRIF Agrif_Set_Parent(dtmax, MAX(Agrif_parent(dtmin),dtmaxp*Agrif_Rhot()))
       IF_AGRIF (cflumax > cflcrt) CALL Agrif_Set_Parent(cflumaxmax, cflumax)
       IF_AGRIF (cflvmax > cflcrt) CALL Agrif_Set_Parent(cflvmaxmax, cflvmax)
       IF_MPI (MASTER) THEN 
       INST_AGRIF WRITE(iscreen,*) 'HIIIIIIIII!!!! desired dt=',dtmaxp
       ENDIF_MPI
     ENDIF_AGRIF
   ENDIF

#if defined key_agrif
! Maximum time step may have been restricted by a child grid
! Done whaterver the rank because a grand-daughter must slow down a daughter which will slow down the mother
! The slackening is done with a time step late
! dtmaxi = initialized through paraspec.txt (=dtmax) !!!
   IF (dt > dtmax) THEN
     dt = dtmax
     halfdt=dt*0.5_rlg
     IF_MPI (MASTER) THEN
     WRITE(iscreen,*) 'HIIIIIIIII!!!! nouveau dt=',dt,' due to AGRIF'
     ENDIF_MPI
     dtmax = dtmaxi
   ELSE IF (dt==dtmax) THEN
     dtmax = dtmaxi ! In case of dtmax (from child grid) equals dtmin (of root grid) and to allow the speed up later
   ENDIF

! Maintain fine grid time in agreement with coarse grid time step and time refinement factor
   IF (.NOT.Agrif_Root()) THEN
     dt = Agrif_Parent(dt)/Agrif_Rhot()
     IF (Agrif_Grandmother()) THEN
         dt = Agrif_Parent(dt) * Agrif_irhot() ! For the grandmother grid, Agrif_Parent is Agrif_Root
                                               ! A multiplication is required (instead of a division)
     END IF
     halfdt=dt*0.5_rlg
   ENDIF
#endif


   cflumaxmax=MAX(cflumaxmax,cflumax)
   cflvmaxmax=MAX(cflvmaxmax,cflvmax)

   !***************************************
   ! fin de la periode d observation
   !***************************************

   IF (t > tdebobs+tobs) THEN
     cflmax=MAX(cflumaxmax,cflvmaxmax)
     IF_MPI (MASTER) THEN 
     WRITE(iscreen,*) 'ATTENTION  le ',cdate
     WRITE(iscreen,*) 'cflumaxmax=',cflumaxmax
     WRITE(iscreen,*) 'cflvmaxmax=',cflvmaxmax
     ENDIF_MPI

     ! Reinitialisation for the next observation period 
     cflumaxmax=0.0_rsh
     cflvmaxmax=0.0_rsh
     tdebobs=t

     IF_AGRIF (.NOT.Agrif_Root()) RETURN ! Only RootGrid can accelerate

     IF(cflmax < cflcrt*0.84_rsh) THEN
       IF_MPI (MASTER) THEN 
       WRITE(iscreen,*) 'TURBOOOOOOOOOOOOOOOOOO.......!!!!!! old dt=',dt
       ENDIF_MPI
       dt=MIN(dt*1.2_rlg,dtmax)
       halfdt=dt*0.5_rlg
       IF_MPI (MASTER) THEN 
       WRITE(iscreen,*) 'Nouveau dt : ',dt
       ENDIF_MPI
     ENDIF
   ENDIF


   PRINT_DBG*,'exit DYN3DDT'

  END SUBROUTINE dyn3d_dt
