!**************    including dyn3dx_adv_quick.h      ***********************

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE dyn3dx_adv_quick.h  ***
   !&E
   !&E ** Purpose : estimate of advection terms (3d) of the momentum
   !&E              equation along the x-direction
   !&E
   !&E ** Description : explicit - QUICK scheme (decentre amont)
   !&E
   !&E ** Called by : dyn3dx_tls_coef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : uz,vz_u,tfiusr,wz,yz_exp,dx_u,dx,dy_f,dy_u
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference : Kowalik, Murty "Numerical Modeling of Ocean Dynamics"
   !&E
   !&E ** History :
   !&E       !  2007-09 (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E       !  2009-02 (F. Dumas) Upwind scheme along open boundaries + QUICK at imin+2 and jmin+2
   !&E       !  2009-03  (V. Garnier) land mask
   !&E       !  2011-11  (F. Vandermeirsch) correct spatial extension on which we can estimate quick scheme
   !&E
   !&E---------------------------------------------------------------------

   ! ----------------
   ! Horizontal terms
   ! ----------------

#ifdef key_shiftobc_verif
!to get exactly the same results as without any shift of obc
   IF (i >= imin+2+ishift_obc_west .AND. i <= imax-2-ishift_obc_east .AND. &
       j <= jmax-2-jshift_obc_north .AND. j >= jmin+3+jshift_obc_south) THEN
#else
   IF (i >= imin+2 .AND. i <= imax-2 .AND. j <= jmax-2 .AND. j >= jmin+3) THEN
#endif

   ! QUICK scheme
   ! ------------ 

     DO k=1,kmax

      !u * du/dx

      !Using derivative module
       vel_adv = uz(k,i,j)
       ududx   = vel_adv*quick_duzdx(uz, vel_adv, k, i, j)
 
      !v * du/dy

       !Using derivative module
       vel_adv = vz_u(k,i,j)
       vdudy   = vel_adv*quick_duzdy(uz, vel_adv, k, i, j)

     ! u * v * tg(phi) / R

       uv=uz(k,i,j)*vz_u(k,i,j)*tfiusr(i,j)

     ! add all terms
       yz_exp(k,i,j) = yz_exp(k,i,j) - ududx - vdudy + uv

#ifdef key_diagdyn_details
   !   u * du/dx
       diagx_nlt_ududx(k,i,j)=ududx
   !   v * du/dy : fully explicit 
       diagx_nlt_vdudy(k,i,j)=vdudy
#endif

     END DO

   ELSE

   ! Upwind scheme
   ! -------------

     DO k=1,kmax

     ! u * du/dx

       vel_adv = uz(k,i,j)
       ududx=vel_adv*upwind_duzdx(uz, vel_adv, k, i, j)

     ! v * du/dy

       vel_adv = vz_u(k,i,j)
       vdudy=vel_adv*upwind_duzdy(uz, vel_adv, k, i, j)

     ! u * v * tg(phi) / R

       uv=uz(k,i,j)*vz_u(k,i,j)*tfiusr(i,j)

     ! add all terms
       yz_exp(k,i,j) = yz_exp(k,i,j) - ududx - vdudy + uv

#ifdef key_diagdyn_details
   !   u * du/dx
       diagx_nlt_ududx(k,i,j)=ududx
   !   v * du/dy : fully explicit 
       diagx_nlt_vdudy(k,i,j)=vdudy
#endif

     END DO

   END IF


   ! -------------------------
   ! Vertical term : w * du/dz
   ! -------------------------

   k=1

     wmo=0.25_rsh*(wz(k,i,j)+wz(k,i+1,j))
     wm=0.5_rsh*(wmo-ABS(wmo))
     wdudz=wm*(uz(k+1,i,j)-uz(k,i,j))/dsigw(k)

     !add term
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdudz

#ifdef key_diagdyn_details
   !   w * du/dz
     diagx_nlt_wdudz(k,i,j)=wdudz
#endif

   k=2

     !Using derivative module
      vel_adv = 0.25_rsh*(wz(k,i,j)+wz(k,i+1,j)+wz(k-1,i,j)+wz(k-1,i+1,j))
      wdudz   = vel_adv*upwind_duzdz(uz, vel_adv, dsigw(k-1), dsigw(k), k, i, j)
 
     !add term
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdudz

#ifdef key_diagdyn_details
   !   w * du/dz
     diagx_nlt_wdudz(k,i,j)=wdudz
#endif

   DO k=3,kmax-2

     !Using derivative module
     vel_adv = 0.25_rsh*(wz(k,i,j)+wz(k,i+1,j)+wz(k-1,i,j)+wz(k-1,i+1,j))
     wdudz   = vel_adv*quick_duzdz(uz, vel_adv, dsigw(k), k, i, j)

     !add term
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdudz

#ifdef key_diagdyn_details
   !   w * du/dz
       diagx_nlt_wdudz(k,i,j)=wdudz
#endif

   END DO

   k=kmax-1

     !Using derivative module
     vel_adv = 0.25_rsh*(wz(k,i,j)+wz(k,i+1,j)+wz(k-1,i,j)+wz(k-1,i+1,j))
     wdudz   = vel_adv*upwind_duzdz(uz, vel_adv, dsigw(k-1), dsigw(k), k, i, j)

     !add term
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdudz

#ifdef key_diagdyn_details
   !   w * du/dz
     diagx_nlt_wdudz(k,i,j)=wdudz
#endif

   k=kmax

     wmo=0.25_rsh*(wz(k-1,i+1,j)+wz(k-1,i,j))
     wp=0.5_rsh*(wmo+ABS(wmo))
     wdudz=wp*(uz(k,i,j)-uz(k-1,i,j))/dsigw(k-1)

     !add term
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdudz

#ifdef key_diagdyn_details
   !   w * du/dz
     diagx_nlt_wdudz(k,i,j)=wdudz
#endif

!************** end  including dyn3dx_adv_quick.h    ****************************
