!**************    including dyn3dx_adv_quickest_lans.h      ***********************

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE dyn3dx_adv_quick.h  ***
   !&E
   !&E ** Purpose : estimate of advection terms (3d) of the momentum
   !&E              equation along the x-direction
   !&E
   !&E ** Description : explicit - QUICK scheme (decentre amont)
   !&E
   !&E ** Called by : dyn3dx_tls_coef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : uz,vz_u,tfiusr,wz,yz_exp,dx_u,dx,dy_f,dy_u
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference : Kowalik, Murty "Numerical Modeling of Ocean Dynamics"
   !&E
   !&E ** History :
   !&E       !  2007-09 (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E       !  2009-02 (F. Dumas) Upwind scheme along open boundaries + QUICK at imin+2 and jmin+2
   !&E       !  2009-03  (V. Garnier) land mask
   !&E       !  2011-11  (F. Vandermeirsch) correct spatial extension on which we can estimate quick scheme
   !&E
   !&E--------------------------------------------------------------------- 

   ! ----------------
   ! Horizontal terms
   ! ----------------

#ifdef key_shiftobc_verif
!to get exactly the same results as without any shift of obc
   IF (i >= imin+2+ishift_obc_west .AND. i <= imax-2-ishift_obc_east .AND. &
       j <= jmax-2-jshift_obc_north .AND. j >= jmin+3+jshift_obc_south) THEN
#else
   IF (i >= imin+2 .AND. i <= imax-2 .AND. j <= jmax-2 .AND. j >= jmin+3) THEN
#endif

   ! QUICK scheme
   ! ------------ 

     DO k=1,kmax

      !Non linear terms

!1-Classic
      !us * du/dx

         vel_adv = uzs(k,i,j)
         usdudx  = vel_adv*quickest_duzdx(uz, vel_adv, k, i, j)

      !vs * du/dy

         vel_adv  = vzs_u(k,i,j)
         vsdudy   = vel_adv*quickest_duzdy(uz, vel_adv, k, i, j)

!2-Terms to ensure the Kelvin circulation theorem

         lans_nl  = 0.

#ifdef key_lans_turb
	 lans_nl     = lans_nl_quickest_lon(uz, uzs, vz_u, vzs_u, k, i, j) 
#endif

      !us * vs * tg(phi) / R

       uv=uzs(k,i,j)*vzs_u(k,i,j)*tfiusr(i,j)

      !add all terms

       	 yz_exp(k,i,j) = yz_exp(k,i,j) - usdudx - vsdudy - lans_nl + uv 

#ifdef key_diagdyn_details
   !   us * du/dx
       diagx_nlt_ududx(k,i,j)=usdudx
   !   vs * du/dy : fully explicit 
       diagx_nlt_vdudy(k,i,j)=vsdudy
#endif

     END DO

   ELSE

   ! Upwind scheme
   ! -------------

     DO k=1,kmax

      !Non linear terms

      !1-Classic


	! us * du/dx
        vel_adv = uzs(k,i,j)
        usdudx  = vel_adv*upwind_duzdx(uz, vel_adv, k, i, j)

     	! vs * du/dy
        vel_adv = vzs_u(k,i,j)
        vsdudy  = vel_adv*upwind_duzdy(uz, vel_adv, k, i, j)

     ! u * v * tg(phi) / R

       	uv=uzs(k,i,j)*vzs_u(k,i,j)*tfiusr(i,j)

     ! add all terms

       	yz_exp(k,i,j) = yz_exp(k,i,j) - usdudx - vsdudy  + uv

#ifdef key_diagdyn_details
   !   u * du/dx
       diagx_nlt_ududx(k,i,j)=ududx
   !   v * du/dy : fully explicit 
       diagx_nlt_vdudy(k,i,j)=vdudy
#endif
     END DO

   END IF

   ! -------------------------
   ! Vertical term : ws * du/dz
   ! -------------------------

   k=1

!1-Classic
     wmo=0.25_rsh*(wzs(k,i,j)+wzs(k,i+1,j))
     wm=0.5_rsh*(wmo-ABS(wmo))
     wsdudz=wm*(uz(k+1,i,j)-uz(k,i,j))/dsigw(k)

!2-Terms to ensure the Kelvin circulation theorem


! add term

     yz_exp(k,i,j) = yz_exp(k,i,j) - wsdudz

#ifdef key_diagdyn_details
   !   ws * du/dz
     diagx_nlt_wdudz(k,i,j)=wsdudz
#endif

   k=2

     !ws * du/dz
     vel_adv = 0.25_rsh*(wzs(k,i,j)+wzs(k,i+1,j)+wzs(k-1,i,j)+wzs(k-1,i+1,j))
     wsdudz  = vel_adv*upwind_duzdz(uz, vel_adv, dsigw(k-1), dsigw(k),  k, i, j)


! add term

     yz_exp(k,i,j) = yz_exp(k,i,j) - wsdudz


#ifdef key_diagdyn_details
   !   ws * du/dz
     diagx_nlt_wdudz(k,i,j)=wsdudz
#endif

   DO k=3,kmax-2

   ! ws * du/dz

      vel_adv  = 0.25_rsh*(wzs(k,i,j)+wzs(k,i+1,j)+wzs(k-1,i,j)+wzs(k-1,i+1,j))
      wsdudz   = vel_adv*quickest_duzdz(uz, vel_adv, dsigw(k), k, i, j)

! add term

      yz_exp(k,i,j) = yz_exp(k,i,j) - wsdudz

#ifdef key_diagdyn_details
   !   ws * du/dz
       diagx_nlt_wdudz(k,i,j)=wsdudz
#endif

   END DO

   k=kmax-1

   ! ws * du/dz

     vel_adv = 0.25_rsh*(wzs(k,i,j)+wzs(k,i+1,j)+wzs(k-1,i,j)+wzs(k-1,i+1,j))
     wsdudz  = vel_adv*upwind_duzdz(uz, vel_adv, dsigw(k-1), dsigw(k),  k, i, j)

! add term

     yz_exp(k,i,j) = yz_exp(k,i,j) - wsdudz

#ifdef key_diagdyn_details
   !   ws * du/dz
     diagx_nlt_wdudz(k,i,j)=wdudz
#endif

   k=kmax
     wmo=0.25_rsh*(wzs(k-1,i+1,j)+wzs(k-1,i,j))
     wp=0.5_rsh*(wmo+ABS(wmo))
     wsdudz=wp*(uz(k,i,j)-uz(k-1,i,j))/dsigw(k-1)

! add term

     yz_exp(k,i,j) = yz_exp(k,i,j) - wsdudz

#ifdef key_diagdyn_details
   !   ws * du/dz
     diagx_nlt_wdudz(k,i,j)=wsdudz
#endif

!************** end  including dyn3dx_adv_quickest_lans.h    ****************************
