!*********   including dyn3dx_ipg_cs.h   ***********************************

   !&E---------------------------------------------------------------------
   !&E                 ***  dyn3dx_ipg_cs.h  ***
   !&E
   !&E ** Purpose : estimate the trend term relative to the internal
   !&E              pressure gradient for the zonal momentum forcing
   !&E              (3D equation)
   !&E
   !&E ** Description : explicit
   !&E                  The vertical interpolation with a cubic spline
   !&E                  is done inside dyn3dyx.F90
   !&E                  Here we estimate the pressure gradient itself
   !&E                  Here we estimate the pressure gradient itself
   !&E                  The algorithm is based on the reconstruction of
   !&E                  the density field and the physical z coordinate
   !&E                  as continuous functions of transformed coordinates
   !&E                  with subsequent analytical integration to compute
   !&E                  the pressure-radient force
   !&E
   !&E ** Called by : included into dyn3dxtlscoef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : h0,zr,boz,pr,dx_u
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference :
   !&E         Shchepetkin and Mc Williams, JGR, 2003
   !&E
   !&E ** History :
   !&E       !  2006-01  (V. Garnier)  Original code
   !&E       !  2006-06  (V. Garnier)  inlining
   !&E       !  2007-09  (V. Garnier) homogeneneity of dx dy...
   !&E
   !&E---------------------------------------------------------------------

   !*******************************************
   !     Pressure gradient along component x
   !*******************************************

   DO k = kmax, 1, -1

     IF ( i==ig(j) .OR. h0(i-1,j)<=-valmanq ) THEN
       dxzr = 0.0_rsh
       dxboz = 0.0_rsh
     ELSE
       dxzr  = zr(k,i  ,j)-zr(k,i-1,j)
#ifdef key_profil_back
       dxboz = boz2(k,i  ,j)-boz2(k,i-1,j)
#else
       dxboz = boz(k,i  ,j)-boz(k,i-1,j)
#endif

     END IF

     dxzrp1 = zr(k,i+1,j)-zr(k,i  ,j)
#ifdef key_profil_back
     dxbozp1 = boz2(k,i+1,j)-boz2(k,i  ,j)
#else
     dxbozp1 = boz(k,i+1,j)-boz(k,i  ,j)
#endif


     IF ( i==id(j)-1 ) THEN
       dxzrp2 = 0.0_rsh
       dxbozp2 = 0.0_rsh
     ELSE IF ( h0(i+2,j)<=-valmanq ) THEN
       dxzrp2 = 0.0_rsh
       dxbozp2 = 0.0_rsh
     ELSE
       dxzrp2 = zr(k,i+2,j)-zr(k,i+1,j)
#ifdef key_profil_back
       dxbozp2 = boz2(k,i+2,j)-boz2(k,i+1,j)
#else
       dxbozp2 = boz(k,i+2,j)-boz(k,i+1,j)
#endif
     END IF

     cff = 2.0_rsh*dxzr*dxzrp1
     IF (cff > epsilon) THEN
       dxzr = cff / (dxzr+dxzrp1)
     ELSE
       dxzr = 0.0_rsh
     END IF
     cff = 2.0_rsh*dxzrp1*dxzrp2
     IF (cff > epsilon) THEN
       dxzrp1 = cff / (dxzrp1+dxzrp2)
     ELSE
       dxzrp1 = 0.0_rsh
     END IF

     cff = 2.0_rsh*dxboz*dxbozp1
     IF (cff > epsilon) THEN
       dxboz = cff / (dxboz+dxbozp1)
     ELSE
       dxboz = 0.0_rsh
     END IF
     cff = 2.0_rsh*dxbozp1*dxbozp2
     IF (cff > epsilon) THEN
       dxbozp1 = cff / (dxbozp1+dxbozp2)
     ELSE
       dxbozp1 = 0.0_rsh
     END IF


#ifdef key_profil_back
     tbc = pr(k,i+1,j) - pr(k,i,j) +                                       &
           (   demi*(  (boz2(k,i+1,j)+boz2(k,i,j))*(zr(k,i+1,j)-zr(k,i,j))   &
           - uncinq*( (dxbozp1-dxboz)*(zr(k,i+1,j)-zr(k,i,j)   &
                             -undouz*(dxzrp1+dxzr))            & 
                     -(dxzrp1-dxzr)*(boz2(k,i+1,j)-boz2(k,i,j) &
                             -undouz*(dxbozp1+dxboz)) )  )   )
#else
     tbc = pr(k,i+1,j) - pr(k,i,j) +                                       &
           (   demi*(  (boz(k,i+1,j)+boz(k,i,j))*(zr(k,i+1,j)-zr(k,i,j))     &
           - uncinq*( (dxbozp1-dxboz)*(zr(k,i+1,j)-zr(k,i,j)   &
                             -undouz*(dxzrp1+dxzr))            & 
                     -(dxzrp1-dxzr)*(boz(k,i+1,j)-boz(k,i,j)   &
                             -undouz*(dxbozp1+dxboz)) )  )   )
#endif
#ifdef key_ktest_iniv3d
     ipx(k,i,j)=tbc/dx_u(i,j)
#endif
     yz_exp(k,i,j) = yz_exp(k,i,j)+tbc/dx_u(i,j)

   END DO

!*********** end including dyn3dx_ipg_cs.h *********************************
