!**************    including dyn3dy_adv_quick.h      ***********************

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE dyn3dy_adv_quick.h  ***
   !&E
   !&E ** Purpose : estimate of advection terms (3d) of the momentum
   !&E              equation along the y-direction
   !&E
   !&E ** Description : explicit - QUICK scheme (decentre amont)
   !&E
   !&E ** Called by : dyn3dy_tls_coef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : uz_v,vz,tfivsr,wz,yz_exp,dx_v,dy,dx_f,dy_v
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference : Kowalik, Murty "Numerical Modeling of Ocean Dynamics"
   !&E
   !&E ** History :
   !&E       !  2007-09 (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E       !  2009-02 (F. Dumas) Upwind scheme along open boundaries + QUICK at imin+2 and jmin+2
   !&E       !  2009-03  (V. Garnier) land mask
   !&E       !  2011-11  (F. Vandermeirsch) correct spatial extension on which we can estimate quick scheme
   !&E
   !&E---------------------------------------------------------------------

   ! ----------------
   ! Horizontal terms
   ! ----------------

#ifdef key_shiftobc_verif
!to get exactly the same results as without any shift of obc
   IF (i >= imin+3+ishift_obc_west .AND. i <= imax-2-ishift_obc_east .AND. &
       j <= jmax-2-jshift_obc_north .AND. j >= jmin+2+jshift_obc_south) THEN 
#else
   IF (i >= imin+3 .AND. i <= imax-2 .AND. j <= jmax-2 .AND. j >= jmin+2) THEN 
#endif

   ! QUICK scheme
   ! ------------

     DO k=1,kmax

     ! v * dv/dy


       vel_adv = vz(k,i,j)
       vdvdy   = vel_adv*quick_dvzdy(vz, vel_adv, k, i, j)

     ! u * dv/dx

       vel_adv = uz_v(k,i,j)
       udvdx   = vel_adv*quick_dvzdx(vz, vel_adv, k, i, j)

     ! u**2 * tg(phi) / R

       uv=uz_v(k,i,j)*uz_v(k,i,j)*tfivsr(i,j)

     ! add all terms

       yz_exp(k,i,j) = yz_exp(k,i,j) - vdvdy - udvdx - uv


#ifdef key_diagdyn_details
   !   v * dv/dy
       diagy_nlt_vdvdy(k,i,j)=vdvdy
   !   u * dv/dx
       diagy_nlt_udvdx(k,i,j)=udvdx
#endif

     END DO

   ELSE

   ! Upwind scheme
   ! -------------

     DO k=1,kmax

     ! v * dv/dy

       vel_adv = vz(k,i,j)
       vdvdy=vel_adv*upwind_dvzdy(vz, vel_adv, k, i, j)

     ! u * dv/dx

       vel_adv = uz_v(k,i,j)
       udvdx=vel_adv*upwind_dvzdx(vz, vel_adv, k, i, j)

     ! u**2 * tg(phi) / R

       uv=uz_v(k,i,j)*uz_v(k,i,j)*tfivsr(i,j)

     ! add all terms

       yz_exp(k,i,j) = yz_exp(k,i,j) - vdvdy - udvdx - uv

#ifdef key_diagdyn_details
   !   v * dv/dy
       diagy_nlt_vdvdy(k,i,j)=vdvdy
   !   u * dv/dx
       diagy_nlt_udvdx(k,i,j)=udvdx
#endif

     END DO

   END IF


   ! -------------------------
   ! Vertical term : w * dv/dz
   ! -------------------------

   k=1
     wmo=0.25_rsh*(wz(k,i,j)+wz(k,i,j+1))
     wm=0.5_rsh*(wmo-ABS(wmo))
     wdvdz=wm*(vz(k+1,i,j)-vz(k,i,j))/dsigw(k)
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdvdz

#ifdef key_diagdyn_details
   !   w * dv/dz
     diagy_nlt_wdvdz(k,i,j)=wdvdz
#endif

   k=2
!Direct

     vel_adv = 0.25_rsh*(wz(k,i,j)+wz(k,i,j+1)+wz(k-1,i,j)+wz(k-1,i,j+1))
     wdvdz   = vel_adv*upwind_dvzdz(vz, vel_adv, dsigw(k-1), dsigw(k), k, i, j)

     !add term
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdvdz

#ifdef key_diagdyn_details
   !   w * dv/dz
     diagy_nlt_wdvdz(k,i,j)=wdvdz
#endif

   DO k=3,kmax-2

     vel_adv = 0.25_rsh*(wz(k,i,j)+wz(k,i,j+1)+wz(k-1,i,j)+wz(k-1,i,j+1))
     wdvdz   = vel_adv*quick_dvzdz(vz, vel_adv, dsigw(k), k, i, j)

     !add term
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdvdz

#ifdef key_diagdyn_details
   !   w * dv/dz
     diagy_nlt_wdvdz(k,i,j)=wdvdz
#endif

   END DO

   k=kmax-1

     vel_adv = 0.25_rsh*(wz(k,i,j)+wz(k,i,j+1)+wz(k-1,i,j)+wz(k-1,i,j+1))
     wdvdz   = vel_adv*upwind_dvzdz(vz, vel_adv, dsigw(k-1), dsigw(k), k, i, j)

     !add term
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdvdz

#ifdef key_diagdyn_details
   !   w * dv/dz
     diagy_nlt_wdvdz(k,i,j)=wdvdz
#endif

   k=kmax
     wmo=0.25_rsh*(wz(k-1,i,j)+wz(k-1,i,j+1))
     wp=0.5_rsh*(wmo+ABS(wmo))
     wdvdz=wp*(vz(k,i,j)-vz(k-1,i,j))/dsigw(k-1)
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdvdz

#ifdef key_diagdyn_details
   !   w * dv/dz
     diagy_nlt_wdvdz(k,i,j)=wdvdz
#endif

!*****************     end  including dyn3dy_adv_quick.h      ******************
