!*********   including dyn3dy_ipg_cs.h  ************************************ 

   !&E---------------------------------------------------------------------
   !&E                 ***  dyn3dy_ipg_cs.h  ***
   !&E
   !&E ** Purpose : estimate the trend term relative to the internal
   !&E              pressure gradient for the meridional momentum forcing
   !&E              (3D equation)
   !&E
   !&E ** Description : explicit
   !&E                  The vertical interpolation with a cubic spline
   !&E                  is done inside dyn3dyx.F90
   !&E                  Here we estimate the pressure gradient itself 
   !&E                  The algorithm is based on the reconstruction of
   !&E                  the density field and the physical z coordinate
   !&E                  as continuous functions of transformed coordinates
   !&E                  with subsequent analytical integration to compute
   !&E                  the pressure-gradient force
   !&E
   !&E ** Called by : included into dyn3dytlscoef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : h0,zr,boz,pr,dx_v
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference :
   !&E         Shchepetkin and Mc Williams, JGR, 2003
   !&E
   !&E ** History :
   !&E       !  2006-01  (V. Garnier)  Original code
   !&E       !  2006-06  (V. Garnier)  inlining
   !&E       !  2007-09  (V. Garnier) homogeneneity of dx dy...
   !&E
   !&E---------------------------------------------------------------------

   !*******************************************
   !     Pressure gradient along component x
   !*******************************************

   DO k = kmax, 1, -1

     IF ( j==jb(i) .OR. h0(i,j-1)<=-valmanq ) THEN
       dyzr  = 0.0_rsh
       dyboz = 0.0_rsh
     ELSE
       dyzr  = zr(k,i,j)-zr(k,i,j-1)
#ifdef key_profil_back
       dyboz = boz2(k,i,j)-boz2(k,i,j-1)
#else
       dyboz = boz(k,i,j)-boz(k,i,j-1)
#endif
     END IF

     dyzrp1  = zr(k,i,j+1)-zr(k,i,j)
#ifdef key_profil_back
     dybozp1 = boz2(k,i,j+1)-boz2(k,i,j)
#else
     dybozp1 = boz(k,i,j+1)-boz(k,i,j)
#endif

     IF ( j==jh(i)-1 ) THEN
       dyzrp2  = 0.0_rsh
       dybozp2 = 0.0_rsh
     ELSE IF (h0(i,j+2)<=-valmanq ) THEN
       dyzrp2  = 0.0_rsh
       dybozp2 = 0.0_rsh
     ELSE
       dyzrp2  = zr(k,i,j+2)-zr(k,i,j+1)
#ifdef key_profil_back
       dybozp2 = boz2(k,i,j+2)-boz2(k,i,j+1)
#else
       dybozp2 = boz(k,i,j+2)-boz(k,i,j+1)
#endif
     END IF

     cff = 2.0_rsh*dyzr*dyzrp1
     IF (cff > epsilon) THEN
       dyzr = cff / (dyzr+dyzrp1)
     ELSE
       dyzr = 0.0_rsh
     ENDIF
     cff = 2.0_rsh*dyzrp1*dyzrp2
     IF (cff > epsilon) THEN
       dyzrp1 = cff / (dyzrp1+dyzrp2)
     ELSE
       dyzrp1 = 0.0_rsh
     END IF

     cff = 2.0_rsh*dyboz*dybozp1
     IF (cff > epsilon) THEN
       dyboz   = cff / (dyboz+dybozp1)
     ELSE
       dyboz   = 0.0_rsh
     ENDIF
     cff = 2.0_rsh*dybozp1*dybozp2
     IF (cff > epsilon) THEN
       dybozp1 = cff / (dybozp1+dybozp2)
     ELSE
       dybozp1 = 0.0_rsh
     ENDIF

#ifdef key_profil_back
     tbc = pr(k,i,j+1) - pr(k,i,j) +                                       &
           (   demi*(  (boz2(k,i,j+1)+boz2(k,i,j))*(zr(k,i,j+1)-zr(k,i,j))   &
           - uncinq*( (dybozp1-dyboz)*(zr(k,i,j+1)-zr(k,i,j)   &
                             -undouz*(dyzrp1+dyzr))            & 
                     -(dyzrp1-dyzr)*(boz2(k,i,j+1)-boz2(k,i,j) &
                             -undouz*(dybozp1+dyboz)) )  )   )
#else
     tbc = pr(k,i,j+1) - pr(k,i,j) +                                       &
           (   demi*(  (boz(k,i,j+1)+boz(k,i,j))*(zr(k,i,j+1)-zr(k,i,j))     &
           - uncinq*( (dybozp1-dyboz)*(zr(k,i,j+1)-zr(k,i,j)   &
                             -undouz*(dyzrp1+dyzr))            & 
                     -(dyzrp1-dyzr)*(boz(k,i,j+1)-boz(k,i,j)   &
                             -undouz*(dybozp1+dyboz)) )  )   )
#endif
#ifdef key_ktest_iniv3d
     ipy(k,i,j)=tbc/dy_v(i,j)
#else
     yz_exp(k,i,j) = yz_exp(k,i,j)+tbc/dy_v(i,j)
#endif

   END DO

!*********** end including dyn3dy_ipg_cs.h *******************************
