   !!======================================================================

  SUBROUTINE    dyn3dyx(v,      &        !input predictor barotropic meridian velocity
                        u,      &        !input predictor barotropic zonal velocity
#if defined key_lans_turb || defined key_leray_turb
                        vs,     &       !input zonal smooth barotropic velocity
                        us,     &       !input meridian smooth barotropic velocity
#endif
                        xeu,    &       !input intermediate sea surface height

                        uv,     &       !output corrector barotropic meridian velocity
#if defined key_lans_turb || defined key_leray_turb
                        uvs,    &       !output corrector barotropic meridian smooth velocity
#endif
                        uv2,    &       !output corrector barotropic zonal velocity
#if defined key_lans_turb || defined key_leray_turb
                        uv2s,   &       !output corrector barotropic zonal smooth velocity
#endif
                        xeuv,   &       !output final sea surface height

                        vz,     &       !input predictor and output corrector 3d meridian velocity
#if defined key_lans_turb || defined key_leray_turb
                        vzs,    &       !input predictor and output corrector 3d meridian smooth velocity
#endif
                        uz,     &       !input predictor and output corrector 3d zonal velocity
#if defined key_lans_turb || defined key_leray_turb
                        uzs,    &       !input predictor and output corrector 3d smooth zonal velocity
#endif
                        boz     &       !input buoyancy
                       )

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE dyn3dyx  ***
   !&E
   !&E ** Purpose : resolution of Saint-Venant equations (barotropic flow)
   !&E              and of 3D momentum equations (barotropic + baroclinic)
   !&E
   !&E ** Description : estimate of uz,vz and xeuv. xeuv and vz are the last
   !&E                  estimates and balance each other.
   !&E
   !&E ** Called by : step3dyx.F90
   !&E
   !&E ** External calls : turb_smago,dyn3d_height_mask,
   !&E                     obc_apply_u,obc_apply_uz,obc_apply_v,obc_apply_vz
   !&E                     dyn2dy_tls_obc,dyn2dy_tls_solve,dyn2dy_cyc_solve
   !&E                     dyn2dx_tls_obc,dyn2dx_tls_solve,dyn2dx_cyc_solve
   !&E                     dyn3dy_tls_coef,dyn3dy_ztls_solve,dyn3dy_epg
   !&E                     dyn3dx_tls_coef,dyn3dx_ztls_solve,dyn3dx_epg
   !&E
   !&E ** Used ij-arrays : igdu,igdv,igdu_old,xeu,u,v,boz
   !&E
   !&E ** Modified variables : uv,xeuv,uv2,uz,vz,ustar,igdv_old,xeint
   !&E
   !&E ** Reference :
   !&E                T. Duhaut, M. Honnorat, L. Debreu, 2008 : Rapport
   !&E                contrat PREVIMER - Ref : 06/2 210 290 (pages 100-125)
   !&E
   !&E ** History :
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E       !  2008-08  (T. Odaka) MPI
   !&E       !  2008-08  (T. Duhaut) time scheme Rq : schema des traceurs
   !&E                               semble instable si u est decomposee en
   !&E                               alpha u et (alpha - 1) u
   !&E       !  2009-04  (A. Mano V. Garnier) MPI2D
   !&E       !  2009-04  (F. Dumas) generalized sigma
   !&E       !  2009-05  (F. Dumas)  AGRIF to new time scheme
   !&E       !  2009-05  (F. Vandermeirsch) characteristics method
   !&E       !  2009-07  (F. Vandermeirsch)  MC 2nd order scheme
   !&E       !  2009-07  (V. Garnier) MPI2d for characteristics method
   !&E       !  2010-04  (T. Odaka) MPI2D cyclic
   !&E       !  2010-07  (L. Debreu, F. Dumas) rewritting of ADI
   !&E       !  2010-09  (L. Debreu, F. Dumas) FCT to avoid negative water depths if wet-drying
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E       !  2011-12  (S. Petton, F. Dumas) Implicitation of the bottom stress (consistence between 2D en 3D)
   !&E                                         removing of min,max
   !&E       !  2012-02  (F. Vandermeirsch) Complement of previous evolution for key_no_bottom_stress
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE bottom_stress
   USE interpolation_methods
   USE derivative_methods
   USE advection_methods
#if defined key_lans_turb || defined key_leray_turb
   USE extendfield
   USE smoothing
#endif

   USE comcolig2d,    ONLY : igdu_old,igdv_old,igdu,igdv,av,bv,cv,y2dv
#if defined key_lans_turb || defined key_leray_turb
   USE comcolig2d, ONLY   : uz2d, vz2d, uzs2d, vzs2d
#endif
#if defined key_lans_turb || defined key_leray_turb
   USE comcolig2d, ONLY   : uv_ext, uv2_ext
#endif
!MBo#ifdef key_lans_turb
#if defined key_lans_turb || defined key_leray_turb
   USE comcolig2d, ONLY   : y2dv_aux, y2dv_moy, y2dvs
#endif
   USE comvars2d,     ONLY : hex,hey,h0,surf,dx_v,dy_u,hty,z0b
   USE comvars2d,     ONLY : halfdt
   USE comvars2d,     ONLY : l_obc_cycl_x,l_obc_cycl_y,l_smagor
   USE comvars2d,     ONLY : jb,jh,ig,id,dx_u,fwet,fwetp
   USE comvars2d,     ONLY : frot_lin
   USE comfluxdo,     ONLY : fxeau,fyeau,phieau
#if defined key_lans_turb || defined key_leray_turb
   USE comfluxdo,     ONLY : fxeau_lans,fyeau_lans
#endif
   USE comcolig3d,    ONLY : intz_exp,cdvit,intz_exp_ustar,intz_exp_vstar,uz_v
#if defined key_lans_turb || defined key_leray_turb
   USE comcolig3d,   ONLY : uzs_v, vz_u
#endif
   USE comvars3d,     ONLY : dsigu,sig
#ifdef key_lans_turb
  USE comcolig3d, ONLY : dzdx, dzdy
#endif
#if defined key_siggen || defined key_gencoord
   USE comsiggen,     ONLY : hzex,hzey,hzuty
#endif
#ifdef key_wave_qeuler_dynamics
   USE comwave,      ONLY : USX_u,USY_v
#endif
   USE_MPI toolmpi,   ONLY : ex_j_int,ex_j_rsh,ex_j_rlg
   USE_MPI toolmpi,   ONLY : ex_i_rsh,ex_i_rlg,ex_peri_y_rsh,ex_peri_x_rsh,ex_i_int
   USE_MPI toolmpi,   ONLY : ex_peri_x_avcv,ex_peri_y_avcv
   USE_MPI comcolig_mpi,  ONLY : transpose_col,trback_col,transpose_row,trback_row
   USE_MPI comcolig_mpi,  ONLY : cy2dxe,caxe,cbxe,ccxe,cxeuv
   USE_MPI comcolig_mpi,  ONLY : ry2dxe,raxe,rbxe,rcxe,rxeuv

   USE obcapplyu
   USE obcapplyv
   USE obcapplyuz
   USE obcapplyvz

   USE obccombine,   ONLY : l_obc_char,l_obc_north,l_obc_south,l_obc_east,l_obc_west

   USE_MPI toolmpi, ONLY: ADD_TAG,min_all_mpi_rsh,max_all_mpi_rsh

   IMPLICIT NONE

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2)     ,INTENT(in) :: xeu


   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2)     ,INTENT(in) :: u,v
#if defined key_lans_turb || defined key_leray_turb
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2)     ,INTENT(in) :: us, vs
#endif
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2)     ,INTENT(inout) :: xeuv

   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2)     ,INTENT(inout) :: uv,uv2
#if defined key_lans_turb || defined key_leray_turb
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2)     ,INTENT(inout) :: uvs,uv2s
#endif
   REAL(KIND=rsh),DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(inout) :: vz, uz
#if defined key_lans_turb || defined key_leray_turb
   REAL(KIND=rsh),DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(inout) :: vzs, uzs
#endif
   REAL(KIND=rsh),DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in   ) :: boz
   !! * Local declarations
   LOGICAL                                  :: Agrif_Root_OMP,Agrif_GrandMother_OMP,bidon
   INTEGER                                  :: i,j,k,i1,i2,j1,j2,test
   REAL(KIND=rsh)                           :: umoy
#if defined key_lans_turb || defined key_leray_turb
   REAL(KIND=rsh)                           :: usmoy
#endif
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2) :: xeint

   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2) :: ustar 

#if defined key_lans_turb || defined key_leray_turb
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2) :: us_star
   REAL(KIND=rsh),DIMENSION(liminm4:limaxp4,ljminm4:ljmaxp4) :: ustar_ext
   REAL(KIND=rlg), DIMENSION(1:ijdu,1:ijdu) :: MatHelm
#endif
   REAL(KIND=rsh),DIMENSION(limin:limax)                     :: tfh,tfb
   REAL(KIND=rsh),DIMENSION(ljmin:ljmax)                     :: tfg,tfd
#ifdef key_dyn_wetdry_fct
   REAL(KIND=rlg)                           :: A
   REAL(KIND=rlg),DIMENSION(limin:limaxp1,ljmin:ljmaxp1) :: phimax
#endif
   REAL(kind=rsh)  :: vitf,cdf

   REAL(KIND=rsh)  :: value

!MBo#ifdef key_lans_turb
#if defined key_lans_turb || defined key_leray_turb
   REAL(kind=rsh)  :: ft, u_vtmp, v_utmp, alpha_lans
#endif

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter DYN3DYX routine'

#ifdef key_MPI_2D
   ! To make sur AGRIF conv will take into account these required variables
   INST_AGRIF bidon = allocated(ry2dxe)
   INST_AGRIF bidon = allocated(raxe)
   INST_AGRIF bidon = allocated (rbxe)
   INST_AGRIF bidon = allocated (rcxe)
   INST_AGRIF bidon = allocated(cy2dxe)
   INST_AGRIF bidon = allocated(caxe)
   INST_AGRIF bidon = allocated(cbxe)
   INST_AGRIF bidon = allocated(ccxe)
#endif

! because the variables (pointer?) used in Agrif_Root function are not seen by all the cpus in OMP
   Agrif_Root_OMP = agrif_root()
   INST_AGRIF Agrif_GrandMother_OMP = agrif_grandmother()

!$OMP PARALLEL default(none) shared(xeu,igdu,igdv,tfb,tfh,l_smagor   &
!$OMP&,u,uz,vz,boz,v,ig,id,jb,jh,uv2,xeuv,uv,dx_u,xeint              &
!$OMP&,ustar,hex,hey,igdu_old,igdv_old,tfg,tfd,intz_exp              &
!$OMP&,limin,ljmax,ljmin,limax,av,bv,cv,y2dv                         &
#ifdef key_lans_turb
!$OMP& y2dvs                                                         &
#endif
!$OMP&,ljminm1,ljminp1,ljminp2,ljminm2,ljmaxp2,ljmaxm1               &
!$OMP&,liminm1,liminp1,liminp2,liminm2,limaxp2,limaxm1               &
!$OMP&,hty,z0b,cdvit,intz_exp_ustar,intz_exp_vstar                   &
#ifdef key_MPI_2D
!$OMP& ,ry2dxe,raxe,rbxe,rcxe,rxeuv,cy2dxe,caxe,cbxe,ccxe,cxeuv      &

#ifdef key_dyn_wetdry_fct
!$OMP& ,ljmaxp1,limaxp1                                              &
#endif
#endif
#ifdef key_dyn_wetdry_fct
!$OMP&,phieau                                                        &
#endif
!$OMP& ,l_obc_char,l_obc_cycl_x,l_obc_cycl_y,fxeau,fyeau             &
!$OMP& ,l_obc_north,l_obc_south,l_obc_east,l_obc_west                &
#if defined key_lans_turb || defined key_leray_turb
!$OMP& ,fxeau_lans,fyeau_lans                                        &
#endif
#if defined key_siggen || defined key_gencoord
!$OMP& ,hzex,hzey,hzuty                                              &
#else
!$OMP& ,sig                                                          &
#endif
#ifdef key_dyn_wetdry_fct
!$OMP& ,phimax,h0,surf,dx_v,dy_u,fwet,fwetp                          &
#endif
#if key_linear_bottom_stress
!$OMP& ,frot_lin                                                     &
#endif
!$OMP& ,halfdt,dsigu,test)                                           &
#ifdef key_agrif
!$OMP& SHARED(imin,imax,jmin,jmax)                                   &
!$OMP& FIRSTPRIVATE(Agrif_GrandMother_OMP)                           &
#endif
!$OMP& FIRSTPRIVATE(Agrif_Root_OMP)                                  &
!$OMP& PRIVATE(umoy,uz_v,vitf,cdf,                                   &
#ifdef key_dyn_wetdry_fct
!$OMP& A,                                                            &
#endif
!$OMP& i,j,j1,j2,i1,i2,k)

   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   !                                                                  !
   !                             Part I                               !
   ! Estimate of intermediate variables :                             !
   !                         XE^n+1*, v_i^n+1, and vz_i^n+1           !
   !                                                                  !
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   !-----------------------------------------------------------------------
   ! Calculation of heights hex and hey
   ! used in continuity equation (and for tracers advection)
   ! Calculation of heights htx and hty, hzuty and hzuty, hzwty and hzwty
   ! used in momentum equation
   ! Definition of the type of cell (igdu and igdv),
   ! the type of extremity of column used in dyn2dytlsobc (tfd,tfg,tfb,tfh)
   ! the mask used for the viscosity term (mo and me)
   !-----------------------------------------------------------------------

   test = 2

   CALL dyn3d_height_mask(xeu,tfg,tfd,tfb,tfh,test)

#if defined key_lans_turb || defined key_leray_turb
     IF (smooth_method == 1) THEN

             igdu_ext(limin:limax, ljmin:ljmax) = igdu(limin:limax, ljmin:ljmax)
             igdv_ext(limin:limax, ljmin:ljmax) = igdv(limin:limax, ljmin:ljmax)

             CALL_MPI ex_i_int(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,igdu_ext(liminm4:limaxp4,ljminm4:ljmaxp4))
             CALL_MPI ex_j_int(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,igdu_ext(liminm4:limaxp4,ljminm4:ljmaxp4))

             CALL_MPI ex_i_int(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,igdv_ext(liminm4:limaxp4,ljminm4:ljmaxp4))
             CALL_MPI ex_j_int(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,igdv_ext(liminm4:limaxp4,ljminm4:ljmaxp4))

             CALL smoothuInd(Indu)
             CALL smoothvInd(Indv)
     END IF
#endif

   IF_AGRIF ( .NOT. Agrif_GrandMother_OMP) THEN

   ! Estimate of viscosity coefficients
   ! (smagorinsky formulation)
   !-----------------------------------

!   IF ( l_smagor .AND. mod(nbouc,5) == 3 ) THEN    ! ATTENTION MPI si utilise
   IF ( l_smagor ) THEN

     CALL turb_smago(uz,vz,igdv)

   END IF

   ! Estimate of ustar :
   ! Add the external pressure gradient to Ustar to complete Ustar estimate.
   ! Finaly, Ustar contains the verticaly integrated right hand side
   ! explicitly evaluated in stepuv and involving the use
   ! of uz^n, vz^n and u_i^n+1/2 :
   ! u*^n+1 = u_i^n+1/2 + RHS(uz^n,vz^n) - gdt/2*d(xe^n+1/2)/dx
   !---------------------------------------------------------


!$OMP DO SCHEDULE(RUNTIME)
   !DO j=ljminp2,ljmaxm1
     !DO i=MAX0(limin,ig(j)),MIN0(limax,id(j)-1)
! strange, the following lines should be sufficient
   DO j=ljmin,ljmax
     DO i=limin,limax
       IF (igdu(i,j)==1 .AND. igdu_old(i,j)==1) THEN
         ustar(i,j) = u(i,j)+halfdt*(intz_exp_ustar(i,j) - 1.0_rlg*grav*forward_dxedx(xeu, i, j) mpext)
      ELSE
         ustar(i,j) = u(i,j)
       END IF
     END DO
   END DO
!$OMP END DO

   ! At the beginning Ustar contains nothing so we forced it to u
   !-------------------------------------------------------------
! comment out this line if you exchange the calling order of stepuv and stepvu
!   IF ( nbouc==1)  ustar(:,:)=u(:,:)

   ! Apply open boundary conditions to ustar
   !---------------------------------------- 

OMPAGRIF BARRIER
OMPAGRIF MASTER
   IF (l_obc_char) THEN
     CALL obc_apply_u_mc(u,ustar,v,v,xeu,xeuv,.FALSE.)
   ELSE
     CALL obc_apply_u(ustar)
   END IF
OMPAGRIF END MASTER
OMPAGRIF BARRIER
OMPAGRIF FLUSH(ustar)

#if defined key_lans_turb || defined key_leray_turb
   CALL extend_u(ustar, ustar_ext)
! smooth ustar
!!-------------
   CALL smoothu(ustar_ext, us_star)
! Apply open boundary conditions to us_star
!----------------------------------------
OMPAGRIF BARRIER
OMPAGRIF MASTER
     CALL obc_apply_u(us_star)
OMPAGRIF END MASTER
OMPAGRIF BARRIER
OMPAGRIF FLUSH(us_star)
#endif


   ! Construction of 2D and 3D matrixes for calculation along y
   ! 1st part : estimate of right hand side :
   ! RHS(uz_i^n+1/2,vz^n+1/2) for NL advection, internal pressure gradient,
   !                    horizontal viscosity, atmospherical forcing
   !                    bottom friction (2d part only)
   ! RHS(baroclinic uz_i^n+1/2,ustar) for coriolis term
   ! 2nd part : estimate of tridiagonal linear system coefficients :
   ! av,bv,cv,y2dv,axe,bxe,cxe,y2dxe
   !--------------------------------------------------------------------

   !CALL dyn3dy_tls_coef(vz,boz,uz,u,v,xeu,ustar)

   CALL dyn3dy_tls_coef(vz,        &
#if defined key_lans_turb || defined key_leray_turb
                        vzs,       &
#endif
                        boz,       &
                        uz,        &
#if defined key_lans_turb || defined key_leray_turb
                        uzs,       &
#endif
                        u,         &
                        v,         &
#if defined key_lans_turb || defined key_leray_turb
                        us,        &
                        vs,        &  
#endif
                        xeu,       &

#if defined key_lans_turb || defined key_leray_turb
                        us_star,   &
#endif
                        ustar      &
                        )

#if defined key_lans_turb || defined key_leray_turb
!Exchange y2dv_aux
!-----------------------------------------------------------------------
OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rlg(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,y2dv_aux(liminm4:limaxp4,ljminm4:ljmaxp4))
   CALL_MPI ex_j_rlg(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,y2dv_aux(liminm4:limaxp4,ljminm4:ljmaxp4))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(y2dv_aux)

!Smooth y2dv_aux
!--------------------------------------------------------------------
   CALL smoothv_y2dv(y2dv_aux, y2dvs)

!$OMP DO SCHEDULE(RUNTIME)
   DO j=ljminp1,ljmaxm1
     DO i=MAX0(limin,2+ishift_obc_west), MIN0(limax,imax-1-ishift_obc_east)
            IF (igdv(i,j)/=4) THEN 
                       IF (igdv(i,j)==1) THEN
                               ft=halfdt*cdvit(i,j)/hty(i,j)
                               y2dv(i,j)  = y2dv_aux(i,j) + ft*v(i,j)
                               y2dvs(i,j) = y2dvs(i,j)    + ft*vs(i,j)
                               y2dv_moy(i,j)  = gam*y2dv(i,j) + gams*y2dvs(i,j)
                       ELSE IF (igdv(i,j)==5) THEN
                               y2dv(i,j)  = y2dv_aux(i,j)
                               y2dvs(i,j) = y2dv(i,j)
                               y2dv_moy(i,j)  = gam*y2dv(i,j) + gams*y2dvs(i,j)
                       END IF
           END IF
      END DO
   END DO
!$OMP END DO
#endif

   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   ! Solve the barotropic system along y !
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   !-----------------------------------------------------------------------
   ! Add the open boundary conditions to the right hand side
   !-----------------------------------------------------------------------

   IF ( .NOT.l_obc_cycl_y .AND. (l_obc_north .OR. l_obc_south) ) THEN
     IF_MPI ( jmin==ljmin .OR. jmax==ljmax ) THEN
!$OMP DO SCHEDULE(dynamic,2)
     DO i=liminp2,limaxm1
       !!     introduction des conditions aux limites en niveaux
       !!     liees a l implicitation du gradient de pression
       !!     Modification de coefficient
       j1=jb(i)
       j2=jh(i)
       CALL dyn2dy_tls_obc(j1,j2,i,tfb(i),tfh(i),v,xeu,xeuv)
     END DO
!$OMP END DO
     ENDIF_MPI
   END IF

   !------------------------------------------------------------------
   ! Resolve the 2D tridiagonal system to evaluate
   ! an intermediate estimate of the meridional velocity : V_i^n+1
   ! and a first guess for the free surface: XE^n+1,*
   !------------------------------------------------------------------

   ! pre solve
   ! ---------
   CALL dyn2dy_tls_presolve

   ! ----------------------------
   ! solve the sea surface height
   ! ----------------------------

   OMPMPI barrier
   OMPMPI master
   ! echange throughout the columns for lines from ciminp2 to cimaxm1
   CALL_MPI transpose_col
   OMPMPI end master
   OMPMPI barrier
   OMPMPI flush(cy2dxe,caxe,cbxe,ccxe)

#ifdef key_MPI_2D
   IF (l_obc_cycl_y) THEN
     CALL dyn2dy_cyc_solve(cxeuv)
   ELSE
     CALL dyn2dy_tls_solve(cxeuv)
   END IF
#else
   IF (l_obc_cycl_y) THEN
     CALL dyn2dy_cyc_solve(xeuv)
   ELSE
     CALL dyn2dy_tls_solve(xeuv)
   END IF
#endif

OMPMPI barrier
OMPMPI master
   ! exchange back to each CPU (limin:limax and ljmi:ljmax)
   CALL_MPI trback_col(xeuv)
OMPMPI end master
OMPMPI barrier
OMPMPI flush(xeuv)

   !---------------------------------------------------------
   ! Apply open boundary conditions to the sea surface height
   !---------------------------------------------------------

!$OMP SINGLE
   IF (l_obc_cycl_y) THEN
#ifdef key_MPI_2D
#ifdef key_shiftobc_curv
     CALL_MPI ex_peri_y_rsh(3,1,liminm2,limaxp2,ljminm2,ljmaxp2,xeuv(liminm2:limaxp2,ljminm2:ljmaxp2))

#else
     CALL_MPI ex_peri_y_rsh(2,1,liminm2,limaxp2,ljminm2,ljmaxp2,xeuv(liminm2:limaxp2,ljminm2:ljmaxp2))
#endif
#else
     DO i=limin,limax ! changed from DO i=imin+2,imax-1
       xeuv(i,jb(i)-1)=xeuv(i,jh(i)-2)
       xeuv(i,jb(i)  )=xeuv(i,jh(i)-1)
       xeuv(i,jh(i)  )=xeuv(i,jb(i)+1)
#ifdef key_shiftobc_curv
       xeuv(i,jb(i)-2)=xeuv(i,jh(i)-3)
       xeuv(i,jh(i)+1)=xeuv(i,jb(i)+2)
       xeuv(i,jh(i)+2)=xeuv(i,jb(i)+3)
#endif
     END DO
#endif
   END IF
   IF (l_obc_cycl_x) THEN
#ifdef key_MPI_2D
#ifdef key_shiftobc_curv
     CALL_MPI ex_peri_x_rsh(3,1,liminm2,limaxp2,ljminm2,ljmaxp2,xeuv(liminm2:limaxp2,ljminm2:ljmaxp2))
#else
     CALL_MPI ex_peri_x_rsh(2,1,liminm2,limaxp2,ljminm2,ljmaxp2,xeuv(liminm2:limaxp2,ljminm2:ljmaxp2))
#endif
#else
     DO j=ljmin,ljmax
       xeuv(ig(j)-1,j)=xeuv(id(j)-2,j)
       xeuv(ig(j)  ,j)=xeuv(id(j)-1,j)
       xeuv(id(j)  ,j)=xeuv(ig(j)+1,j)
#ifdef key_shiftobc_curv
       xeuv(ig(j)-2,j)=xeuv(id(j)-3,j)
       xeuv(id(j)+1,j)=xeuv(ig(j)+2,j)
       xeuv(id(j)+2,j)=xeuv(ig(j)+3,j)
#endif
     END DO
#endif
   END IF
!$OMP END SINGLE

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,xeuv(liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL_MPI ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,xeuv(liminm2:limaxp2,ljminm2:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(xeuv)

#ifdef key_infos_adi_step
!   WRITE(*,*)"min_xe_corr1=",MINVAL(xeuv)
#endif

   ! --------------------------------------------------
   ! post solve : estimate of the barotropic velocity V
   ! --------------------------------------------------

!$OMP DO SCHEDULE(RUNTIME)
   DO j=ljmin,ljmax
     DO i=liminp2,limaxm1
       IF (j.GE.jb(i) .AND. j.LE.jh(i)-1) THEN
         IF (ljmin==jmin) av(i,jb(i))=0.0_rlg
         IF (ljmax==jmax) cv(i,jh(i)-1)=0.0_rlg
         uv(i,j)  = (y2dv(i,j)  - (cv(i,j)*xeuv(i,j+1)+av(i,j)*xeuv(i,j)))*bv(i,j)
#if defined key_lans_turb || defined key_leray_turb
         uvs(i,j) = (y2dvs(i,j) - (cv(i,j)*xeuv(i,j+1)+av(i,j)*xeuv(i,j)))*bv(i,j)
#endif
       END IF
     END DO
   END DO
!$OMP END DO

!$OMP SINGLE
   IF (l_obc_cycl_y) THEN

     CALL_MPI ex_peri_y_avcv(liminm1,limax,ljminm1,ljmax,av(liminm1:limax,ljminm1:ljmax),cv(liminm1:limax,ljminm1:ljmax))

     IF_MPI (ljmax == jmax) THEN
     DO i=liminp2,limaxm1
#ifdef key_MPI_2D
       uv(i,jh(i)-1)  = uv(i,jh(i)-1) - cv(i,jh(i)-1)*xeuv(i,jh(i))*bv(i,jh(i)-1)
#if defined key_lans_turb || defined key_leray_turb
       uvs(i,jh(i)-1) = uvs(i,jh(i)-1)  - cv(i,jh(i)-1)*xeuv(i,jh(i))*bv(i,jh(i)-1)
#endif
#else
       uv(i,jh(i)-1)  = uv(i,jh(i)-1)   - cv(i,jb(i))*xeuv(i,jh(i))*bv(i,jh(i)-1)
#if defined key_lans_turb || defined key_leray_turb
       uvs(i,jh(i)-1) = uvs(i,jh(i)-1) - cv(i,jb(i))*xeuv(i,jh(i))*bv(i,jh(i)-1)
#endif
#endif
     END DO
     ENDIF_MPI

     IF_MPI (ljmin == jmin) THEN
     DO i=liminp2,limaxm1
#ifdef key_MPI_2D
       uv(i,jb(i))  = uv(i,jb(i))  - av(i,jb(i))*xeuv(i,jb(i))*bv(i,jb(i))
#if defined key_lans_turb || defined key_leray_turb
       uvs(i,jb(i)) = uvs(i,jb(i)) - av(i,jb(i))*xeuv(i,jb(i))*bv(i,jb(i))
#endif
#else
       uv(i,jb(i))  = uv(i,jb(i))  - av(i,jh(i)-1)*xeuv(i,jb(i))*bv(i,jb(i))
#if defined key_lans_turb || defined key_leray_turb
       uvs(i,jb(i)) = uvs(i,jb(i)) - av(i,jh(i)-1)*xeuv(i,jb(i))*bv(i,jb(i))
#endif
#endif
     END DO
     ENDIF_MPI
   END IF

!$OMP END SINGLE

   !------------------------------------------------------
   ! apply open boundary conditions to barotropic velocity
   !------------------------------------------------------

OMPAGRIF BARRIER
OMPAGRIF MASTER
   IF (l_obc_char) THEN
     IF (.NOT.l_obc_cycl_y .AND. Agrif_Root_OMP) THEN
       CALL obc_apply_v_mc(u,ustar,v,uv,xeu,xeuv,.TRUE.)
#if defined key_lans_turb || defined key_leray_turb
       CALL obc_apply_v_mc(us,us_star,vs,uvs,xeu,xeuv,.TRUE.)
#endif
     END IF
     IF_AGRIF (.NOT.l_obc_cycl_y .AND. .NOT.Agrif_Root_OMP) CALL agrif_obc_v(uv)
#if defined key_lans_turb || defined key_leray_turb
     IF_AGRIF (.NOT.l_obc_cycl_y .AND. .NOT.Agrif_Root_OMP) CALL agrif_obc_v(uvs)
#endif
   ELSE
     IF_AGRIF (.NOT. Agrif_Root_OMP) CALL agrif_obc_v(uv)
#if defined key_lans_turb || defined key_leray_turb
    IF_AGRIF (.NOT. Agrif_Root_OMP) CALL agrif_obc_v(uvs)
#endif
     IF (Agrif_Root_OMP) THEN
        CALL obc_apply_v(uv)
#if defined key_lans_turb || defined key_leray_turb
        CALL obc_apply_v(uvs)
#endif
     END IF
   END IF
OMPAGRIF END MASTER
OMPAGRIF BARRIER
OMPAGRIF FLUSH(uv)
#if defined key_lans_turb || defined key_leray_turb
OMPAGRIF FLUSH(uvs)
#endif

!#if defined key_lans_turb || defined key_leray_turb
! extend for parallelization
!----------------------------------------
!   CALL extend_v(uv, uv_ext)
! smooth ustar
!----------------------------------------
!   CALL smoothv(uv_ext, uvs)
!
! Apply open boundary conditions to ustar
!----------------------------------------
!OMPAGRIF BARRIER
!OMPAGRIF MASTER
!     CALL obc_apply_v(uvs)
!OMPAGRIF END MASTER
!OMPAGRIF BARRIER
!OMPAGRIF FLUSH(uvs)
!#endif

#ifdef key_infos_adi_steps
!   WRITE(*,*)"max_v_corr=",MAXVAL(uv)
#endif
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   ! Intermediate estimate of the 3d meridional velocity              !
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   ! Add the external pressure gradient (using XE^n+1,*)
   ! to right hand side of the meridional momentum equation
   !--------------------------------------------------------

   ! 28- gradient de pression externe a l instant n
   CALL dyn3dy_epg(xeu,xeuv)

   ! complete estimate of the 3d meridional velocity : VZ_i^n+1 taking
   ! all the explicit terms (yz_exp)
   ! and the implicit vertical diffusivity or mixing into account
   !----------------------------------------------------------

   CALL dyn3dy_ztls_solve(vz,uv)

   ! Apply open boundary conditions to intermediate estimate of VZ_i^n+1
   !--------------------------------------------------------------------

OMPAGRIF BARRIER
OMPAGRIF MASTER
   IF (l_obc_char) THEN
     IF (.NOT.l_obc_cycl_y .AND. Agrif_Root_OMP)                     &
       CALL obc_apply_vz_mc(uz,vz,boz)
     IF_AGRIF (.NOT.l_obc_cycl_y .AND. .NOT.Agrif_Root_OMP) CALL agrif_obc_vz(vz,uv,xeuv)
   ELSE
     IF_AGRIF (.NOT.Agrif_Root_OMP) CALL agrif_obc_vz(vz,uv,xeuv)
     IF (Agrif_Root_OMP) CALL obc_apply_vz(vz)
   END IF
OMPAGRIF END MASTER
OMPAGRIF BARRIER
OMPAGRIF FLUSH(vz)

 
#if defined key_lans_turb || defined key_leray_turb
!$OMP DO SCHEDULE(RUNTIME)
   DO k=1,kmax

        vz2d(liminm2:limaxp2,ljminm2:ljmaxp2) = vz(k,liminm2:limaxp2,ljminm2:ljmaxp2)

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,vz2d(liminm4:limaxp4,ljminm4:ljmaxp4))
   CALL_MPI ex_j_rsh(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,vz2d(liminm4:limaxp4,ljminm4:ljmaxp4))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(vz2d)

        CALL  smoothv(vz2d, vzs2d)

        vzs(k,liminm2:limaxp2,ljminm2:ljmaxp2) = vzs2d(liminm2:limaxp2,ljminm2:ljmaxp2)

   END DO
!$OMP END DO

! Apply open boundary conditions to intermediate estimate of VZS_i^n+1
!--------------------------------------------------------------------

OMPAGRIF BARRIER
OMPAGRIF MASTER
     IF_AGRIF (.NOT.Agrif_Root_OMP) CALL agrif_obc_vz(vzs,uv,xeuv)
     IF (Agrif_Root_OMP) CALL obc_apply_vz(vzs)
OMPAGRIF END MASTER
OMPAGRIF BARRIER
OMPAGRIF FLUSH(vzs)
#endif

! convergence of the 3d velocity towards the barotropic estimate
!----------------------------------------------------------------

!$OMP DO SCHEDULE(RUNTIME)
   DO j=ljminp1,ljmaxm1
     DO i=liminp2,limaxm1
       IF (igdv(i,j)/=4) THEN
#if defined key_siggen || defined key_gencoord
         umoy=SUM(vz(:,i,j)*hzey(:,i,j)*dsigu(:))/hey(i,j)
#else
         umoy=SUM(vz(:,i,j)*dsigu(:))
#if defined key_lans_turb || defined key_leray_turb
         usmoy=SUM(vzs(:,i,j)*dsigu(:))
#endif
#endif
         vz(:,i,j)=vz(:,i,j)-umoy+uv(i,j)
#if defined key_lans_turb || defined key_leray_turb
         vzs(:,i,j)=vzs(:,i,j)-usmoy+uvs(i,j)
#endif
       ELSE
         vz(:,i,j)=uv(i,j)
#if defined key_lans_turb || defined key_leray_turb
         vzs(:,i,j)=uvs(i,j)
#endif
       END IF
     END DO
   END DO
!$OMP END DO

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2,ljmaxp2,vz(:,liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL_MPI ex_j_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2,ljmaxp2,vz(:,liminm2:limaxp2,ljminm2:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(vz)

#if defined key_lans_turb || defined key_leray_turb
OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2,ljmaxp2,vzs(:,liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL_MPI ex_j_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2,ljmaxp2,vzs(:,liminm2:limaxp2,ljminm2:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(vzs)
#endif

#ifdef key_infos_adi_steps
!   WRITE(*,*)"max_vz_corr=",MAXVAL(vz)
#endif

   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   !                                                                  !
   !                             Part II                              !
   ! Estimate of final variables :                                    !
   !                         XE^n+1, u^n+1, and uz^n+1                !
   !                                                                  !
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   ! Start building Vstar from the integrated right hand side
   ! At this step, Vstar does not contains the external
   ! pressure gradient term :
   ! v*^n+1 = v_i^n+1 + RHS(uz_i^n+1/2,vz^n+1/2)
   !---------------------------------------------------------

!$OMP DO SCHEDULE(RUNTIME)
   DO j=ljminp1,ljmaxm1
     DO i = liminp2, limaxm1
       IF (igdv(i,j)==1) THEN

         !uz_v=0.25_rsh*(uz(1,i,j)+uz(1,i-1,j)+uz(1,i,j+1)+uz(1,i-1,j+1))
         !vitf=SQRT(uz_v(1,i,j)*uz_v(1,i,j)+vz(1,i,j)*vz(1,i,j))
#if defined key_siggen || defined key_gencoord
         cdf=(0.4_rsh/LOG(0.5_rsh*dsigu(1)*hzuty(1,i,j)/z0b(i,j)+1.0_rsh))**2
#else
         cdf = drag_coef(hty(i,j), sig(1), z0b(i,j))
#endif
#ifdef key_no_bottom_stress
         cdf=0.0_rsh
#endif
#if key_linear_bottom_stress
         cdvit(i,j)=frot_lin
#else
         cdvit(i,j) = drag_times_vit(uz_v(1,i,j), vz(1,i,j), cdf)
#endif
#if defined key_lans_turb || defined key_leray_turb
         intz_exp_vstar(i,j) = intz_exp(i,j)
#else
         intz_exp(i,j)=intz_exp(i,j)-cdvit(i,j)*vz(1,i,j)/hty(i,j)*cfroi
         intz_exp_vstar(i,j) = intz_exp(i,j)
#endif
!        vstar(i,j)=uv(i,j)+halfdt*intz_exp(i,j)
       END IF
       igdv_old(i,j)=igdv(i,j)
     END DO
   END DO
!$OMP END DO

   !-----------------------------------------------------------------------
   ! Update of heights hex and hey
   ! used in continuity equation (and for tracers advection)
   ! Update of heights htx and hty, estimates of heights hzuty and hzuty,
   ! hzwty and hzwty, heights in momentum equation
   ! Definition of the type of cell (igdu and igdv),
   ! the type of extremity of the row (tfg,tfd) used in dyn2dxtlsobc
   ! the mask used for the viscosity term (mn and ms)
   !-----------------------------------------------------------------------

   test = 1

!$OMP DO SCHEDULE(RUNTIME)
   DO j=ljminm2,ljmaxp2
     xeint(liminm2:limaxp2,j) = alphap*xeu(liminm2:limaxp2,j) + alphan*xeuv(liminm2:limaxp2,j)
   END DO
!$OMP END DO

   CALL dyn3d_height_mask(xeint,tfg,tfd,tfb,tfh,test)

#if defined key_lans_turb || defined key_leray_turb
     IF (smooth_method == 1) THEN

             igdu_ext(limin:limax, ljmin:ljmax) = igdu(limin:limax, ljmin:ljmax)
             igdv_ext(limin:limax, ljmin:ljmax) = igdv(limin:limax, ljmin:ljmax)

             CALL_MPI ex_i_int(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,igdu_ext(liminm4:limaxp4,ljminm4:ljmaxp4))
             CALL_MPI ex_j_int(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,igdu_ext(liminm4:limaxp4,ljminm4:ljmaxp4))

             CALL_MPI ex_i_int(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,igdv_ext(liminm4:limaxp4,ljminm4:ljmaxp4))
             CALL_MPI ex_j_int(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,igdv_ext(liminm4:limaxp4,ljminm4:ljmaxp4))

             CALL smoothuInd(Indu)
             CALL smoothvInd(Indv)
     END IF
#endif

   ! Construction of 2D and 3D matrixes for calculation along x
   ! 1st part : estimate of right hand side :
   ! RHS(uz_i^n+1/2,vz_i^n+1) for NL advection, internal pressure gradient
   !                      horizontal viscosity, atmospherical forcing
   !                      bottom friction (2d part only)
   ! RHS(baroclinic vz^n,u_i^n+1/2) for coriolis term
   ! 2nd part : estimate of tridiagonal linear system coefficients :
   ! av,bv,cv,y2dv,axe,bxe,cxe,y2dxe
   !-----------------------------------------------------------------

   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   !!!!!!!!!!!!!CALL dyn3dx_tls_coef(vz,boz,uz,u,v,xeu,uv)!!!!!!!!!!!!!!
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   CALL dyn3dx_tls_coef(vz,     &
#if defined key_lans_turb || defined key_leray_turb
                        vzs,    &
#endif
                        boz,    &
                        uz,     &
#if defined key_lans_turb || defined key_leray_turb
                        uzs,    &
#endif
                        u,      &
                        v,      &
#if defined key_lans_turb || defined key_leray_turb
                        us,     &
                        vs,     &
#endif
                        xeu,    &

#if defined key_lans_turb || defined key_leray_turb
                        uvs,    &
#endif
                        uv      &
                        )

#if defined key_lans_turb || defined key_leray_turb
!Exchange y2dv_aux
!-----------------
OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rlg(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,y2dv_aux(liminm4:limaxp4,ljminm4:ljmaxp4))
   CALL_MPI ex_j_rlg(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,y2dv_aux(liminm4:limaxp4,ljminm4:ljmaxp4))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(y2dv_aux)

!Smooth y2dv_aux
!---------------

   CALL smoothu_y2dv(y2dv_aux, y2dvs)

!Bottom stress correction
!------------------------

!$OMP DO SCHEDULE(RUNTIME)
   DO j=ljminp2,ljmaxm1
        DO i=MAX0(ig(j),limin),MIN0(limax,id(j)-1)
              IF (igdu(i,j)/=4) THEN
                        IF (igdu(i,j)==1) THEN
                                ft=halfdt*cdvit(i,j)/htx(i,j)
                                y2dv(i,j)  = y2dv_aux(i,j) + ft*u(i,j)
                                y2dvs(i,j) = y2dvs(i,j)    + ft*us(i,j)
                                y2dv_moy(i,j)  = gam*y2dv(i,j) + gams*y2dvs(i,j)
                        ELSE IF (igdu(i,j)==5) THEN
                                y2dv(i,j)  = y2dv_aux(i,j)
                                y2dvs(i,j) = y2dv(i,j)                 
                                y2dv_moy(i,j)  = gam*y2dv(i,j) + gams*y2dvs(i,j)              
                        END IF
               END IF
      END DO
   END DO
!$OMP END DO
#endif

   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   ! Solve the barotropic system along x !
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   !-----------------------------------------------------------------------
   ! Add the open boundary conditions to the right hand side
   !-----------------------------------------------------------------------
   IF ( .NOT.l_obc_cycl_x  .AND. (l_obc_east .OR. l_obc_west) ) THEN
     IF_MPI ( imin==limin .OR. imax==limax ) THEN
#ifdef key_MPI_2D
!$OMP DO SCHEDULE(RUNTIME)
#else
!$OMP DO SCHEDULE(dynamic,2)
#endif
     DO j=ljminp2,ljmaxm1
       i1=ig(j)
       i2=id(j)
       !!     introduction des conditions aux limites en niveaux
       !!     liees a l implicitation du gradient de pression
       !!     Modification de coefficient
       CALL dyn2dx_tls_obc(i1,i2,j,tfg(j),tfd(j),u,xeu,xeuv)
     END DO
!$OMP END DO
     ENDIF_MPI
   END IF
   !-----------------------------------------------------------------------
   ! Resolve the 2D tridiagonal system to evaluate
   ! the final estimate of the meridional velocity : U^n+1
   ! and the final estimate of the free surface: XE^n+1
   !------------------------------------------------------------------

   ! pre solve
   ! ---------
   CALL dyn2dx_tls_presolve

   ! ----------------------------
   ! solve the sea surface height
   ! ----------------------------

OMPMPI barrier
OMPMPI master
   ! echange throughout the lines for columns from rjminp2 to rjmaxm1
   CALL_MPI transpose_row
OMPMPI end master
OMPMPI barrier
OMPMPI flush(ry2dxe,raxe,rbxe,rcxe)

#ifdef key_MPI_2D
   IF (l_obc_cycl_x) THEN
     CALL dyn2dx_cyc_solve(rxeuv)
   ELSE
     CALL dyn2dx_tls_solve(rxeuv)
   END IF
#else
   IF (l_obc_cycl_x) THEN
     CALL dyn2dx_cyc_solve(xeuv)
   ELSE
     CALL dyn2dx_tls_solve(xeuv)
   END IF
#endif

OMPMPI barrier
OMPMPI master
   ! exchange back to each CPU (limin:limax and ljmi:ljmax)
   CALL_MPI trback_row(xeuv)
   CALL_MPI ex_i_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,xeuv(liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL_MPI ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,xeuv(liminm2:limaxp2,ljminm2:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(xeuv)


  !---------------------------------------------------------
   ! Apply open boundary conditions to the sea surface height
   !---------------------------------------------------------

!$OMP SINGLE
   IF (l_obc_cycl_x) THEN
#ifdef key_MPI_2D
#ifdef key_shiftobc_curv
     CALL_MPI ex_peri_x_rsh(3,1,liminm2,limaxp2,ljminm2,ljmaxp2,xeuv(liminm2:limaxp2,ljminm2:ljmaxp2))
#else
     CALL_MPI ex_peri_x_rsh(2,1,liminm2,limaxp2,ljminm2,ljmaxp2,xeuv(liminm2:limaxp2,ljminm2:ljmaxp2))
#endif
#else
     DO j=ljmin,ljmax ! changed from DO j=jmin+2,jmax-1
       xeuv(ig(j)-1,j)=xeuv(id(j)-2,j)
       xeuv(ig(j)  ,j)=xeuv(id(j)-1,j)
       xeuv(id(j)  ,j)=xeuv(ig(j)+1,j)
#ifdef key_shiftobc_curv
       xeuv(ig(j)-2,j)=xeuv(id(j)-3,j)
       xeuv(id(j)+1,j)=xeuv(ig(j)+2,j)
       xeuv(id(j)+2,j)=xeuv(ig(j)+3,j)
#endif
     END DO
#endif
   END IF
   IF (l_obc_cycl_y) THEN
#ifdef key_MPI_2D
#ifdef key_shiftobc_curv
     CALL_MPI ex_peri_y_rsh(3,1,liminm2,limaxp2,ljminm2,ljmaxp2,xeuv(liminm2:limaxp2,ljminm2:ljmaxp2))
#else
     CALL_MPI ex_peri_y_rsh(2,1,liminm2,limaxp2,ljminm2,ljmaxp2,xeuv(liminm2:limaxp2,ljminm2:ljmaxp2))
#endif
#else
     DO i=limin,limax
       xeuv(i,jb(i)-1)=xeuv(i,jh(i)-2)
       xeuv(i,jb(i)  )=xeuv(i,jh(i)-1)
       xeuv(i,jh(i)  )=xeuv(i,jb(i)+1)
#ifdef key_shiftobc_curv
       xeuv(i,jb(i)-2)=xeuv(i,jh(i)-3)
       xeuv(i,jh(i)+1)=xeuv(i,jb(i)+2)
       xeuv(i,jh(i)+2)=xeuv(i,jb(i)+3)
#endif
     END DO
#endif
   END IF
!$OMP END SINGLE

#ifdef key_infos_adi_step
!   WRITE(*,*)"min_xe_corr=",MINVAL(xeuv)
#endif
   ! --------------------------------------------------
   ! post solve : estimate of the barotropic velocity U
   ! --------------------------------------------------

!$OMP DO SCHEDULE(RUNTIME)
   DO j=ljminp2,ljmaxm1
     DO i=MAX0(limin,ig(j)),MIN0(limax,id(j)-1)
       IF (limin==imin) av(ig(j),j)=0.0_rlg
       IF (limax==imax) cv(id(j)-1,j)=0.0_rlg     
       uv2(i,j)  = (y2dv(i,j)  - (cv(i,j)*xeuv(i+1,j)+av(i,j)*xeuv(i,j)))*bv(i,j)
#if defined key_lans_turb || defined key_leray_turb
       uv2s(i,j) = (y2dvs(i,j) - (cv(i,j)*xeuv(i+1,j)+av(i,j)*xeuv(i,j)))*bv(i,j)
#endif
     END DO
   END DO
!$OMP END DO

!$OMP SINGLE
   IF (l_obc_cycl_x) THEN

     CALL_MPI ex_peri_x_avcv(liminm1,limax,ljminm1,ljmax,av(liminm1:limax,ljminm1:ljmax),cv(liminm1:limax,ljminm1:ljmax))

     IF_MPI (limax == imax) THEN
     DO j=ljminp2,ljmaxm1
#ifdef key_MPI_2D
       uv2(id(j)-1,j)  = uv2(id(j)-1,j)  - cv(id(j)-1,j)*xeuv(id(j),j)*bv(id(j)-1,j)
#if defined key_lans_turb || defined key_leray_turb
       uv2s(id(j)-1,j) = uv2s(id(j)-1,j) - cv(id(j)-1,j)*xeuv(id(j),j)*bv(id(j)-1,j)
#endif
#else
       uv2(id(j)-1,j)  = uv2(id(j)-1,j)  - cv(ig(j),j)*xeuv(id(j),j)*bv(id(j)-1,j)
#if defined key_lans_turb || defined key_leray_turb
       uv2s(id(j)-1,j) = uv2s(id(j)-1,j) - cv(ig(j),j)*xeuv(id(j),j)*bv(id(j)-1,j
#endif
#endif
     END DO
     ENDIF_MPI

     IF_MPI (limin == imin) THEN
     DO j=ljminp2,ljmaxm1
#ifdef key_MPI_2D
       uv2(ig(j),j)  = uv2(ig(j),j)  - av(ig(j),j)*xeuv(ig(j),j)*bv(ig(j),j)
#if defined key_lans_turb || defined key_leray_turb
       uv2s(ig(j),j) = uv2s(ig(j),j) - av(ig(j),j)*xeuv(ig(j),j)*bv(ig(j),j)
#endif
#else
       uv2(ig(j),j)  = uv2(ig(j),j)  - av(id(j)-1,j)*xeuv(ig(j),j)*bv(ig(j),j)
#if defined key_lans_turb || defined key_leray_turb
       uv2s(ig(j),j) = uv2s(ig(j),j) - av(id(j)-1,j)*xeuv(ig(j),j)*bv(ig(j),j)
#endif
#endif
     END DO
     ENDIF_MPI
   END IF
!$OMP END SINGLE

   !----------------------------------------------------------
   ! Apply open boundary conditions to the barotropic velocity
   !----------------------------------------------------------

OMPAGRIF BARRIER
OMPAGRIF MASTER
   IF (l_obc_char) THEN
     IF (.NOT.l_obc_cycl_x .AND. Agrif_Root_OMP) THEN
       CALL obc_apply_u_mc(u,uv2,v,uv,xeu,xeuv,.TRUE.)
#if defined key_lans_turb || defined key_leray_turb
       CALL obc_apply_u_mc(us,uv2s,vs,uvs,xeu,xeuv,.TRUE.)
#endif
     END IF
     IF_AGRIF (.NOT.l_obc_cycl_x .AND. .NOT.Agrif_Root_OMP) CALL agrif_obc_u(uv2)
#if defined key_lans_turb || defined key_leray_turb
     IF_AGRIF (.NOT.l_obc_cycl_x .AND. .NOT.Agrif_Root_OMP) CALL agrif_obc_u(uv2s)
#endif
   ELSE 
     IF_AGRIF (.NOT. Agrif_Root_OMP) CALL agrif_obc_u(uv2)
#if defined key_lans_turb || defined key_leray_turb
     IF_AGRIF (.NOT. Agrif_Root_OMP) CALL agrif_obc_u(uv2s)
#endif
     IF (Agrif_Root_OMP) THEN
        CALL obc_apply_u(uv2)
#if defined key_lans_turb || defined key_leray_turb
        CALL obc_apply_u(uv2s)
#endif
     END IF
   END IF
OMPAGRIF END MASTER
OMPAGRIF BARRIER
OMPAGRIF FLUSH(uv2)
#if defined key_lans_turb || defined key_leray_turb
OMPAGRIF FLUSH(uv2s)
#endif

!#if defined key_lans_turb || defined key_leray_turb
! extend for parallelization
!----------------------------------------
!   CALL extend_u(uv2, uv2_ext)
! smooth ustar
!----------------------------------------
!   CALL smoothu(uv2_ext, uv2s)
!
! Apply open boundary conditions to ustar
!----------------------------------------
!OMPAGRIF BARRIER
!OMPAGRIF MASTER
!     CALL obc_apply_u(uv2s)
!OMPAGRIF END MASTER
!OMPAGRIF BARRIER
!OMPAGRIF FLUSH(uv2s)
!#endif

#ifdef key_infos_adi_steps
    !WRITE(*,*)"max_u_corr=",MAXVAL(uv2)
#endif

   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   ! Solve the complete system (barotropic+baroclinic zonal velocity)      !
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   ! Add the external pressure gradient (using XE^n+1)
   ! to right hand side of the zonal momentum equation
   !-------------------------------------------------------

    CALL dyn3dx_epg(xeu,xeuv)

   ! Estimate of the zonal velocity UZ^n+1 taking
   ! all the explicit terms (yz_exp)
   ! and the implicit vertical diffusivity or mixing into account
   !----------------------------------------------------------

   CALL dyn3dx_ztls_solve(uz,uv2)

   ! apply open boundary conditions
   !-------------------------------

OMPAGRIF BARRIER
OMPAGRIF MASTER
   IF (l_obc_char) THEN
   IF (.NOT.l_obc_cycl_x .AND. Agrif_Root_OMP)                     &
     CALL obc_apply_uz_mc(uz,vz,boz)
   IF_AGRIF (.NOT.l_obc_cycl_x .AND. .NOT.Agrif_Root_OMP) CALL agrif_obc_uz(uz,uv2,xeuv)
   ELSE
     IF_AGRIF (.NOT.Agrif_Root_OMP) CALL agrif_obc_uz(uz,uv2,xeuv)
     IF (Agrif_Root_OMP) CALL obc_apply_uz(uz)
   END IF
OMPAGRIF END MASTER
OMPAGRIF BARRIER
OMPAGRIF FLUSH(uz)

!Compute 3d smooth velocies
#if defined key_lans_turb || defined key_leray_turb
!$OMP DO SCHEDULE(RUNTIME)
   DO k=1,kmax

        uz2d(liminm2:limaxp2,ljminm2:ljmaxp2) = uz(k,liminm2:limaxp2,ljminm2:ljmaxp2)

OMPMPI barrier
OMPMPI master
        CALL_MPI ex_i_rsh(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,uz2d(liminm4:limaxp4,ljminm4:ljmaxp4))
        CALL_MPI ex_j_rsh(-4,4,1,liminm4,limaxp4,ljminm4,ljmaxp4,uz2d(liminm4:limaxp4,ljminm4:ljmaxp4))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(uz2d)

        CALL smoothu(uz2d, uzs2d)

        uzs(k,liminm2:limaxp2,ljminm2:ljmaxp2) = uzs2d(liminm2:limaxp2,ljminm2:ljmaxp2)

   END DO
!$OMP END DO

   ! apply open boundary conditions
   !-------------------------------

OMPAGRIF BARRIER
OMPAGRIF MASTER
     IF_AGRIF (.NOT.Agrif_Root_OMP) CALL agrif_obc_uz(uzs,uv2,xeuv)
     IF (Agrif_Root_OMP) CALL obc_apply_uz(uzs)
OMPAGRIF END MASTER
OMPAGRIF BARRIER
OMPAGRIF FLUSH(uzs)
#endif

! convergence of the 3d velocity towards the barotropic estimate
! final estimate of the zonal velocity UZ^n+1
!----------------------------------------------------------------

!$OMP DO SCHEDULE(RUNTIME)
   DO j=ljminp2,ljmaxm1
     DO i=liminp1,limaxm1
       IF (igdu(i,j)/=4) THEN
#if defined key_siggen || defined key_gencoord
         umoy=SUM(uz(:,i,j)*hzex(:,i,j)*dsigu(:))/hex(i,j)
#else
         umoy=SUM(uz(:,i,j)*dsigu(:))
#if defined key_lans_turb || defined key_leray_turb
         usmoy=SUM(uzs(:,i,j)*dsigu(:))
#endif
#endif
         uz(:,i,j)=uz(:,i,j)-umoy+uv2(i,j)
#if defined key_lans_turb || defined key_leray_turb
         uzs(:,i,j)=uzs(:,i,j)-usmoy+uv2s(i,j)
#endif
       END IF
     END DO
   END DO
!$OMP END DO

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2,ljmaxp2,uz(:,liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL_MPI ex_j_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2,ljmaxp2,uz(:,liminm2:limaxp2,ljminm2:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(uz)
#if defined key_lans_turb || defined key_leray_turb
OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2,ljmaxp2,uzs(:,liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL_MPI ex_j_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2,ljmaxp2,uzs(:,liminm2:limaxp2,ljminm2:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(uzs)
#endif

#ifdef key_infos_adi_steps
   WRITE(*,*)"max_uz_corr=",MAXVAL(uz)
#endif

   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   !                                                                  !
   !                             Part III                             !
   ! Estimate of water fluxes used by the tracer advection part       !
   !                                                                  !
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!$OMP DO SCHEDULE(RUNTIME)
   DO j=ljmin,ljmax
     IF ( j>=jmin+2 .AND. j<=jmax-1 ) THEN
       DO i=MAX0(limin,ig(j)),MIN0(limax,id(j)-1)

#if defined key_siggen || defined key_gencoord
         fxeau(:,i,j)=hzex(:,i,j)*uz(:,i,j)*halfdt
#else
#ifdef key_wave_qeuler_dynamics
         fxeau(:,i,j)=hex(i,j)*(uz(:,i,j)+USX_u(:,i,j))*halfdt
#else
         fxeau(:,i,j)=hex(i,j)*uz(:,i,j)*halfdt
#if defined key_lans_turb || defined key_leray_turb
         fxeau_lans(:,i,j)=hex(i,j)*uzs(:,i,j)*halfdt
#endif
#endif
#endif

       END DO
     END IF
     DO i=liminp2,limaxm1
#if defined key_siggen || defined key_gencoord
       IF ( j>=jb(i) .AND. j<=jh(i)-1 ) fyeau(:,i,j)=hzey(:,i,j)*vz(:,i,j)*halfdt
#else
#ifdef key_wave_qeuler_dynamics
       IF ( j>=jb(i) .AND. j<=jh(i)-1 ) fyeau(:,i,j)=hey(i,j)*(vz(:,i,j)+USY_v(:,i,j))*halfdt
#else
       IF ( j>=jb(i) .AND. j<=jh(i)-1 ) fyeau(:,i,j)=hey(i,j)*vz(:,i,j)*halfdt
#if defined key_lans_turb || defined key_leray_turb
       IF ( j>=jb(i) .AND. j<=jh(i)-1 ) fyeau_lans(:,i,j)=hey(i,j)*vzs(:,i,j)*halfdt
#endif
#endif
#endif

     END DO
   END DO
!$OMP END DO

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rlg(-1,0,kmax,liminm1,limax,ljmin,ljmax,fxeau(:,liminm1:limax,ljmin:ljmax))
   CALL_MPI ex_j_rlg(-1,0,kmax,limin,limax,ljminm1,ljmax,fyeau(:,limin:limax,ljminm1:ljmax))
#if defined key_lans_turb || defined key_leray_turb
   CALL_MPI ex_i_rlg(-1,0,kmax,liminm1,limax,ljmin,ljmax,fxeau_lans(:,liminm1:limax,ljmin:ljmax))
   CALL_MPI ex_j_rlg(-1,0,kmax,limin,limax,ljminm1,ljmax,fyeau_lans(:,limin:limax,ljminm1:ljmax))
#endif
OMPMPI end master
OMPMPI barrier
#if defined key_lans_turb || defined key_leray_turb
OMPMPI flush(fxeau,fyeau,fxeau_lans,fyeau_lans)
#else
OMPMPI flush(fxeau,fyeau)
#endif

#ifdef key_dyn_wetdry_fct

   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   !                                                                  !
   !                 Part IV : wet-drying purpose                     !
   !  Correction of the sea surface height and the velocities         !
   !  over very small depths (key_dyn_wetdry_fct cpp key)             !
   !                                                                  !
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!$OMP DO SCHEDULE(runtime)
   DO j=ljmin,ljmax
     phimax(:,j)=1.0_rlg
   END DO
!$OMP END DO

   !----------------------------------------------------------------------
   ! For negative water thicknesses, removing of fluxes responsible of the
   ! decreasing of the sea surface height
   !----------------------------------------------------------------------

!$OMP DO SCHEDULE(runtime)
   DO j=ljminp2,ljmaxm1
     DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)
       IF (h0(i,j)+xeuv(i,j) < 0.0_rsh) THEN
         A=fwetp(i,j)*surf(i,j)*(xeu(i,j)+h0(i,j))/(MAX(0.0_rlg,SUM(dsigu(1:kmax)*fyeau(1:kmax,i,j  )))*dx_v(i,j  ) &
                                                  -MIN(0.0_rlg,SUM(dsigu(1:kmax)*fyeau(1:kmax,i,j-1)))*dx_v(i,j-1) &
                                                  +MAX(0.0_rlg,SUM(dsigu(1:kmax)*fxeau(1:kmax,i,j  )))*dy_u(i,j  ) &
                                                  -MIN(0.0_rlg,SUM(dsigu(1:kmax)*fxeau(1:kmax,i-1,j)))*dy_u(i-1,j)+epsilon)
         phimax(i,j)=MAX(MIN(1.0_rlg, A),0.0_rlg)
       END IF
     END DO
   END DO
!$OMP END DO

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rlg(0,1,1,limin,limaxp1,ljmin,ljmaxp1,phimax(limin:limaxp1,ljmin:ljmaxp1))
   CALL_MPI ex_j_rlg(0,1,1,limin,limaxp1,ljmin,ljmaxp1,phimax(limin:limaxp1,ljmin:ljmaxp1))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(phimax)

   !----------------------------------------------------------------------
   ! Limitation of the fluxes over small depths (phimax=1 everywhere else)
   !----------------------------------------------------------------------

!$OMP DO SCHEDULE(runtime)
   DO j=ljmin,ljmax
     IF ((j.GE.jmin+2).AND.(j.LE.jmax-1)) THEN
       DO i=MAX0(limin,ig(j)),MIN0(limax,id(j)-1)
         IF (fxeau(1,i,j) > 0.0_rlg) THEN
           fxeau(:,i,j)= phimax(i  ,j)*fxeau(:,i,j)
         ELSE
           fxeau(:,i,j)= phimax(i+1,j)*fxeau(:,i,j)
         END IF
       END DO
     END IF
     DO i=liminp2,limaxm1
       IF ((j.GE.jb(i)).AND.(j.LE.jh(i)-1)) THEN
         IF (fyeau(1,i,j) > 0.0_rlg) THEN
           fyeau(:,i,j)= phimax(i,j  )*fyeau(:,i,j)
         ELSE
           fyeau(:,i,j)= phimax(i,j+1)*fyeau(:,i,j)
         END IF
       END IF
     END DO
   END DO
!$OMP END DO

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rlg(-1,0,kmax,liminm1,limax,ljmin,ljmax,fxeau(:,liminm1:limax,ljmin:ljmax))
   CALL_MPI ex_j_rlg(-1,0,kmax,limin,limax,ljminm1,ljmax,fyeau(:,limin:limax,ljminm1:ljmax))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(fxeau,fyeau)

   !-----------------------------------
   ! Estimate of the sea surface height
   !-----------------------------------

!$OMP DO SCHEDULE(runtime)
   DO j=ljminp2,ljmaxm1
     DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)
       xeuv(i,j)=(xeu(i,j)*fwetp(i,j)-(SUM(dsigu(1:kmax)*fxeau(1:kmax,i  ,j))*dy_u(i  ,j) &
                                      -SUM(dsigu(1:kmax)*fxeau(1:kmax,i-1,j))*dy_u(i-1,j) &
                                      +SUM(dsigu(1:kmax)*fyeau(1:kmax,i  ,j))*dx_v(i  ,j) &
                                      -SUM(dsigu(1:kmax)*fyeau(1:kmax,i,j-1))*dx_v(i,j-1) &
                                      -SUM(phieau(1:kmax,i,j))*halfdt )/surf(i,j))/fwet(i,j)
     END DO
   END DO
!$OMP END DO

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,xeuv(liminm2:limaxp2,ljminm2:ljmaxp2))
!utile   CALL_MPI ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,xeuv(liminm2:limaxp2,ljminm2:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(xeuv)

   !---------------------------------------------------------
   ! Apply open boundary conditions to the sea surface height
   !---------------------------------------------------------

!$OMP SINGLE
   IF (l_obc_cycl_x) THEN
#ifdef key_MPI_2D
#ifdef key_shiftobc_curv
     CALL_MPI ex_peri_x_rsh(3,1,liminm2,limaxp2,ljminm2,ljmaxp2,xeuv(liminm2:limaxp2,ljminm2:ljmaxp2))
#else
     CALL_MPI ex_peri_x_rsh(2,1,liminm2,limaxp2,ljminm2,ljmaxp2,xeuv(liminm2:limaxp2,ljminm2:ljmaxp2))
#endif
#else
     DO j=ljmin,ljmax ! changed from DO j=jmin+2,jmax-1
       xeuv(ig(j)-1,j)=xeuv(id(j)-2,j)
       xeuv(ig(j)  ,j)=xeuv(id(j)-1,j)
       xeuv(id(j)  ,j)=xeuv(ig(j)+1,j)
#ifdef key_shiftobc_curv
       xeuv(ig(j)-2,j)=xeuv(id(j)-3,j)
       xeuv(id(j)+1,j)=xeuv(ig(j)+2,j)
       xeuv(id(j)+2,j)=xeuv(ig(j)+3,j)
#endif
     END DO
#endif
   END IF
   IF (l_obc_cycl_y) THEN
#ifdef key_MPI_2D
#ifdef key_shiftobc_curv
     CALL_MPI ex_peri_y_rsh(3,1,liminm2,limaxp2,ljminm2,ljmaxp2,xeuv(liminm2:limaxp2,ljminm2:ljmaxp2))
#else
     CALL_MPI ex_peri_y_rsh(2,1,liminm2,limaxp2,ljminm2,ljmaxp2,xeuv(liminm2:limaxp2,ljminm2:ljmaxp2))
#endif
#else
     DO i=limin,limax
       xeuv(i,jb(i)-1)=xeuv(i,jh(i)-2)
       xeuv(i,jb(i)  )=xeuv(i,jh(i)-1)
       xeuv(i,jh(i)  )=xeuv(i,jb(i)+1)
#ifdef key_shiftobc_curv
       xeuv(i,jb(i)-2)=xeuv(i,jh(i)-3)
       xeuv(i,jh(i)+1)=xeuv(i,jb(i)+2)
       xeuv(i,jh(i)+2)=xeuv(i,jb(i)+3)
#endif
     END DO
#endif
   END IF
!$OMP END SINGLE

   !-------------------------------------------------------
   ! Estimate of the velocities :
   !   - they balance the external pressure gradient
   !   - they do not correspond to the water fluxes (fyeau)
   !-------------------------------------------------------

!$OMP DO SCHEDULE(RUNTIME)
   DO j=ljminp2,ljmaxm1
     DO i=MAX0(limin,ig(j)),MIN0(limax,id(j)-1)
       IF (limin==imin) av(ig(j),j)=0.0_rlg
       IF (limax==imax) cv(id(j)-1,j)=0.0_rlg
       uv2(i,j)=(y2dv(i,j)-(cv(i,j)*xeuv(i+1,j)+av(i,j)*xeuv(i,j)))*bv(i,j)
     END DO
   END DO
!$OMP END DO

!$OMP SINGLE
   IF (l_obc_cycl_x) THEN

     CALL_MPI ex_peri_x_avcv(liminm1,limax,ljminm1,ljmax,av(liminm1:limax,ljminm1:ljmax),cv(liminm1:limax,ljminm1:ljmax))

     IF_MPI (limax == imax) THEN
     DO j=ljminp2,ljmaxm1
#ifdef key_MPI_2D
       !uv2(imax-1,j) = uv2(imax-1,j)-cv(imax-1,j)*xeuv(imax,j)*bv(imax-1,j)
       uv2(id(j)-1,j) = uv2(id(j)-1,j)-cv(id(j)-1,j)*xeuv(id(j),j)*bv(id(j)-1,j)
#else
       !uv2(imax-1,j) = uv2(imax-1,j)-cv(imin+1,j)*xeuv(imax,j)*bv(imax-1,j)
       uv2(id(j)-1,j) = uv2(id(j)-1,j)-cv(ig(j),j)*xeuv(id(j),j)*bv(id(j)-1,j)
#endif
     END DO
     ENDIF_MPI

     IF_MPI (limin == imin) THEN
     DO j=ljminp2,ljmaxm1
#ifdef key_MPI_2D
       !uv2(imin+1,j) = uv2(imin+1,j)-av(imin+1,j)*xeuv(imin+1,j)*bv(imin+1,j)
       uv2(ig(j),j) = uv2(ig(j),j)-av(ig(j),j)*xeuv(ig(j),j)*bv(ig(j),j)
#else
       !uv2(imin+1,j) = uv2(imin+1,j)-av(imax-1,j)*xeuv(imin+1,j)*bv(imin+1,j)
       uv2(ig(j),j) = uv2(ig(j),j)-av(id(j)-1,j)*xeuv(ig(j),j)*bv(ig(j),j)
#endif
     END DO
     ENDIF_MPI

   END IF
!$OMP END SINGLE


   !----------------------------------------------------------
   ! Apply open boundary conditions to the barotropic velocity
   !----------------------------------------------------------

OMPAGRIF BARRIER
OMPAGRIF MASTER
   IF (l_obc_char) THEN
     IF (.NOT.l_obc_cycl_x .AND. Agrif_Root_OMP)                   &
       CALL obc_apply_u_mc(u,uv2,v,uv,xeu,xeuv,.TRUE.)
     IF_AGRIF (.NOT.l_obc_cycl_x .AND. .NOT.Agrif_Root_OMP) CALL agrif_obc_u(uv2)
   ELSE
     IF_AGRIF (.NOT. Agrif_Root_OMP) CALL agrif_obc_u(uv2)
     IF (Agrif_Root_OMP) CALL obc_apply_u(uv2)
   END IF
OMPAGRIF END MASTER
OMPAGRIF BARRIER
OMPAGRIF FLUSH(uv2)


   ! apply open boundary conditions
   !-------------------------------

OMPAGRIF BARRIER
OMPAGRIF MASTER
   IF (l_obc_char) THEN
   IF (.NOT.l_obc_cycl_x .AND. Agrif_Root_OMP)                     &
     CALL obc_apply_uz_mc(uz,vz,boz)
   IF_AGRIF (.NOT.l_obc_cycl_x .AND. .NOT.Agrif_Root_OMP) CALL agrif_obc_uz(uz,uv2,xeuv)
   ELSE
     IF_AGRIF (.NOT.Agrif_Root_OMP) CALL agrif_obc_uz(uz,uv2,xeuv)
     IF (Agrif_Root_OMP) CALL obc_apply_uz(uz)
   END IF
OMPAGRIF END MASTER
OMPAGRIF BARRIER
OMPAGRIF FLUSH(uz)

   ! convergence of the 3d velocity towards the barotropic estimate
   ! final estimate of the zonal velocity UZ^n+1
   !----------------------------------------------------------------

!$OMP DO SCHEDULE(RUNTIME)
   DO j=ljminp2,ljmaxm1
     DO i=liminp1,limaxm1
       IF (igdu(i,j)/=4) THEN
#if defined key_siggen || defined key_gencoord
         umoy=SUM(uz(:,i,j)*hzex(:,i,j)*dsigu(:))/hex(i,j)
#else
         umoy=SUM(uz(:,i,j)*dsigu(:))
#endif
         uz(:,i,j)=uz(:,i,j)-umoy+uv2(i,j)
       END IF
     END DO
   END DO
!$OMP END DO

OMPAGRIF BARRIER
OMPAGRIF MASTER
   IF (l_obc_cycl_x .OR. l_obc_cycl_y) THEN
     IF_AGRIF (.NOT.Agrif_Root_OMP) CALL agrif_obc_uz(uz,uv2,xeuv)
     IF (Agrif_Root_OMP) CALL obc_apply_uz(uz)
   END IF
OMPAGRIF END MASTER
OMPAGRIF BARRIER
OMPAGRIF FLUSH(uz)

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2,ljmaxp2,uz(:,liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL_MPI ex_j_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2,ljmaxp2,uz(:,liminm2:limaxp2,ljminm2:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(uz)
#endif
   ENDIF_AGRIF
 
!$OMP END PARALLEL

   PRINT_DBG*, 'exit DYN3DYX routine'

  END SUBROUTINE dyn3dyx
