MODULE advection_methods 

   !!======================================================================
   !!                   ***  MODULE calc_gradient  ***
   !!
   !! ** Description :

   !!      !!======================================================================
   !! * Modules used
   USE parameters

   USE comcolig2d, ONLY : mn,ms,whx               !
 
   IMPLICIT NONE

   PUBLIC::  quick_advection_3d_xdir, quick_advection_3d_ydir, quick_advection_zdir
   
   CONTAINS


!------------------------------------------
!COMPUTE  ADVECTION TERM USING QUICK SCHEME
!------------------------------------------

        FUNCTION quick_advection_3d_xdir(phi, vel_adv, dx, k, i, j) RESULT(phi_adv)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2, ljminm2:ljmaxp2), INTENT(IN) :: phi
                REAL(KIND=rsh), INTENT(IN) :: vel_adv, dx
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh)             :: phi_adv, vel_adv_pos, vel_adv_neg

                vel_adv_pos = 0.5_rsh*(vel_adv + ABS(vel_adv))
                vel_adv_neg = 0.5_rsh*(vel_adv - ABS(vel_adv))

                phi_adv = (vel_adv_pos*phi(k,i-2,j)*whx(i-1,j) + (-7.0_rsh*vel_adv_pos - 3.0_rsh*vel_adv_neg)*phi(k,i-1,j) &
                        + (3.0_rsh*vel_adv_pos-3.0_rsh*vel_adv_neg)*phi(k,i,j) + (3.0_rsh*vel_adv_pos+7.0_rsh*vel_adv_neg)*phi(k,i+1,j) &
                        - vel_adv_neg*phi(k,i+2,j)*whx(i+1,j))/(8.0_rsh*dx)

        END FUNCTION quick_advection_3d_xdir

        FUNCTION quick_advection_3d_ydir(phi, vel_adv, dy, k, i, j) RESULT(phi_adv)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2, ljminm2:ljmaxp2), INTENT(IN) :: phi
                REAL(KIND=rsh), INTENT(IN) :: vel_adv, dy
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh)             :: phi_adv, vel_adv_pos, vel_adv_neg

                vel_adv_pos = 0.5_rsh*(vel_adv + ABS(vel_adv))
                vel_adv_neg = 0.5_rsh*(vel_adv - ABS(vel_adv))

                phi_adv = (vel_adv_pos*phi(k,i,j-2)*ms(i,j-1)*ms(i,j) + (-7.0_rsh*vel_adv_pos - 3.0_rsh*vel_adv_neg)*phi(k,i,j-1) &
                        + (3.0_rsh*vel_adv_pos-3.0_rsh*vel_adv_neg)*phi(k,i,j) + (3.0_rsh*vel_adv_pos+7.0_rsh*vel_adv_neg)*phi(k,i,j+1) &
                        - vel_adv_neg*phi(k,i,j+2)*mn(i,j+1)*mn(i,j))/(8.0_rsh*dy)

        END FUNCTION quick_advection_3d_ydir

        FUNCTION quick_advection_zdir(phi, vel_adv, dsig, k, i, j) RESULT(phi_adv)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2), INTENT(IN) :: phi
                REAL(KIND=rsh), INTENT(IN) :: vel_adv, dsig
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh)             :: phi_adv, vel_adv_pos, vel_adv_neg

                vel_adv_pos = 0.5_rsh*(vel_adv + ABS(vel_adv))
                vel_adv_neg = 0.5_rsh*(vel_adv - ABS(vel_adv))

                phi_adv = (vel_adv_pos*phi(k-2,i,j)+ (-7.0_rsh*vel_adv_pos-3.0_rsh*vel_adv_neg)*phi(k-1,i,j) &
                        + (3.0_rsh*vel_adv_pos-3.0_rsh*vel_adv_neg)*phi(k,i,j)+(3.0_rsh*vel_adv_pos+7.0_rsh*vel_adv_neg)*phi(k+1,i,j) &
                        - vel_adv_neg*phi(k+2,i,j))/(8.0_rsh*dsig)

        END FUNCTION quick_advection_zdir


!------------------------------------------
!COMPUTE  ADVECTION TERM USING UPWIND SCHEME
!------------------------------------------

        FUNCTION upwind_advection_3d_xdir(phi, vel_adv, dxij, dxip1j, k, i, j) RESULT(phi_adv)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2), INTENT(IN) :: phi
                REAL(KIND=rsh), INTENT(IN) :: vel_adv, dxij, dxip1j
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh)             :: phi_adv, vel_adv_pos, vel_adv_neg

                vel_adv_pos = 0.5_rsh*(vel_adv + ABS(vel_adv))
                vel_adv_neg = 0.5_rsh*(vel_adv - ABS(vel_adv))

                phi_adv = vel_adv_pos*(phi(k,i,j)-phi(k,i-1,j))/dxij + vel_adv_neg*(phi(k,i+1,j)-phi(k,i,j))/dxip1j


        END FUNCTION upwind_advection_3d_xdir

        FUNCTION upwind_advection_3d_ydir(phi, vel_adv, dyijm1, dyij, k, i, j) RESULT(phi_adv)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2), INTENT(IN) :: phi
                REAL(KIND=rsh), INTENT(IN) :: vel_adv, dyijm1, dyij
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh)             :: phi_adv, vel_adv_pos, vel_adv_neg

                vel_adv_pos = 0.5_rsh*(vel_adv + ABS(vel_adv))
                vel_adv_neg = 0.5_rsh*(vel_adv - ABS(vel_adv))

                phi_adv = vel_adv_pos*(phi(k,i,j)-phi(k,i,j-1))/dyijm1 + vel_adv_neg*(phi(k,i,j+1)-phi(k,i,j))/dyij

        END FUNCTION upwind_advection_3d_ydir

        FUNCTION upwind_advection_zdir(phi, vel_adv, dsigkm1, dsigk,  k, i, j) RESULT(phi_adv)

                IMPLICIT NONE

                !Inpout parameter
                REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2), INTENT(IN) :: phi
                REAL(KIND=rsh), INTENT(IN) :: vel_adv, dsigkm1, dsigk
                INTEGER,        INTENT(IN) :: k, i, j

                !Output parameter
                REAL(kind=rsh)             :: phi_adv

                !Local declaration
                REAL(kind=rsh)             :: vel_adv_pos, vel_adv_neg

                vel_adv_pos = 0.5_rsh*(vel_adv + ABS(vel_adv))
                vel_adv_neg = 0.5_rsh*(vel_adv - ABS(vel_adv))

                phi_adv = vel_adv_pos*(phi(k,i,j)-phi(k-1,i,j))/dsigkm1 + vel_adv_neg*(phi(k+1,i,j)-phi(k,i,j))/dsigk

        END FUNCTION upwind_advection_zdir


END MODULE advection_methods 
