   !!======================================================================

  SUBROUTINE sflx_turb_ayina(ztemp,zsat,zwindmod,zpa,zrh,zrad,zirlos,zrain, &
                             zsenslos,zlatlos,zevap,ztosx,ztosy)
                              

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE sflx_turb_ayina  ***
   !&E
   !&E ** Purpose : Computation of the heat fluxes and wind stress
   !&E              coefficients at the ocean surface
   !&E
   !&E ** Description : The Wind stress and the flux of heat at
   !&E              the ocean surfaces are derived from
   !&E              semi-empirical ( or bulk ) formulae which
   !&E              relate the flux to the properties of the surface
   !&E              and of the lower atmosphere.
   !&E
   !&E              zpa      ! atmospherical pressure             [Pa]
   !&E              zsat     ! air temperature                    [Celsius]
   !&E              ztemp    ! sea surface temperature            [Celsius]
   !&E              zwindmod ! module of wind at 10 m             [m/s]
   !&E              zrh      ! relative humidity of air           [kg/kg]
   !&E              suswe    ! relative humidity of the ocean     [kg/kg]
   !&E
   !&E ** Called by : sflxsurf.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference : Ayina et Bentamy (2007), Ayina et al (2008)
   !&E                METEOCEAN report (2009)
   !&E
   !&E ** History :
   !&E       !  2003-10  (L.-H. Ayina)  Original code
   !&E       !  2009-06  (V. Garnier) MARS implementation
   !&E
   !&E----------------------------------------------------------------------
   USE comvars2d,  ONLY : jb,jh,ig,id,nbouc
!   USE cometdyn,   ONLY : tosx, tosy, u10, v10

   !! * Arguments
   REAL(KIND=rsh), INTENT( IN )    :: ztemp,zsat,zwindmod,zpa,zrh, &
                                      zrad,zirlos,zrain
   REAL(KIND=rsh), INTENT( OUT )   :: zsenslos,zlatlos,zevap
   REAL(KIND=rsh), INTENT( INOUT ) :: ztosx,ztosy

   !! * Local declarations
#ifdef key_sfls_coef
   REAL(KIND=rsh), DIMENSION(limin:limax,ljmin:ljmax) ::   &
         zchs,            &  ! coefficient for sensible heat flux
         zces                ! coefficient for latent heat flux
#endif

   REAL(KIND=rsh), PARAMETER ::          &     ! wind scaling
                        cv0=0.0_rsh, cv2=2.0_rsh, cv3=3.0_rsh,       &
                        cv4=4.0_rsh, cv5=5.0_rsh, cv6=6.0_rsh,       &
                        cv7=7.0_rsh, cv8=8.0_rsh, cv9=9.0_rsh,       &
                        cv10=10.0_rsh, cv11=11.0_rsh, cv13=13.0_rsh, &
                        cv14=14.0_rsh, cv18=18.0_rsh, cv19=19.0_rsh, &
                        cv24=24.0_rsh, cv25=25.0_rsh, cv52=52.0_rsh

   REAL(KIND=rsh), PARAMETER ::    &
                        zal=0.032_rsh,     &
                        zgr= 10.0_rsh,     &
                        zga= 287.04_rsh,   &  !   []
                        zc1= 0.7859_rsh,   &
                        zc2= 0.03477_rsh,  &
                        zc3= 0.00412_rsh,  &
                        zc4= 0.622_rsh,    &
                        zc5= 1.0_rsh-zc4,  &
                        zc6= 0.61_rsh,     &
                        zc8= 2.2e-3,       &
                        zzv= 10.0_rsh,     &  ! def  [m]
                        zca= 0.4_rsh,      &
                        zm1= 13.2_rsh,     &
                        zm2= 0.0_rsh,      &
                        zm3= 1.e-06,       &
                        zco= 1.e-04_rsh,   &
                        cqs= 1.32e-3,      &
                        cql= 1.15e-3,      &
                        zcodu= 1.e-3,      &
                        zcodt= 1.e-3,      &
                        zcodq = 1.e-3,     &
                        zhlw= 2.5e+06,     &
                        zhse= 1004.0_rsh,  &
                        zzero= 0.0_rsh,    &
                        krenf= 1.2_rsh,    &
                        zdept= 10.0_rsh,   &
                        zvita= 1.0_rsh,    &
                        zvito= 0.5_rsh,    &
                        zzone= 1.0_rsh,    &
                        coest= -0.005_rsh, &
                        coein= -0.005_rsh, &
                        a1= -0.146785_rsh, &
                        b1= -0.292400_rsh, &
                        c1= -2.207748_rsh, &
                        d1=  1.612292_rsh, &
                        csss=0.527_rsh

   INTEGER zitime

   REAL(KIND=rsh)    ::   &
                        zu_star, zz_utas, zz_usta,               &
                        zu_plog, zu_rapu, zu_dist,zt_sta1,       &
                        zt_sta2,zt_dist , zq_sta1,zq_sta2,       &
                        zq_dist,zu_refer, zq_evapo,              &
                        zq_hlate,zt_hsens, zu_zotes,zlmob,       &
                        zt_artz, zt_usta, zt_plog,               &
                        zu_prlog,zq_artz,zq_usta,                &
                        zq_plog, zkav,zdvp,zkov,zkot,            &
                        zkaq,dift,difq,zwind,                    &
                        zvis,zcin,zdaq,zkoq,zdiq,zdit,           &
                        nkov,nkot,nkoq,ztax,ztay,zten,           &
                        ztqa,zesa,zaqa,zrha,ztva,ztpa,ztka,zvpa, &

                        ztqo,zeso,zaqo,ztvo,ztpo,ztko,zvpo,ztmo, &
                        zcte,zkma,zcoq,zkmi,                     &
                        zcqe,zkat,zcor,zobmi,zobms,              &

                        st_zpsiv,st_zpsit,st_zpsiq,              &
                        in_zpsiv,in_ta,in_zpsit,st_zobmi,        &
                        in_zpsiq,in_zobmi,in_zx,in_l1,in_l2,     &
                        zlma,zpsiv,zpsit,zpsiq

   !!----------------------------------------------------------------------
   !! * Executable part


   !------------------------------------------------------------------!
   !  computation of wind direction                                   !
   !------------------------------------------------------------------!

         zwind  = MAX(zwindmod,0.5_rsh)          ! Warning : a minimum is required [m/s]
         zvis   = zm1 + zm2*zsat                 ! [degrees Celsius] []
         zcin   = zvis*zm3                       ! kinematic viscosity []

   !------------------------------------------------------------------!
   !   Atmospherical parameters                                       !
   !------------------------------------------------------------------!

         ztqa   = (zc1+zc2*zsat)/(1.0_rsh+zc3*zsat)
         ztqa   = 10.0_rsh**ztqa                     !  saturation vapor pressure [mbar]
         ztqa   = 100.0_rsh*ztqa                     !  [mbr] -----> [Pa]
         ztqa   = zrh*ztqa                           !  relative humidity
         zesa   = zc4*ztqa/(zpa-zc5*ztqa)            !  specific humidity

         ztka   = zsat + kelvin           ! Air temperature [Kelvin]
         ztpa   = ztka + zt*0.0098_rsh    ! Potential temperature
                                          ! the air temperature increases by 0.98 degrees Celsius per 100 meters in adiabatic conditions

         ztva   =  ztka*(1.0_rsh+zc6*zesa)    ! Virtual Temperature

         zvpa   =  ztva                  ! Virtual Potential temperature
                                         ! normalement on doit ajouter un alpha
                                         ! qui est egale zt*0.0098. ATTENTION approximation
         zaqa   =  ztqa/(zga*ztka)       ! Absolue Humidity    []
         zrha   =  zpa/(zga*ztka)        ! air humidity density -> CLIO  []

   !------------------------------------------------------------------!
   !   Ocean parameters                                               !
   !------------------------------------------------------------------!

         ztqo = (zc1+zc2*ztemp)/(1.0_rsh+zc3*ztemp)
         ztqo = 10.0_rsh**ztqo                  ! saturation vapor pressure [mbar]
         ztqo = 100.0_rsh*ztqo                  ! [mbr] -----> [Pa]
         ztqo = suswe*ztqo                      ! salinity effect
         zeso = zc4*ztqo/(zpa-zc5*ztqo)         ! specific humidity -> CLIO

         ztko = ztemp + kelvin                  ! Sea surface temperature [Kelvin]
         ztpo = ztko                            ! Potential temperature  ATTENTION approximation
         ztvo = ztko*(1.0_rsh+zc6*zeso)         ! Virtual Temperature
         zvpo = ztvo                            ! Virtual Potential temperature ATTENTION approximation
         zaqo = ztqo/(zga*ztko)                 ! Absolue Humidity -> CLIO ??? []

   !-------------------------------------------------------------------!
   !      Computation of stability criterios                           !
   !                               : terme de gradient adiabatique     !
   !-------------------------------------------------------------------!

                                         ! difference of humidity [] [kg/kg](au lieu
         zdiq   =   zesa - zeso          ! de prendre la dif d humidite absolue
                                         ! je vais prendre celle de l humidite
                                         ! specifique)
         zdaq   =   zeso - zesa

         zdit   =   ztpo - ztpa          ! difference of potential temperatures  [K]

         zdvp   =   zvpo - zvpa          ! difference of virtual potential temperatures [K]

         ztmo   =   ztka*(1.0_rsh + zc8*ztka*zesa)

   !------------------------------------------------------------------!
   !   Initialization parameters with Smith et al . 1998 (neutral)    !
   !------------------------------------------------------------------!

         zkot = 1.3_rsh    ! Drag coefficient []
         zkov = 1.3_rsh

         IF ((zwind.ge.cv0).AND.(zwind.le.cv52)) THEN
           zkot = a1*EXP(b1*(zwind+c1))+(d1/zwind)+1.0_rsh    ! Drag coefficient (smith)
         ENDIF

         IF ((zwind.ge.cv0).AND.(zwind.le.cv2)) THEN
           zkov = 1.0_rsh-0.01_rsh*zwind
         ELSE IF ((zwind.ge.cv3).AND.(zwind.le.cv4)) THEN
           zkov =  0.91_rsh+0.02_rsh*zwind
         ELSE IF ((zwind.ge.cv5).AND.(zwind.le.cv6)) THEN
           zkov =  0.83_rsh+0.04_rsh*zwind
         ELSE IF ((zwind.ge.cv7).AND.(zwind.le.cv8)) THEN
           zkov =  0.71_rsh+0.06_rsh*zwind
         ELSE IF ((zwind.ge.cv9).AND.(zwind.le.cv10)) THEN
           zkov =  0.70_rsh+0.06_rsh*zwind
         ELSE IF ((zwind.ge.cv11).AND.(zwind.le.cv13)) THEN
           zkov =  0.80_rsh+0.05_rsh*zwind
         ELSE IF ((zwind.ge.cv14).AND.(zwind.le.cv18)) THEN
           zkov =  0.81_rsh+0.05_rsh*zwind
         ELSE IF ((zwind.ge.cv19).AND.(zwind.le.cv24)) THEN
           zkov =  0.80_rsh+0.05_rsh*zwind
         ELSE IF ((zwind.ge.cv25).AND.(zwind.le.cv52)) THEN
           zkov =  1.04_rsh+0.04_rsh*zwind
         END IF

         zkov  = zkov*1.e-3
         zkot  = zkot*1.e-3
         zkoq  = zkot
         zkav  = zkov
         zkat  = zkot
         zkaq  = zkot
         nkov  = zkov
         nkot  = zkot
         nkoq  = zkoq
         zkma  = 2.9e-3
         zkmi  = 1.1e-3


!------------------------------------------------------------------!
!   Obukhov  parameters                                            !
!------------------------------------------------------------------!

         IF ((ABS(zdiq).ge.zzero).AND.(ABS(zdit).ge.zzero)) THEN

           zitime = 0
           zu_dist = 1.0_rsh
           zt_dist = 1.0_rsh
           zq_dist = 1.0_rsh

           !---------------------------------------------------------------------!
           ! Controle de la coherence entre les deux termes definisants les 3 flux!
           !---------------------------------------------------------------------!
           DO WHILE ((zu_dist > zcodu) .OR. (zt_dist > zcodt) .OR. (zq_dist > zcodq))

             zitime = zitime + 1

             zcor    =   zwind*zwind*ztva      ! U*U*Tv
             zcor    =   zcor*(zkov**1.5_rsh)  ! U*U*TvCd***1.5
             zcor    =   zca*zgr*zzv*zkot/zcor ! K*Z*g*Ct/U*U*Tv*Cd***1.5

             zcte    =   zdit*zcor

             zcoq    =   zc6*ztka*zkoq         !
             zcoq    =   zcoq/(zkot)
             zcqe    =   zcoq*zdaq*zcor        !

             zobms   =   coest*(zcte + zcqe)    !
             zobmi   =   coein*(zcte + zcqe)    ! (Z/L) parametre de stabilite
             zlmob   =   10.0_rsh/coein*(zcte + zcqe)

   !------------------------------------------------------------------!
   !   Stable conditions                                              !
   !------------------------------------------------------------------!

             st_zobmi  = MAX(zzero,zobms)
             st_zpsiv  =  -7.0_rsh * st_zobmi  ! stability function for wind
             st_zpsit  =  st_zpsiv             ! stability function for hsensible
             st_zpsiq  =  st_zpsiv             ! stability function for hlatent

   !------------------------------------------------------------------!
   !   Instable conditions                                            !
   !------------------------------------------------------------------!

             in_zobmi  = MIN(zzero,zobmi)
             in_zx     =  (1.0_rsh - 16.0_rsh * in_zobmi)**0.25_rsh
             in_l1     =  2.0_rsh * LOG((1.0_rsh+in_zx)/2.0_rsh)
             in_l2     =  LOG((1.0_rsh+ in_zx*in_zx)/2.0_rsh)
             in_ta     =  2.0_rsh * ATAN(in_zx)
             in_zpsiv  =  in_l1 + in_l2 - in_ta + 3.1416_rsh/2.0_rsh
             in_zpsit  =  2.0_rsh*in_l2
             in_zpsiq  =  in_zpsit

   !------------------------------------------------------------------!
   !   Computation of stability function                              !
   !------------------------------------------------------------------!

             zlma     =   MAX(zzero,SIGN(zzone,zdvp)) ! zlma = 0. instable
                                                      !      = 1. stable
             zpsiv    =   zlma*in_zpsiv + (1.0_rsh - zlma)*st_zpsiv
             zpsit    =   zlma*in_zpsit + (1.0_rsh - zlma)*st_zpsit
             zpsiq    =   zpsit

   !------------------------------------------------------------------!
   !  calcul de u_star    : correction sur le cd et le ustatr         !
   !------------------------------------------------------------------!

             zu_star  =   (zkov**0.5_rsh)*zwind         ! old u*
             zz_utas  =   zal*zu_star*zu_star/zgr       ! z* by charnock
             zz_usta  =   zz_utas +  zcin/(zgr*zu_star) ! z*+effet de faible vent

             zu_plog  =   (LOG(zzv/zz_usta)-zpsiv)/zca  ! profil log de u
             zu_rapu  =   zwind/zu_star                 ! rapport des u
             zu_star  =   zu_star*zu_rapu/zu_plog       ! new u* corrige
             zkov  =   (zu_star/zwind)**2               ! new coef de transfere
             zu_dist  =   zu_plog - zu_rapu             ! diff of news profils
             zu_dist  =   ABS(zu_dist)

             IF (zkov > zkma)   zkov = nkov + zkmi
             IF (zkov < zkmi)   zkov = nkov*krenf

   !-------------------------------------------------------------------!
   !  calcul de  Q_star : correction sur la temperature et le ct       !
   !-------------------------------------------------------------------!

             IF (zitime == 1) then
               zt_artz   =   zca*zca/(zkot*LOG(10.0_rsh/zz_usta))
               zt_usta   =   10.0_rsh/EXP(zt_artz)
               zt_plog   =   LOG(zt/zt_usta)-zpsit
             ENDIF

             zt_sta1   =   1.0_rsh/(LOG(zzv/zt_usta)-zpsit)
             zt_sta2   =   zkot*zwind/(zca*zu_star)
             zkot   =   zkot*zt_sta1/zt_sta2
             zt_dist   =   zdit*(zt_sta1 - zt_sta2)
             zt_dist   =   ABS(zt_dist)

             IF (zkot > zkma) zkot = nkot + zkmi
             IF (zkot < zkmi) zkot = nkot*krenf

   !-------------------------------------------------------------------!
   !  calcul de q_star     : correction sur lhumidité  et le c        !
   !-------------------------------------------------------------------!

             IF (zitime == 1) THEN
               zq_artz   =   zca*zca/(zkoq*LOG(10.0_rsh/zz_usta))
               zq_usta   =   10.0_rsh/EXP(zq_artz)
               zq_plog   =   LOG(zt/zq_usta)-zpsiq
             ENDIF

             zq_sta1   =   1.0_rsh/(LOG(zzv/zq_usta)-zpsiq)
             zq_sta2   =   zkoq*zwind/(zca*zu_star)
             zkoq   =   zkoq*zq_sta1/zq_sta2
             zq_dist   =   zdiq*(zq_sta1 - zq_sta2)
             zq_dist   =   ABS(zq_dist)

             IF (zkoq > zkma)   zkoq = nkov + zkmi
             IF (zkoq < zkmi)   zkoq = nkoq*krenf

   !-------------------------------------------------------------------!
   !  cas non convergent                                               !
   !-------------------------------------------------------------------!

             IF (zitime == 10)  then
               zt_dist = 0.00001_rsh    ! convergence criterium is satisfied
               zq_dist = 0.00001_rsh    ! convergence criterium is satisfied
               zu_dist = 0.00001_rsh    ! convergence criterium is satisfied
               zkot    = nkot           ! keep the values of the last iteration
               zkoq    = nkoq
               zkov    = nkov
               dift    = zdit           ! difference of temperature
               difq    = zdaq
             ENDIF

           END DO

   !---------------------------------------------------------------------!
   !       calcul de Long Obukhov & Monin et de U_ref                    !
   !---------------------------------------------------------------------!

           zu_prlog = (LOG(zzv/zz_usta) - zpsiv)/zca
           zu_refer = zu_star*zu_prlog
           zu_refer = zwind

         ELSE                                          ! cas neutre et stable

           zkoq   =   nkoq
           zkot   =   nkot
           zkov   =   nkov
           zu_refer  =  zwind
           zu_star   =  (zkov**0.5_rsh)*zwind
           zz_utas   =  zal*zu_star*zu_star/zgr          ! z* by charnock
           zz_usta   =  zz_utas + zcin/(zgr*zu_star)     ! z*+effet de faible vent
           zt_sta2   =  zdit*zkot*zwind/(zca*zu_star)
           zq_sta2   =  zdaq*zkoq*zwind/(zca*zu_star)
           zu_plog  =   LOG(zzv/zz_usta)/zca
           zkov  =   (1.0_rsh/zu_plog)**2
           zkoq   =   nkoq
           zkot   =   nkot

         ENDIF

   !---------------------------------------------------------------------!
   ! Calcul des trois flux : tension de vent,                            !
   !                        flux de chaleur latente et sensible          !
   !---------------------------------------------------------------------!

         zq_evapo  = zrha*zkoq*zu_refer*(zeso-zesa)        ! evaporation m/s
         zq_hlate  = - zhlw*zq_evapo                       ! latent heat flux

         zt_hsens  = - zrha*zhse*zkot*zu_refer*(ztko-ztka) ! sensible heat flux

         zu_zotes  = zrha*zkov                        ! wind
         zu_zotes  = zu_zotes*zu_refer
         zu_zotes  = zu_zotes*zu_refer                ! wind stress

         ! Wind stress
         ztosx=zrha*zkov*ztosx
         ztosy=zrha*zkov*ztosy

         ! zu_refer is the modified module of the wind 
         ! from which all the exchange coefficients have been estimated.
         ! ztosx = ||V||*Vx ; ztosy = ||V||*Vy, so zu_refer/zwindmod removes
         ! the module of the wind in the formula,
         ! the second factor zu_refer/zwindmod corrects the component of the wind

         IF (zwindmod > 0.0_rsh) THEN
           ztosx=ztosx*(zu_refer/zwindmod)**2
           ztosy=ztosy*(zu_refer/zwindmod)**2
         ENDIF

#ifdef key_windstress_module
         ztosx=zrha*zkov*zwindmod*zwindmod
         ztosy=zu_zotes
#endif


   !---------------------------------------------------------------------!
   ! Assign out variables                                                !
   !---------------------------------------------------------------------!

         zsenslos = zt_hsens
         zlatlos  = zq_hlate
         zevap    = zq_evapo
#ifdef key_sfls_coef
         zchs(i,j) = zkot
         zces(i,j) = zkoq
       ELSE
         zchs(i,j) = 0.0_rsh
         zces(i,j) = 0.0_rsh
#endif

  END SUBROUTINE sflx_turb_ayina
