  SUBROUTINE sflx_turb_large(ztemp,zsat,zwindmod,zpa,zrh,zrad,zirlos,zrain, &
                             zsenslos,zlatlos,zevap,ztosx,ztosy)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE sflx_turb_large  ***
   !&E
   !&E ** Purpose : latent and sensible heat fluxes estimates
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : sflxsurf.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference : Large and Yeager (2004)
   !&E                METEOCEAN report (2009)
   !&E
   !&E ** History :
   !&E             issued from OPA 9.0
   !&E       !  2004-08  (U. Schweckendiek)  Original code
   !&E       !  2005-04  (L. Brodeau, A.M. Treguier) severals modifications:
   !&E                   - new bulk routine for efficiency
   !&E                   - WINDS ARE NOW ASSUMED TO BE AT T POINTS in input files
   !&E       !  2006-02  (S. Masson, G. Madec)  IOM read + NEMO "style"
   !&E       !  2006-12  (L. Brodeau) handle both 2m and 10m input fields
   !&E       !  2009-06  (B. Levier, V. Garnier) MARS implementation
   !&E
   !&E Rq vl : Symphonie utilise 6 iterations et remonte la temp potentielle de l eau en surface
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE comvars2d,  ONLY : jb,jh,ig,id,dt,nbouc,iscreenlog
   USE cometdyn,   ONLY : l_sflx_ath2m
   USE_MPI toolmpi,  ONLY : findmax_mpi_3,MASTER

   !! * Arguments
   REAL(KIND=rsh), INTENT( IN )      :: ztemp,zsat,zwindmod,zpa,zrh, &
                                        zrad,zirlos,zrain
   REAL(KIND=rsh), INTENT( OUT )     :: zsenslos,zlatlos,zevap
   REAL(KIND=rsh), INTENT( INOUT )   :: ztosx,ztosy

   !! * Local declarations
!   REAL(KIND=rsh)            ::   &
!         zzu , zzv              ! scalar 
   REAL(KIND=rsh)                                   ::   &
         zsst,            &  ! SST                                    [K]
         zat,             &  ! air temperature                        [K]
         zssu,            &  ! zonal sea surface current              [m/s]
         zssv,            &  ! meridional sea surface current         [m/s]
         dUnormt,         &  ! relative wind (norm) on T points       [m/s]
!         ztaux, ztauy,    &  ! wind stress computed at T-point
         sh_w,            &  ! specific humidity at zSST              [kg/kg]
         sh_a,            &  ! air spec. hum.                         [kg/kg]
         cheat,           &  ! coefficient for sensible heat flux
         cevap,           &  ! coefficient for latent heat flux
         cdrag,           &  ! coefficient for wind stress
         zat_zu,          &  ! air temperature at wind speed height   [K]
         sh_a_zu             ! air spec. hum.  at wind speed height   [kg/kg]
   REAL(KIND=rsh), PARAMETER ::   &
         rhoair  = 1.22_rsh,  &  ! air density                        [kg/m3]
         cp    = 1000.5_rsh,  &  ! specific heat of air               [J/kg/K]
         zu    = 10.0_rsh        ! height for dU                      [m]
   INTEGER :: ii,jj

   !!----------------------------------------------------------------------
   !! * Executable part   

   ! --------------
   ! Initialization
   ! --------------

   ! Temperatures in Kelvin
   ! ----------------------

   ! sea temperature [Kelvin]
   ! Caution set to kelvin over land, not 0 Kelvin (otherwise floating point exception in bulk computation) 
   zsst = MAX( (ztemp + kelvin) , 0.5_rsh )

   ! air potential temperature [Kelvin]
   ! the air temperature increases by 0.98 degrees Celsius per 100 meters
   zat = zsat + kelvin + 0.0098_rsh*zt


   ! Sea surface current to estimate scalar wind ( = | U10m - SSU | )
   ! It is important to take into account the sea surface courant
   ! ----------------------------------------------------------------

   ! Interpolation of surface current at T-point, zssu and zssv :
!   DO j = 2, ljmaxm1
!      DO i = 1, imax
!         zssu(i,j) = ( gu(i-1,j  ) + gu(i,j) ) * 0.5_rsh
!         zssv(i,j) = ( gv(i  ,j-1) + gv(i,j) ) * 0.5_rsh
!      END DO
!   END DO
#ifdef key_courant_notreadyyet
   zssu = un(i,j)
   zssv = vn(i,j)
#else
   zssu = 0.0_rsh
   zssv = 0.0_rsh
#endif


   ! Module of relative wind
   ! -----------------------

   ! Wind components are provided on T-points within the netcdf input file.
   ! Thus, the wind module is computded at T-points taking into account the sea

!      zzu = MIN(ABS(u10 - zssu),50.0_rsh)
!      zzv = MIN(ABS(v10 - zssv),50.0_rsh)
!      dUnormt = SQRT( zzu*zzu + zzv*zzv )
       dUnormt = zwindmod

   ! specific humidities
   ! -------------------
   sh_w = suswe*640380.0_rsh*exp(-5107.4_rsh/zsst)/rhoair
   sh_a = zrh*640380.0_rsh*exp(-5107.4_rsh/zat)/rhoair


   ! ---------------------------------------------------------------------
   ! CORE iterative algo for computation of cdrag, cheat, cevap at T-point
   ! ---------------------------------------------------------------------

   IF ( l_sflx_ath2m ) THEN   ! air temp. and spec. hum. are given at different height (2m) than wind (10m)

     IF ( nbouc == 1 )   THEN
       IF_MPI (MASTER) THEN
       WRITE(iscreenlog,*) 
       WRITE(iscreenlog,*) 'Calling coef_large_2z for bulk transfert coefficients'
       WRITE(iscreenlog,*) 
       ENDIF_MPI
     ENDIF
     CALL coef_large_2z(zt,zu,zsst, zat, sh_w, sh_a, &
         &              dUnormt, cdrag, cheat, cevap, zat_zu, sh_a_zu)

   ELSE                ! air temp. and spec. hum. are given at same height than wind (10m)

     IF( nbouc == 1 )    THEN
       IF_MPI (MASTER) THEN
       WRITE(iscreenlog,*) 
       WRITE(iscreenlog,*) 'Calling coef_large_1z for bulk transfert coefficients'
       ENDIF_MPI
     ENDIF
     CALL coef_large_1z(zu, zsst, zat, sh_w, sh_a,  &
         &              dUnormt, cdrag, cheat, cevap )
   ENDIF


   ! -------------------------
   ! Momentum over ocean
   ! -------------------------

   ztosx=rhoair*cdrag*ztosx
   ztosy=rhoair*cdrag*ztosy

   ! dUnormt is the modified module of the wind 
   ! from which all the exchange coefficients have been estimated.
   ! ztosx = ||V||*Vx ; ztosy = ||V||*Vy, so dUnormt/zwindmod removes
   ! the module of the wind in the formula,
   ! the second factor dUnormt/zwindmod corrects the component of the wind

   IF (zwindmod > 0.0_rsh) THEN
     ztosx=ztosx*(dUnormt/zwindmod)**2
     ztosy=ztosy*(dUnormt/zwindmod)**2
   ENDIF

#ifdef key_windstress_module
   ztosx=rhoair*cdrag*zwindmod*zwindmod
   ztosy=rhoair*cdrag*dUnormt*dUnormt
#endif


   ! ---------------------------
   ! Turbulent fluxes over ocean
   ! ---------------------------

   IF( l_sflx_ath2m ) THEN

      ! Values of temp. and hum. adjusted to 10m must be used instead of 2m values
      zsenslos = -rhoair*cp*cheat*(zsst - zat_zu)*dUnormt
      zevap    = rhoair*cevap*(sh_w - sh_a_zu)*dUnormt

   ELSE

      zsenslos = -rhoair*cp*cheat*(zsst - zat)*dUnormt
      zevap    = rhoair*cevap*(sh_w - sh_a)*dUnormt

   END IF

   zlatlos =  -lnu * zevap

  END SUBROUTINE sflx_turb_large

   !!======================================================================

  SUBROUTINE coef_large_1z(zu, sst, T_a, q_w, q_a,   &
   &                     dU, cdrag, cheat, cevap   )
   !!----------------------------------------------------------------------
   !!                      ***  ROUTINE  turb_core  ***
   !!
   !! ** Purpose :   Computes turbulent transfert coefficients of surface
   !!                fluxes according to Large & Yeager (2004)
   !!
   !! ** Method  :   I N E R T I A L   D I S S I P A T I O N   M E T H O D
   !!      Momentum, Latent and sensible heat exchange coefficients
   !!      Caution: this procedure should only be used in cases when air
   !!      temperature (T_air), air specific humidity (q_air) and wind (dU)
   !!      are provided at the same height 'zzu' (10 m)!
   !!
   !! References :
   !!   Large & Yeager, 2004 : Diurnal to decadal global forcing
   !!                          for ocean and sea-ice models: the data sets
   !!                          and flux climatologies. CGD Division of the National Center
   !!                          for Atmospheric Research, NCAR Technical Note: NCAR/TN-460+STR.
   !!
   !! History :
   !!        !  XX-XX  (???     )  Original code
   !!   9.0  !  05-08  (L. Brodeau) Rewriting and optimization
   !!----------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : grav

   !! * Arguments
   REAL(KIND=rsh), INTENT(in) :: zu           ! altitude of wind measurement       [m]
   REAL(KIND=rsh), INTENT(in)                                   ::  &
         sst,      &       ! sea surface temperature         [Kelvin]
         T_a,      &       ! potential air temperature       [Kelvin]
         q_w,      &       ! sea surface specific humidity   [kg/kg]
         q_a,      &       ! specific air humidity           [kg/kg]
         dU                ! wind module |U(zu)-U(0)|        [m/s]
   REAL(KIND=rsh), intent(out)                                    :: &
         cdrag,    &       ! transfert coefficient for momentum       (tau)
         cheat,    &       ! transfert coefficient for temperature (Q_sens)
         cevap             ! transfert coefficient for evaporation  (Q_lat)

   !! * Local declarations
   REAL(KIND=rsh)                                    ::   &
         dU10,           & ! relative wind dU                     [m/s]
         dT,             & ! air/sea temperature differeence      [K]
         dq,             & ! air/sea humidity difference          [kg/kg]
         cdrag_n10,      & ! 10m neutral drag coefficient
         cevap_n10,      & ! 10m neutral latent coefficient
         cheat_n10,      & ! 10m neutral sensible coefficient
         sqrt_cdrag_n10, & ! root square of cdrag_n10
         sqrt_cdrag,     & ! root square of cdrag
         T_vpot,         & ! virtual potential temperature        [K]
         T_star,         & ! turbulent scale of tem. fluct.
         q_star,         & ! turbulent humidity of temp. fluct.
         U_star,         & ! turb. scale of velocity fluct.
         L,              & ! Monin-Obukov length                  [m]
         zeta,           & ! stability parameter at height zu
         U_n10,          & ! neutral wind velocity at 10m         [m]   
         xlogt, xct, zpsi_h, zpsi_m
   REAL(KIND=rsh), PARAMETER        :: milli = 0.001_rsh
   !!
   INTEGER :: j_itt
   INTEGER, PARAMETER :: nb_itt = 3
   INTEGER                                    ::   &
         stab              ! 1st guess stability test integer

   REAL(KIND=rsh), PARAMETER ::                        &
         karman  = 0.4_rsh          ! von Karman s constant
 
   !! * Start
   !! Air/sea differences
   dU10 = max(0.5_rsh, dU) ! we don t want to fall under 0.5 m/s
   dT = T_a - sst          ! assuming that T_a is allready the potential temp. at zzu
   dq = q_a - q_w
   !!    
   !! Virtual potential temperature
   T_vpot = T_a*(1.0_rsh + 0.608_rsh*q_a)
   !!
   !! Neutral Drag Coefficient
   stab    = 0.5_rsh + sign(0.5_rsh,dT)    ! stable : stab = 1 ; unstable : stab = 0 
   cdrag_n10  = milli * ( 2.7_rsh/dU10 + 0.142_rsh + dU10/13.09_rsh )    !   L & Y eq. (6a)
   sqrt_cdrag_n10 = sqrt(cdrag_n10)
   cevap_n10  = milli * ( 34.6_rsh * sqrt_cdrag_n10 )                     !   L & Y eq. (6b)
   cheat_n10  = milli*sqrt_cdrag_n10*(18.0_rsh*stab + 32.7_rsh*(1-stab))  !   L & Y eq. (6c), (6d)
   !!
   !! Initializing transfert coefficients with their first guess neutral equivalents :
   cdrag = cdrag_n10 ;  cevap = cevap_n10 ;  cheat = cheat_n10 ;  sqrt_cdrag = sqrt(cdrag)

   !! * Now starting iteration loop
   DO j_itt=1, nb_itt
      !! Turbulent scales :
      U_star  = sqrt_cdrag*dU10                !   L & Y eq. (7a)
      T_star  = cheat/sqrt_cdrag*dT            !   L & Y eq. (7b)
      q_star  = cevap/sqrt_cdrag*dq            !   L & Y eq. (7c)

      !! Estimate the Monin-Obukov length      ! L & Y eq. (8a)
      L  = (U_star**2)/( karman*grav*(T_star/T_vpot + q_star/(q_a + 1.0_rsh/0.608_rsh)) )

      !! Stability parameters :
      zeta = zu/L ;  zeta   = sign( min(abs(zeta),10.0_rsh), zeta )
      zpsi_h = psi_h(zeta)
      zpsi_m = psi_m(zeta)

      !! Shifting the wind speed to 10m and neutral stability :
      U_n10 = dU10*1.0_rsh/(1.0_rsh + sqrt_cdrag_n10/karman*(log(zu/10.0_rsh) - zpsi_m)) !  L & Y eq. (9a)

      !! Updating the neutral 10m transfer coefficients :
      cdrag_n10  = milli * (2.7_rsh/U_n10 + 0.142_rsh + U_n10/13.09_rsh)     !  L & Y eq. (6a)
      sqrt_cdrag_n10 = sqrt(cdrag_n10)
      cevap_n10  = milli * (34.6_rsh * sqrt_cdrag_n10)                       !  L & Y eq. (6b)
      stab    = 0.5_rsh + sign(0.5_rsh,zeta)
      cheat_n10  = milli*sqrt_cdrag_n10*(18.0_rsh*stab + 32.7_rsh*(1-stab))  !  L & Y eq. (6c), (6d)

      !! Shifting the neutral  10m transfer coefficients to ( zu , zeta ) :
      !!
      xct = 1.0_rsh + sqrt_cdrag_n10/karman*(log(zu/10) - zpsi_m)
      cdrag  = cdrag_n10/(xct*xct) ;  sqrt_cdrag = sqrt(cdrag)     !  L & Y eq. (10a)
      !!
      xlogt = log(zu/10.0_rsh) - zpsi_h
      !!
      xct = 1.0_rsh + cheat_n10*xlogt/karman/sqrt_cdrag_n10
      cheat  = cheat_n10*sqrt_cdrag/sqrt_cdrag_n10/xct             !  L & Y eq. (10b)
      !!
      xct = 1.0_rsh + cevap_n10*xlogt/karman/sqrt_cdrag_n10
      cevap  = cevap_n10*sqrt_cdrag/sqrt_cdrag_n10/xct             !  L & Y eq. (10c)
      !!
   END DO
   !!
  END SUBROUTINE coef_large_1z

   !!======================================================================

  SUBROUTINE coef_large_2z(zt, zu, sst, T_zt, q_w, q_zt, dU, cdrag, cheat, cevap, T_zu, q_zu)
   !!----------------------------------------------------------------------
   !!                      ***  ROUTINE  turb_core  ***
   !!
   !! ** Purpose :   Computes turbulent transfert coefficients of surface 
   !!                fluxes according to Large & Yeager (2004).
   !!
   !! ** Method  :   I N E R T I A L   D I S S I P A T I O N   M E T H O D
   !!      Momentum, Latent and sensible heat exchange coefficients
   !!      Caution: this procedure should only be used in cases when air
   !!      temperature (T_air) and air specific humidity (q_air) are at 2m
   !!      whereas wind (dU) is at 10m.
   !!
   !! References :
   !!      Large & Yeager, 2004 : ???
   !! History :
   !!   9.0  !  06-12  (L. Brodeau) Original code for 2Z
   !!----------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : grav

   !! * Arguments
   REAL(KIND=rsh), INTENT(in)   :: &
         zt,      &     ! height for T_zt and q_zt                   [m]
         zu             ! height for dU                              [m]
   REAL(KIND=rsh), INTENT(in)                                   ::  &
         sst,     &     ! sea surface temperature              [Kelvin]
         T_zt,    &     ! air potential temperature            [Kelvin]
         q_w,     &     ! sea surface specific humidity         [kg/kg]
         q_zt,    &     ! specific air humidity                 [kg/kg]
         dU             ! relative wind module |U(zu)-U(0)|       [m/s]
   REAL(KIND=rsh), INTENT(out)                                    ::  &
         cdrag,   &     ! transfer coefficient for momentum         (tau)
         cheat,   &     ! transfer coefficient for sensible heat (Q_sens)
         cevap,   &     ! transfert coefficient for evaporation   (Q_lat)
         T_zu,    &     ! air temp. shifted at zu                     [K]
         q_zu           ! spec. hum.  shifted at zu               [kg/kg]

   !! * Local declarations
   REAL(KIND=rsh), PARAMETER        :: milli = 0.001_rsh, &
                                       karman  = 0.4_rsh      ! von Karman s constant
   REAL(KIND=rsh)                                   ::  &
         dU10,           & ! relative wind dU                  [m/s]
         dT,             & ! air/sea temperature differeence   [K]
         dq,             & ! air/sea humidity difference       [kg/kg]
         cdrag_n10,      & ! 10m neutral drag coefficient
         cevap_n10,      & ! 10m neutral latent coefficient
         cheat_n10,      & ! 10m neutral sensible coefficient
         sqrt_cdrag_n10, & ! root square of cdrag_n10
         sqrt_cdrag,     & ! root square of cdrag
         T_vpot_u,       & ! virtual potential temperature        [K]
         T_star,         & ! turbulent scale of tem. fluct.
         q_star,         & ! turbulent humidity of temp. fluct.
         U_star,         & ! turb. scale of velocity fluct.
         L,              & ! Monin-Obukov length                  [m]
         zeta_u,         & ! stability parameter at height zu
         zeta_t,         & ! stability parameter at height zt
         U_n10,          & ! neutral wind velocity at 10m        [m]
         xlogt, xct, zpsi_hu, zpsi_ht, zpsi_m

   INTEGER :: j_itt
   INTEGER, PARAMETER :: nb_itt = 3   ! number of itterations
   INTEGER                                   :: &
           &     stab                ! 1st stability test integer

   !!  * Start

   !! Initial air/sea differences
   dU10 = max(0.5_rsh, dU)      !  we don t want to fall under 0.5 m/s
   dT = T_zt - sst 
   dq = q_zt - q_w

   !! Neutral Drag Coefficient :
   stab = 0.5_rsh + sign(0.5_rsh,dT)                 ! stab = 1  if dT > 0  -> STABLE
   cdrag_n10  = milli*( 2.7_rsh/dU10 + 0.142_rsh + dU10/13.09_rsh )          !   L & Y eq. (6a)
   sqrt_cdrag_n10 = sqrt(cdrag_n10)
   cevap_n10  = milli*( 34.6_rsh * sqrt_cdrag_n10 )                          !   L & Y eq. (6b)
   cheat_n10  = milli*sqrt_cdrag_n10*(18.0_rsh*stab + 32.7_rsh*(1 - stab))   !   L & Y eq. (6c, 6d)

   !! Initializing transf. coeff. with their first guess neutral equivalents :
   cdrag = cdrag_n10 ;  cevap = cevap_n10 ;  cheat = cheat_n10 ;  sqrt_cdrag = sqrt(cdrag)

   !! Initializing z_u values with z_t values :
   T_zu = T_zt ;  q_zu = q_zt

   !!  * Now starting iteration loop
   DO j_itt=1, nb_itt
      dT = T_zu - sst ;  dq = q_zu - q_w ! Updating air/sea differences
      T_vpot_u = T_zu*(1.0_rsh + 0.608_rsh*q_zu)    ! Updating virtual potential temperature at zu
      U_star = sqrt_cdrag*dU10                ! Updating turbulent scales :   (L & Y eq. (7a))
      T_star  = cheat/sqrt_cdrag*dT           !   L & Y eq. (7b)
      q_star  = cevap/sqrt_cdrag*dq           !   L & Y eq. (7c)
      !!

      IF (T_star + 0.608_rsh*T_zu*q_star /= 0.0_rsh) THEN
        !! Estimate the Monin-Obukov length at height zu    ! L & Y eq. (8a)
        !L  = (U_star**2)/( karman*grav*(T_star/T_vpot_u + q_star/(q_zu + 1.0_rsh/0.608_rsh)) )
        !FAUXL = (U_star**2) / (karman*grav/T_vpot_u*(T_star*(1.0_rsh+0.608_rsh*q_zu) + 0.608_rsh*T_zu*q_star))
        L = (U_star**2) / (karman*grav/T_vpot_u*(T_star + 0.608_rsh*T_zu*q_star))
        !! Stability parameters :
        zeta_u  = zu/L  ;  zeta_u = sign( min(abs(zeta_u),10.0_rsh), zeta_u )
        zeta_t  = zt/L  ;  zeta_t = sign( min(abs(zeta_t),10.0_rsh), zeta_t )
      ELSE
        ! not really clean but... it happens very rarely and by luck not at the last iteration ?
        zeta_u  = 0.0_rsh
        zeta_t  = 0.0_rsh
      END IF
      zpsi_hu = psi_h(zeta_u)     !   L & Y eq. (8c, 8e)
      zpsi_ht = psi_h(zeta_t)     !   L & Y eq. (8c, 8e)
      zpsi_m  = psi_m(zeta_u)     !   L & Y eq. (8c, 8d)
      !!
      !! Shifting the wind speed to 10m and neutral stability : (L & Y eq.(9a))
      U_n10 = dU10/(1.0_rsh + sqrt_cdrag_n10/karman*(log(zu/10.0_rsh) - zpsi_m))
      !!
      !! Shifting temperature and humidity at zu :          (L & Y eq. (9b-9c))
      !! Main difference with coef_large_1z !!!
      T_zu = T_zt - T_star/karman*(log(zt/zu) + zpsi_hu - zpsi_ht)
      q_zu = q_zt - q_star/karman*(log(zt/zu) + zpsi_hu - zpsi_ht)
      !!
      !! q_zu cannot have a negative value : forcing 0
      stab = 0.5_rsh + sign(0.5_rsh,q_zu) ;  q_zu = stab*q_zu
      !!
      !! Updating the neutral 10m transfer coefficients :
      cdrag_n10  = milli * (2.7_rsh/U_n10 + 0.142_rsh + U_n10/13.09_rsh)    ! L & Y eq. (6a)
      sqrt_cdrag_n10 = sqrt(cdrag_n10)
      cevap_n10  = milli * (34.6_rsh * sqrt_cdrag_n10)                 ! L & Y eq. (6b)
      stab    = 0.5_rsh + sign(0.5_rsh,zeta_u)
      cheat_n10  = milli*sqrt_cdrag_n10*(18.0_rsh*stab + 32.7_rsh*(1-stab)) ! L & Y eq. (6c-6d)
      !!
      !!
      !! Shifting the neutral 10m transfer coefficients to (zu,zeta_u) :
      xct = 1.0_rsh + sqrt_cdrag_n10/karman*(log(zu/10.0_rsh) - zpsi_m)
      cdrag = cdrag_n10/(xct*xct) ; sqrt_cdrag = sqrt(cdrag)     !  L & Y eq. (10a)
      !!
      xlogt = log(zu/10.0_rsh) - zpsi_hu
      !!
      xct = 1.0_rsh + cheat_n10*xlogt/karman/sqrt_cdrag_n10
      cheat  = cheat_n10*sqrt_cdrag/sqrt_cdrag_n10/xct           !  L & Y eq. (10b)
      !!
      xct = 1.0_rsh + cevap_n10*xlogt/karman/sqrt_cdrag_n10
      cevap  = cevap_n10*sqrt_cdrag/sqrt_cdrag_n10/xct           !  L & Y eq. (10c)
      !!
      !!
   END DO
  END SUBROUTINE coef_large_2z

   !!======================================================================

  FUNCTION psi_m(zta)
   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE psi_m  ***
   !&E
   !&E ** Purpose : evaluate stability function for wind speed and scalars
   !&E              for the momentum variables
   !&E
   !&E ** Description : matches Kansas and free convection forms with weighting
   !&E             convective form follows Fairall et al (1996) with profile constants
   !&E             from Grachev et al (2000) BLM
   !&E             stable form from Beljaars and Holtslag (1991)
   !&E
   !&E ** Called by : sflx_turb_large
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference : Psis, L & Y eq. (8c), (8d), (8e)
   !&E
   !&E ** History :
   !&E          issued from OPA 9.0
   !&E       !  2009-06  (V. Garnier) MARS implementation
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used

   REAL(KIND=rsh),                                   INTENT(in) :: zta

   REAL(KIND=rsh), PARAMETER :: pi = 3.14159_rsh
   REAL(KIND=rsh)                                               :: psi_m
   REAL(KIND=rsh)                                               :: X2, X, stabit
   X2 = sqrt(abs(1.0_rsh - 16.0_rsh*zta))  ;  X2 = max(X2 , 1.0_rsh) ;  X  = sqrt(X2)
   stabit    = 0.5_rsh + sign(0.5_rsh,zta)
   psi_m = -5.0_rsh*zta*stabit  &                                                  ! Stable
        & + (1 - stabit)*(2.0_rsh*log((1.0_rsh + X)/2.0_rsh) + log((1.0_rsh + X2)/2.0_rsh) - 2.0_rsh*atan(X) + pi/2.0_rsh)  ! Unstable 
  END FUNCTION psi_m

   !!======================================================================

  FUNCTION psi_h(zta)
   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE psi_h  ***
   !&E
   !&E ** Purpose : evaluate stability function for wind speed and scalars
   !&E              for the heat fluxes variables
   !&E
   !&E ** Description : matches Kansas and free convection forms with weighting
   !&E             convective form follows Fairall et al (1996) with profile constants
   !&E             from Grachev et al (2000) BLM
   !&E             stable form from Beljaars and Holtslag (1991)
   !&E
   !&E ** Called by : sflx_turb_large
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference : Psis, L & Y eq. (8c), (8d), (8e)
   !&E
   !&E ** History :
   !&E          issued from OPA 9.0
   !&E       !  2009-06  (V. Garnier) MARS implementation
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used

   REAL(KIND=rsh),                                   INTENT(in) :: zta

   REAL(KIND=rsh)                                               :: psi_h
   REAL(KIND=rsh)                                               :: X2, X, stabit
   X2 = sqrt(abs(1.0_rsh - 16.0_rsh*zta))  ;  X2 = max(X2 , 1.0_rsh) ;  X  = sqrt(X2)
   stabit    = 0.5_rsh + sign(0.5_rsh,zta)
   psi_h = -5.0_rsh*zta*stabit  &                                       ! Stable
        & + (1 - stabit)*(2.0_rsh*log( (1.0_rsh + X2)/2.0_rsh ))        ! Unstable
    END FUNCTION psi_h

