 MODULE toolgeom
   !!======================================================================
   !!                   ***  MODULE toolgeom  ***
   !! Mathematical functions
   !!======================================================================
   !! * Modules used
   USE parameters, ONLY : rsh

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC f_dzu,f_dzw, &              ! function called by biolodynzwat
          f_hzu,       &              ! vertical scale factor at T-location
          f_hzw,       &              ! vertical scale factor at w-location (interface)
          f_hzu_x,     &              ! vertical scale factor at u-location
          f_hzw_x,     &              ! vertical scale factor at the interface and u-location
          f_hzu_y,     &              ! vertical scale factor at v-location
          f_hzw_y,     &              ! vertical scale factor at the interface and v-location
          f_zu,        &              ! depth at T-location
          f_zw,        &              ! depth at w-location (interface)
          f_zu_x,      &              ! depth at U-location (middle of the cell along the z-axis)
          f_zu_y,      &              ! depth at V-location (middle of the cell along the z-axis)
          f_zw_x,      &              ! depth at the interface at hx-location (upper right corner along xz-slice)
          f_zw_y                      ! depth at the interface at hy-location (upper right corner along yz-slice)

   !! * Shared or public module variables

   !! * Private variables

 CONTAINS

   !!======================================================================
 
  FUNCTION f_dzu(h0,xe,k,i,j)
 
   !&E---------------------------------------------------------------------
   !&E                 ***  FUNCTION f_dzu  ***
   !&E
   !&E ** Purpose : estimate the layer thickness between 2 interfaces (w-locations)
   !&E
   !&E ** Description : the u-location (along z-axis) is the middle of the cell
   !&E
   !&E ** Called by : biolodynzwat
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2009-10  (M. Sourisseau)  Original code
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE comvars3d,  ONLY : sig,dsigu
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : hc_sig,dcusds
#if ! defined key_siggen_g2
   USE comsiggen, ONLY : hzdcusds
#endif
#endif

   !! * Arguments
   INTEGER, INTENT( in )        :: k,i,j         ! indexes of the cell
   REAL(KIND=rsh), INTENT( in ) :: h0,xe
   REAL(KIND=rsh) :: f_dzu                       ! thickness

   !! * Local declarations
   REAL(KIND=rsh) :: d,hz_u

   !!----------------------------------------------------------------------
   !! * Executable part

#if defined key_siggen || defined key_gencoord
#ifdef key_siggen_g2
     hz_u=(xe+h0)/(hc_sig(i,j)+h0) * (hc_sig(i,j)*dsigu(k)+h0*dcusds(k))
#else
!     hz_u=xe+hc_sig(i,j)+(h0-hc_sig(i,j))*dcusds(k)
     hz_u=xe+hzdcusds(k,i,j)
    ! write(*,*) 'test',sum(dcusds(1:kmax)*dsigu(1:kmax)) ! must be equal to 1
#endif
     f_dzu=hz_u*dsigu(k)
#else
     d=h0+xe
     f_dzu=d*dsigu(k)
#endif

  END FUNCTION f_dzu

   !!======================================================================
 
  FUNCTION f_dzw(h0,xe,k,i,j)
 
   !&E---------------------------------------------------------------------
   !&E                 ***  FUNCTION f_dzw  ***
   !&E
   !&E ** Purpose : estimate the layer thickness between 2 u-locations
   !&E              (w location)
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : biolodynzwat
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2009-10  (M. Sourisseau)  Original code
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE comvars3d,  ONLY : sigw,dsigw
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : hc_sig,dcwsds
#if ! defined key_siggen_g2
   USE comsiggen, ONLY : hzdcwsds
#endif
#endif

   !! * Arguments
   INTEGER, INTENT( in )        :: k,i,j         ! indexes of the cell
   REAL(KIND=rsh), INTENT( in ) :: h0,xe
   REAL(KIND=rsh) :: f_dzw                       ! thickness

   !! * Local declarations
   REAL(KIND=rsh) :: d,hz_w

   !!----------------------------------------------------------------------
   !! * Executable part

#if defined key_siggen || defined key_gencoord
#ifdef key_siggen_g2
     hz_w=(xe+h0)/(hc_sig(i,j)+h0) * (hc_sig(i,j)*dsigw(k)+h0*dcwsds(k))
#else
!     hz_w=xe+hc_sig(i,j)+(h0-hc_sig(i,j))*dcwsds(k)
     hz_w=xe+hzdcwsds(k,i,j)
#endif
     f_dzw=hz_w*dsigw(k)
#else
     d=h0+xe
     f_dzw=d*dsigw(k)
#endif

  END FUNCTION f_dzw

   !!======================================================================
 
  FUNCTION f_hzu(h0,xe,k,i,j)
 
   !&E---------------------------------------------------------------------
   !&E                 ***  FUNCTION f_hzu  ***
   !&E
   !&E ** Purpose : estimate the vertical scale factor Hz at the middle of
   !&E              cell (tracer location)
   !&E
   !&E ** Description : the u-location (along z-axis) is the middle of the cell
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2010-03  (V. Garnier) Original code
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
#if defined key_siggen || defined key_gencoord
#if key_siggen_g2
   USE comvars3d, ONLY : dsigu
   USE comsiggen, ONLY : hc_sig,dcusds
#else
   USE comsiggen, ONLY : hzdcusds
#endif
#endif

   !! * Arguments
   INTEGER, INTENT( in )        :: k,i,j         ! indexes of the cell
   REAL(KIND=rsh), INTENT( in ) :: h0,xe
   REAL(KIND=rsh) :: f_hzu                       ! vertical scale factor

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

#ifdef key_siggen
#ifdef key_siggen_g2
     f_hzu=(xe+h0)/(hc_sig(i,j)+h0) * (hc_sig(i,j)*dsigu(k)+h0*dcusds(k))
#else
     f_hzu=xe+hzdcusds(k,i,j)
#endif
#elif defined key_gencoord
     f_hzu=0.0_rsh
     f_hzu=xe+hzdcusds(k,i,j)
#else
     f_hzu=0.0_rsh   ! il manque h0.
! Fonction bcp utilisee dans dyn3dheightmask, obcinitheight, initheight
! et ce seulement pour les siggen donc pas de modif generale pour l instant. 
#endif

  END FUNCTION f_hzu

   !!======================================================================
 
  FUNCTION f_hzw(h0,xe,k,i,j)
 
   !&E---------------------------------------------------------------------
   !&E                 ***  FUNCTION f_hzw  ***
   !&E
   !&E ** Purpose : estimate the vertical scale factor Hz at the w-location
   !&E
   !&E ** Description : the w-location (along z-axis) is at the interface
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2010-03  (V. Garnier) Original code
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
#if defined key_siggen || defined key_gencoord
#if key_siggen_g2
   USE comvars3d, ONLY : dsigw
   USE comsiggen, ONLY : hc_sig,dcwsds
#else
   USE comsiggen, ONLY : hzdcwsds
#endif
#endif

   !! * Arguments
   INTEGER, INTENT( in )        :: k,i,j         ! indexes of the cell
   REAL(KIND=rsh), INTENT( in ) :: h0,xe
   REAL(KIND=rsh) :: f_hzw                       ! vertical scale factor

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

#ifdef key_siggen
#ifdef key_siggen_g2
     f_hzw=(xe+h0)/(hc_sig(i,j)+h0) * (hc_sig(i,j)*dsigw(k)+h0*dcwsds(k))
#else
     f_hzw=xe+hzdcwsds(k,i,j)
#endif
#elif defined key_gencoord
     f_hzw=0.0_rsh
     f_hzw=xe+hzdcwsds(k,i,j)
#else
     f_hzw=0.0_rsh
#endif

  END FUNCTION f_hzw

   !!======================================================================
 
  FUNCTION f_hzu_x(hx,xe,k,i,j)
 
   !&E---------------------------------------------------------------------
   !&E                 ***  FUNCTION f_hzu_x  ***
   !&E
   !&E ** Purpose : estimate the vertical scale factor Hz at the u-location
   !&E              (along x and z axes)
   !&E
   !&E ** Description : the u-location (along z-axis) is the middle of the cell
   !&E                  and dx/2 at the right of the tracer
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2010-03  (V. Garnier) Original code
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
#if defined key_siggen || defined key_gencoord
#if key_siggen_g2
   USE comvars3d, ONLY : dsigu
   USE comsiggen, ONLY : hcx_sig,dcusds
#else
   USE comsiggen, ONLY : hzdcusds_x
#endif
#endif

   !! * Arguments
   INTEGER, INTENT( in )        :: k,i,j         ! indexes of the cell
   REAL(KIND=rsh), INTENT( in ) :: hx,xe
   REAL(KIND=rsh) :: f_hzu_x                     ! vertical scale factor

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

#ifdef key_siggen
#ifdef key_siggen_g2
     f_hzu_x=(xe+hx)/(hcx_sig(i,j)+hx) * (hcx_sig(i,j)*dsigu(k)+hx*dcusds(k))
#else
     f_hzu_x=xe+hzdcusds_x(k,i,j)
#endif
#elif defined key_gencoord
     f_hzu_x=0.0_rsh
     f_hzu_x=xe+hzdcusds_x(k,i,j)
#else
     f_hzu_x=0.0_rsh
#endif

  END FUNCTION f_hzu_x

   !!======================================================================
 
  FUNCTION f_hzw_x(hx,xe,k,i,j)
 
   !&E---------------------------------------------------------------------
   !&E                 ***  FUNCTION f_hzw_x  ***
   !&E
   !&E ** Purpose : estimate the vertical scale factor Hz at the interface
   !&E              and u-location along x axis
   !&E
   !&E ** Description : 
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2010-03  (V. Garnier) Original code
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
#if defined key_siggen || defined key_gencoord
#if key_siggen_g2
   USE comvars3d, ONLY : dsigw
   USE comsiggen, ONLY : hcx_sig,dcwsds
#else
   USE comsiggen, ONLY : hzdcwsds_x
#endif
#endif

   !! * Arguments
   INTEGER, INTENT( in )        :: k,i,j         ! indexes of the cell
   REAL(KIND=rsh), INTENT( in ) :: hx,xe
   REAL(KIND=rsh) :: f_hzw_x                     ! vertical scale factor

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

#ifdef key_siggen
#ifdef key_siggen_g2
     f_hzw_x=(xe+hx)/(hcx_sig(i,j)+hx) * (hcx_sig(i,j)*dsigw(k)+hx*dcwsds(k))
#else
     f_hzw_x=xe+hzdcwsds_x(k,i,j)
#endif
#elif defined key_gencoord
     f_hzw_x=0.0_rsh
     f_hzw_x=xe+hzdcwsds_x(k,i,j)
#else
     f_hzw_x=0.0_rsh
#endif

  END FUNCTION f_hzw_x

   !!======================================================================

  FUNCTION f_hzu_y(hy,xe,k,i,j)
 
   !&E---------------------------------------------------------------------
   !&E                 ***  FUNCTION f_hzu_y  ***
   !&E
   !&E ** Purpose : estimate the vertical scale factor Hz at the v-location
   !&E              (along y and z axes)
   !&E
   !&E ** Description : the v-location (along z-axis) is the middle of the cell
   !&E                  and dy/2 up to the tracer
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2010-03  (V. Garnier) Original code
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
#if defined key_siggen || defined key_gencoord
#if key_siggen_g2
   USE comvars3d, ONLY : dsigu
   USE comsiggen, ONLY : hcy_sig,dcusds
#else
   USE comsiggen, ONLY : hzdcusds_y
#endif
#endif

   !! * Arguments
   INTEGER, INTENT( in )        :: k,i,j         ! indexes of the cell
   REAL(KIND=rsh), INTENT( in ) :: hy,xe
   REAL(KIND=rsh) :: f_hzu_y                     ! vertical scale factor

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

#ifdef key_siggen
#ifdef key_siggen_g2
     f_hzu_y=(xe+hy)/(hcy_sig(i,j)+hy) * (hcy_sig(i,j)*dsigu(k)+hy*dcusds(k))
#else
     f_hzu_y=xe+hzdcusds_y(k,i,j)
#endif
#elif defined key_gencoord
     f_hzu_y=0.0_rsh
     f_hzu_y=xe+hzdcusds_y(k,i,j)
#else
     f_hzu_y=0.0_rsh
#endif

  END FUNCTION f_hzu_y

   !!======================================================================
 
  FUNCTION f_hzw_y(hy,xe,k,i,j)
 
   !&E---------------------------------------------------------------------
   !&E                 ***  FUNCTION f_hzw_y  ***
   !&E
   !&E ** Purpose : estimate the vertical scale factor Hz at the interface
   !&E              and u-location along x axis
   !&E
   !&E ** Description : 
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2010-03  (V. Garnier) Original code
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
#if defined key_siggen || defined key_gencoord
#if key_siggen_g2
   USE comvars3d, ONLY : dsigw
   USE comsiggen, ONLY : hcy_sig,dcwsds
#else
   USE comsiggen, ONLY : hzdcwsds_y
#endif
#endif

   !! * Arguments
   INTEGER, INTENT( in )        :: k,i,j         ! indexes of the cell
   REAL(KIND=rsh), INTENT( in ) :: hy,xe
   REAL(KIND=rsh) :: f_hzw_y                     ! vertical scale factor

   !! * Local declarations

   !!----------------------------------------------------------------------
   !! * Executable part

#ifdef key_siggen
#ifdef key_siggen_g2
     f_hzw_y=(xe+hy)/(hcy_sig(i,j)+hy) * (hcy_sig(i,j)*dsigw(k)+hy*dcwsds(k))
#else
     f_hzw_y=xe+hzdcwsds_y(k,i,j)
#endif
#elif defined key_gencoord
     f_hzw_y=0.0_rsh
     f_hzw_y=xe+hzdcwsds_y(k,i,j)
#else
     f_hzw_y=0.0_rsh
#endif

  END FUNCTION f_hzw_y

   !!======================================================================
 
  FUNCTION f_zu(h0,xe,k,i,j)
 
   !&E---------------------------------------------------------------------
   !&E                 ***  FUNCTION f_zu  ***
   !&E
   !&E ** Purpose : estimate the depth at u level
   !&E
   !&E ** Description : the u-location (along z-axis) is the middle of the cell
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2009-10  (V. Garnier)  Original code
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE comvars3d,  ONLY : sig
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : hc_sig,Csu_sig
#endif

   !! * Arguments
   INTEGER, INTENT( in )        :: k,i,j        ! indexes of the cell
   REAL(KIND=rsh), INTENT( in ) :: h0,xe
   REAL(KIND=rsh) :: f_zu                       ! depth

   !! * Local declarations
   REAL(KIND=rsh) :: d

   !!----------------------------------------------------------------------
   !! * Executable part

#if defined key_siggen || defined key_gencoord
#ifdef key_siggen_g2
     f_zu=xe + (xe+h0)/(hc_sig(i,j)+h0) * (hc_sig(i,j)*sig(k)+h0*Csu_sig(k))
#else
     f_zu=xe+(xe+hc_sig(i,j))*sig(k) + (h0-hc_sig(i,j))*Csu_sig(k)
#endif
#else
     d=h0+xe
     f_zu=d*sig(k)+xe
#endif

  END FUNCTION f_zu

   !!======================================================================

  FUNCTION f_zu_x(hx,xe,k,i,j)

   !&E---------------------------------------------------------------------
   !&E                 ***  FUNCTION f_zu_x  ***
   !&E
   !&E ** Purpose : estimate the depth at u level and u location
   !&E
   !&E ** Description : the u-location (along z-axis) is the middle of the cell
   !&E                  and at the right of the center of the cell (along x-axis)
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2011-12  (S. Petton)  Original code
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE comvars3d,  ONLY : sig
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : hcx_sig,Csu_sig
#endif

   !! * Arguments
   INTEGER, INTENT( in )        :: k,i,j        ! indexes of the cell
   REAL(KIND=rsh), INTENT( in ) :: hx,xe
   REAL(KIND=rsh) :: f_zu_x                     ! depth

   !! * Local declarations
   REAL(KIND=rsh) :: d

   !!----------------------------------------------------------------------
   !! * Executable part

#if defined key_siggen || defined key_gencoord
#ifdef key_siggen_g2
     f_zu_x=xe + (xe+hx)/(hcx_sig(i,j)+hx) * (hcx_sig(i,j)*sig(k)+hx*Csu_sig(k))
#else
     f_zu_x=xe+(xe+hcx_sig(i,j))*sig(k) + (hx-hcx_sig(i,j))*Csu_sig(k)
#endif
#else
     d=hx+xe
     f_zu_x=d*sig(k)+xe
#endif

  END FUNCTION f_zu_x

   !!======================================================================

  FUNCTION f_zu_y(hy,xe,k,i,j)

   !&E---------------------------------------------------------------------
   !&E                 ***  FUNCTION f_zu_y  ***
   !&E
   !&E ** Purpose : estimate the depth at u level and u location
   !&E
   !&E ** Description : the u-location (along z-axis) is the middle of the cell
   !&E                  and at the top of the center of the cell (along y-axis)
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2011-12  (S. Petton)  Original code
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE comvars3d,  ONLY : sig
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : hcy_sig,Csu_sig
#endif

   !! * Arguments
   INTEGER, INTENT( in )        :: k,i,j        ! indexes of the cell
   REAL(KIND=rsh), INTENT( in ) :: hy,xe
   REAL(KIND=rsh) :: f_zu_y                     ! depth

   !! * Local declarations
   REAL(KIND=rsh) :: d

   !!----------------------------------------------------------------------
   !! * Executable part

#if defined key_siggen || defined key_gencoord
#ifdef key_siggen_g2
     f_zu_y=xe + (xe+hy)/(hcy_sig(i,j)+hy) * (hcy_sig(i,j)*sig(k)+hy*Csu_sig(k))
#else
     f_zu_y=xe+(xe+hcy_sig(i,j))*sig(k) + (hy-hcy_sig(i,j))*Csu_sig(k)
#endif
#else
     d=hy+xe
     f_zu_y=d*sig(k)+xe
#endif

  END FUNCTION f_zu_y

   !!======================================================================

  FUNCTION f_zw(h0,xe,k,i,j)
 
   !&E---------------------------------------------------------------------
   !&E                 ***  FUNCTION f_zw  ***
   !&E
   !&E ** Purpose : estimate the depth at w level
   !&E
   !&E ** Description : the w-location (along z-axis) is the upper interface
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2009-10  (V. Garnier)  Original code
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE comvars3d,  ONLY : sigw
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : hc_sig,Csw_sig
#endif

   !! * Arguments
   INTEGER, INTENT( in )        :: k,i,j        ! indexes of the cell
   REAL(KIND=rsh), INTENT( in ) :: h0,xe
   REAL(KIND=rsh) :: f_zw                       ! depth

   !! * Local declarations
   REAL(KIND=rsh) :: d

   !!----------------------------------------------------------------------
   !! * Executable part

#if defined key_siggen || defined key_gencoord
#ifdef key_siggen_g2
     f_zw=xe + (xe+h0)/(hc_sig(i,j)+h0) * (hc_sig(i,j)*sigw(k)+h0*Csw_sig(k))
#else
     f_zw=xe+(xe+hc_sig(i,j))*sigw(k) + (h0-hc_sig(i,j))*Csw_sig(k)
#endif
#else
     d=h0+xe
     f_zw=d*sigw(k)+xe
#endif

  END FUNCTION f_zw

   !!======================================================================

  FUNCTION f_zw_x(hx,xe,k,i,j)

   !&E---------------------------------------------------------------------
   !&E                 ***  FUNCTION f_zw_x  ***
   !&E
   !&E ** Purpose : estimate the depth at w level (interface) and hx location
   !&E
   !&E ** Description : location : the upper right corner of the cell (along zx-slice)
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2012-02  (V. Garnier) Original code
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE comvars3d,  ONLY : sigw
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : hcx_sig,Csw_sig
#endif

   !! * Arguments
   INTEGER, INTENT( in )        :: k,i,j        ! indexes of the cell
   REAL(KIND=rsh), INTENT( in ) :: hx,xe
   REAL(KIND=rsh) :: f_zw_x                     ! depth

   !! * Local declarations
   REAL(KIND=rsh) :: d

   !!----------------------------------------------------------------------
   !! * Executable part

#if defined key_siggen || defined key_gencoord
#ifdef key_siggen_g2
     f_zw_x=xe + (xe+hx)/(hcx_sig(i,j)+hx) * (hcx_sig(i,j)*sigw(k)+hx*Csw_sig(k))
#else
     f_zw_x=xe+(xe+hcx_sig(i,j))*sigw(k) + (hx-hcx_sig(i,j))*Csw_sig(k)
#endif
#else
     d=hx+xe
     f_zw_x=d*sigw(k)+xe
#endif

  END FUNCTION f_zw_x

   !!======================================================================

  FUNCTION f_zw_y(hy,xe,k,i,j)

   !&E---------------------------------------------------------------------
   !&E                 ***  FUNCTION f_zw_y  ***
   !&E
   !&E ** Purpose : estimate the depth at w level (interface) and hy location
   !&E
   !&E ** Description : location : the upper right corner of the cell (along zy-slice)
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2012-02  (V. Garnier) Original code
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE comvars3d,  ONLY : sigw
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : hcy_sig,Csw_sig
#endif

   !! * Arguments
   INTEGER, INTENT( in )        :: k,i,j        ! indexes of the cell
   REAL(KIND=rsh), INTENT( in ) :: hy,xe
   REAL(KIND=rsh) :: f_zw_y                     ! depth

   !! * Local declarations
   REAL(KIND=rsh) :: d

   !!----------------------------------------------------------------------
   !! * Executable part

#if defined key_siggen || defined key_gencoord
#ifdef key_siggen_g2
     f_zw_y=xe + (xe+hy)/(hcy_sig(i,j)+hy) * (hcy_sig(i,j)*sigw(k)+hy*Csw_sig(k))
#else
     f_zw_y=xe+(xe+hcy_sig(i,j))*sigw(k) + (hy-hcy_sig(i,j))*Csw_sig(k)
#endif
#else
     d=hy+xe
     f_zw_y=d*sigw(k)+xe
#endif

  END FUNCTION f_zw_y

   !!======================================================================

 END MODULE
