MODULE toolmarsatmcpl
#ifdef key_oasis_mars_atm
   !!===========================================================================
   !!                   ***  MODULE  toolmarsatmcpl  ***
   !!
   !! Module used for coupling applications between MARS and WRF or Meso-NH with OASIS3-MCT 
   !!
   !!===========================================================================

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC snd_fields_to_atm 
   PUBLIC rcv_fields_from_atm

  CONTAINS


   !!======================================================================

  
  SUBROUTINE snd_fields_to_atm(id_oasis_time)

   !&E---------------------------------------------------------------------
   !&E                 *** ROUTINE snd_fields_to_atm  ***
   !&E
   !&E ** Purpose : Send coupling fields to WRF or Meso-NH
   !&E
   !&E ** Description : 
   !&E
   !&E ** Called by : 
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E        2013-07 : A. THEVENIN (CERFACS) : Creation 
   !&E
   !&E---------------------------------------------------------------------

   USE parameters,   ONLY : limax, ljmax, kmax, &
                         &  liminp1, ljminp1
   USE tooloasismct, ONLY : snd_fld, cpl_oasis_snd, il_nb_snd                           
   USE comvarp2d,    ONLY : temp
   USE comvarp3d,    ONLY : uz, vz                         
                          
   !! * Argument
   INTEGER, INTENT(IN) :: id_oasis_time
   
   !! * Local declaration
   REAL(kind=8), DIMENSION(limax-liminp1+1, ljmax-ljminp1+1) :: rla_oasis_snd
   INTEGER                                                   :: ib_do 
   LOGICAL                                                   :: ll_action   
   
   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter SND_FIELDS_FROM_ATM routine'

   DO ib_do = 1, il_nb_snd

      IF (snd_fld(ib_do)%cl_field_name == 'MARS_MSK') THEN
         rla_oasis_snd(:,:) = 1.0 
      ENDIF

      IF (snd_fld(ib_do)%cl_field_name == 'MARS_SST') THEN
         rla_oasis_snd(:,:) = DBLE(temp(kmax,liminp1:limax,ljminp1:ljmax))      
      ENDIF 

      IF (snd_fld(ib_do)%cl_field_name == 'MARS__UZ') THEN
         rla_oasis_snd(:,:) = DBLE(uz(kmax,liminp1:limax,ljminp1:ljmax))
      ENDIF
 
      IF (snd_fld(ib_do)%cl_field_name == 'MARS__VZ') THEN
         rla_oasis_snd(:,:) = DBLE(vz(kmax,liminp1:limax,ljminp1:ljmax))
      ENDIF

      ! ici on envoie a chaque pas de temps, du coup oasis peut cumuler faire
      ! des moyennes... puisque oasis recoit l info mais ne la redonne pas a
      ! WRF ou Meso-NH quand ce n est pas la date de couplage. 
      ! si on cherche a limiter les copies ci-dessus, on pert cette
      ! possibilite. La limitation peut se faire en precisant en dur la pas
      ! de couplage (MODt,120) par exemple.
      CALL cpl_oasis_snd(ib_do, id_oasis_time, rla_oasis_snd, ll_action)
      
   ENDDO

   PRINT_DBG*, 'exit SND_FIELDS_FROM_ATM routine'

  END SUBROUTINE snd_fields_to_atm


   !!======================================================================

  
  SUBROUTINE rcv_fields_from_atm(id_oasis_time)

   !&E---------------------------------------------------------------------
   !&E                 *** ROUTINE rcv_fields_from_atm  ***
   !&E
   !&E ** Purpose : Receive coupling fields from WRF or Meso-NH
   !&E
   !&E ** Description : 
   !&E
   !&E ** Called by : 
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E        2013-07 : A. THEVENIN (CERFACS) : Creation 
   !&E
   !&E---------------------------------------------------------------------
  
#include "toolcpp.h"

   USE parameters,   ONLY : limax, ljmax, rsh, liminm2, limaxp2, &
                     &      ljminm2, ljmaxp2, liminp1, ljminp1
   USE tooloasismct, ONLY : rcv_fld, cpl_oasis_rcv, il_nb_rcv 
   USE cometdyn,     ONLY : pa, tosx, tosy                             
   USE sflxatm,      ONLY : rad, irlos, latlos, senslos
   USE_MPI toolmpi,  ONLY : ex_i_rsh,ex_j_rsh
   USE comfluxdo,    ONLY : rain, evap    
                         
   !! * Argument
   INTEGER, INTENT(IN) :: id_oasis_time
   
   !! * Local declaration
   REAL(kind=8), DIMENSION(limax-liminp1+1, ljmax-ljminp1+1) :: rla_oasis_rcv
   INTEGER                                                   :: ib_do 
   LOGICAL                                                   :: ll_action   
   
   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter RCV_FIELDS_FROM_ATM routine'

   DO ib_do = 1, il_nb_rcv

      IF (rcv_fld(ib_do)%cl_field_name == 'MARSRAIN') THEN
         CALL cpl_oasis_rcv(ib_do, id_oasis_time, rla_oasis_rcv, ll_action)
         IF (ll_action) THEN
            rain(liminp1:limax,ljminp1:ljmax) = REAL(rla_oasis_rcv, rsh)
         ENDIF
      ENDIF

      IF (rcv_fld(ib_do)%cl_field_name == 'MARSEVAP') THEN
         CALL cpl_oasis_rcv(ib_do, id_oasis_time, rla_oasis_rcv, ll_action)
         IF (ll_action) THEN
            evap(liminp1:limax,ljminp1:ljmax) = REAL(rla_oasis_rcv, rsh)
         ENDIF
      ENDIF

      IF (rcv_fld(ib_do)%cl_field_name == 'MARS_QSR') THEN
         CALL cpl_oasis_rcv(ib_do, id_oasis_time, rla_oasis_rcv, ll_action)
         IF (ll_action) THEN
            rad(liminp1:limax,ljminp1:ljmax) = REAL(rla_oasis_rcv, rsh)
         ENDIF
      ENDIF

      IF (rcv_fld(ib_do)%cl_field_name == 'MARSHEAT') THEN
         CALL cpl_oasis_rcv(ib_do, id_oasis_time, rla_oasis_rcv, ll_action)
         IF (ll_action) THEN
            ! (SW + Qlat + Qsens in one of the corresponding arrays)
            irlos(liminp1:limax,ljminp1:ljmax) = REAL(rla_oasis_rcv, rsh)
         ENDIF
      ENDIF

      IF (rcv_fld(ib_do)%cl_field_name == 'MARS__IR') THEN
         CALL cpl_oasis_rcv(ib_do, id_oasis_time, rla_oasis_rcv, ll_action)
         IF (ll_action) THEN
            irlos(liminp1:limax,ljminp1:ljmax) = REAL(rla_oasis_rcv, rsh)
         ENDIF
      ENDIF

      IF (rcv_fld(ib_do)%cl_field_name == 'MARS_LAT') THEN
         CALL cpl_oasis_rcv(ib_do, id_oasis_time, rla_oasis_rcv, ll_action)
         IF (ll_action) THEN
            latlos(liminp1:limax,ljminp1:ljmax) = REAL(rla_oasis_rcv, rsh)
         ENDIF
      ENDIF

      IF (rcv_fld(ib_do)%cl_field_name == 'MARS_SEN') THEN
         CALL cpl_oasis_rcv(ib_do, id_oasis_time, rla_oasis_rcv, ll_action)
         IF (ll_action) THEN
            senslos(liminp1:limax,ljminp1:ljmax) = REAL(rla_oasis_rcv, rsh)
         ENDIF
      ENDIF

      IF (rcv_fld(ib_do)%cl_field_name == 'MARS__PA') THEN
         CALL cpl_oasis_rcv(ib_do, id_oasis_time, rla_oasis_rcv, ll_action)
         IF (ll_action) THEN
            pa(liminp1:limax,ljminp1:ljmax) = REAL(rla_oasis_rcv, rsh)
            CALL_MPI ex_i_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,pa(liminm2:limaxp2,ljminm2:ljmaxp2))
            CALL_MPI ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,pa(liminm2:limaxp2,ljminm2:ljmaxp2))
         ENDIF
      ENDIF

      IF (rcv_fld(ib_do)%cl_field_name == 'MARSTAUX') THEN
         CALL cpl_oasis_rcv(ib_do, id_oasis_time, rla_oasis_rcv, ll_action)
         IF (ll_action) THEN
            tosx(liminp1:limax,ljminp1:ljmax) = REAL(rla_oasis_rcv, rsh)
         ENDIF
      ENDIF

      IF (rcv_fld(ib_do)%cl_field_name == 'MARSTAUY') THEN
         CALL cpl_oasis_rcv(ib_do, id_oasis_time, rla_oasis_rcv, ll_action)
         IF (ll_action) THEN
            tosy(liminp1:limax,ljminp1:ljmax) = REAL(rla_oasis_rcv, rsh)
         ENDIF
      ENDIF

   ENDDO

   PRINT_DBG*, 'exit RCV_FIELDS_FROM_ATM routine'

  END SUBROUTINE rcv_fields_from_atm

#else
   !! Dummy module 
#endif
END MODULE toolmarsatmcpl
