  MODULE tooloasismct
#ifdef key_oasis
   !!===========================================================================
   !!                   ***  MODULE  tooloasismct  ***
   !!
   !! Generic Module used for coupling applications with OASIS3-MCT 
   !!
   !!===========================================================================

#include "toolcpp.h"

   USE comoasis
   USE mod_oasis                                      ! OASIS3-MCT module 

   IMPLICIT NONE
   PRIVATE

   INTEGER               :: il_compid                 ! Component model ID returned by oasis_init_comp
   CHARACTER(len=6)      :: cl_model_name = 'mars3D'  ! Model name (same as in namcouple)   
   INTEGER               :: il_err                    ! Return error code
   INTEGER, PUBLIC       :: il_nb_rcv, il_nb_snd      ! Number of coupling fields    
   INTEGER, PARAMETER    :: ip_maxfld=50              ! Maximum number of coupling fields
   
   TYPE, PUBLIC          :: CPL_FIELD                 ! Type for coupling field information
      CHARACTER(len = 8) :: cl_field_name             ! Name of the coupling field   
      INTEGER            :: il_field_id               ! Field ID
   END TYPE CPL_FIELD

   TYPE(CPL_FIELD), DIMENSION(ip_maxfld), PUBLIC :: rcv_fld, snd_fld   ! Coupling fields

   !! * Accessibility
   PUBLIC cpl_oasis_init
   PUBLIC cpl_oasis_grid
   PUBLIC cpl_oasis_define 
   PUBLIC cpl_oasis_snd
   PUBLIC cpl_oasis_rcv
   PUBLIC cpl_oasis_finalize   

  CONTAINS

  SUBROUTINE cpl_oasis_init(id_lcomm)

   !&E---------------------------------------------------------------------
   !&E                 *** ROUTINE cpl_oasis_init  ***
   !&E
   !&E ** Purpose : Coupling initialisation 
   !&E
   !&E ** Description : 
   !&E
   !&E ** Called by : 
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E        2013-07 : A. THEVENIN (CERFACS) : Creation 
   !&E
   !&E---------------------------------------------------------------------

   !! * Argument
   INTEGER, INTENT(OUT) :: id_lcomm                   ! Model local communicator
   
   !! * Local declaration
   
   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter CPL_OASIS_INIT routine'
   
   !! Initialise the coupling
   CALL oasis_init_comp (il_compid, cl_model_name, il_err)
   IF (il_err /= 0) THEN
      CALL oasis_abort(il_compid, 'cpl_oasis_init', 'Problem during oasis_init_comp')
   ENDIF
   
   !! Get the value of a local MPI communicator to be used by MARS for its internal parallelisation
   CALL oasis_get_localcomm (id_lcomm, il_err)
   IF (il_err /= 0) THEN
      CALL oasis_abort(il_compid, 'cpl_oasis_init', 'Problem during oasis_get_localcomm')
   ENDIF

   PRINT_DBG*, 'exit CPL_OASIS_INIT routine'

  END SUBROUTINE cpl_oasis_init


   !!======================================================================

  SUBROUTINE cpl_oasis_grid(ld_master,id_lcomm)

   !&E---------------------------------------------------------------------
   !&E                 *** ROUTINE cpl_oasis_grid  ***
   !&E
   !&E ** Purpose : Grid data file definition 
   !&E
   !&E ** Description : 
   !&E
   !&E ** Called by : 
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E        2013-10 : A. THEVENIN (CERFACS) : Creation 
   !&E
   !&E---------------------------------------------------------------------
   USE comvars2d,  ONLY : lon2d_g,  lat2d_g,  &
                          lon2du_g, lat2du_g, &
                          lon2dv_g, lat2dv_g, &
                          lon2df_g, lat2df_g
                          
   USE parameters, ONLY : imin, imax, jmin, jmax, &
                          valmanq, pi 
   
   USE comvars2d,  ONLY : h0_g, hx_g, hy_g,  &
                          h0fond, radearth,  &
                          l_corio_cst, fisac
                          
   !! * Argument
   LOGICAL, INTENT(IN) :: ld_master                  ! MASTER process or not
   INTEGER, INTENT(IN) :: id_lcomm                   ! Model local communicator
   
   !! * Local declaration
   INTEGER              :: nlon, nlat, i,j
   REAL(KIND=8), DIMENSION(imax-imin, jmax-jmin, 4) :: clon, clat, clonu, clatu, clonv, clatv
   REAL(KIND=8), DIMENSION(imax-imin, jmax-jmin)    :: area, area_u, area_v
   REAL(KIND=8), DIMENSION(imax-imin, jmax-jmin)    :: dx, dx_u, dx_v 
   REAL(KIND=8), DIMENSION(imax-imin, jmax-jmin)    :: dy, dy_u, dy_v 
   REAL(KIND=8) :: fi, radeg
   REAL(KIND=8)            :: xbox, ybox, radian
!   REAL(KIND=8), PARAMETER :: pi = 3.141592653589793
!   REAL(KIND=8), PARAMETER :: earth_radius = 6378000.0 !Earth radius (in meters)
   INTEGER, DIMENSION(imax-imin, jmax-jmin) :: mask
   
   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter CPL_OASIS_GRID routine'


   IF (ld_master) THEN
      nlon = imax-imin
      nlat = jmax-jmin

      ! Model grid longitudes and latitudes
      CALL oasis_write_grid ('ocnt', nlon, nlat,                &
            &                lon2d_g(imin+1:imax,jmin+1:jmax),  &
            &                lat2d_g(imin+1:imax,jmin+1:jmax)   &
            &                )

      CALL oasis_write_grid ('ocnu', nlon, nlat,                &
            &                lon2du_g(imin+1:imax,jmin+1:jmax), &
            &                lat2du_g(imin+1:imax,jmin+1:jmax)  &
            &               )

      CALL oasis_write_grid ('ocnv', nlon, nlat,                &
            &                lon2dv_g(imin+1:imax,jmin+1:jmax), &
            &                lat2dv_g(imin+1:imax,jmin+1:jmax)  &
            &               )

      ! Model grid cell corner longitudes and latitudes
      xbox=lon2d_g(1,2)-lon2d_g(1,1)
      clon(:,:,1)=lon2d_g(imin+1:imax,jmin+1:jmax)+xbox/2.
      clon(:,:,2)=lon2d_g(imin+1:imax,jmin+1:jmax)-xbox/2.
      clon(:,:,3)=lon2d_g(imin+1:imax,jmin+1:jmax)-xbox/2.
      clon(:,:,4)=lon2d_g(imin+1:imax,jmin+1:jmax)+xbox/2.
     
      ybox=lat2d_g(2,1)-lat2d_g(1,1)
      clat(:,:,1)=lat2d_g(imin+1:imax,jmin+1:jmax)+ybox/2.
      clat(:,:,2)=lat2d_g(imin+1:imax,jmin+1:jmax)+ybox/2.
      clat(:,:,3)=lat2d_g(imin+1:imax,jmin+1:jmax)-ybox/2.
      clat(:,:,4)=lat2d_g(imin+1:imax,jmin+1:jmax)-ybox/2.

      CALL oasis_write_corner('ocnt', nlon, nlat, 4, clon, clat)
      
      xbox=lon2du_g(1,2)-lon2du_g(1,1)
      clonu(:,:,1)=lon2du_g(imin+1:imax,jmin+1:jmax)+xbox/2.
      clonu(:,:,2)=lon2du_g(imin+1:imax,jmin+1:jmax)-xbox/2.
      clonu(:,:,3)=lon2du_g(imin+1:imax,jmin+1:jmax)-xbox/2.
      clonu(:,:,4)=lon2du_g(imin+1:imax,jmin+1:jmax)+xbox/2.
     
      ybox=lat2du_g(2,1)-lat2du_g(1,1)
      clatu(:,:,1)=lat2du_g(imin+1:imax,jmin+1:jmax)+ybox/2.
      clatu(:,:,2)=lat2du_g(imin+1:imax,jmin+1:jmax)+ybox/2.
      clatu(:,:,3)=lat2du_g(imin+1:imax,jmin+1:jmax)-ybox/2.
      clatu(:,:,4)=lat2du_g(imin+1:imax,jmin+1:jmax)-ybox/2.
      
      CALL oasis_write_corner('ocnu', nlon, nlat, 4, clonu, clatu)
      
      xbox=lon2dv_g(1,2)-lon2dv_g(1,1)
      clonv(:,:,1)=lon2dv_g(imin+1:imax,jmin+1:jmax)+xbox/2.
      clonv(:,:,2)=lon2dv_g(imin+1:imax,jmin+1:jmax)-xbox/2.
      clonv(:,:,3)=lon2dv_g(imin+1:imax,jmin+1:jmax)-xbox/2.
      clonv(:,:,4)=lon2dv_g(imin+1:imax,jmin+1:jmax)+xbox/2.
     
      ybox=lat2dv_g(2,1)-lat2dv_g(1,1)
      clatv(:,:,1)=lat2dv_g(imin+1:imax,jmin+1:jmax)+ybox/2.
      clatv(:,:,2)=lat2dv_g(imin+1:imax,jmin+1:jmax)+ybox/2.
      clatv(:,:,3)=lat2dv_g(imin+1:imax,jmin+1:jmax)-ybox/2.
      clatv(:,:,4)=lat2dv_g(imin+1:imax,jmin+1:jmax)-ybox/2.  
      
      CALL oasis_write_corner('ocnv', nlon, nlat, 4, clonv, clatv)

      ! Model grid cell areas
!      radian = 180./ pi
!      
!      xbox=lon2d_g(1,2)-lon2d_g(1,1)
!      ybox=lat2d_g(2,1)-lat2d_g(1,1)
!      area(:,:)=xbox/radian * 2*sin(ybox/(2.*radian)) *          &
!         &      cos(lat2d_g(imin+1:imax,jmin+1:jmax)/radian) *   &
!         &      earth_radius*earth_radius

!      CALL oasis_write_area('ocnt', nlon, nlat, area)
!      
!      xbox=lon2du_g(1,2)-lon2du_g(1,1)
!      ybox=lat2du_g(2,1)-lat2du_g(1,1)
!      area_u(:,:)=xbox/radian * 2*sin(ybox/(2.*radian)) *           &
!         &        cos(lat2du_g(imin+1:imax,jmin+1:jmax)/radian) *   &
!         &        earth_radius*earth_radius

!      CALL oasis_write_area('ocnu', nlon, nlat, area_u)
!      
!      xbox=lon2dv_g(1,2)-lon2dv_g(1,1)
!      ybox=lat2dv_g(2,1)-lat2dv_g(1,1) 
!      area_v(:,:)=xbox/radian * 2*sin(ybox/(2.*radian)) *           &
!         &        cos(lat2dv_g(imin+1:imax,jmin+1:jmax)/radian) *   &
!         &        earth_radius*earth_radius

!      CALL oasis_write_area('ocnv', nlon, nlat, area_v)

      radeg = pi/180.0
      DO j=jmin+1,jmax
         DO i =imin+1,imax
       
            ! T location
!           g = REAL(lon2d(i,j),rsh)
            fi = lat2d_g(i,j)
            IF (l_corio_cst) fi=fisac
            dy(i,j)=radearth*(lat2dv_g(i,j)-lat2dv_g(i,j-1))*radeg
            dx(i,j)=radearth*COS(fi*radeg)*(lon2du_g(i,j)-lon2du_g(i-1,j))*radeg

            ! U location
            fi = lat2du_g(i,j)
            IF (l_corio_cst) fi=fisac
            dy_u(i,j)=radearth*(lat2df_g(i,j)-lat2df_g(i,j-1))*radeg
            IF (i==imax) THEN
               dx_u(i,j)=radearth*COS(fi*radeg)*(lon2d_g(i,j)-lon2d_g(i-1,j))*radeg
            ELSE
               dx_u(i,j)=radearth*COS(fi*radeg)*(lon2d_g(i+1,j)-lon2d_g(i,j))*radeg
            END IF

            ! V location
            fi = lat2dv_g(i,j)
            IF (l_corio_cst) fi=fisac
            IF (j==jmax) THEN
               dy_v(i,j)=radearth*(lat2d_g(i,j)-lat2d_g(i,j-1))*radeg
            ELSE
               dy_v(i,j)=radearth*(lat2d_g(i,j+1)-lat2d_g(i,j))*radeg
            END IF
            dx_v(i,j)=radearth*COS(fi*radeg)*(lon2df_g(i,j)-lon2df_g(i-1,j))*radeg
            
         ENDDO
      ENDDO
      
      
      ! govlet et goulet non pris en compte
      area(:,:)=dy(:,:)*dx(:,:)
      area_u(:,:)=dy_u(:,:)*dx_u(:,:)
      area_v(:,:)=dy_v(:,:)*dx_v(:,:)
      
      CALL oasis_write_area('ocnt', nlon, nlat, area)
      CALL oasis_write_area('ocnu', nlon, nlat, area_u)
      CALL oasis_write_area('ocnv', nlon, nlat, area_v)


      ! Model grid mask : 0=sea / 1=land
      mask(:,:)=1
      DO j=jmin+1,jmax
         DO i=imin+1,imax
            IF ( h0_g(i,j) /= -valmanq ) mask(i,j)=0 
         ENDDO
      ENDDO
      CALL oasis_write_mask('ocnt', nlon, nlat, mask)

      mask(:,:)=1
      DO j=jmin+1,jmax
         DO i=imin+1,imax
            IF ( hx_g(i,j) /= -valmanq-h0fond ) mask(i,j)=0
         ENDDO
      ENDDO
      CALL oasis_write_mask('ocnu', nlon, nlat, mask)      

      mask(:,:)=1
      DO j=jmin+1,jmax
         DO i=imin+1,imax
            IF ( hy_g(i,j) /= -valmanq-h0fond) mask(i,j)=0
         ENDDO
      ENDDO
      CALL oasis_write_mask('ocnv', nlon, nlat, mask) 
      
      ! Terminate grid writing
      CALL oasis_terminate_grids_writing()

   ENDIF

   CALL MPI_Barrier(id_lcomm, il_err)

   PRINT_DBG*, 'exit CPL_OASIS_GRID routine'

  END SUBROUTINE cpl_oasis_grid


   !!======================================================================


  
  SUBROUTINE cpl_oasis_define

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cpl_oasis_define  ***
   !&E
   !&E ** Purpose : Partition definition and coupling fields declaration 
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : 
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       2013-07 : A. THEVENIN (CERFACS) : Creation
   !&E
   !&E---------------------------------------------------------------------

   USE parameters, ONLY : imax,    jmax,    &
                          limax,   ljmax,   &
                          liminm1, ljminm1, &
                          liminp1, ljminp1, &
                          l_testcase
                           
   !! * Argument
   
   !! * Local declarations
   INTEGER                 :: ib_i, rank
   INTEGER                 :: il_part_id      ! PartitionID
   INTEGER, DIMENSION(5)   :: ila_paral       ! Description of the local partition in the global index space
   INTEGER, DIMENSION(2,2) :: ila_shape       ! Vector giving the min & max index for each dim of the fields
   INTEGER, DIMENSION(2)   :: ila_var_nodims  ! rank of fields & number of bundles (1 with OASIS3-MCT)
   LOGICAL                 :: ll_mpi_file     ! to check if there an mpi.txt file for domain decompasition
   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter CPL_OASIS_DEFINE routine'

   INQUIRE(file = 'mpi.txt', EXIST = ll_mpi_file)
  
   !! * Define the partition : OASIS BOX partition
   IF (ll_mpi_file .OR. l_testcase) THEN
      ila_paral(1) = 2
      ila_paral(2) = imax*ljminm1 + liminm1 
      ila_paral(3) = limax-liminp1+1
      ila_paral(4) = ljmax-ljminp1+1 
      ila_paral(5) = imax 
   ELSE
      IF (imax .GT. jmax) THEN
         ila_paral(1) = 2
         ila_paral(2) = liminm1
         ila_paral(3) = limax-liminp1+1
         ila_paral(4) = ljmax-ljminp1+1
         ila_paral(5) = imax
      ELSE
         ila_paral(1) = 2
         ila_paral(2) = ljminm1
         ila_paral(3) = limax-liminp1+1
         ila_paral(4) = ljmax-ljminp1+1
         ila_paral(5) = jmax
      ENDIF
   ENDIF

   CALL oasis_def_partition (il_part_id, ila_paral, il_err, imax*jmax)

 
   IF (il_err /= 0) THEN
      CALL oasis_abort(il_compid, 'cpl_oasis_define', 'Problem during oasis_def_partition')
   ENDIF

   ila_shape(:,1) = (/1, limax-liminp1+1 /)
   ila_shape(:,2) = (/1, ljmax-ljminp1+1 /)

   !! * Coupling fields declaration
   ila_var_nodims(1) = 2    ! rank of fields array
   ila_var_nodims(2) = 1    ! always 1 with OASIS3-MCT 2.0
      
   CALL get_list_exch_field (rcv_fld, snd_fld, il_nb_rcv, il_nb_snd)



   !! Send coupling fields  
   DO ib_i = 1, il_nb_snd
      CALL oasis_def_var (snd_fld(ib_i)%il_field_id     &
           &            , snd_fld(ib_i)%cl_field_name   &
           &            , il_part_id                    & 
           &            , ila_var_nodims                &
           &            , OASIS_Out                     &
           &            , ila_shape                     &
           &            , OASIS_Real                    &
           &            , il_err )
      
      IF (il_err /= 0) THEN
         CALL oasis_abort(il_compid, 'cpl_oasis_define', 'Problem during oasis_def_var')
      ENDIF         
   ENDDO
   

   !! Received coupling fields  
   DO ib_i = 1, il_nb_rcv
      CALL oasis_def_var (rcv_fld(ib_i)%il_field_id    &
           &            , rcv_fld(ib_i)%cl_field_name  &
           &            , il_part_id                   & 
           &            , ila_var_nodims               &
           &            , OASIS_In                     &
           &            , ila_shape                    &
           &            , OASIS_Real                   &
           &            , il_err )
      
      IF (il_err /= 0) THEN
         CALL oasis_abort(il_compid, 'cpl_oasis_define', 'Problem during oasis_def_var')
      ENDIF         
   ENDDO
   
   !! * End of definition phase
   CALL oasis_enddef(il_err)
   IF (il_err /= 0) THEN
      CALL oasis_abort(il_compid, 'cpl_oasis_define', 'Problem during oasis_enddef')
   ENDIF

   PRINT_DBG*, 'exit CPL_OASIS_DEFINE routine'

  END SUBROUTINE cpl_oasis_define


   !!======================================================================


  SUBROUTINE cpl_oasis_snd(id_nb, id_time, rda_field, ld_action)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cpl_oasis_snd  ***
   !&E
   !&E ** Purpose : In the model time step loop, each process sends its parts of the coupling field
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : 
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       2013-07 : A. THEVENIN (CERFACS) : Creation
   !&E
   !&E---------------------------------------------------------------------

   !! * Arguments
   INTEGER, INTENT(IN)   :: id_nb                         ! Number of the field to be send
   INTEGER, INTENT(IN)   :: id_time                       ! Atmosphere time-step in seconds
   REAL(KIND=8), DIMENSION(:,:), INTENT(IN) :: rda_field  ! Coupling field array to be send
   LOGICAL, INTENT(OUT)  :: ld_action                     ! Action performed
   
   !! * Local declarations
   INTEGER :: il_info                                     ! OASIS3-MCT info argument
   
   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter CPL_OASIS_SND routine'

   CALL oasis_put ( snd_fld(id_nb)%il_field_id &
   &              , id_time                    &
   &              , rda_field                  &
   &              , il_info                    &
   &                )

   ld_action = il_info == OASIS_Sent     .OR. il_info == OASIS_ToRest .OR.   &
   &           il_info == OASIS_SentOut  .OR. il_info == OASIS_ToRestOut

   PRINT_DBG*, 'exit CPL_OASIS_SND routine'

  END SUBROUTINE cpl_oasis_snd    


   !!======================================================================


  SUBROUTINE cpl_oasis_rcv(id_nb, id_time, rda_field, ld_action)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cpl_oasis_rcv  ***
   !&E
   !&E ** Purpose : In the model time step loop, each process receives its parts of the coupling field
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : 
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       2013-07 : A. THEVENIN (CERFACS) : Creation
   !&E
   !&E---------------------------------------------------------------------

   !! * Argument
   INTEGER, INTENT(IN)   :: id_nb                          ! Number of the field to be received
   INTEGER, INTENT(IN)   :: id_time                        ! Ocean time-step in seconds
   REAL(KIND=8), DIMENSION(:,:), INTENT(OUT) :: rda_field  ! Coupling field array to be received
   LOGICAL, INTENT(OUT)  :: ld_action                      ! Action performed

   !! * Local declarations
   INTEGER :: il_info                                      ! OASIS3-MCT info argument
   
   !!----------------------------------------------------------------------
   !! * Executable part
  
   PRINT_DBG*, 'enter CPL_OASIS_RCV routine'

   CALL oasis_get ( rcv_fld(id_nb)%il_field_id &
   &              , id_time                    &
   &              , rda_field                  &
   &              , il_info                    &
   &                )
   
   ld_action = il_info == OASIS_Recvd   .OR. il_info == OASIS_FromRest .OR.   &
   &           il_info == OASIS_RecvOut .OR. il_info == OASIS_FromRestOut

   PRINT_DBG*, 'exit CPL_OASIS_RCV routine'

  END SUBROUTINE cpl_oasis_rcv


   !!======================================================================


  SUBROUTINE cpl_oasis_finalize

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cpl_prism_finalize  ***
   !&E
   !&E ** Purpose : Finalizes the coupling  
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : 
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       2013-07 : A. THEVENIN (CERFACS) : Creation
   !&E
   !&E---------------------------------------------------------------------

   !! * Argument
   
   !! * Local declarations
   
   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter CPL_OASIS_FINALIZE routine'

   CALL oasis_terminate (il_err)
   
   IF (il_err /= 0) THEN
      CALL oasis_abort(il_compid, 'cpl_prism_finalize', 'Problem during oasis_terminate')
   ENDIF  

   PRINT_DBG*, 'exit CPL_OASIS_FINALIZE routine'

  END SUBROUTINE cpl_oasis_finalize


   !!======================================================================


  SUBROUTINE get_list_exch_field(rcv, snd, id_nb_rcv, id_nb_snd)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE get_list_exch_field  ***
   !&E
   !&E ** Purpose : Provides the list of coupling fields  
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : 
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       2013-07 : A. THEVENIN (CERFACS) : Creation
   !&E
   !&E---------------------------------------------------------------------

   !! * Argument
   TYPE(CPL_FIELD), DIMENSION(ip_maxfld), INTENT (INOUT) :: rcv, snd
   INTEGER, INTENT(INOUT) :: id_nb_rcv, id_nb_snd 
     
   !! * Local declarations
   
   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter GET_LIST_EXCH_FIELD routine'

   ! Ocean variables
   ! ---------------

   !! Coupling fields send by MARS
   id_nb_snd = 0

   ! MARS Mask
   IF (l_cpl_ocn_maskt) THEN
     id_nb_snd=id_nb_snd+1
     snd(id_nb_snd)%cl_field_name='MARS_MSK'
   END IF
   
   ! temp : sea surface temperature (Degrees C)
   IF (l_cpl_ocn_sst) THEN
     id_nb_snd=id_nb_snd+1
     snd(id_nb_snd)%cl_field_name='MARS_SST'
   END IF
 
   ! Banc découvrant pour la SSH
   IF (l_cpl_ocn_maskt) THEN
     id_nb_snd=id_nb_snd+1
     snd(id_nb_snd)%cl_field_name='MARS_BDH'
   END IF
   
   ! Banc découvrant pour uz
   IF (l_cpl_ocn_masku) THEN
     id_nb_snd=id_nb_snd+1
     snd(id_nb_snd)%cl_field_name='MARS_BDU'
   END IF
   
   ! Banc découvrant pour vz
   IF (l_cpl_ocn_maskv) THEN
     id_nb_snd=id_nb_snd+1
     snd(id_nb_snd)%cl_field_name='MARS_BDV'
   END IF

   ! ssh : sea surface height (m)
   IF (l_cpl_ocn_ssh) THEN
     id_nb_snd=id_nb_snd+1
     snd(id_nb_snd)%cl_field_name='MARS_SSH'
   END IF
 
   ! uz : sea surface zonal currents (m.s-1)
   IF (l_cpl_ocn_ucur) THEN
     id_nb_snd=id_nb_snd+1
     snd(id_nb_snd)%cl_field_name='MARS__UZ'
   END IF

   ! vz : sea surface meridional currents (m.s-1)
   IF (l_cpl_ocn_vcur) THEN
     id_nb_snd=id_nb_snd+1
     snd(id_nb_snd)%cl_field_name='MARS__VZ'
   END IF

   ! Atmospheric variables
   ! ---------------------

   !! Coupling fields received by MARS
   id_nb_rcv = 0   

   ! RAIN (mm.s-1)
   IF (l_cpl_atm_rain) THEN
     id_nb_rcv = id_nb_rcv +1
     rcv(id_nb_rcv)%cl_field_name='MARSRAIN'
   END IF

   ! Evaporation (mm.s-1)
   IF (l_cpl_atm_evap) THEN
     id_nb_rcv = id_nb_rcv +1
     rcv(id_nb_rcv)%cl_field_name='MARSEVAP'
   END IF

   ! Solar Flux (W.m-2)
   IF (l_cpl_atm_rad) THEN
     id_nb_rcv = id_nb_rcv +1
     rcv(id_nb_rcv)%cl_field_name='MARS_QSR'
   END IF

   ! Total heat flux (LW + SW + Qlat + Qsens) (W.m-2)
   IF (l_cpl_atm_heat) THEN
     id_nb_rcv = id_nb_rcv +1
     rcv(id_nb_rcv)%cl_field_name='MARSHEAT'
   END IF

   ! Long-wave thermal flux (W.m-2)
   IF (l_cpl_atm_ir) THEN
     id_nb_rcv = id_nb_rcv +1
     rcv(id_nb_rcv)%cl_field_name='MARS__IR'
   END IF

   ! Latent heat flux (W.m-2)
   IF (l_cpl_atm_lat) THEN
     id_nb_rcv = id_nb_rcv +1
     rcv(id_nb_rcv)%cl_field_name='MARS_LAT'
   END IF

   ! Sensible heat flux (W.m-2)
   IF (l_cpl_atm_sens) THEN
     id_nb_rcv = id_nb_rcv +1
     rcv(id_nb_rcv)%cl_field_name='MARS_SEN'
   END IF
 
   ! Surface Pressure (Pa)
   IF (l_cpl_atm_pa) THEN
     id_nb_rcv = id_nb_rcv +1
     rcv(id_nb_rcv)%cl_field_name='MARS__PA'
   END IF

   ! Zonal Wind Stress (N.m-2)
   IF (l_cpl_atm_fwsu) THEN
     id_nb_rcv = id_nb_rcv +1
     rcv(id_nb_rcv)%cl_field_name='MARSTAUX'
   END IF

   ! Meridional Wind Stress (N.m-2)
   IF (l_cpl_atm_fwsv) THEN
     id_nb_rcv = id_nb_rcv +1
     rcv(id_nb_rcv)%cl_field_name='MARSTAUY' 
   END IF
   
   ! Wave variables
   ! --------------

   ! mask to manage wet-drying
   IF (l_cpl_wave_mask) THEN
     id_nb_rcv = id_nb_rcv +1
     rcv(id_nb_rcv)%cl_field_name='MARS_DRY'
   END IF

   ! tom1 / wave_t0m1 : mean period (s)
   IF (l_cpl_wave_tmn) THEN
     id_nb_rcv = id_nb_rcv +1
     rcv(id_nb_rcv)%cl_field_name='MARST0M1'
   END IF

   ! hs / wave_hs : HS significant wave height (m)
   IF (l_cpl_wave_hs) THEN
     id_nb_rcv = id_nb_rcv +1
     rcv(id_nb_rcv)%cl_field_name='MARS__HS'
   END IF

   ! THM / wave_thm : mean direction (degrees)
   IF (l_cpl_wave_dir) THEN
     id_nb_rcv = id_nb_rcv +1
     rcv(id_nb_rcv)%cl_field_name='MARS_DIR'
   END IF

   ! BHD / wave_bhd : wave-induced Bernoulli head pressure (bhd in N.m-1)
   IF (l_cpl_wave_bhd) THEN
     id_nb_rcv = id_nb_rcv +1
     rcv(id_nb_rcv)%cl_field_name='MARS_BHD'
   END IF

   IF (l_cpl_wave_two) THEN
     ! tauox / wave_tauox : x-component of the wave-ocean momentum flux (tauox in m2.s-2)
     id_nb_rcv = id_nb_rcv +1
     rcv(id_nb_rcv)%cl_field_name='MARSTWOX'
     ! tauoy / wave_tauoy : y-component of the wave-ocean momentum flux (tauox in m2.s-2)
     id_nb_rcv = id_nb_rcv +1
     rcv(id_nb_rcv)%cl_field_name='MARSTWOY'
   END IF

   ! uba / wave_uba : rms amplitude of orbital bottom velocity of the waves (m/s)
   IF (l_cpl_wave_uba) THEN
     id_nb_rcv = id_nb_rcv +1
     rcv(id_nb_rcv)%cl_field_name='MARS_UBR'
   END IF
   
   ! phioc / wave_phioc : Wave-to-ocean TKE flux (phioc in W.m-2)
   IF (l_cpl_wave_foc) THEN
     id_nb_rcv = id_nb_rcv +1
     rcv(id_nb_rcv)%cl_field_name='MARS_FOC'
   END IF

   IF (l_cpl_wave_taw) THEN
     ! tauwix / wave_tauwx : Net wave-supported stress, x-component (tauwix in m2.s-2)
     id_nb_rcv = id_nb_rcv +1
     rcv(id_nb_rcv)%cl_field_name='MARSTAWX'
     ! tauwiy / wave_tauwy : Net wave-supported stress, y-component (tauwiy in m2.s-2)
     id_nb_rcv = id_nb_rcv +1
     rcv(id_nb_rcv)%cl_field_name='MARSTAWY'
   END IF

   ! wlm / wave_wlm : mean length wave (m)
   IF (l_cpl_wave_lm) THEN
     id_nb_rcv = id_nb_rcv +1
     rcv(id_nb_rcv)%cl_field_name='MARS__LM'
   END IF

   PRINT_DBG*, 'exit GET_LIST_EXCH_FIELD routine'

  END SUBROUTINE get_list_exch_field
#else
   !! Dummy module 
#endif
END MODULE tooloasismct
