#ifdef key_substance

  SUBROUTINE parsub_eq(xe2,xe3,cv_wat,setvel,niter,num_ws)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE parsub_eq_wquick_sig  ***
   !&E
   !&E ** Purpose : other possibility of subroutine body for the computation 
   !&E              of transport of tracers with fall velocity. 
   !&E
   !&E ** Description : A QUICK scheme is used for depths higher than hm.
   !&E                  For water heights lower than hm, a special computation
   !&E                  is performed in the subroutine parsub_eq_hm.
   !&E                  This subroutine body replaces the default body
   !&E                  when the key key_parsub_wquickupwind is actived.
   !&E
   !&E ** Important remark about the settling velocity with the use of key_subs_part_eqsubdt cpp key
   !&E              To save computation time, we assume the substances have the same settling velocity
   !&E              in a group (since the group and the corresponding sub time step depend on
   !&E              the diameter and the density of the particule).
   !&E              This assumption is not necessary valid for the biology. If so, modify slightly
   !&E              the maximum settling velocity in variable.dat file, to create a new group.
   !&E
   !&E ** Called by : stepu,stepv
   !&E
   !&E ** External calls : parsub_advdifx,parsub_advdify,parsub_advz_quickupwind
   !&E
   !&E ** Used ij-arrays : cv_wat,h0,xe2,xe3,kz,flx_s2w,flx_w2s,flx_w2s_sum
   !&E                     fx,fy,fz,phicon,dy_u,dx_v,surf
   !&E  
   !&E ** Modified variables : cv_wat
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2010-03  (P. Douillet) substances with settling velocities
   !&E       !                         issued from tssub_eq_wquick_sig.h
   !&E       !  2010-06  (V.M. Faure, N. Bousserez, M. Baklouti) coupling with eco3m
   !&E       !  2010-06  (F. Cayocca) coupling with the module of sedimentology
   !&E       !  2011-02 (B. Thouvenin) correction for key_subs_part_eqsubdt key
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E       !  2014-01  (B. Thouvenin - F. Cayocca) introduction of key_sand2D
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d,     ONLY : ig,id,h0,hm,dy_u,dx_v,surf,l_tssubadv_over_smalldepth,halfdt,t,fwet,fwetp
   USE comvars3d,     ONLY : kz,dsigw,dsigu
#if defined key_siggen || defined key_gencoord
   USE toolgeom,      ONLY : f_hzu,f_hzw
#endif
   USE comvarp3d,     ONLY : uz,vz
   USE comsubstance,  ONLY : fx,fy,fz,phicon,f90,nv_ws_max,iv_ws_max,l_bilan,tdebsubs
#ifdef key_parsub_macho3d
   USE comsubstance,     ONLY : nbouc_machop
#endif
#if defined key_biolo || defined key_contaminant || defined key_eco3m
   USE comsubstance,  ONLY : dcdt
#endif
#if defined key_sedim
   USE comsubstance,ONLY : flx_s2w,flx_w2s,flx_w2s_sum,isand1,isand2
#ifdef key_sand2D
   USE sedimento,    ONLY : l_sand2D
#endif
#endif
   USE parsubobc3dapply, ONLY : parsub_obc3d_apply
   USE submassbalance,   ONLY : parsub_budget_flxcum
   USE_AGRIF parsubobc3dapply, ONLY : parsub_obc3d_apply_agrif

   IMPLICIT NONE

   !! * Arguments
   INTEGER,                                                              INTENT(in)    :: niter,num_ws
   REAL(KIND=rsh),                                                       INTENT(in)    :: setvel
   REAL(kind=rsh),DIMENSION(limin:limax,ljmin:ljmax),                    INTENT(in)    :: xe2
   REAL(kind=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),            INTENT(in)    :: xe3
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(inout) :: cv_wat

   !! * Local declarations
   INTEGER                                :: i,j,k,l,ll,l1,iv
   REAL(KIND=rsh)                         :: dtm,hz3_u,hz2_u,hz3_w,hz3m1_w,h3

   REAL(KIND=rlg),DIMENSION(kmax,nb_var)  :: y
   REAL(KIND=rlg)                         :: xk,yk
   REAL(KIND=rlg),DIMENSION(kmax)         :: a,b,c,u0,v0,p0,q0

#ifdef key_sand2D
   LOGICAL                                         :: l_sand2D_l
#endif   

   !!----------------------------------------------------------------------
   !! * Executable part


!$OMP PARALLEL DEFAULT(none) PRIVATE(hz2_u,hz3_u,h3,hz3_w,hz3m1_w, &
#if defined key_sand2D
!$OMP& l_sand2D_l,                                                 &
#endif
!$OMP& i,j,k,l,iv,ll,l1,xk,yk,dtm,p0,q0,u0,v0,a,b,c,y)             &
!$OMP& SHARED(halfdt,l_tssubadv_over_smalldepth,xe2,xe3,hm,ig,id,h0  &
!$OMP& ,limin,limax,ljmin,ljmax,ljminp2,ljmaxm1,fwet,fwetp           &
!$OMP& ,liminm1,limaxp2,ljminm1,ljmaxp2,l_bilan,t,tdebsubs         &
#if defined key_siggen || defined key_gencoord
#endif
!$OMP& ,uz,vz,cv_wat,fx,fy,fz,phicon,f90,nv_ws_max,iv_ws_max       &
#if defined key_sedim
!$OMP& ,flx_s2w,flx_w2s,flx_w2s_sum,isand1,isand2                 &
#if defined key_sand2D
!$OMP& ,l_sand2D                                                  &
#endif
#endif
#ifdef key_parsub_macho3d
!$OMP& ,nbouc_machop                                              &
#endif
#if defined key_biolo || defined key_contaminant || defined key_eco3m
!$OMP& ,dcdt                                                       &
#endif
!$OMP& ,niter,num_ws,setvel,kz,dsigw,dsigu,dy_u,dx_v,surf)

  DO ll =1,niter

   dtm=halfdt/REAL(niter,rlg)

   ! calcul des flux horizontaux et verticaux (advectifs et diffusifs)
   !------------------------------------------------------------------

#if ! defined key_1DV

#ifdef key_parsub_macho3d
!$OMP SINGLE
   nbouc_machop(num_ws)=nbouc_machop(num_ws)+1
!$OMP END SINGLE
   CALL parsub_advdif_macho3d(xe2,xe3,setvel,cv_wat,ll,niter,num_ws)
#else
   CALL parsub_advdifx(cv_wat,niter,num_ws)
   CALL parsub_advdify(cv_wat,niter,num_ws)
   CALL parsub_advz_quickupwind(xe2,xe3,setvel,cv_wat,ll,niter,num_ws)
#endif

#else
   CALL parsub_advz_quickupwind(xe2,xe3,setvel,cv_wat,ll,niter,num_ws)

#endif

!$OMP DO SCHEDULE(RUNTIME)
   DO j=ljminp2,ljmaxm1
     DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)

       h3=h0(i,j)+xe3(i,j)
       IF(h3 > hm) THEN

   ! matrice tridiagonale du modele vertical
   ! (resolution a faire pour chaque traceur du fait de la prise en compte
   ! des depots se faisant a des vitesses de chute differentes)
   !----------------------------------------

         DO l1=1,nv_ws_max(num_ws)         ! loop on variable number in each group of settling velocity
                                           ! num_ws, number of different settling velocity
           iv=iv_ws_max(num_ws,l1)         ! corresponding index of variable in cv_wat

#if defined key_sedim && defined key_sand2D
           l_sand2D_l=.FALSE.
           IF( iv.GE.isand1 .AND. iv.LE.isand2 ) l_sand2D_l=l_sand2D(iv)
           IF( iv.GE.isand1 .AND. iv.LE.isand2 .AND. l_sand2D_l) THEN     ! for variables "sable" or "sand"

             cv_wat(iv,2:kmax,i,j)=0.0_rsh  ! concentration in the upper layers =0
   !-------------------------
             k=1
   !-------------------------

             hz2_u=f_hzu(h0(i,j),(REAL(ll-1,rsh)*(xe3(i,j)-xe2(i,j))/REAL(niter,rsh)+xe2(i,j)),k,i,j)
             hz3_u=f_hzu(h0(i,j),(REAL(ll  ,rsh)*(xe3(i,j)-xe2(i,j))/REAL(niter,rsh)+xe2(i,j)),k,i,j)
             b(k)=hz3_u*fwet(i,j)+dtm*flx_w2s(iv,i,j)
             y(k,iv)=hz2_u*fwetp(i,j)*cv_wat(iv,k,i,j)-((dy_u(i,j)*fx(iv,k,i,j)-dy_u(i-1,j)*fx(iv,k,i-1,j) &
                                  +dx_v(i,j)*fy(iv,k,i,j)-dx_v(i,j-1)*fy(iv,k,i,j-1)) &
                                  -(phicon(iv,k,i,j)/REAL(niter,rsh)))/surf(i,j) &
                                  +flx_s2w(iv,i,j)*dtm
             u0(k)=y(k,iv)/b(k)
             cv_wat(iv,k,i,j)=f90(iv)*u0(k)   ! concentration in the row k=1 is the average concentration
           ELSE
#endif
   !-------------------------
           k=1
   !-------------------------
           hz2_u=f_hzu(h0(i,j),(REAL(ll-1,rsh)*(xe3(i,j)-xe2(i,j))/REAL(niter,rsh)+xe2(i,j)),k,i,j)
           hz3_u=f_hzu(h0(i,j),(REAL(ll  ,rsh)*(xe3(i,j)-xe2(i,j))/REAL(niter,rsh)+xe2(i,j)),k,i,j)
           hz3_w=f_hzw(h0(i,j),(REAL(ll  ,rsh)*(xe3(i,j)-xe2(i,j))/REAL(niter,rsh)+xe2(i,j)),k,i,j)
           c(k)=-dtm*kz(k,i,j)/(hz3_w*dsigw(k))/dsigu(k)
           b(k)=hz3_u*fwet(i,j)+dtm*kz(k,i,j)/(hz3_w*dsigw(k))/dsigu(k)
#ifdef key_sedim
           b(k)=b(k)+dtm*flx_w2s(iv,i,j)/dsigu(k)
#endif

           a(k)=0.0_rlg

           q0(k)=-c(k)/b(k)        ! factorisation LU

   !-------------------------
           DO k=2,kmax-1
   !-------------------------
             hz2_u=f_hzu(h0(i,j),(REAL(ll-1,rsh)*(xe3(i,j)-xe2(i,j))/REAL(niter,rsh)+xe2(i,j)),k,i,j)
             hz3_u=f_hzu(h0(i,j),(REAL(ll  ,rsh)*(xe3(i,j)-xe2(i,j))/REAL(niter,rsh)+xe2(i,j)),k,i,j)
             hz3_w=f_hzw(h0(i,j),(REAL(ll  ,rsh)*(xe3(i,j)-xe2(i,j))/REAL(niter,rsh)+xe2(i,j)),k,i,j)
             hz3m1_w=f_hzw(h0(i,j),(REAL(ll,rsh)*(xe3(i,j)-xe2(i,j))/REAL(niter,rsh)+xe2(i,j)),k-1,i,j)
             c(k)=-dtm*kz(k,i,j)/(hz3_w*dsigw(k))/dsigu(k)
             a(k)=-dtm*kz(k-1,i,j)/(hz3m1_w*dsigw(k-1))/dsigu(k)
             b(k)=hz3_u*fwet(i,j)+dtm*(kz(k,i,j)/(hz3_w*dsigw(k))+kz(k-1,i,j)/(hz3m1_w*dsigw(k-1)))/dsigu(k)

             p0(k)=a(k)*q0(k-1)+b(k)                ! factorisation LU
             q0(k)=-c(k)/p0(k)

   !-------------------------
           ENDDO
   !-------------------------

   !-------------------------
           k=kmax
   !-------------------------
           hz2_u=f_hzu(h0(i,j),(REAL(ll-1,rsh)*(xe3(i,j)-xe2(i,j))/REAL(niter,rsh)+xe2(i,j)),k,i,j)
           hz3_u=f_hzu(h0(i,j),(REAL(ll  ,rsh)*(xe3(i,j)-xe2(i,j))/REAL(niter,rsh)+xe2(i,j)),k,i,j)
           hz3m1_w=f_hzw(h0(i,j),(REAL(ll,rsh)*(xe3(i,j)-xe2(i,j))/REAL(niter,rsh)+xe2(i,j)),k-1,i,j)
           a(k)=-dtm*kz(k-1,i,j)/(hz3m1_w*dsigw(k-1))/dsigu(k)
           b(k)=hz3_u*fwet(i,j)+dtm*kz(k-1,i,j)/(hz3m1_w*dsigw(k-1))/dsigu(k)
           c(k)=0.0_rlg

           p0(k)=a(k)*q0(k-1)+b(k)                ! factorisation LU


   ! estimation des traceurs
   !------------------------

           DO k=1,kmax
             hz2_u=f_hzu(h0(i,j),(REAL(ll-1,rsh)*(xe3(i,j)-xe2(i,j))/REAL(niter,rsh)+xe2(i,j)),k,i,j)
             y(k,iv)=hz2_u*fwetp(i,j)*cv_wat(iv,k,i,j)-(dy_u(i,j)*fx(iv,k,i,j)-dy_u(i-1,j)*fx(iv,k,i-1,j) &
                                          +dx_v(i,j)*fy(iv,k,i,j)-dx_v(i,j-1)*fy(iv,k,i,j-1) &
                                          -(phicon(iv,k,i,j)/REAL(niter,rsh))/dsigu(k))/surf(i,j)
             y(k,iv)=y(k,iv)-(fz(iv,k,i,j)-fz(iv,k-1,i,j))/dsigu(k)
#if defined key_biolo || defined key_contaminant || defined key_eco3m
             y(k,iv)=y(k,iv)+dcdt(iv,k,i,j)*dtm*hz2_u*fwetp(i,j)
#endif
           ENDDO
#ifdef key_sedim
           y(1,iv)=y(1,iv)+flx_s2w(iv,i,j)*dtm/dsigu(1)  
#endif
           u0(1)=y(1,iv)/b(1)
           DO k=2,kmax
             u0(k)=(y(k,iv)-a(k)*u0(k-1))/p0(k)
           ENDDO
           xk=u0(kmax)
           cv_wat(iv,kmax,i,j)=f90(iv)*(xk)
           DO k=kmax-1,1,-1
             xk=q0(k)*xk+u0(k)
             cv_wat(iv,k,i,j)=f90(iv)*(xk)
           ENDDO
#ifdef key_sedim
#if defined key_sand2D
           ENDIF ! test on iv sand
#endif
           ! integration of effectif settling => flx_w2s_sum=kg/m2
           flx_w2s_sum(iv,i,j)=flx_w2s_sum(iv,i,j)+flx_w2s(iv,i,j)*cv_wat(iv,1,i,j)*dtm
#endif
         ENDDO

       ENDIF ! fin test sur la hauteur
     ENDDO ! fin boucle en i
   ENDDO ! fin boucle en j

!$OMP END DO

   IF (l_tssubadv_over_smalldepth) CALL parsub_eq_hm(xe2,xe3,cv_wat,ll,niter,num_ws)

   IF (l_bilan .AND. t >= tdebsubs) CALL parsub_budget_flxcum(xe2,xe3,cv_wat,dtm,niter,num_ws,ll)

   ! Apply open boundary conditions on temperature, salinity and substances

OMPAGRIF BARRIER
OMPAGRIF MASTER
   IF_AGRIF (Agrif_Root()) THEN
#ifdef key_substance
     CALL parsub_obc3d_apply(uz,vz,cv_wat,dtm,num_ws)
#endif
   ELSE_AGRIF
#ifdef key_substance
     CALL_AGRIF parsub_obc3d_apply_agrif(num_ws)
#endif
   ENDIF_AGRIF
OMPAGRIF END MASTER
OMPAGRIF BARRIER
 
  END DO  ! ends loop on subdt


!$OMP END PARALLEL

  END SUBROUTINE parsub_eq

#endif
