#ifdef key_substance
  SUBROUTINE parsub_advdif_macho3d(xe2,xe3,setvel,cv_wat,ll,niter,num_ws)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE parsub_advdif_macho3d  ***
   !&E
   !&E ** Purpose : call the good advection scheme depending of the cpp key
   !&E
   !&E ** Description : use macho splitting in 3 directions X,Y,Z
   !&E                  The tracer advection is computed in a different order at each time step:
   !&E                  along directions XYZ - XZY - YXZ - YZX - ZXY - ZYX
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : temp,sal,cv_wat
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference : Leonard B. P., Lock A. P. and M. K. MacVean (1996) :
   !&E                Conservative Explicit Unrestricted-Time-Step
   !&E                Multidimensional COnstancy-Preserving Advections Schemes.
   !&E                Monthly Weather Review, 124, 2588:2605
   !&E
   !&E                Debreu, Duhaut (2011) - Contrat PREVIMER 09/2 211 097
   !&E
   !&E ** History :
   !&E       !  2010-12  (V. Garnier) Original code
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! Modules used
   USE parameters, ONLY : rsh,imin,imax,jmin,ljminm2,ljmaxp2,jmax,kmax,ljmin,ljmax, &
                          liminm2,liminm1,limaxp1,limaxp2,limin,limax,ljminm1,ljmaxp1
#ifdef key_substance
   USE parameters, ONLY : nb_var
#endif
   USE comvars2d,  ONLY : nbouc,ierrorlog,nbouc_macho,t
#ifdef key_substance
   USE comsubstance, ONLY : c2,c3
#ifdef key_parsub_macho3d
   USE comsubstance, ONLY : nbouc_machop
#endif
#endif

   !! * Arguments
   INTEGER,                                                              INTENT(in)    :: niter,num_ws,ll
   REAL(KIND=rsh),                                                       INTENT(in)    :: setvel
   REAL(kind=rsh),DIMENSION(limin:limax,ljmin:ljmax),                    INTENT(in)    :: xe2
   REAL(kind=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),            INTENT(in)    :: xe3
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(inout) :: cv_wat


   !! * Local declarations
   INTEGER :: nxyz

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter PARSUB_ADVDIF_MACHO3D.F90 routine'
#ifdef key_parsub_macho3d
   nxyz=MOD(nbouc+nbouc_machop(num_ws),6)

   SELECT CASE (nxyz)

     CASE(1) ! XYZ

     CALL parsub_advx_macho(cv_wat,c2,niter,num_ws)
     CALL parsub_advy_macho(c2,c3,niter,num_ws)
     ! Advection along x
#ifdef key_parsub_adv_ultimatequickest
     CALL parsub_advdifx_uq(cv_wat,niter,num_ws) 
#elif key_parsub_adv_fifthorder
     CALL parsub_advdifx_fifthorder(cv_wat,niter,num_ws)
#elif key_parsub_adv_fourthorder
     CALL parsub_advdifx_fourthorder(cv_wat,niter,num_ws)
#else
     CALL parsub_advdifx_quickupwind(cv_wat,niter,num_ws)
#endif


     ! Advection along y
#ifdef key_parsub_adv_ultimatequickest
     CALL parsub_advdify_uq(c2,niter,num_ws)
#elif key_parsub_adv_fifthorder
     CALL parsub_advdify_fifthorder(c2,niter,num_ws)
!#elif key_parsub_adv_fourthorder
!     CALL parsub_advdify_fourthorder(c2,niter,num_ws)
#else
     CALL parsub_advdify_quickupwind(c2,niter,num_ws)
#endif

     ! Advection along z
#if defined key_parsub_wquickupwind
     CALL parsub_advz_quickupwind(xe2,xe3,setvel,c3,ll,niter,num_ws)
#elif defined key_parsub_wcompact
     CALL parsub_advz_compact(xe2,xe3,setvel,c3,ll,niter,num_ws)
#endif

   CASE(2) ! XZY

     CALL parsub_advx_macho(cv_wat,c2,niter,num_ws) 
     CALL parsub_advz_macho(xe2,xe3,c2,c3,setvel,ll,niter,num_ws)

     ! Advection along x
#ifdef key_parsub_adv_ultimatequickest
     CALL parsub_advdifx_uq(cv_wat,niter,num_ws) 
#elif key_parsub_adv_fifthorder
     CALL parsub_advdifx_fifthorder(cv_wat,niter,num_ws)
!!#elif key_parsub_adv_fourthorder
!!     CALL parsub_advdifx_fourthorder(cv_wat,niter,num_ws)
#else
     CALL parsub_advdifx_quickupwind(cv_wat,niter,num_ws)
#endif

     ! Advection along y
#ifdef key_parsub_adv_ultimatequickest
     CALL parsub_advdify_uq(c3,niter,num_ws)
#elif key_parsub_adv_fifthorder
     CALL parsub_advdify_fifthorder(c3,niter,num_ws)
#elif key_parsub_adv_fourthorder
     CALL parsub_advdify_fourthorder(c3,niter,num_ws)
#else
     CALL parsub_advdify_quickupwind(c3,niter,num_ws)
#endif

     ! Advection along z
#if defined key_parsub_wquickupwind
     CALL parsub_advz_quickupwind(xe2,xe3,setvel,c2,ll,niter,num_ws)
#elif defined key_parsub_wcompact
     CALL parsub_advz_compact(xe2,xe3,setvel,c2,ll,niter,num_ws)
#endif

   CASE(3) ! YXZ

     CALL parsub_advy_macho(cv_wat,c2,niter,num_ws)
     CALL parsub_advx_macho(c2,c3,niter,num_ws)

     ! Advection along x
#ifdef key_parsub_adv_ultimatequickest
     CALL parsub_advdifx_uq(c2,niter,num_ws) 
#elif key_parsub_adv_fifthorder
     CALL parsub_advdifx_fifthorder(c2,niter,num_ws)
#elif key_parsub_adv_fourthorder
     CALL parsub_advdifx_fourthorder(c2,niter,num_ws)
#else
     CALL parsub_advdifx_quickupwind(c2,niter,num_ws)
#endif

     ! Advection along y
#ifdef key_parsub_adv_ultimatequickest
     CALL parsub_advdify_uq(cv_wat,niter,num_ws)
#elif key_parsub_adv_fifthorder
     CALL parsub_advdify_fifthorder(cv_wat,niter,num_ws)
#elif key_parsub_adv_fourthorder
     CALL parsub_advdify_fourthorder(cv_wat,niter,num_ws)
#else
     CALL parsub_advdify_quickupwind(cv_wat,niter,num_ws)
#endif

     ! Advection along z
#if defined key_parsub_wquickupwind
     CALL parsub_advz_quickupwind(xe2,xe3,setvel,c3,ll,niter,num_ws)
#elif defined key_parsub_wcompact
     CALL parsub_advz_compact(xe2,xe3,setvel,c3,ll,niter,num_ws)
#endif


   CASE(4) ! YZX

     CALL parsub_advy_macho(cv_wat,c2,niter,num_ws)
     CALL parsub_advz_macho(xe2,xe3,c2,c3,setvel,ll,niter,num_ws)

     ! Advection along x
#ifdef key_parsub_adv_ultimatequickest
     CALL parsub_advdifx_uq(c3,niter,num_ws)
#elif key_parsub_adv_fifthorder
     CALL parsub_advdifx_fifthorder(c3,niter,num_ws)
#elif key_parsub_adv_fourthorder
     CALL parsub_advdifx_fourthorder(c3,niter,num_ws)
#else
     CALL parsub_advdifx_quickupwind(c3,niter,num_ws)
#endif

     ! Advection along y
#ifdef key_parsub_adv_ultimatequickest
     CALL parsub_advdify_uq(cv_wat,niter,num_ws)
#elif key_parsub_adv_fifthorder
     CALL parsub_advdify_fifthorder(cv_wat,niter,num_ws)
#elif key_parsub_adv_fourthorder
     CALL parsub_advdify_fourthorder(cv_wat,niter,num_ws)
#else
     CALL parsub_advdify_quickupwind(cv_wat,niter,num_ws)
#endif

     ! Advection along z
#if defined key_parsub_wquickupwind
     CALL parsub_advz_quickupwind(xe2,xe3,setvel,c2,ll,niter,num_ws)
#elif defined key_parsub_wcompact
     CALL parsub_advz_compact(xe2,xe3,setvel,c2,ll,niter,num_ws)
#endif

   CASE(5) ! ZXY

     CALL parsub_advz_macho(xe2,xe3,cv_wat,c2,setvel,ll,niter,num_ws)
     CALL parsub_advx_macho(c2,c3,niter,num_ws)

     ! Advection along x
#ifdef key_parsub_adv_ultimatequickest
     CALL parsub_advdifx_uq(c2,niter,num_ws) 
#elif key_parsub_adv_fifthorder
     CALL parsub_advdifx_fifthorder(c2,niter,num_ws)
#elif key_parsub_adv_fourthorder
     CALL parsub_advdifx_fourthorder(c2,niter,num_ws)
#else
     CALL parsub_advdifx_quickupwind(c2,niter,num_ws)
#endif

     ! Advection along y
#ifdef key_parsub_adv_ultimatequickest
     CALL parsub_advdify_uq(c3,niter,num_ws)
#elif key_parsub_adv_fifthorder
     CALL parsub_advdify_fifthorder(c3,niter,num_ws)
#elif key_parsub_adv_fourthorder
     CALL parsub_advdify_fourthorder(c3,niter,num_ws)
#else
     CALL parsub_advdify_quickupwind(c3,niter,num_ws)
#endif

     ! Advection along z
#if defined key_parsub_wquickupwind

     CALL parsub_advz_quickupwind(xe2,xe3,setvel,cv_wat,ll,niter,num_ws)
#elif defined key_parsub_wcompact
     CALL parsub_advz_compact(xe2,xe3,setvel,cv_wat,ll,niter,num_ws)
#endif

   CASE(0) ! ZYX
     CALL parsub_advz_macho(xe2,xe3,cv_wat,c2,setvel,ll,niter,num_ws)
     CALL parsub_advy_macho(c2,c3,niter,num_ws)

     ! Advection along x
#ifdef key_parsub_adv_ultimatequickest
     CALL parsub_advdifx_uq(c3,niter,num_ws)
#elif key_parsub_adv_fifthorder
     CALL parsub_advdifx_fifthorder(c3,niter,num_ws)
#elif key_parsub_adv_fourthorder
     CALL parsub_advdifx_fourthorder(c3,niter,num_ws)
#else
     CALL parsub_advdifx_quickupwind(c3,niter,num_ws)
#endif

#ifdef key_parsub_adv_ultimatequickest
     CALL parsub_advdify_uq(c2,niter,num_ws)
#elif key_parsub_adv_fifthorder
     CALL parsub_advdify_fifthorder(c2,niter,num_ws)
#elif key_parsub_adv_fourthorder
     CALL parsub_advdify_fourthorder(c2,niter,num_ws)
#else
     CALL parsub_advdify_quickupwind(c2,niter,num_ws)
#endif

     ! Advection along z
#if defined key_parsub_wquickupwind
     CALL parsub_advz_quickupwind(xe2,xe3,setvel,cv_wat,ll,niter,num_ws)
#elif defined key_parsub_wcompact
     CALL parsub_advz_compact(xe2,xe3,setvel,cv_wat,ll,niter,num_ws)
#endif

   CASE DEFAULT
     WRITE(ierrorlog,*) ' '
     WRITE(ierrorlog,*) ' routine parsub_ADVDIF_MACHO3D '
     WRITE(ierrorlog,*) ' you should never see this line '
     WRITE(ierrorlog,*) ' '
   END SELECT
#endif
   PRINT_DBG*, 'exit parsub_ADVDIF_MACHO3D.F90 routine'

  END SUBROUTINE parsub_advdif_macho3d
#endif
