#ifdef key_substance
  SUBROUTINE parsub_advdifx_fifthorder(c,niter,num_ws)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE parsub_advdifx_fifthorder  ***
   !&E
   !&E ** Purpose : Estimate of zonal fluxes of particulate substances (advection and diffusion)
   !&E              Advection uses FIFTH order scheme.
   !&E
   !&E ** Description : No horizontal diffusion at lower depths ( < hminkxky)
   !&E                  - case of cyclicity in y : correct for eastward propagation,
   !&E                    uncorrect for westward propagation (stencil not long enough)
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : hex,hx,fxeau,c(cv_wat),hzex,dx_u,fx 
   !&E  
   !&E ** Modified variables : fsalx,ftempx,fx
   !&E
   !&E ** Reference : Debreu, Duhaut (2011) contrat PREVIMER 09/2 211 097 
   !&E
   !&E ** History :
   !&E           !  2014-10 (B.Thouvenin) Original code (from parsubadvdifxultimatequickest.F90 and tssubadvdifxfifthorder.F90)
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

#ifdef key_parsub_adv_fifthorder

   !! * Modules used
   USE parameters
   USE comvars2d, ONLY : ig,id,htx,hex,hx,kx,h0fond,hminkxky,dx_u,halfdt,l_obc_cycl_x
#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
   USE comvars2d, ONLY : dy_u
#endif
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : hzex
#endif
   USE comfluxdo, ONLY : fxeau
   USE comsubstance, ONLY: fx,iv_ws_max,nv_ws_max,nv_adv,nvp,c5
   USE_MPI toolmpi, ONLY : ex_i_rsh,ex_j_rsh
#ifdef key_sedim
   USE sedimento,    ONLY : corflux
#endif
 

   IMPLICIT NONE

   !! * Arguments
   INTEGER,                                                              INTENT(in) :: niter,num_ws
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in) :: c

   !! * Local declarations
   INTEGER                   :: i,j,k,l,iv
   REAL(KIND=rsh), PARAMETER :: dsmall = 1.0d-20
   REAL(KIND=rsh)            :: uu,da,db,dc,dd,de,uuu,cfl,kxb
   REAL(KIND=rsh), DIMENSION(nv_adv)    :: ya,yb,yc,yd,ye,delta

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter PARSUB_ADVDIFX_FIFTHORDER.F90 routine'

!$OMP SINGLE
          DO l=1,nv_ws_max(num_ws)
           iv=iv_ws_max(num_ws,l)
           c5(iv,:,liminm1:limaxp2,ljminm1:ljmaxp2)=c(iv,:,liminm1:limaxp2,ljminm1:ljmaxp2)
          END DO
!$OMP END SINGLE
OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-2,3,nvp*kmax,liminm2,limaxp3,ljminm2,ljmaxp3,c5(1:nvp,:,liminm2:limaxp3,ljminm2:ljmaxp3))
   CALL_MPI ex_j_rsh(-2,3,nvp*kmax,liminm2,limaxp3,ljminm2,ljmaxp3,c5(1:nvp,:,liminm2:limaxp3,ljminm2:ljmaxp3))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(c5)

!$OMP DO schedule(runtime),private(i,j,k,kxb               &
!$OMP& ,l,iv,ya,yb,yc,yd,ye,delta                          &
!$OMP& ,da,db,dc,dd,de,uu,uuu,cfl)

   DO j=ljminp2,ljmaxm1
     DO i=MAX0(limin,ig(j)),MIN0(limax,id(j)-1)
       DO k=1,kmax
         IF (htx(i,j) <= 0.0_rsh .OR. hx(i,j) == -valmanq-h0fond) THEN
          DO l=1,nv_ws_max(num_ws)
           iv=iv_ws_max(num_ws,l)
           fx(iv,k,i,j)=0.0_rsh
          END DO
         ELSE
           kxb=kx
           IF (htx(i,j) < hminkxky) kxb=0.0_rsh ! no diffusion for small water depth
           uu=fxeau(k,i,j)
#if defined key_siggen || defined key_gencoord
           uuu = uu/(halfdt*hzex(k,i,j))
#else
           uuu = uu/(halfdt*hex(i,j))
#endif
           cfl = uuu*halfdt/dx_u(i,j)

           IF (uu <= 0.0_rsh) THEN  ! westward advection

             de=(1.0_rsh/120.0_rsh)*(4.0_rsh-5.0_rsh*cfl**2+cfl**4)
             dd=(1.0_rsh/120.0_rsh)*(-26.0_rsh+5.0_rsh*cfl+30.0_rsh*cfl**2-5.0_rsh*cfl**3-4.0_rsh*cfl**4)
             dc=(1.0_rsh/120.0_rsh)*(94.0_rsh-75.0_rsh*cfl-40.0_rsh*cfl**2+15.0_rsh*cfl**3+6.0_rsh*cfl**4)
             db=(1.0_rsh/120.0_rsh)*(54.0_rsh+75.0_rsh*cfl+10.0_rsh*cfl**2-15.0_rsh*cfl**3-4.0_rsh*cfl**4)
             da=(1.0_rsh/120.0_rsh)*(-6.0_rsh-5.0_rsh*cfl+5.0_rsh*cfl**2+5.0_rsh*cfl**3+cfl**4)

             DO l=1,nv_ws_max(num_ws)
              iv=iv_ws_max(num_ws,l)
              yc(iv)=c5(iv,k,i+1,j)
              yb(iv)=c5(iv,k,i,j)

              IF (htx(i+1,j) > 0.0_rsh .AND. (i+1 < id(j) .OR. l_obc_cycl_x)) THEN
                yd(iv)=c5(iv,k,i+2,j)
              ELSE
                yd(iv)=yc(iv)
              END IF
              IF (i+2<=imax .AND. htx(MIN0(i+2,imax),j) > 0.0_rsh .AND. (i+2 < id(j) .OR. l_obc_cycl_x)) THEN
                ye(iv)=c5(iv,k,i+3,j)
              ELSE
                ye(iv)=yd(iv)
              END IF
              IF (htx(i-1,j) > 0.0_rsh .AND. (i > ig(j) .OR. l_obc_cycl_x)) THEN
              !IF (hex(i-1,j) > 0.0_rsh .AND. i-1 > ig(j)) THEN
                ya(iv)=c5(iv,k,i-1,j)
              ELSE
                ya(iv)=yb(iv)
              END IF
              delta(iv)=yc(iv)-yb(iv)
             ENDDO

           ELSE  ! eastward advection

             da=(1.0_rsh/120.0_rsh)*(4.0_rsh-5.0_rsh*cfl**2+cfl**4)
             db=(1.0_rsh/120.0_rsh)*(-26.0_rsh-5.0_rsh*cfl+30.0_rsh*cfl**2+5.0_rsh*cfl**3-4.0_rsh*cfl**4)
             dc=(1.0_rsh/120.0_rsh)*(94.0_rsh+75.0_rsh*cfl-40.0_rsh*cfl**2-15.0_rsh*cfl**3+6.0_rsh*cfl**4)
             dd=(1.0_rsh/120.0_rsh)*(54.0_rsh-75.0_rsh*cfl+10.0_rsh*cfl**2+15.0_rsh*cfl**3-4.0_rsh*cfl**4)
             de=(1.0_rsh/120.0_rsh)*(-6.0_rsh+5.0_rsh*cfl+5.0_rsh*cfl**2-5.0_rsh*cfl**3+cfl**4)

             DO l=1,nv_ws_max(num_ws)
              iv=iv_ws_max(num_ws,l)
              yc(iv)=c5(iv,k,i,j)
              yd(iv)=c5(iv,k,i+1,j)

              IF (htx(i-1,j) > 0.0_rsh .AND. (i > ig(j) .OR. l_obc_cycl_x)) THEN
!               IF (hex(i-1,j) > 0.0_rsh) THEN
                yb(iv)=c5(iv,k,i-1,j)
              ELSE
                yb(iv)=yc(iv)
              END IF
              IF (i-2>=0 .AND. htx(MAX0(i-2,imin),j) > 0.0_rsh .AND. (i-1 > ig(j) .OR. l_obc_cycl_x)) THEN
                ya(iv)=c5(iv,k,i-2,j)
              ELSE
                ya(iv)=yb(iv)
              END IF
              IF (htx(i+1,j) > 0.0_rsh .AND. (i+1 < id(j) .OR. l_obc_cycl_x)) THEN
                ye(iv)=c5(iv,k,i+2,j)
              ELSE
                ye(iv)=yd(iv)
              END IF
              delta(iv)=yd(iv)-yc(iv)
             ENDDO

           END IF    ! ends test on velocity sign


   ! advection substances
   ! -------------------------------------------------
           DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)
             fx(iv,k,i,j) = uu*(da*ya(iv)+db*yb(iv)+dc*yc(iv)+dd*yd(iv)+de*ye(iv))
           END DO

           DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)
#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
   ! Advection term (iso-sigma) alone
   ! --------------------------------
             fx(iv,k,i,j) = fx(iv,k,i,j)*dy_u(i,j)
#else
   ! Advection/diffusion term (iso-sigma)
   ! ----------------------------------
#if defined key_siggen || defined key_gencoord
             fx(iv,k,i,j) = fx(iv,k,i,j) - hzex(k,i,j)*kxb*delta(iv)/dx_u(i,j)*halfdt
#else
             fx(iv,k,i,j) = fx(iv,k,i,j) - hex(i,j)*kxb*delta(iv)/dx_u(i,j)*halfdt
#endif
#endif
#ifdef key_sedim
             IF (k == 1) fx(iv,k,i,j)=fx(iv,k,i,j)*corflux(iv,i,j)
#endif
           END DO

         END IF ! ends test on water depth
       END DO ! ends loop on k
     END DO ! ends loop on i
   END DO ! ends loop on j
!$OMP END DO

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-1,0,nv_adv*kmax,liminm1,limax,ljmin,ljmax,fx(:,:,liminm1:limax,ljmin:ljmax))

OMPMPI end master
OMPMPI barrier
OMPMPI flush(fx)

#else

   USE parameters, ONLY : rsh,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,nb_var
   INTEGER,                                                              INTENT(in) :: niter,num_ws
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in) :: c

#endif

   PRINT_DBG*, 'exit PARSUB_ADVDIFX_FIFTHORDER.F90 routine'

  END SUBROUTINE parsub_advdifx_fifthorder
#endif
