#ifdef key_substance

  SUBROUTINE parsub_advdifx_quickupwind(c,niter,num_ws)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE parsub_advdifx_quickupwind  ***
   !&E
   !&E ** Purpose : Estimate of zonal fluxes of temperature and salinity
   !&E              and other substances (advection and diffusion)
   !&E              Advection uses TVD scheme (QUICK and upwind) owing to limiter.
   !&E
   !&E ** Description : No horizontal diffusion at lower depths ( < hminkxky)
   !&E
   !&E ** Cas cyclique en y : correct for eastward propagation
   !&E                        uncorrect for westward propagation (stencil not long enough)
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : hex,hx,c(cv_wat),hzex,dx_u ,fx
   !&E
   !&E ** Modified variables : fx
   !&E
   !&E ** Reference : Kowalik Z. and T. S. Murty : Numerical modeling of ocean
   !&E                dynamics in Advanced series on ocean engineering, vol 5.
   !&E
   !&E ** History :
   !&E           !  2010-03  (P. Douillet) substances with settling velocities
   !&E                                     issued from tssub_advdifx_quickupwind
   !&E           !  2011-02  (B. Thouvenin) correction diffusion term of substances
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d,    ONLY : ig,id,htx,hex,hx,kx,qmax,h0fond,hminkxky,dx_u,halfdt,l_obc_cycl_x
#if defined key_siggen || defined key_gencoord
   USE comsiggen,    ONLY : hzex
#endif
   USE comfluxdo,    ONLY : fxeau
   USE comsubstance, ONLY : fx,iv_ws_max,nv_ws_max,nv_adv
#ifdef key_sedim
   USE sedimento,    ONLY : corflux
#endif
   USE_MPI toolmpi,      ONLY : ex_i_rsh

   IMPLICIT NONE

   !! * Arguments
   INTEGER,                                                                INTENT(in) :: niter,num_ws
   REAL(KIND=rsh), DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2), INTENT(in) :: c

   !! * Local declarations
   LOGICAL                 :: decentt,decents
   INTEGER                 :: i,j,k
   REAL(KIND=rsh)          :: unsqmax,uu,yr,qot,dyds,dygs,dydt
   REAL(KIND=rsh)          :: dygt,kxb,epsi
   LOGICAL, DIMENSION(nv_adv)           :: decent
   INTEGER                              :: l,iv
   REAL(KIND=rsh)                       :: dyg
   REAL(KIND=rsh), DIMENSION(nv_adv)    :: ya,yb,yc,dyd

   !!----------------------------------------------------------------------
   !! * Executable part

   unsqmax=1.0_rsh/qmax
   epsi=1.0e-10

!$OMP DO schedule(runtime),private(i,j,k,kxb               &
!$OMP& ,uu,yr,qot,decents,decentt,dyds,dygs,dydt,dygt      &
!$OMP& ,l,iv,ya,yb,yc,decent,dyd,dyg)
   DO j=ljminp2,ljmaxm1
     DO i=MAX0(limin,ig(j)),MIN0(limax,id(j)-1)
       DO k=1,kmax
         IF (htx(i,j) <= 0.0_rsh .OR. hx(i,j) == -valmanq-h0fond) THEN
           DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)
             fx(iv,k,i,j)=0.0_rsh
           END DO
         ELSE
           kxb=kx
           IF (htx(i,j) < hminkxky) kxb=0.0_rsh ! no diffusion for small water depth
           uu=fxeau(k,i,j)
           IF (uu <= 0.0_rsh) THEN  ! westward advection
            DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)
             yb(iv)=c(iv,k,i+1,j)
             yc(iv)=c(iv,k,i,j)
             IF (htx(i+1,j) > 0.0_rsh .AND. i+1 < id(j)) THEN
               ya(iv)=c(iv,k,i+2,j)
             ELSE
               ya(iv)=yb(iv)
             END IF

               dyd(iv)=yb(iv)-yc(iv)
               dyg=yb(iv)-ya(iv)
               decent(iv)=(dyd(iv)*dyg) >= 0.0_rsh
               IF (.NOT.decent(iv)) THEN
                 qot=ABS(dyd(iv)/MAX(ABS(dyg),epsi))
                 IF (qot <= qmax .AND. qot >= unsqmax) THEN
                   decent(iv)=.FALSE.
                 ELSE
                   decent(iv)=.TRUE.
                 END IF
               END IF
            END DO

           ELSE  ! eastward advection

            DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)
             yb(iv)=c(iv,k,i,j)
             yc(iv)=c(iv,k,i+1,j)
             IF (htx(i-1,j) > 0.0_rsh .AND. (i > ig(j) .OR. l_obc_cycl_x)) THEN
!              IF (hex(i-1,j) > 0.0_rsh) THEN
               ya(iv)=c(iv,k,i-1,j)
             ELSE
               ya(iv)=yb(iv)
             END IF

               dyd(iv)=yc(iv)-yb(iv)
               dyg=yb(iv)-ya(iv)
               decent(iv)=(dyd(iv)*dyg) <= 0.0_rsh
               IF (.NOT.decent(iv)) THEN
                 qot=ABS(dyd(iv)/MAX(ABS(dyg),epsi))
                 IF (qot <= qmax .AND. qot >= unsqmax) THEN
                   decent(iv)=.FALSE.
                 ELSE
                   decent(iv)=.TRUE.
                 END IF
               END IF
            END DO
           END IF    ! ends test on velocity sign



   ! advection flux of other substances
   ! ----------------------------------

           DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)

#if defined key_ts_diffh_geo || defined key_ts_diffh_rho

  ! advection flux alone
  ! --------------------
             IF (decent(iv)) THEN
               fx(iv,k,i,j)=fxeau(k,i,j)/REAL(niter,rsh)*yb(iv)*dy_u(i,j)
             ELSE
               yr=0.375_rsh*yc(iv)+0.75_rsh*yb(iv)-0.125_rsh*ya(iv)
               fx(iv,k,i,j)=fxeau(k,i,j)/REAL(niter,rsh)*yr*dy_u(i,j)
             END IF

#else

  ! diffusion/advection flux
  ! ------------------------
             IF (decent(iv)) THEN
#if defined key_siggen || defined key_gencoord
               fx(iv,k,i,j)=(fxeau(k,i,j)*yb(iv)-hzex(k,i,j)*kxb*dyd(iv)/dx_u(i,j)*halfdt)/REAL(niter,rsh)
#else
               fx(iv,k,i,j)=(fxeau(k,i,j)*yb(iv)-hex(i,j)*kxb*dyd(iv)/dx_u(i,j)*halfdt)/REAL(niter,rsh)
#endif
             ELSE
               yr=0.375_rsh*yc(iv)+0.75_rsh*yb(iv)-0.125_rsh*ya(iv)
#if defined key_siggen || defined key_gencoord
               fx(iv,k,i,j)=(fxeau(k,i,j)*yr/REAL(niter,rsh) - hzex(k,i,j)*kxb*dyd(iv)/dx_u(i,j)*halfdt)/REAL(niter,rsh)
#else
               fx(iv,k,i,j)=(fxeau(k,i,j)*yr/REAL(niter,rsh) - hex(i,j)*kxb*dyd(iv)/dx_u(i,j)*halfdt)/REAL(niter,rsh)
#endif
             END IF

#endif

#ifdef key_sedim
             IF (k.eq.1) fx(iv,k,i,j)=fx(iv,k,i,j)*corflux(iv,i,j)
#endif
           END DO

          END IF ! ends test on water depth
        END DO ! ends loop on k
      END DO ! ends loop on i
   END DO ! ends loop on j
!$OMP END DO

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-1,0,nv_adv*kmax,liminm1,limax,ljmin,ljmax,fx(:,:,liminm1:limax,ljmin:ljmax))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(fx)

  END SUBROUTINE parsub_advdifx_quickupwind

#else

  SUBROUTINE parsub_advdifx_quickupwind_empty
    WRITE(*,*) 'parsub_advdifx_quickupwind_empty routine :'
    WRITE(*,*) 'You should not have seen this print!  error?'
    return
  END SUBROUTINE parsub_advdifx_quickupwind_empty

#endif
