#ifdef key_substance

  SUBROUTINE parsub_advdifx_uqm(c,niter,num_ws)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE parsub_advdifx_ultimatequickestmacho  ***
   !&E
   !&E ** Purpose : Estimate of zonal fluxes of substances with settling velocity
   !&E              (advection and diffusion)
   !&E              Advection uses QUICKEST scheme with ULTIMATE limiter and
   !&E              its MACHO multidimensional extension.
   !&E
   !&E ** Description : No horizontal diffusion at lower depths ( < hminkxky)
   !&E                  - case of cyclicity in y: correct for eastward propagation,
   !&E                    uncorrect for westward propagation (stencil not long enough)
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : hex,hx,hey,hy,fxeau,fyeau,c(cv_wat),hzex,dx_u,dy,
   !&E                     fx 
   !&E
   !&E ** Modified variables : fx
   !&E
   !&E ** Reference : Leonard B. P., Lock A. P. and M. K. MacVean (1996) :
   !&E                Conservative Explicit Unrestricted-Time-Step
   !&E                Multidimensional COnstancy-Preserving Advections Schemes.
   !&E                Monthly Weather Review, 124, 2588:2605
   !&E
   !&E ** History :
   !&E           !  2010-03  (P. Douillet) substances with settling velocities
   !&E           !                         issued from tssub_advdifx_uqm
   !&E           !  2011-02  (B. Thouvenin) correction diffusion term of substances + upwind estimate
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d,    ONLY : ig,id,htx,hex,hx,kx,h0fond,hminkxky,dx_u,dy,hty,hey, &
                            jb,jh,hy,halfdt,l_obc_cycl_x
#if defined key_siggen || defined key_gencoord
   USE comsiggen,    ONLY : hzex,hzey
#endif
   USE comfluxdo,    ONLY : fxeau,fyeau
   USE comsubstance, ONLY : nv_adv,nv_tot,fx,nvp,iv_ws_max,nv_ws_max,c2
#ifdef key_sedim
   USE sedimento,    ONLY : corflux
#endif
 
   USE_MPI toolmpi,  ONLY : ex_i_rsh,ex_peri_x_rsh

   IMPLICIT NONE

   !! * Arguments
   INTEGER,                                                              INTENT(in) :: niter,num_ws
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in) :: c

   !! * Local declarations
   INTEGER                   :: i,j,k
   REAL(KIND=rsh), PARAMETER :: dsmall = 1.0d-20
   REAL(KIND=rsh)            :: uu,dyds,dygs,dydt,d0,d1,uuu,courant,vv,  &
                                dygt,kxb,thetap,psip
   REAL(KIND=rlg), DIMENSION(kmax) :: fvb,fvt
   REAL(KIND=rsh), DIMENSION(kmax) :: salb,salt,tempb,tempt
   INTEGER                              :: l,iv
   REAL(KIND=rsh), DIMENSION(nv_adv)    :: dyd,dyg,ya,yb,yc,delta
   REAL(KIND=rsh), DIMENSION(1:nvp,kmax) :: cb,ct

   !!----------------------------------------------------------------------
   !! * Executable part

   ! estimate of intermediate tracers (from upwind scheme)
   ! -----------------------------------------------------

   ! at open boundaries (east and west)

!$OMP SINGLE

   DO l=1,nv_ws_max(num_ws)
   iv=iv_ws_max(num_ws,l)
   IF_MPI (limin==imin) THEN
     c2(iv,:,imin:imin+1,ljmin:ljmax) = c(iv,:,imin:imin+1,ljmin:ljmax)
   ENDIF_MPI
   IF_MPI (limax==imax) THEN
     c2(iv,:,imax,ljmin:ljmax) = c(iv,:,imax,ljmin:ljmax)
   ENDIF_MPI
   END DO

!$OMP END SINGLE

   ! inside the domain

!$OMP DO SCHEDULE(RUNTIME),PRIVATE(i,j,k,l,iv, &
!$OMP& ct,fvt,cb,fvb,vv)

   DO i=liminp2,limaxm1
     DO j=ljminp1m1,ljmax
       DO k=1,kmax

#ifdef key_MPI_2D
         IF (j == jb(i) .OR. j==ljminp1m1) THEN
#else
         IF (j == jb(i) .OR. j==ljminp1-1) THEN
#endif

           IF (hty(i,j) <= 0.0_rsh .OR. hy(i,j)==-valmanq-h0fond) THEN
            DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)
             ct(iv,k)=0.0_rsh
            END DO
             fvt(k)=0.0_rlg
           ELSE

             fvt(k)=fyeau(k,i,j)   ! h*vz*dt/2

             DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)
             ! upwind scheme 
             IF (fvt(k) <= 0.0_rlg) THEN
               ct(iv,k)=c(iv,k,i,j+1)
             ELSE
               ct(iv,k)=c(iv,k,i,j)
             END IF   ! ends test on the direction of propagation
             END DO

           END IF   ! ends test on the water depth

#ifdef key_MPI_2D
         ELSE IF (j >= jb(i)+1 .AND. j <= jh(i)-1 .AND. j>ljminp1m1) THEN
#else
         ELSE IF (j >= jb(i)+1 .AND. j <= jh(i)-1 .AND. j>ljminp1-1) THEN
#endif

           ! bottom of the stencil
           DO l=1,nv_ws_max(num_ws)
           iv=iv_ws_max(num_ws,l)
           cb(iv,k)=ct(iv,k)
           END DO
           fvb(k)=fvt(k)

           ! right of the stencil
           IF (hty(i,j) <= 0.0_rsh .OR. hy(i,j)==-valmanq-h0fond) THEN
            DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)
             ct(iv,k)=0.0_rsh
            END DO
             fvt(k)=0.0_rlg
           ELSE

             fvt(k)=fyeau(k,i,j)   ! h*vz*dt/2

             DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)

             ! upwind scheme 
             IF (fvt(k) <= 0.0_rlg) THEN
               ct(iv,k)=c(iv,k,i,j+1)
             ELSE
               ct(iv,k)=c(iv,k,i,j)
             END IF   ! ends test on the direction of propagation
             END DO

           END IF   ! ends test on the water depth

   ! estimate of intermediate tracers (from upwind scheme)
           IF ( (fvb(k)>0.0_rlg) .AND. (fvt(k)>0.0_rlg) ) THEN
#if defined key_siggen || defined key_gencoord
             vv = fvb(k)/(halfdt*hzey(k,i,j-1))
#else
             vv = fvb(k)/(halfdt*hey(i,j-1))
#endif
           ELSE IF ( (fvb(k)<0.0_rlg) .AND. (fvt(k)<0.0_rlg) ) THEN
!cval toseelater Normal qu il n y ait pas de sous pas de temps ?
#if defined key_siggen || defined key_gencoord
             vv = fvt(k)/(halfdt*hzey(k,i,j))
#else
             vv = fvt(k)/(halfdt*hey(i,j))
#endif
           ELSE
             ! no time evolution of the intermediate temperature
             ! along the boundaries
             vv = 0.0_rsh
           END IF

           ! apply euler (time) - upwind (x) scheme
           DO l=1,nv_ws_max(num_ws)
           iv=iv_ws_max(num_ws,l)
           c2(iv,k,i,j)  = c(iv,k,i,j)  - (halfdt/REAL(niter,rsh)/dy(i,j))*vv*(ct(iv,k)-cb(iv,k))
           END DO

         END IF   ! ends test for selecting right j loop

       END DO   ! ends loop on k
     END DO   ! ends loop on i
   END DO   ! ends loop on j
!$OMP END DO

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-1,2,nv_tot*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,c2(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))
OMPMPI end master
OMPMPI barrier
#if ! defined key_offline
#endif
OMPMPI flush(c2)

   ! estimate of fluxes (from ultimate quickest scheme)
   ! --------------------------------------------------

!$OMP DO SCHEDULE(RUNTIME),PRIVATE(i,j,k,kxb,l, &
!$OMP& iv,dyd,dyg,ya,yb,yc,delta,               &
!$OMP& d0,d1,uu,uuu,courant,thetap,psip)

   DO j=ljminp2,ljmaxm1
     DO i=MAX0(limin,ig(j)),MIN0(limax,id(j)-1)
       DO k=1,kmax
         IF (htx(i,j) <= 0.0_rsh .OR. hx(i,j) == -valmanq-h0fond) THEN
          DO l=1,nv_ws_max(num_ws)
           iv=iv_ws_max(num_ws,l)
           fx(iv,k,i,j)=0.0_rsh
          END DO
         ELSE
           kxb=kx
           IF (htx(i,j) < hminkxky) kxb=0.0_rsh ! no diffusion for small water depth
           uu=fxeau(k,i,j)
#if defined key_siggen || defined key_gencoord
           uuu = uu/(halfdt*hzex(k,i,j))
#else
           uuu = uu/(halfdt*hex(i,j))
#endif
           courant = ABS(uuu)*halfdt/dx_u(i,j)/REAL(niter,rsh)
           d0=(1.0_rsh/6.0_rsh)*(2.0_rsh-courant)*(1.0_rsh-courant)
           d1=(1.0_rsh/6.0_rsh)*(1.0_rsh-courant)*(1.0_rsh+courant)
             
           IF (uu <= 0.0_rsh) THEN  ! westward advection
            DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)
             yb(iv)=c2(iv,k,i+1,j)
             yc(iv)=c2(iv,k,i,j)
             IF (htx(i+1,j) > 0.0_rsh .AND. i+1 < id(j)) THEN
               ya(iv)=c2(iv,k,i+2,j)
             ELSE
               ya(iv)=yb(iv)
             END IF
             dyd(iv)=yc(iv)-yb(iv)
             dyg(iv)=yb(iv)-ya(iv)
             delta(iv)=yb(iv)-yc(iv)
            END DO

           ELSE  ! eastward advection

            DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)
             yb(iv)=c2(iv,k,i,j)
             yc(iv)=c2(iv,k,i+1,j)
             IF (htx(i-1,j) > 0.0_rsh .AND. (i > ig(j) .OR. l_obc_cycl_x)) THEN
!              IF (hex(i-1,j) > 0.0_rsh) THEN
               ya(iv)=c2(iv,k,i-1,j)
             ELSE
               ya(iv)=yb(iv)
             END IF
             dyg(iv)=yb(iv)-ya(iv)
             dyd(iv)=yc(iv)-yb(iv)
             delta(iv)=yc(iv)-yb(iv)
            END DO

           END IF    ! ends test on velocity sign


   ! advection flux of other substances
   ! ----------------------------------

           DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)
             IF ((ABS(dyd(iv)).LE.dsmall).OR.(ABS(dyg(iv)).LE.dsmall)) THEN
               ! no horizontal gradient
               thetap = 0.0_rsh
               psip = 0.0_rsh
             ELSE
               ! ultimate limiter
               thetap=(dyg(iv)+dsmall)/(dyd(iv)+dsmall)
               psip = d0+d1*thetap
               psip=MAX(0.0_rsh,MIN(MIN(1.0_rsh,psip),((1.0_rsh-courant)/(courant+dsmall))*thetap))
             END IF
             fx(iv,k,i,j) = uu*(yb(iv)+psip*dyd(iv))
           END DO


           DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)

#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
   ! Advection term (iso-sigma) alone
   ! --------------------------------
             fx(iv,k,i,j) = fx(iv,k,i,j)/REAL(niter,rsh)*dy_u(i,j)

#else
   ! Advection/diffusion term (iso-sigma)
   ! ------------------------------------
#if defined key_siggen || defined key_gencoord
             fx(iv,k,i,j) = (fx(iv,k,i,j) - hzex(k,i,j)*kxb*delta(iv)/dx_u(i,j)*halfdt)/REAL(niter,rsh)
#else
             fx(iv,k,i,j) = (fx(iv,k,i,j) - hex(i,j)*kxb*delta(iv)/dx_u(i,j)*halfdt)/REAL(niter,rsh)
#endif

#endif
#ifdef key_sedim
             IF (k == 1) fx(iv,k,i,j)=fx(iv,k,i,j)*corflux(iv,i,j)
#endif

           END DO

         END IF ! ends test on water depth
       END DO ! ends loop on k
     END DO ! ends loop on i
   END DO ! ends loop on j
!$OMP END DO

!$OMP SINGLE
   ! periodic conditions along x-axis
   IF (l_obc_cycl_x) THEN
#ifdef key_MPI_2D
     DO l=1,nv_ws_max(num_ws)
       iv=iv_ws_max(num_ws,l)
!cval not sure it will work
#ifdef key_shiftobc
       CALL_MPI ex_peri_x_rsh(3,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,c2(iv,:,liminm1:limaxp2,ljminm1:ljmaxp2))
#else
       CALL_MPI ex_peri_x_rsh(1,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,c2(iv,:,liminm1:limaxp2,ljminm1:ljmaxp2))
#endif
     END DO
#else
     DO j=ljmin,ljmax
       DO l=1,nv_ws_max(num_ws)
          iv=iv_ws_max(num_ws,l)
   ! western boundary
          c2(iv,:,ig(j),j)=c2(iv,:,id(j)-1,j)
          c2(iv,:,ig(j)-1,j)=c2(iv,:,id(j)-2,j)
#ifdef key_shiftobc
          c2(iv,:,ig(j)-2,j)=c2(iv,:,id(j)-3,j)
#endif
   ! eastern boundary
          c2(iv,:,id(j),j)=c2(iv,:,ig(j)+1,j)
#ifdef key_shiftobc
          c2(iv,:,id(j)+1,j)=c2(iv,:,ig(j)+2,j)
          c2(iv,:,id(j)+2,j)=c2(iv,:,ig(j)+3,j)
#endif

       ENDDO
     END DO
#endif
   END IF
!$OMP END SINGLE

!$OMP SINGLE
   ! periodic conditions along x-axis
   IF (l_obc_cycl_x) THEN
#ifdef key_MPI_2D
     DO l=1,nv_ws_max(num_ws)
       iv=iv_ws_max(num_ws,l)
!cval not sure it will work
#ifdef key_shiftobc
       CALL_MPI ex_peri_x_rsh(3,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,c2(iv,:,liminm1:limaxp2,ljminm1:ljmaxp2))
#else
       CALL_MPI ex_peri_x_rsh(1,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,c2(iv,:,liminm1:limaxp2,ljminm1:ljmaxp2))
#endif
     END DO
#else
     DO j=ljmin,ljmax
       DO l=1,nv_ws_max(num_ws)
          iv=iv_ws_max(num_ws,l)
   ! western boundary
          c2(iv,:,ig(j),j)=c2(iv,:,id(j)-1,j)
          c2(iv,:,ig(j)-1,j)=c2(iv,:,id(j)-2,j)
#ifdef key_shiftobc
          c2(iv,:,ig(j)-2,j)=c2(iv,:,id(j)-3,j)
#endif
   ! eastern boundary
          c2(iv,:,id(j),j)=c2(iv,:,ig(j)+1,j)
#ifdef key_shiftobc
          c2(iv,:,id(j)+1,j)=c2(iv,:,ig(j)+2,j)
          c2(iv,:,id(j)+2,j)=c2(iv,:,ig(j)+3,j)
#endif

       ENDDO
     END DO
#endif
   END IF
!$OMP END SINGLE

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-1,0,nv_adv*kmax,liminm1,limax,ljmin,ljmax,fx(:,:,liminm1:limax,ljmin:ljmax))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(fx)

  END SUBROUTINE parsub_advdifx_uqm

#else

  SUBROUTINE parsub_advdifx_uqm_empty
    WRITE(*,*) 'parsub_advdifx_uqm_empty routine :'
    WRITE(*,*) 'You should not have seen this print!  error?'
    return
  END SUBROUTINE parsub_advdifx_uqm_empty

#endif
