#ifdef key_substance

  SUBROUTINE parsub_advdify_fifthorder(c,niter,num_ws)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE parsub_advdify_fifthorder  ***
   !&E
   !&E ** Purpose : Estimate of meridional fluxes of substances  with settling velocity (advection and diffusion)
   !&E              Advection uses 5TH order scheme.
   !&E
   !&E ** Description : No horizontal diffusion at lower depths ( < hminkxky)
   !&E                 - case of cyclicity in y : correct for northward propagation,
   !&E                    uncorrect for southward propagation (stencil not long enough)
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : hey,hy,fyeauc(cv_wat),hzey,dy_v,fy
   !&E
   !&E ** Modified variables : fy
   !&E
   !&E ** Reference : Debreu, Duhaut (2011) contrat PREVIMER 09/2 211 097 
   !&E
   !&E ** History :
   !&E           !  2014-10 (B.Thouvenin) Original code (from parsubadvdifyultimatequickest.F90 and tssubadvdifyfifthorder.F90)
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

#ifdef key_parsub_adv_fifthorder

   !! * Modules used
   USE parameters
   USE comvars2d,   ONLY : jb,jh,hty,hey,hy,h0fond,ky,hminkxky,dy_v,halfdt,l_obc_cycl_y
#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
   USE comvars2d,   ONLY : dx_v
#endif
#if defined key_siggen || defined key_gencoord
   USE comsiggen,   ONLY : hzey
#endif
   USE comfluxdo,   ONLY : fyeau
   USE comsubstance,ONLY : nv_adv,nvp,fy,iv_ws_max,nv_ws_max,c5
#ifdef key_sedim
   USE sedimento,    ONLY : corfluy
#endif
   USE_MPI toolmpi, ONLY : ex_j_rsh,ex_i_rsh

   IMPLICIT NONE

   !! * Arguments
   INTEGER,                                                              INTENT(in) :: niter,num_ws
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in) :: c

   !! * Local declarations
   INTEGER                   :: i,j,k,iv,l
   REAL(KIND=rsh), PARAMETER :: dsmall = 1.0d-20
   REAL(KIND=rsh)            :: vv,da,db,dc,dd,de,vvv,cfl,     &
                                dygt,kyb
   REAL(KIND=rsh), DIMENSION(nv_adv)    :: ya,yb,yc,yd,ye,delta

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter PARSUB_ADVDIFY_FIFTHORDER.F90 routine'

!$OMP SINGLE
   DO l=1,nv_ws_max(num_ws)
      iv=iv_ws_max(num_ws,l)
      c5(iv,:,liminm1:limaxp2,ljminm1:ljmaxp2) = c(iv,:,liminm1:limaxp2,ljminm1:ljmaxp2)
   ENDDO
!$OMP END SINGLE
OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-2,3,nvp*kmax,liminm2,limaxp3,ljminm2,ljmaxp3,c5(1:nvp,:,liminm2:limaxp3,ljminm2:ljmaxp3))
   CALL_MPI ex_j_rsh(-2,3,nvp*kmax,liminm2,limaxp3,ljminm2,ljmaxp3,c5(1:nvp,:,liminm2:limaxp3,ljminm2:ljmaxp3))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(c5)

!$OMP DO schedule(runtime),private(i,j,k,kyb      &
!$OMP& ,l,iv,ya,yb,yc,yd,ye,delta                   &
!$OMP&,da,db,dc,dd,de,vv,vvv,cfl)

   DO j=ljmin,ljmax
     DO i=liminp2,limaxm1

       IF (j >= jb(i) .AND. j <= jh(i)-1) THEN
         DO k=1,kmax
           IF (hty(i,j) <= 0.0_rsh .OR. hy(i,j)==-valmanq-h0fond) THEN
             DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
               fy(iv,k,i,j)=0.0_rsh
             ENDDO
           ELSE
             kyb=ky
             IF (hty(i,j) < hminkxky) kyb=0.0_rsh ! no diffusion for small water depth
             vv=fyeau(k,i,j)
#if defined key_siggen || defined key_gencoord
             vvv = vv/(halfdt*hzey(k,i,j))
#else
             vvv = vv/(halfdt*hey(i,j))
#endif
             cfl = vvv*halfdt/dy_v(i,j)
                           
             IF (vv <= 0.0_rsh) THEN  ! southward advection

               de=(1.0_rsh/120.0_rsh)*(4.0_rsh-5.0_rsh*cfl**2+cfl**4)
               dd=(1.0_rsh/120.0_rsh)*(-26.0_rsh+5.0_rsh*cfl+30.0_rsh*cfl**2-5.0_rsh*cfl**3-4.0_rsh*cfl**4)
               dc=(1.0_rsh/120.0_rsh)*(94.0_rsh-75.0_rsh*cfl-40.0_rsh*cfl**2+15.0_rsh*cfl**3+6.0_rsh*cfl**4)
               db=(1.0_rsh/120.0_rsh)*(54.0_rsh+75.0_rsh*cfl+10.0_rsh*cfl**2-15.0_rsh*cfl**3-4.0_rsh*cfl**4)
               da=(1.0_rsh/120.0_rsh)*(-6.0_rsh-5.0_rsh*cfl+5.0_rsh*cfl**2+5.0_rsh*cfl**3+cfl**4)

               DO l=1,nv_ws_max(num_ws)
                 iv=iv_ws_max(num_ws,l)
 
                 yc(iv)=c5(iv,k,i,j+1)
                 yb(iv)=c5(iv,k,i,j)

                 IF (hty(i,j+1) > 0.0_rsh .AND. (j+1 < jh(i) .OR. l_obc_cycl_y)) THEN
                   yd(iv)=c5(iv,k,i,j+2)
                 ELSE
                   yd(iv)=yc(iv)
                 END IF
                 IF (j+2<=jmax .AND. hty(i,MIN0(j+2,jmax)) > 0.0_rsh .AND. (j+2 < jh(i) .OR. l_obc_cycl_y)) THEN
                   ye(iv)=c5(iv,k,i,j+3)
                 ELSE
                   ye(iv)=yd(iv)
                 END IF
                 IF (hty(i,j-1) > 0.0_rsh .AND. (j > jb(i) .OR. l_obc_cycl_y)) THEN
                   ya(iv)=c5(iv,k,i,j-1)
                 ELSE
                   ya(iv)=yb(iv)
                 END IF
                 delta(iv)=yc(iv)-yb(iv)
               ENDDO

             ELSE      ! northward advection

               da=(1.0_rsh/120.0_rsh)*(4.0_rsh-5.0_rsh*cfl**2+cfl**4)
               db=(1.0_rsh/120.0_rsh)*(-26.0_rsh-5.0_rsh*cfl+30.0_rsh*cfl**2+5.0_rsh*cfl**3-4.0_rsh*cfl**4)
               dc=(1.0_rsh/120.0_rsh)*(94.0_rsh+75.0_rsh*cfl-40.0_rsh*cfl**2-15.0_rsh*cfl**3+6.0_rsh*cfl**4)
               dd=(1.0_rsh/120.0_rsh)*(54.0_rsh-75.0_rsh*cfl+10.0_rsh*cfl**2+15.0_rsh*cfl**3-4.0_rsh*cfl**4)
               de=(1.0_rsh/120.0_rsh)*(-6.0_rsh+5.0_rsh*cfl+5.0_rsh*cfl**2-5.0_rsh*cfl**3+cfl**4)

               DO l=1,nv_ws_max(num_ws)
                 iv=iv_ws_max(num_ws,l)
                 yc(iv)=c5(iv,k,i,j)
                 yd(iv)=c5(iv,k,i,j+1)
                 IF (hty(i,j-1) > 0.0_rsh .AND. (j > jb(i) .OR. l_obc_cycl_y)) THEN
                   yb(iv)=c5(iv,k,i,j-1)
                 ELSE
                   yb(iv)=yc(iv)
                 END IF
                 IF (j-2>=jmin .AND. hty(i,MAX0(j-2,jmin)) > 0.0_rsh .AND. (j-1 > jb(i) .OR. l_obc_cycl_y)) THEN
                   ya(iv)=c5(iv,k,i,j-2)
                 ELSE
                   ya(iv)=yb(iv)
                 END IF
                 IF (hty(i,j+1) > 0.0_rsh .AND. (j+1 < jh(i) .OR. l_obc_cycl_y)) THEN
                   ye(iv)=c5(iv,k,i,j+2)
                 ELSE
                   ye(iv)=yd(iv)
                 END IF
                 delta(iv)=yd(iv)-yc(iv)
                ENDDO
             END IF       ! ends test on velocity sign


   ! advection flux of salinity/temperature/substances
   ! -------------------------------------------------
             DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
               fy(iv,k,i,j) = vv*(da*ya(iv)+db*yb(iv)+dc*yc(iv)+dd*yd(iv)+de*ye(iv))
             END DO

             DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
   ! Advection term alone
   ! --------------------
               fy(iv,k,i,j) = fy(iv,k,i,j)*dx_v(i,j)
#else
   ! Advection/diffusion term (iso-sigma)
   ! ------------------------------------
#if defined key_siggen || defined key_gencoord
               fy(iv,k,i,j) = fy(iv,k,i,j) - hzey(k,i,j)*kyb*delta(iv)/dy_v(i,j)*halfdt
#else
               fy(iv,k,i,j) = fy(iv,k,i,j) - hey(i,j)*kyb*delta(iv)/dy_v(i,j)*halfdt
#endif
#endif
#ifdef key_sedim
             IF (k == 1) fy(iv,k,i,j)=fy(iv,k,i,j)*corfluy(iv,i,j)
#endif

             END DO

           END IF ! ends test on water depth
         END DO ! ends loop on k
       END IF ! for selecting right j loop
     END DO ! ends loop on i
   END DO ! ends loop on j
!$OMP END DO

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_j_rsh(-1,0,nv_adv*kmax,limin,limax,ljminm1,ljmax,fy(:,:,limin:limax,ljminm1:ljmax))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(fy)

#else

   USE parameters, ONLY : rsh,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,nb_var
   INTEGER,                                                              INTENT(in) :: niter,num_ws
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in) :: c

#endif

   PRINT_DBG*, 'exit parsub_ADVDIFY_FIFTHORDER.F90 routine'

  END SUBROUTINE parsub_advdify_fifthorder
#endif
