#ifdef key_substance
  SUBROUTINE parsub_advdify_fourthorder(c,niter,num_ws)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE parsub_advdify_fourthorder  ***
   !&E
   !&E ** Purpose : Estimate of meridional fluxes of substances  with settling velocity (advection and diffusion)
   !&E              Advection uses a fourth order centered scheme equivalent to QUICKEST
   !&E
   !&E ** Description : No horizontal diffusion at lower depths ( < hminkxky)
   !&E                 - case of cyclicity in y : correct for northward propagation,
   !&E                    uncorrect for southward propagation (stencil not long enough)
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : hey,hy,fyeau,c(cv_wat),hzey,dy_v
   !&E                     fy
   !&E
   !&E ** Modified variables : fy
   !&E
   !&E ** Reference : Schema centre d ordre 4 corespondant au quickest 
   !&E
   !&E ** History :
   !&E           !  2010-10 (L. Debreu, T. Duhaut) original code
   !&E           !                         issued from tssub_advdify_fourthorder
   !&E           !  2011-02  (B. Thouvenin) correction diffusion term of substances
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d,   ONLY : jb,jh,hty,hey,hy,h0fond,ky,hminkxky,dy_v,halfdt,l_obc_cycl_y,dx_v
#if defined key_siggen || defined key_gencoord
   USE comsiggen,   ONLY : hzey
#endif
   USE comfluxdo,   ONLY : fyeau
   USE comsubstance,ONLY : iv_ws_max,nv_ws_max,nv_adv,fy
#ifdef key_sedim
   USE sedimento,    ONLY : corfluy
#endif
   USE_MPI toolmpi, ONLY : ex_j_rsh

   IMPLICIT NONE

   !! * Arguments

   INTEGER,                                                              INTENT(in) :: niter,num_ws
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in) :: c

   !! * Local declarations
   INTEGER                   :: i,j,k
!   REAL(KIND=rsh), PARAMETER :: dsmall = 1.0d-20
   REAL(KIND=rsh)            :: vv,vvv,cfl,d1,d2,d3,d4,kyb
   INTEGER                              :: l,iv
   REAL(KIND=rsh), DIMENSION(nv_adv)    :: ya,yb,yc,yd,delta

   !!----------------------------------------------------------------------
   !! * Executable part

!$OMP DO schedule(runtime),private(i,j,k,kyb   &
!$OMP& ,l,iv,ya,yb,yc,yd,delta                 &
!$OMP&,d1,d2,d3,d4,vv,vvv,cfl)

   DO j=ljmin,ljmax
     DO i=liminp2,limaxm1

       IF (j >= jb(i) .AND. j <= jh(i)-1) THEN
         DO k=1,kmax
           IF (hty(i,j) <= 0.0_rsh .OR. hy(i,j)==-valmanq-h0fond) THEN
            DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)
             fy(iv,k,i,j)=0.0_rsh
            END DO
           ELSE
             kyb=ky
             IF (hty(i,j) < hminkxky) kyb=0.0_rsh ! no diffusion for small water depth
             vv=fyeau(k,i,j)
#if defined key_siggen || defined key_gencoord
             vvv = vv/(halfdt*hzey(k,i,j))
#else
             vvv = vv/(halfdt*hey(i,j))
#endif
             cfl = vvv*halfdt/dy_v(i,j)/REAL(niter,rsh)
             d1 = (1.0_rsh/24.0_rsh)*(-2.0_rlg+cfl+2.0_rlg*cfl**2-cfl**3)
             d2 = (1.0_rsh/24.0_rsh)*(14.0_rlg-15.0_rlg*cfl-2.0_rlg*cfl**2+3.0_rlg*cfl**3)
             d3 = (1.0_rsh/24.0_rsh)*(14.0_rlg+15.0_rlg*cfl-2.0_rlg*cfl**2-3.0_rlg*cfl**3)
             d4 = (1.0_rsh/24.0_rsh)*(-2.0_rlg-cfl+2.0_rlg*cfl**2+cfl**3)

             DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
               yb(iv)=c(iv,k,i,j+1)
               yc(iv)=c(iv,k,i,j)
               delta(iv)=yb(iv)-yc(iv)

               IF (hty(i,j+1) > 0.0_rsh .AND. j+1 < jh(i)) THEN
                 ya(iv)=c(iv,k,i,j+2)
               ELSE
                 ya(iv)=yb(iv)
               END IF

               IF (hty(i,j-1) > 0.0_rsh .AND. (j > jb(i) .OR. l_obc_cycl_y)) THEN
                 ya(iv)=c(iv,k,i,j-1)
               ELSE
                 yd(iv)=yc(iv)
               END IF
             END DO




   ! advection flux of other substances
   ! ----------------------------------

             DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
!               IF ((ABS(dyd(iv)).LE.dsmall).OR.(ABS(dyg(iv)).LE.dsmall)) THEN
!                 ! no horizontal gradient
!               ELSE
!               END IF
               fy(iv,k,i,j) = vv*(d1*ya(iv)+d2*yb(iv)+d3*yc(iv)+d4*yd(iv))
             END DO


             DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
   ! Advection term alone
   ! --------------------
               fy(iv,k,i,j) = fy(iv,k,i,j)*dx_v(i,j)/REAL(niter,rsh)
#else
   ! Advection/diffusion term (iso-sigma)
   ! ------------------------------------
#if defined key_siggen || defined key_gencoord
               fy(iv,k,i,j) = (fy(iv,k,i,j) - hzey(k,i,j)*kyb*delta(iv)/dy_v(i,j)*halfdt)/REAL(niter,rsh)
#else
               fy(iv,k,i,j) = (fy(iv,k,i,j) - hey(i,j)*kyb*delta(iv)/dy_v(i,j)*halfdt)/REAL(niter,rsh)
#endif
#endif
#ifdef key_sedim
               IF (k == 1) fy(iv,k,i,j)=fy(iv,k,i,j)*corfluy(iv,i,j)
#endif

             END DO

           END IF ! ends test on water depth
         END DO ! ends loop on k
       END IF ! for selecting right j loop
     END DO ! ends loop on i
   END DO ! ends loop on j
!$OMP END DO

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_j_rsh(-1,0,nv_adv*kmax,limin,limax,ljminm1,ljmax,fy(:,:,limin:limax,ljminm1:ljmax))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(fy)

  END SUBROUTINE parsub_advdify_fourthorder

#else

  SUBROUTINE parsub_advdify_fourthorder_empty
    WRITE(*,*) 'parsub_advdify_fourthorder_empty routine :'
    WRITE(*,*) 'You should not have seen this print!  error?'
    return
  END SUBROUTINE parsub_advdify_fourthorder_empty

#endif
