#ifdef key_substance

  SUBROUTINE parsub_advdify_uq(c,niter,num_ws)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE parsub_advdify_ultimatequickest  ***
   !&E
   !&E ** Purpose : Estimate of meridional fluxes of substances  with settling velocity (advection and diffusion)
   !&E              Advection uses QUICKEST scheme with ULTIMATE limiter.
   !&E
   !&E ** Description : No horizontal diffusion at lower depths ( < hminkxky)
   !&E                 - case of cyclicity in y : correct for northward propagation,
   !&E                    uncorrect for southward propagation (stencil not long enough)
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : hey,hy,fyeau,c(cv_wat),hzey,dy_v
   !&E                     fy
   !&E
   !&E ** Modified variables : fy
   !&E
   !&E ** Reference : Leonard B. P. (1991) : The ULTIMATE conservative
   !&E                difference scheme applied to unsteady one-dimensional
   !&E                advection. Computer Methods in Applied Mechanics and
   !&E                Engineering, 88, 17-74
   !&E
   !&E ** History :
   !&E           !  2010-03  (P. Douillet) substances with settling velocities
   !&E           !                         issued from tssub_advdify_uq
   !&E           !  2011-02  (B. Thouvenin) correction diffusion term of substances
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d,   ONLY : jb,jh,hty,hey,hy,h0fond,ky,hminkxky,dy_v,halfdt,l_obc_cycl_y
#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
   USE comvars2d,   ONLY : dx_v
#endif
#if defined key_siggen || defined key_gencoord
   USE comsiggen,   ONLY : hzey
#endif
   USE comfluxdo,   ONLY : fyeau
   USE comsubstance,ONLY : fy,iv_ws_max,nv_ws_max,nv_adv
#ifdef key_sedim
   USE sedimento,    ONLY : corfluy
#endif

   USE_MPI toolmpi, ONLY : ex_j_rsh

   IMPLICIT NONE

   !! * Arguments
   INTEGER,                                                              INTENT(in) :: niter,num_ws
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in) :: c

   !! * Local declarations
   INTEGER                   :: i,j,k
   REAL(KIND=rsh), PARAMETER :: dsmall = 1.0d-20
   REAL(KIND=rsh)            :: vv,dyds,dygs,dydt,d0,d1,vvv,courant,     &
                                dygt,kyb,thetap,psip
   INTEGER                              :: l,iv
   REAL(KIND=rsh), DIMENSION(nv_adv)    :: dyd,dyg,ya,yb,yc,delta

   !!----------------------------------------------------------------------
   !! * Executable part

!$OMP DO schedule(runtime),private(i,j,k,dyds,dygs,dydt    &
!$OMP&,dygt,kyb,l,iv,dyd,dyg,ya,yb,yc,delta                &
!$OMP&,d0,d1,vv,vvv,courant,thetap,psip)

   DO j=ljmin,ljmax
     DO i=liminp2,limaxm1

       IF (j >= jb(i) .AND. j <= jh(i)-1) THEN
         DO k=1,kmax
           IF (hty(i,j) <= 0.0_rsh .OR. hy(i,j)==-valmanq-h0fond) THEN
            DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)
             fy(iv,k,i,j)=0.0_rsh
            END DO
           ELSE
             kyb=ky
             IF (hty(i,j) < hminkxky) kyb=0.0_rsh ! no diffusion for small water depth
             vv=fyeau(k,i,j)
#if defined key_siggen || defined key_gencoord
             vvv = vv/(halfdt*hzey(k,i,j))
#else
             vvv = vv/(halfdt*hey(i,j))
#endif
             courant = ABS(vvv)*halfdt/dy_v(i,j)/REAL(niter,rsh)
             d0=(1.0_rsh/6.0_rsh)*(2.0_rsh-courant)*(1.0_rsh-courant)
             d1=(1.0_rsh/6.0_rsh)*(1.0_rsh-courant)*(1.0_rsh+courant)
                           
             IF (vv <= 0.0_rsh) THEN  ! southward advection
              DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
               yb(iv)=c(iv,k,i,j+1)
               yc(iv)=c(iv,k,i,j)

               IF (hty(i,j+1) > 0.0_rsh .AND. j+1 < jh(i)) THEN
                 ya(iv)=c(iv,k,i,j+2)
               ELSE
                 ya(iv)=yb(iv)
               END IF

               dyg(iv)=yb(iv)-ya(iv)
               dyd(iv)=yc(iv)-yb(iv)
               delta(iv)=yb(iv)-yc(iv)
              END DO

             ELSE      ! northward advection

              DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
               yb(iv)=c(iv,k,i,j)
               yc(iv)=c(iv,k,i,j+1)
               IF (hty(i,j-1) > 0.0_rsh .AND. (j > jb(i) .OR. l_obc_cycl_y)) THEN
                 ya(iv)=c(iv,k,i,j-1)
               ELSE
                 ya(iv)=yb(iv)
               END IF

               dyg(iv)=yb(iv)-ya(iv)
               dyd(iv)=yc(iv)-yb(iv)
               delta(iv)=yc(iv)-yb(iv)
              END DO

             END IF       ! ends test on velocity sign


   ! advection flux of other substances
   ! ----------------------------------

             DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
               IF ((ABS(dyd(iv)).LE.dsmall).OR.(ABS(dyg(iv)).LE.dsmall)) THEN
                 ! no horizontal gradient
                 thetap = 0.0_rsh
                 psip = 0.0_rsh
               ELSE
                 ! ultimate limiter
                 thetap = (dyg(iv)+dsmall)/(dyd(iv)+dsmall)
                 psip = d0+d1*thetap
                 psip=MAX(0.0_rsh,MIN(MIN(1.0_rsh,psip),((1.0_rsh-courant)/(courant+dsmall))*thetap))
               END IF
               fy(iv,k,i,j) = vv*(yb(iv)+psip*dyd(iv))
             END DO


             DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)

#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
   ! Advection term (iso-sigma) alone
   ! --------------------------------
               fy(iv,k,i,j) = fy(iv,k,i,j)/REAL(niter,rsh)*dx_v(i,j)

#else
   ! Advection/diffusion term (iso-sigma)
   ! ------------------------------------
#if defined key_siggen || defined key_gencoord
               fy(iv,k,i,j) = (fy(iv,k,i,j) - hzey(k,i,j)*kyb*delta(iv)/dy_v(i,j)*halfdt)/REAL(niter,rsh)
#else
               fy(iv,k,i,j) = (fy(iv,k,i,j) - hey(i,j)*kyb*delta(iv)/dy_v(i,j)*halfdt)/REAL(niter,rsh)
#endif

#endif
#ifdef key_sedim
               IF (k == 1) fy(iv,k,i,j)=fy(iv,k,i,j)*corfluy(iv,i,j)
#endif

             END DO

           END IF ! ends test on water depth
         END DO ! ends loop on k
       END IF ! for selecting right j loop
     END DO ! ends loop on i
   END DO ! ends loop on j
!$OMP END DO

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_j_rsh(-1,0,nv_adv*kmax,limin,limax,ljminm1,ljmax,fy(:,:,limin:limax,ljminm1:ljmax))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(fy)

  END SUBROUTINE parsub_advdify_uq

#else

  SUBROUTINE parsub_advdify_uq_empty
    WRITE(*,*) 'parsub_advdify_uq_empty routine :'
    WRITE(*,*) 'You should not have seen this print!  error?'
    return
  END SUBROUTINE parsub_advdify_uq_empty

#endif
