#ifdef key_substance
  SUBROUTINE parsub_advx_macho(c,c2,niter,num_ws)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE parsub_advx_macho  ***
   !&E
   !&E ** Purpose : Intermediate estimate of the tracers
   !&E
   !&E ** Description : 
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : htx,hx,uz,sal,temp,c(cv_wat),dx
   !&E
   !&E ** Modified variables : c2
   !&E
   !&E ** Reference : Leonard B. P., Lock A. P. and M. K. MacVean (1996) :
   !&E                Conservative Explicit Unrestricted-Time-Step
   !&E                Multidimensional COnstancy-Preserving Advections Schemes.
   !&E                Monthly Weather Review, 124, 2588:2605
   !&E
   !&E ** History :
   !&E           !  2010-05 (V. Garnier) Original code from tssubadvdifxultimatequickestmacho.F90
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d,    ONLY : h0fond,dx,htx,ig,id,hx,halfdt
   USE comvarp3d,    ONLY : uz
#ifdef key_substance
   USE comsubstance, ONLY : nv_adv,nvpi,nv_ws_max,iv_ws_max
#endif
   USE_MPI toolmpi,  ONLY : ex_i_rsh,ex_j_rsh

   IMPLICIT NONE

   !! * Arguments
   INTEGER,                                                              INTENT(in) :: niter,num_ws
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in)  :: c
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(inout) :: c2

   !! * Local declarations
   INTEGER                   :: i,j,k,iv,l
   REAL(KIND=rlg), DIMENSION(kmax) :: uadv

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter PARSUB_ADVX_MACHO.F90 routine'

   ! estimate of intermediate tracers (from upwind scheme)
   ! -----------------------------------------------------

   ! at open boundaries (north and south)

!$OMP SINGLE
   DO l=1,nv_ws_max(num_ws)
     iv=iv_ws_max(num_ws,l)

   IF_MPI (limin==imin) THEN
     c2(iv,:,MAX0(imin,imin+ishift_obc_west-1):imin+1+ishift_obc_west,ljmin:ljmax) = &
                        c(iv,:,MAX0(imin,imin+ishift_obc_west-1):imin+1+ishift_obc_west,ljmin:ljmax)
   ENDIF_MPI
   IF_MPI (limax==imax) THEN
     c2(iv,:,imax-ishift_obc_east:MIN0(imax,imax-ishift_obc_east+2),ljmin:ljmax) = &
                        c(iv,:,imax-ishift_obc_east:MIN0(imax,imax-ishift_obc_east+2),ljmin:ljmax)
   ENDIF_MPI
   IF_MPI (ljmin==jmin) THEN
     c2(iv,:,limin:limax,MAX0(jmin,jmin+jshift_obc_south-1):jmin+1+jshift_obc_south) = &
                        c(iv,:,limin:limax,MAX0(jmin,jmin+jshift_obc_south-1):jmin+1+jshift_obc_south)
   ENDIF_MPI
   IF_MPI (ljmax==jmax) THEN
     c2(iv,:,limin:limax,jmax-jshift_obc_north:MIN0(jmax,jmax-jshift_obc_north+2)) = &
                        c(iv,:,limin:limax,jmax-jshift_obc_north:MIN0(jmax,jmax-jshift_obc_north+2))
   ENDIF_MPI
   ENDDO

!$OMP END SINGLE

   ! inside the domain

!$OMP DO SCHEDULE(RUNTIME),PRIVATE(i,j,k,uadv)

   DO j=ljmin,ljmax
#ifdef key_MPI_2D
     DO i=limin,limax
     IF (i >= ig(j)+1 .AND. i <=id(j)-1) THEN
#else
     DO i=ig(j)+1,id(j)-1
#endif

!       DO k=1,kmax

         ! estimate of the velocities
         IF (htx(i  ,j) <= 0.0_rsh .OR. hx(i  ,j) == -valmanq-h0fond .OR. &
             htx(i-1,j) <= 0.0_rsh .OR. hx(i-1,j) == -valmanq-h0fond) THEN
           uadv(:)=0.0_rlg
         ELSE
           uadv(:)=0.5_rsh*(uz(:,i-1,j)+uz(:,i,j))
#ifdef key_test
           DO k=1,kmax
             IF (uz(k,i,j)*uz(k,i-1,j) > 0.0_rsh) THEN
               IF (uz(k,i,j) > 0.0_rsh) THEN
                 uadv(k)=uz(k,i-1,j)
               ELSE
                 uadv(k)=uz(k,i,j)
               END IF
             ELSE
               uadv(k)=0.0_rsh
             END IF
           END DO
#endif
         END IF

         ! estimate of the intermediate tracers
           DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)
             c2(iv,:,i,j)  = c(iv,:,i,j)-(halfdt/dx(i,j))*uadv(:)*0.5_rsh*(c(iv,:,i+1,j)-c(iv,:,i-1,j))/REAL(niter,rsh)
         END DO

!       END DO   ! ends loop on k
#ifdef key_MPI_2D
     END IF
#endif
     END DO   ! ends loop on i
   END DO   ! ends loop on j
!$OMP END DO

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-1,2,nv_adv*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,c2(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,nv_adv*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,c2(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(c2)


   PRINT_DBG*, 'exit PARSUB_ADVX_MACHO.F90 routine'

  END SUBROUTINE parsub_advx_macho
#endif
