#ifdef key_substance
  SUBROUTINE parsub_advy_macho(c,c2,niter,num_ws)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE parsub_advy_macho  ***
   !&E
   !&E ** Purpose : Estimate of intermediate tracers
   !&E
   !&E ** Description : 
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : hty,hy,vz,sal,temp,c(cv_wat),dy
   !&E
   !&E ** Modified variables : sal2,temp2,c2
   !&E
   !&E ** Reference : Leonard B. P., Lock A. P. and M. K. MacVean (1996) :
   !&E                Conservative Explicit Unrestricted-Time-Step
   !&E                Multidimensional COnstancy-Preserving Advections Schemes.
   !&E                Monthly Weather Review, 124, 2588:2605
   !&E
   !&E ** History :
   !&E           !  2010-05 (V. Garnier) Original code from tssubadvdifxultimatequickestmacho.F90
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d,    ONLY : h0fond,dy,hty,jb,jh,hy,halfdt
   USE comvarp3d,    ONLY : vz
#ifdef key_substance
   USE comsubstance, ONLY : nv_adv,nvpi,nv_ws_max,iv_ws_max
#endif
   USE_MPI toolmpi,  ONLY : ex_i_rsh,ex_j_rsh

   IMPLICIT NONE

   !! * Arguments
   INTEGER,                                                              INTENT(in) :: niter,num_ws
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in)  :: c
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(inout) :: c2


   !! * Local declarations
   INTEGER                   :: i,j,k,iv,l
   REAL(KIND=rlg), DIMENSION(kmax) :: vadv

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter PARSUB_ADVY_MACHO.F90 routine'

   ! estimate of intermediate tracers (from upwind scheme)
   ! -----------------------------------------------------

   ! at open boundaries (east and west)

!$OMP SINGLE

   DO l=1,nv_ws_max(num_ws)
     iv=iv_ws_max(num_ws,l)
   IF_MPI (limin==imin) THEN
     c2(iv,:,MAX0(imin,imin+ishift_obc_west-1):imin+1+ishift_obc_west,ljmin:ljmax) = &
                        c(iv,:,MAX0(imin,imin+ishift_obc_west-1):imin+1+ishift_obc_west,ljmin:ljmax)
   ENDIF_MPI
   IF_MPI (limax==imax) THEN
     c2(iv,:,imax-ishift_obc_east:MIN0(imax,imax-ishift_obc_east+2),ljmin:ljmax) = &
                        c(iv,:,imax-ishift_obc_east:MIN0(imax,imax-ishift_obc_east+2),ljmin:ljmax)
   ENDIF_MPI
   IF_MPI (ljmin==jmin) THEN
     c2(iv,:,limin:limax,MAX0(jmin,jmin+jshift_obc_south-1):jmin+1+jshift_obc_south) = &
                        c(iv,:,limin:limax,MAX0(jmin,jmin+jshift_obc_south-1):jmin+1+jshift_obc_south)
   ENDIF_MPI
   IF_MPI (ljmax==jmax) THEN
     c2(iv,:,limin:limax,jmax-jshift_obc_north:MIN0(jmax,jmax-jshift_obc_north+2)) = &
                        c(iv,:,limin:limax,jmax-jshift_obc_north:MIN0(jmax,jmax-jshift_obc_north+2))
   ENDIF_MPI
   ENDDO
!$OMP END SINGLE

   ! inside the domain

!$OMP DO SCHEDULE(RUNTIME),PRIVATE(i,j,k,vadv)

   DO i=liminp2,limaxm1
     DO j=ljminp1,ljmax
!       DO k=1,kmax

         IF (j >= jb(i)+1 .AND. j <= jh(i)-1) THEN

           ! estimate of the advective velocity
           IF (hty(i,j  ) <= 0.0_rsh .OR. hy(i,j  )==-valmanq-h0fond .OR. &
               hty(i,j-1) <= 0.0_rsh .OR. hy(i,j-1)==-valmanq-h0fond) THEN
             vadv(:)=0.0_rlg
           ELSE
             vadv(:)=0.5_rsh*(vz(:,i,j-1)+vz(:,i,j))
           END IF

           ! estimate of intermediate tracers
           DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)
             c2(iv,:,i,j)  = c(iv,:,i,j)  - (halfdt/dy(i,j))*vadv(:)*0.5_rsh*(c(iv,:,i,j+1)-c(iv,:,i,j-1))/REAL(niter,rsh)
           END DO

         END IF   ! ends test for selecting right j loop

!       END DO   ! ends loop on k
     END DO   ! ends loop on i
   END DO   ! ends loop on j
!$OMP END DO

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-1,2,nv_adv*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,c2(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,nv_adv*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,c2(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(c2)

   PRINT_DBG*, 'exit PARSUB_ADVY_MACHO.F90 routine'

  END SUBROUTINE parsub_advy_macho
#endif
