  MODULE traobc3dapply

   !!======================================================================
   !!                   ***  MODULE  traobc3dapply  ***
   !! Ocean dynamics : apply open boundary conditions of variables
   !!                  temperature and salinity
   !!======================================================================

#include "toolcpp.h"

   !! * Modules used

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC ts_obc3d_apply                ! routine called by stepu.F90 stepv.F90
#ifdef key_agrif
   PUBLIC ts_obc3d_apply_agrif          ! routine called by stepu.F90 stepv.F90
#endif

   !! * Shared module variables

   !! * Private variables

  CONTAINS
   !!======================================================================

  SUBROUTINE ts_obc3d_apply(u,v,sal,temp)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ts_obc3d_apply  ***
   !&E
   !&E ** Purpose : apply open boundary conditions for temperature
   !&E              and salinity field
   !&E
   !&E ** Description : apply external value when water mass enters the domain
   !&E
   !&E ** Called by : stepu, stepv
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : sal,temp,hx,u,obc_sw,obc_tw,dx_u,obc_se,obc_te,
   !&E                     v,obc_ss,obc_ts,dy_v,obc_sn,obc_tn
   !&E
   !&E ** Modified variables : sal,temp
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !                       Original code
   !&E       !  2007-05 (V. Garnier) Mise en forme - introduction de obc_* variables
   !&E       !  2007-09 (V. Garnier) homogeneneity of dx dy...
   !&E       !  2009-04 (A. Mano) MPI2D
   !&E       !  2009-07 (F. Vandermeirsch) MC 2nd order scheme
   !&E       !  2010-04  (T. Odaka) MPI2D cyclic
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : imin,imax,jmin,jmax,kmax,                      &
                          ljminp2,ljmaxm1,ljmin,ljmax,ljminm1,ljmaxp1,   &
                          limin,limax,liminm1,limaxp2,liminp2,limaxm1,   &
                          ljmaxp2,liminm2,ljminm2,rsh,rlg,num_testcase,  &
                          ishift_obc_west,ishift_obc_east,jshift_obc_south,jshift_obc_north

   USE comvars2d,  ONLY : dt,ig,hx,h0fond,dx_u,id,jb,hy,dy_v,jh,l_obc_cycl_x,l_obc_cycl_y
   USE comvars3d,  ONLY : l_testcase,valmanq
   USE obcogcm,    ONLY : obc_tw,obc_se,obc_te,obc_ss,obc_ts,obc_sn,obc_tn,obc_sw,obc_coefrel
   USE obccombine, ONLY : l_obc_char,l_obc_south,l_obc_north,l_obc_east,l_obc_west
   USE_MPI toolmpi,ONLY : ex_i_rsh,ex_j_rsh,ex_peri_x_rsh,ex_peri_y_rsh

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(inout) :: sal,temp
   REAL(KIND=rsh),DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(in) :: u,v

   !! * Local declarations
   INTEGER                   :: i,j,k
   REAL(KIND=rsh)            :: uu,vv,cou,dtrelax
   REAL(KIND=rlg)            :: dtm

   !!----------------------------------------------------------------------
   !! * Executable part

   IF (l_testcase .AND. (num_testcase==1 &
       .OR. num_testcase==33 .OR. num_testcase == 37 &
! cas kelvin a virer
       .OR. num_testcase==3) &
      ) THEN

     ! None

   ELSE ! general

     dtm=0.5_rlg*dt
     dtrelax=dtm*obc_coefrel  ! dt / time scale of relaxation
     IF (obc_coefrel==0.0_rsh) dtrelax=1.0_rsh

!$OMP SINGLE
   ! periodic conditions along x-axis
     IF (l_obc_cycl_x) THEN
#ifdef key_MPI_2D
!!!!!!#if defined key_tssub_adv_fourthorder_testts  || defined key_tssub_adv_fifthorder_testts
#ifdef key_shiftobc
       CALL_MPI ex_peri_x_rsh(3,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,sal(:,liminm1:limaxp2,ljminm1:ljmaxp2))
       CALL_MPI ex_peri_x_rsh(3,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,temp(:,liminm1:limaxp2,ljminm1:ljmaxp2))
#else
       CALL_MPI ex_peri_x_rsh(1,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,sal(:,liminm1:limaxp2,ljminm1:ljmaxp2))
       CALL_MPI ex_peri_x_rsh(1,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,temp(:,liminm1:limaxp2,ljminm1:ljmaxp2))
#endif
#else
       DO j=ljmin,ljmax
   ! western boundary
         sal(:,ig(j),j) = sal(:,id(j)-1,j)
         sal(:,ig(j)-1,j) = sal(:,id(j)-2,j)
         temp(:,ig(j),j) = temp(:,id(j)-1,j)
         temp(:,ig(j)-1,j) = temp(:,id(j)-2,j)
!!!!!!#if defined key_tssub_adv_fourthorder_testts  || defined key_tssub_adv_fifthorder_testts
#ifdef key_shiftobc
         sal(:,ig(j)-2,j) = sal(:,id(j)-3,j)
         temp(:,ig(j)-2,j) = temp(:,id(j)-3,j)
#endif
   ! eastern boundary
         sal(:,id(j),j) = sal(:,ig(j)+1,j)
         temp(:,id(j),j) = temp(:,ig(j)+1,j)
!!!!#if defined key_tssub_adv_fourthorder_testts  || defined key_tssub_adv_fifthorder_testts
#ifdef key_shiftobc
         sal(:,id(j)+1,j) = sal(:,ig(j)+2,j)
         sal(:,id(j)+2,j) = sal(:,ig(j)+3,j)
         temp(:,id(j)+1,j) = temp(:,ig(j)+2,j)
         temp(:,id(j)+2,j) = temp(:,ig(j)+3,j)
#endif
       END DO
#endif
     END IF  ! ends periodic conditions along x-axis

   ! periodic conditions along y-axis
     IF (l_obc_cycl_y) THEN

#ifdef key_MPI_2D
!!!!!#if defined key_tssub_adv_fourthorder_testts  || defined key_tssub_adv_fifthorder_testts
#ifdef key_shiftobc
       CALL_MPI ex_peri_y_rsh(3,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,sal(:,liminm1:limaxp2,ljminm1:ljmaxp2))
       CALL_MPI ex_peri_y_rsh(3,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,temp(:,liminm1:limaxp2,ljminm1:ljmaxp2))
#else
       CALL_MPI ex_peri_y_rsh(1,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,sal(:,liminm1:limaxp2,ljminm1:ljmaxp2))
       CALL_MPI ex_peri_y_rsh(1,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,temp(:,liminm1:limaxp2,ljminm1:ljmaxp2))
#endif
#else
       DO i=limin,limax
   ! southern boundary
         sal(:,i,jb(i)) = sal(:,i,jh(i)-1)
         sal(:,i,jb(i)-1) = sal(:,i,jh(i)-2)
         temp(:,i,jb(i)) = temp(:,i,jh(i)-1)
         temp(:,i,jb(i)-1) = temp(:,i,jh(i)-2)
!!!#if defined key_tssub_adv_fourthorder_testts  || defined key_tssub_adv_fifthorder_testts
#ifdef key_shiftobc
         sal(:,i,jb(i)-2) = sal(:,i,jh(i)-3)
         temp(:,i,jb(i)-2) = temp(:,i,jh(i)-3)
#endif
   ! northern boundary
         sal(:,i,jh(i)) = sal(:,i,jb(i)+1)
         temp(:,i,jh(i)) = temp(:,i,jb(i)+1)
!!!!#if defined key_tssub_adv_fourthorder_testts  || defined key_tssub_adv_fifthorder_testts
#ifdef key_shiftobc
         sal(:,i,jh(i)+1) = sal(:,i,jb(i)+2)
         sal(:,i,jh(i)+2) = sal(:,i,jb(i)+3)
         temp(:,i,jh(i)+1) = temp(:,i,jb(i)+2)
         temp(:,i,jh(i)+2) = temp(:,i,jb(i)+3)
#endif
       END DO
#endif

     END IF  ! ends periodic conditions along y-axis
!$OMP END SINGLE


     IF (.NOT.l_obc_cycl_x .AND. (l_obc_west .OR. l_obc_east)) THEN

       DO j=ljminp2,ljmaxm1

   ! Western boundary
         i=ig(j)
         IF_MPI (i==imin+1+ishift_obc_west .AND. limin<=i .AND. i<=limax) THEN
           IF (hx(i,j) /= -valmanq-h0fond) THEN
             DO k=1,kmax
               IF (l_obc_char) THEN
                 uu=u(k,i-1,j)+u(k,i,j)
               ELSE
                 uu=u(k,i,j)
               END IF
               IF (uu > 0.0_rsh) THEN
                 sal(k,i,j)=sal(k,i,j)*(1.0_rsh-dtrelax)+obc_sw(k,1,j)*dtrelax
                 temp(k,i,j)=temp(k,i,j)*(1.0_rsh-dtrelax)+obc_tw(k,1,j)*dtrelax
               ELSE
                 IF (l_obc_char) THEN
               ! Extrapolation d ordre 2
                   sal(k,i,j)=2.0_rsh*sal(k,i+1,j)-sal(k,i+2,j)
                   temp(k,i,j)=2.0_rsh*temp(k,i+1,j)-temp(k,i+2,j)
                 ELSE
                   cou=uu*dtm/dx_u(i,j)
                   sal(k,i,j)=sal(k,i,j)*(1.0_rsh+cou)-cou*sal(k,i+1,j)
                   temp(k,i,j)=temp(k,i,j)*(1.0_rsh+cou)-cou*temp(k,i+1,j)
                 END IF
               END IF
             END DO
           END IF
         ENDIF_MPI

   ! Eastern boundary
         i=id(j)
         IF_MPI (i==imax-ishift_obc_east .AND. limin<=i .AND. i<=limax) THEN
           IF (hx(i-1,j) /= -valmanq-h0fond) THEN
             DO k=1,kmax
               IF (l_obc_char) THEN
                 uu=u(k,i-1,j)+u(k,i,j)
               ELSE
                 uu=u(k,i-1,j)
               END IF
               IF (uu < 0.0_rsh) THEN
                 sal(k,i,j)=sal(k,i,j)*(1.0_rsh-dtrelax)+obc_se(k,imax,j)*dtrelax
                 temp(k,i,j)=temp(k,i,j)*(1.0_rsh-dtrelax)+obc_te(k,imax,j)*dtrelax
               ELSE
                 IF (l_obc_char) THEN
                   sal(k,i,j)=2.0_rsh*sal(k,i-1,j)-sal(k,i-2,j)
                   temp(k,i,j)=2.0_rsh*temp(k,i-1,j)-temp(k,i-2,j)
                 ELSE
                   cou=uu*dtm/dx_u(i-1,j)
                   sal(k,i,j)=sal(k,i,j)*(1.0_rsh-cou)+cou*sal(k,i-1,j)
                   temp(k,i,j)=temp(k,i,j)*(1.0_rsh-cou)+cou*temp(k,i-1,j)
                 END IF
               END IF
             END DO
           END IF
         ENDIF_MPI

       END DO

     END IF  ! ends test on .not.l_obc_cycl_x

     IF (.NOT.l_obc_cycl_y .AND. (l_obc_north .OR. l_obc_south)) THEN

       DO j=ljmin,ljmax
       DO i=liminp2,limaxm1

   ! Southern boundary
         IF (j == jb(i) .AND. j==jmin+1+jshift_obc_south) THEN
           IF (hy(i,j) /= -valmanq-h0fond) THEN
             DO k=1,kmax
               IF (l_obc_char) THEN
                 vv=v(k,i,j-1)+v(k,i,j)
               ELSE
                 vv=v(k,i,j)
               END IF
               IF (vv > 0.0_rsh) THEN
                 sal(k,i,j)=sal(k,i,j)*(1.0_rsh-dtrelax)+obc_ss(k,i,1)*dtrelax
                 temp(k,i,j)=temp(k,i,j)*(1.0_rsh-dtrelax)+obc_ts(k,i,1)*dtrelax
               ELSE
                 IF (l_obc_char) THEN
                   sal(k,i,j)=2.0_rsh*sal(k,i,j+1)-sal(k,i,j+2)
                   temp(k,i,j)=2.0_rsh*temp(k,i,j+1)-temp(k,i,j+2)
                 ELSE
                   cou=vv*dtm/dy_v(i,j)
                   sal(k,i,j)=sal(k,i,j)*(1.0_rsh+cou)-cou*sal(k,i,j+1)
                   temp(k,i,j)=temp(k,i,j)*(1.0_rsh+cou)-cou*temp(k,i,j+1)
                 END IF
               END IF
             END DO
           END IF

   ! Northern boundary
         ELSE IF (j == jh(i) .AND. j==jmax-jshift_obc_north) THEN
           IF (hy(i,j-1) /= -valmanq-h0fond) THEN
             DO k=1,kmax
               IF (l_obc_char) THEN
                 vv=v(k,i,j-1)+v(k,i,j)
               ELSE
                 vv=v(k,i,j-1)
               END IF
               IF (vv < 0.0_rsh) THEN
                 sal(k,i,j)=sal(k,i,j)*(1.0_rsh-dtrelax)+obc_sn(k,i,jmax)*dtrelax
                 temp(k,i,j)=temp(k,i,j)*(1.0_rsh-dtrelax)+obc_tn(k,i,jmax)*dtrelax
               ELSE
                 IF (l_obc_char) THEN
                   sal(k,i,j)=2.0_rsh*sal(k,i,j-1)-sal(k,i,j-2)
                   temp(k,i,j)=2.0_rsh*temp(k,i,j-1)-temp(k,i,j-2)
                 ELSE
                   cou=vv*dtm/dy_v(i,j-1)
                   sal(k,i,j)=sal(k,i,j)*(1.0_rsh-cou)+cou*sal(k,i,j-1)
                   temp(k,i,j)=temp(k,i,j)*(1.0_rsh-cou)+cou*temp(k,i,j-1)
                 END IF
               END IF
             END DO
           END IF
         END IF

       END DO
       END DO

     END IF  ! ends test on .not.l_obc_cycl_y

   END IF   ! ends test on l_testcase

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,sal(:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,sal(:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_i_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,temp(:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,temp(:,liminm1:limaxp2,ljminm1:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(sal,temp)

  END SUBROUTINE ts_obc3d_apply

   !!======================================================================

  SUBROUTINE ts_obc3d_apply_agrif

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ts_obc3d_apply_agrif  ***
   !&E
   !&E ** Purpose : apply open boundary conditions for temperature
   !&E              and salinity field for AGRIF
   !&E
   !&E ** Description : apply external value when water mass enters the domain
   !&E
   !&E ** Called by : stepu, stepv
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !                       Original code
   !&E       !  2007-05 (V. Garnier) Mise en forme - introduction de obc_* variables
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
#ifdef key_agrif

   USE parameters
   USE comvars2d, ONLY : c_agrif_interp
   USE comvarp2d
   USE agrifcommonid
   USE obccombine, ONLY : l_obc_char,l_obc_south,l_obc_north,l_obc_east,l_obc_west

   USE_MPI toolmpi, ONLY : ex_i_rsh,ex_j_rsh
   USE_MPI toolmpi, ONLY : MPI_EXI_recv,MPI_EXI_send
   USE_MPI toolmpi, ONLY : MPI_EXJ_recv,MPI_EXJ_send

   !! * Arguments

   !! * Local declarations
   External :: interp3dtraceur
   INTEGER :: i,j

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter TS_OBC3D_APPLY_AGRIF routine'

   Agrif_UseSpecialValue = .TRUE.
   Agrif_SpecialValue = valmanq

   c_agrif_interp='traceur_bc'
   CALL Agrif_Bc_Variable(traceur_id,procname=interp3dtraceur)
   Agrif_UseSpecialValue = .FALSE.

   IF (Agrif_GrandMother()) THEN
     IF ((l_obc_west .OR. l_obc_east)) THEN

       DO j=ljminp2,ljmaxm1

   ! Western boundary
         i=ig(j)
         IF_MPI (limin==i-1) THEN
           IF (hx(i,j) /= -valmanq-h0fond) THEN
              sal(:,i,j) = sal(:,i+1,j)
              temp(:,i,j) = temp(:,i+1,j)
           ELSE
              sal(:,i,j) = valmanq
              temp(:,i,j) = valmanq
           END IF
         ENDIF_MPI

   ! Eastern boundary
         i=id(j)
         IF_MPI (limax==i) THEN
           IF (hx(i-1,j) /= -valmanq-h0fond) THEN
              sal(:,i,j) = sal(:,i-1,j)
              temp(:,i,j) = temp(:,i-1,j)
           ELSE
              sal(:,i,j) = valmanq
              temp(:,i,j) = valmanq
           END IF
         ENDIF_MPI

       END DO

     END IF  ! ends test on .not.l_obc_cycl_x

     IF ((l_obc_north .OR. l_obc_south)) THEN

       DO j=ljmin,ljmax
       DO i=liminp2,limaxm1

   ! Southern boundary
         IF (j == jb(i))THEN
           IF (hy(i,j) /= -valmanq-h0fond) THEN
             sal(:,i,j) = sal(:,i,j+1)
             temp(:,i,j) = temp(:,i,j+1)
           ELSE
             sal(:,i,j) = valmanq
             temp(:,i,j) = valmanq
           END IF

   ! Northern boundary
         ELSE IF (j == jh(i))THEN
           IF (hy(i,j-1) /= -valmanq-h0fond) THEN
             sal(:,i,j) = sal(:,i,j-1)
             temp(:,i,j) = temp(:,i,j-1)
           ELSE
             sal(:,i,j) = valmanq
             temp(:,i,j) = valmanq
           END IF
         END IF

       END DO
       END DO

     END IF  ! ends test on .not.l_obc_cycl_y

   END IF

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,sal(:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,sal(:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_i_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,temp(:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,temp(:,liminm1:limaxp2,ljminm1:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(sal,temp)

#endif
   PRINT_DBG*, 'exit TS_OBC3D_APPLY_AGRIF routine'

  END SUBROUTINE ts_obc3d_apply_agrif

  END MODULE traobc3dapply

   !!======================================================================

#ifdef key_agrif

  SUBROUTINE interp3dtraceur(tabres,l1,l2,k1,k2,i1,i2,j1,j2,before)

   USE parameters
   USE comvarp3d
   USE comvars2d
   USE comvarp2d
#if defined key_siggen || defined key_gencoord
   USE toolgeom, ONLY : f_hzu
#endif
   IMPLICIT NONE

   INTEGER i1,i2,j1,j2,k1,k2,l1,l2
   REAL tabres(l1:l2,k1:k2,i1:i2,j1:j2)
   REAL tra_diff(l1:l2,k1:k2,i1:i2,j1:j2)
   REAL(KIND=rsh),DIMENSION(l1:l2,k1:k2,i1:i2,j1:j2) :: Fx, Fy
   INTEGER i,j,k
   LOGICAL :: before
   REAL(KIND=rsh),DIMENSION(i1:i2,j1:j2) :: coeff
   REAL(KIND=rsh) :: visc_sponge
   INTEGER :: dist
   INTEGER :: ii1,ii2,jj1,jj2

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter INTERP3DTRACEUR routine'

   IF (before) THEN
     DO j=j1,j2
       DO i=i1,i2
         IF (h0(i,j)+ssh(i,j) < h0fond) THEN
           DO k=k1,k2
             tabres(1:2,k,i,j) = Agrif_SpecialValue
!            tabres(k,i,j) = 10.0
           END DO
         ELSE
           DO k=k1,k2
             tabres(1,k,i,j) = temp(k,i,j)
             tabres(2,k,i,j) = sal(k,i,j)
           END DO
         END IF

       END DO
     END DO
   ELSE
     SELECT CASE (c_agrif_interp)
       CASE('traceur_init')

         DO j=j1,j2
           DO i=i1,i2
             IF (h0(i,j)==-valmanq) THEN
               DO k=k1,k2
                 temp(k,i,j) = valmanq
                 sal(k,i,j)  = valmanq
               END DO
             ELSE
               DO k=k1,k2
                 temp(k,i,j) = tabres(1,k,i,j)
                 sal(k,i,j)  = tabres(2,k,i,j)
               END DO
             END IF
           END DO
         END DO

       CASE('traceur_bc')

         DO j=j1,j2
           DO i=i1,i2
!         IF (h0(i,j)+ssh(i,j)>=h0fond) THEN
             IF (h0(i,j)==-valmanq) THEN
               DO k=k1,k2
                 IF (i==imin+1.or.i==imax.or.j==jmax.or.j==jmin+1) THEN
                   temp(k,i,j) = valmanq
                   sal(k,i,j)  = valmanq
                 END IF
               END DO
               tra_diff(:,:,i,j) = 0.
             ELSE

               DO k=k1,k2
                 IF (i==imin+1.or.i==imax.or.j==jmax.or.j==jmin+1) THEN
                   temp(k,i,j) = tabres(1,k,i,j)
                   sal(k,i,j)  = tabres(2,k,i,j)
                 END IF
               END DO
               tra_diff(1,:,i,j) = temp(:,i,j) - tabres(1,:,i,j)
               tra_diff(2,:,i,j) = sal(:,i,j) - tabres(2,:,i,j)
             END IF
           END DO
         END DO

         RETURN

!cval WARNING AGRIF en dur !!!!!
         visc_sponge = 0.*1./32.

         ii1 = MAX(limin,i1+1)
         ii1 = MAX(ii1,imin+2)

         ii2 = MIN(limax,i2-1)
         ii2 = MIN(ii2,imax-1)

         jj1 = MAX(ljmin,j1+1)
         jj1 = MAX(jj1,jmin+2)


         jj2 = MIN(ljmax,j2-1)
         jj2 = MIN(jj2,jmax-1)

         coeff = 0.

         DO j=jj1-1,jj2+1
           DO i=ii1-1,ii2+1
             dist = nb_connect*Agrif_Irhox()
             dist = min(dist,i-(imin+1))
             dist = min(dist,imax-i)
             dist = min(dist,j-(jmin+1))
             dist = min(dist,jmax-j)

             coeff(i,j) = (1.-REAL(dist)/real(nb_connect*Agrif_Irhox()))*visc_sponge* &
             (dx(i,j)*dy(i,j))**2/(halfdt*(dx(i,j)**2+dy(i,j)**2))
           END DO
         END DO

         Fx=0.
         DO j=jj1,jj2
           DO i=ii1-1,ii2
             DO k=k1,k2
#if defined key_siggen || defined key_gencoord
               Fx(:,k,i,j) = hzex(k,i,j)*0.5*(coeff(i,j)+coeff(i+1,j))* &
               (tra_diff(:,k,i+1,j)-tra_diff(:,k,i,j))/dx_u(i,j)*halfdt
#else
               Fx(:,k,i,j) = hex(i,j)*0.5*(coeff(i,j)+coeff(i+1,j))* &
               (tra_diff(:,k,i+1,j)-tra_diff(:,k,i,j))/dx_u(i,j)*halfdt
#endif
             END DO
           END DO
         END DO

         Fy = 0.
         DO j=jj1-1,jj2
           DO i=ii1,ii2
             DO k=k1,k2
#if defined key_siggen || defined key_gencoord
               Fy(:,k,i,j) = hzey(k,i,j)*0.5*(coeff(i,j)+coeff(i,j+1))* &
                            (tra_diff(:,k,i,j+1)-tra_diff(:,k,i,j))/dy_v(i,j)*halfdt
#else
               Fy(:,k,i,j) = hey(i,j)*0.5*(coeff(i,j)+coeff(i,j+1))* &
                            (tra_diff(:,k,i,j+1)-tra_diff(:,k,i,j))/dy_v(i,j)*halfdt
#endif
             END DO
           END DO
         END DO

         DO j=jj1,jj2
           DO i=ii1,ii2
             DO k=k1,k2
               IF (h0(i,j)+ssh(i,j) > hm) THEN
                 temp(k,i,j) = temp(k,i,j)+((1./dx(i,j))*(Fx(1,k,i,j)-Fx(1,k,i-1,j))  &
                 +(1./dy(i,j))*(Fy(1,k,i,j)-Fy(1,k,i,j-1)))/ &
#if defined key_siggen || defined key_gencoord
                 f_hzu(h0(i,j),ssh(i,j),k,i,j)
#else
                 (ssh(i,j)+h0(i,j))
#endif
                 sal(k,i,j) = sal(k,i,j)+((1./dx(i,j))*(Fx(2,k,i,j)-Fx(2,k,i-1,j))  &
                                    +(1./dy(i,j))*(Fy(2,k,i,j)-Fy(2,k,i,j-1)))/ &
#if defined key_siggen || defined key_gencoord
                 f_hzu(h0(i,j),ssh(i,j),k,i,j)
#else
                 (ssh(i,j)+h0(i,j))
#endif
               END IF
             END DO
           END DO
         END DO

     END SELECT



   END IF

   PRINT_DBG*, 'exit INTERP3DTRACEUR routine'

   RETURN

  END SUBROUTINE interp3dtraceur

   !!======================================================================

  SUBROUTINE interp3dturb(tabres,l1,l2,k1,k2,i1,i2,j1,j2,before)

   USE parameters
   USE comvarp3d
   USE comvars2d
   USE comvarp2d
   USE comturb
#if defined key_siggen || defined key_gencoord
   USE toolgeom, ONLY : f_hzu
#endif

   IMPLICIT NONE

   INTEGER i1,i2,j1,j2,k1,k2,l1,l2
   REAL tabres(l1:l2,k1:k2,i1:i2,j1:j2)
   INTEGER i,j,k,l
   LOGICAL :: before

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter INTERP3DTURB routine'
!
   IF (before) THEN
     DO j=j1,j2
       DO i=i1,i2
         IF (h0(i,j) == -valmanq) THEN
           DO k=k1,k2
             DO l=l1,l2
               tabres(l,k,i,j) = Agrif_SpecialValue
             END DO
           END DO
         ELSE
           DO k=k1,k2
             tabres(1,k,i,j) = ect(k,i,j)
             tabres(2,k,i,j) = psi(k,i,j)
             IF (turb_nbeq == 2) THEN
               tabres(3,k,i,j) = eps(k,i,j)
             END IF
           END DO
         END IF
       END DO
     END DO
   ELSE
     DO j=j1,j2
       DO i=i1,i2
         IF (h0(i,j)==-valmanq) THEN
           DO k=k1,k2
             ect(k,i,j) = valmanq
             psi(k,i,j) = valmanq
             IF (turb_nbeq == 2) THEN
               eps(k,i,j) = valmanq
             END IF
           END DO
         ELSE
           DO k=k1,k2
             ect(k,i,j) = tabres(1,k,i,j)
             psi(k,i,j) = tabres(2,k,i,j)
             IF (turb_nbeq == 2) THEN
               eps(k,i,j) = tabres(3,k,i,j)
             END IF
           END DO
         END IF
       END DO
     END DO
   END IF

   PRINT_DBG*, 'exit INTERP3DTURB routine'

   RETURN

  END SUBROUTINE interp3dturb

   !!======================================================================

  SUBROUTINE interp3dkz(tabres,k1,k2,i1,i2,j1,j2,before)

   USE parameters
   USE comvarp3d
   USE comvars2d
   USE comvarp2d
   USE comturb
#if defined key_siggen || defined key_gencoord
   USE toolgeom, ONLY : f_hzu
#endif

   IMPLICIT NONE

   INTEGER i1,i2,j1,j2,k1,k2
   REAL tabres(k1:k2,i1:i2,j1:j2)
   INTEGER i,j,k
   LOGICAL :: before

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter INTERP3DKZ routine'
!
   IF (before) THEN
     DO j=j1,j2
       DO i=i1,i2
         IF (h0(i,j) == -valmanq) THEN
           DO k=k1,k2
             tabres(k,i,j) = Agrif_SpecialValue
           END DO
         ELSE
           DO k=k1,k2
             tabres(k,i,j) = kz(k,i,j)
           END DO
         END IF
       END DO
     END DO
   ELSE
     DO j=j1,j2
       DO i=i1,i2
         IF (h0(i,j)==-valmanq) THEN
           DO k=k1,k2
             kz(k,i,j) = valmanq
           END DO
         ELSE
           DO k=k1,k2
             kz(k,i,j)  = tabres(k,i,j)
           END DO
         END IF
       END DO
     END DO
   END IF

   PRINT_DBG*, 'exit INTERP3DKZ routine'

   RETURN

  END SUBROUTINE interp3dkz

   !!======================================================================

  SUBROUTINE interp3dnz(tabres,k1,k2,i1,i2,j1,j2,before)

   USE parameters
   USE comvarp3d
   USE comvars2d
   USE comvarp2d
   USE comturb
#if defined key_siggen || defined key_gencoord
   USE toolgeom, ONLY : f_hzu
#endif

   IMPLICIT NONE

   INTEGER i1,i2,j1,j2,k1,k2
   REAL tabres(k1:k2,i1:i2,j1:j2)
   INTEGER i,j,k
   LOGICAL :: before

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter INTERP3DNZ routine'
!
   IF (before) THEN
     DO j=j1,j2
       DO i=i1,i2
         IF (h0(i,j) == -valmanq) THEN
           DO k=k1,k2
             tabres(k,i,j) = Agrif_SpecialValue
           END DO
         ELSE
           DO k=k1,k2
             tabres(k,i,j) = nz(k,i,j)
           END DO
         END IF
       END DO
     END DO
   ELSE
     DO j=j1,j2
       DO i=i1,i2
         IF (h0(i,j)==-valmanq) THEN
           DO k=k1,k2
             nz(k,i,j) = valmanq
           END DO
         ELSE
           DO k=k1,k2
             nz(k,i,j)  = tabres(k,i,j)
           END DO
         END IF
       END DO
     END DO
   END IF

   PRINT_DBG*, 'exit INTERP3DKZ routine'

   RETURN

  END SUBROUTINE interp3dnz
#endif
