  SUBROUTINE tssub_eq(xe2,xe3,sal,temp        &
#ifdef key_substance
                                      ,cv_wat &
#endif
                                             )

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE tssub_eq_wcen  ***
   !&E
   !&E ** Purpose : first possibility of subroutine body for the computation 
   !&E              of transport of salinity, temperature and tracers without fall velocity. 
   !&E
   !&E ** Description : A centred scheme is used for vertical convection
   !&E                  (resulting in a series of problems as sigma levels are not equidistant).
   !&E                  For low water heights, a special computation is eventually performed
   !&E                  in tssub_eq_hm, depending on the value of hm.
   !&E                  This subroutine body is the selected one by default.
   !&E
   !&E ** Called by : stepu, stepv
   !&E
   !&E ** External calls : tssub_advdifx, tssub_advdify
   !&E
   !&E ** Used ij-arrays : sal,temp,cv_wat,h0,xe2,xe3,wz,kz,flx_w2s,
   !&E                     fsalx,fsaly,fdifsal,phisal,
   !&E                     ftempx,ftempy,fdiftemp,phitemp,
   !&E                     fx,fy,phicon,dy_u,dx_v,surf
   !&E  
   !&E ** Modified variables : sal,temp,cv_wat
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2004-12-27
   !&E       !  2007-09  (V. Garnier) homogeneneity of dx dy...
   !&E       !  2009-04  (A. Mano, V. Garnier) MPI2D
   !&E       !  2009-09  (B. Thouvenin)  dcdt term for contaminant and biolo
   !&E       !  2010-06  (V.M. Faure, N. Bousserez, M. Baklouti) coupling with eco3m
   !&E       !  2010-06  (F. Cayocca) coupling with the module of sedimentology
   !&E       !  2010-10  (T. Duhaut) introduction of diffusion along isopycnes
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters
   USE comvars2d,   ONLY : ig,id,h0,hm,dy_u,dx_v,surf,l_tssubadv_over_smalldepth,halfdt,fwetp,fwet
   USE comvars3d,   ONLY : wz,kz,dsigw,dsigu
#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
   USE comfluxdo,   ONLY : fdifsal,alloc_fdifadsal,dealloc_fdifadsal
   USE comfluxdo,   ONLY : fsalx,fsaly
   USE cometemp,    ONLY : fdiftemp,alloc_fdifadtemp,dealloc_fdifadtemp
   USE cometemp,    ONLY : ftempx,ftempy,phitemp
#else
   USE comfluxdo,   ONLY : fsalx,fsaly
   USE cometemp,    ONLY : ftempx,ftempy,phitemp
#endif
   USE cometemp,    ONLY : phisal
#ifdef key_substance
   USE comsubstance,ONLY : fx,fy,nv_adv,nvp,phicon,f90
#endif
#if defined key_biolo || defined key_contaminant || defined key_eco3m
   USE comsubstance,ONLY : dcdt
#endif
#if defined key_sedim
   USE comsubstance,ONLY : flx_s2w,flx_w2s
#endif

   IMPLICIT NONE

   !! * Arguments
   REAL(kind=rsh),DIMENSION(limin:limax,ljmin:ljmax),                    INTENT(in)    :: xe2
   REAL(kind=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),                INTENT(in)    :: xe3
!cval xe2 not ready for tsdiffh
   REAL(kind=rsh),DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2),       INTENT(inout) :: sal,temp
#ifdef key_substance
   REAL(kind=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(inout) :: cv_wat
#endif

   !! * Local declarations
   INTEGER 				  :: i,j,k,l
   REAL(kind=rsh)  			  :: dtm,d3,d2,unsd3
#ifdef key_substance
   REAL(kind=rlg),DIMENSION(kmax,nb_var)  :: y
#endif
   REAL(kind=rlg)                         :: xk,yk
   REAL(kind=rlg),DIMENSION(kmax)         :: a,b,c,ysal,u0,v0,p0,ytemp
   REAL(kind=rlg),DIMENSION(0:kmax)       :: q0

   !!----------------------------------------------------------------------
   !! * Executable part

#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
   CALL alloc_fdifadsal   !allocating fdifsal fdifsal
   CALL alloc_fdifadtemp  !allocating fdiftemp fadtemp
#endif

!$OMP PARALLEL default(none)PRIVATE(d2,d3,unsd3,i,j,k,l, &
!$OMP&    xk,yk,dtm,p0,q0,u0,v0,a,b,c,                   &
#ifdef key_substance
!$OMP&    y,                                             &
#endif
!$OMP&    ysal,ytemp)                                    &
!$OMP& shared(halfdt,sal,temp,l_tssubadv_over_smalldepth,xe2,xe3 &
!$OMP& ,phisal,phitemp,hm,ig,id,h0                       &
!$OMP& ,limin,limax,ljmin,ljmax,ljminp2,ljmaxm1          &
!$OMP& ,liminm1,limaxp2,ljminm1,ljmaxp2,fwet,fwetp       &
#ifdef key_substance
!$OMP& ,cv_wat,nvp,nv_adv,fx,fy,phicon,f90               &
#if defined key_biolo || defined key_contaminant || defined key_eco3m
!$OMP& ,dcdt                                             &
#endif
#if defined key_sedim
!$OMP& ,flx_s2w,flx_w2s                                  &
#endif
#endif
#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
!$OMP&  ,fdifsal,fdiftemp                                &
#endif
!$OMP& ,wz,kz,dsigw,dsigu,dy_u,fsalx,dx_v,fsaly,surf,ftempx,ftempy)

   dtm=halfdt

! calcul des flux horizontaux (advectifs et diffusifs)
!---------------------------------------------------

#if ! defined key_1DV

#ifdef key_ts_diffh_geo
   CALL ts_diffh_geo(sal,temp,xe2)           ! estimates diffusion fluxes for salinity and temperature along z=cst
#endif
#ifdef key_ts_diffh_rho
   CALL ts_diffh_rho(sal,temp,xe2)           ! estimates diffusion fluxes for salinity and temperature along rho=cst
#endif
#ifdef key_substance
   CALL tssub_advdifx(sal,temp,cv_wat)
   CALL tssub_advdify(sal,temp,cv_wat)
#else
   CALL tssub_advdifx(sal,temp)
   CALL tssub_advdify(sal,temp)
#endif

#endif

!$OMP DO SCHEDULE(RUNTIME)
   DO j=ljminp2,ljmaxm1
     DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)

       d3=h0(i,j)+xe3(i,j)
       d2=h0(i,j)+xe2(i,j)
       IF(d3 > hm) THEN
         unsd3=1.0_rsh/d3

!-------------------------
         k=1
!-------------------------
         c(k)=dtm*(d3*0.5_rlg*wz(k,i,j)-kz(k,i,j)*unsd3/dsigw(k))/dsigu(k)
         b(k)=d3*fwet(i,j)+dtm*(d3*0.5_rlg*wz(k,i,j)+kz(k,i,j)*unsd3/dsigw(k))/dsigu(k)
#ifdef key_sedim
         b(k)=b(k)+dtm*flx_w2s(-1,i,j)/dsigu(k)  ! car flux identique pour toute
                                                 ! variable dissoute et calcule
                                                 ! pour temp(-1) et sal (0)
                                                 ! flx_w2s(-1,i,j)=flx_w2s( 0,i,j)
#endif

#if ! defined key_offline

#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
         ysal(k)=d2*fwetp(i,j)*sal(k,i,j)-(fsalx(k,i,j)-fsalx(k,i-1,j)              &
                +fsaly(k,i,j)-fsaly(k,i,j-1)+fdifsal(k,i,j))/surf(i,j)   &
                +phisal(k,i,j)/dsigu(k)

         ytemp(k)=d2*fwetp(i,j)*temp(k,i,j)-(ftempx(k,i,j)-ftempx(k,i-1,j)            &
                 +ftempy(k,i,j)-ftempy(k,i,j-1)+fdiftemp(k,i,j))/surf(i,j) &
                 +phitemp(k,i,j)/dsigu(k)
#else
         ysal(k)=d2*fwetp(i,j)*sal(k,i,j)-(dy_u(i,j)*fsalx(k,i,j)-dy_u(i-1,j)*fsalx(k,i-1,j) &
                +dx_v(i,j)*fsaly(k,i,j)-dx_v(i,j-1)*fsaly(k,i,j-1))/surf(i,j)     &
                +phisal(k,i,j)/dsigu(k)
#if defined key_sedim
         ysal(k)=ysal(k)+dtm*flx_s2w(0,i,j)/dsigu(k)
#endif

         ytemp(k)=d2*fwetp(i,j)*temp(k,i,j)-(dy_u(i,j)*ftempx(k,i,j)-dy_u(i-1,j)*ftempx(k,i-1,j) &
                 +dx_v(i,j)*ftempy(k,i,j)-dx_v(i,j-1)*ftempy(k,i,j-1))/surf(i,j)      &
                 +phitemp(k,i,j)/dsigu(k)
#if defined key_sedim
         ytemp(k)=ytemp(k)+dtm*flx_s2w(-1,i,j)/dsigu(k)  !! ATTENTION PB dx dy et dt !!!!!
#endif

#endif

   ! key_offline
#endif

         q0(k)=-c(k)/b(k)        ! factorisation LU

#if ! defined key_offline
         u0(k)=ysal(k)/b(k)      ! salinite
         v0(k)=ytemp(k)/b(k)     ! temperature
#endif

!-------------------------
         DO k=2,kmax-1
!-------------------------
           c(k)=dtm*(d3*0.5_rlg*wz(k,i,j)-kz(k,i,j)*unsd3/dsigw(k))/dsigu(k)
           a(k)=-dtm*(d3*0.5_rlg*wz(k-1,i,j)+kz(k-1,i,j)*unsd3/dsigw(k-1))/dsigu(k)
           b(k)=d3*fwet(i,j)+dtm*(d3*0.5_rlg*wz(k,i,j)+kz(k,i,j)*unsd3/dsigw(k)         &
               -d3*0.5_rlg*wz(k-1,i,j)+kz(k-1,i,j)*unsd3/dsigw(k-1))/dsigu(k)

#if ! defined key_offline

#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
           ysal(k)=d2*fwetp(i,j)*sal(k,i,j)-(fsalx(k,i,j)-fsalx(k,i-1,j)             &
                  +fsaly(k,i,j)-fsaly(k,i,j-1)+fdifsal(k,i,j))/surf(i,j)  &
                  +phisal(k,i,j)/dsigu(k)

           ytemp(k)=d2*fwetp(i,j)*temp(k,i,j)-(ftempx(k,i,j)-ftempx(k,i-1,j)            &
                   +ftempy(k,i,j)-ftempy(k,i,j-1)+fdiftemp(k,i,j))/surf(i,j) &
                   +phitemp(k,i,j)/dsigu(k)
#else
           ysal(k)=d2*fwetp(i,j)*sal(k,i,j)-(dy_u(i,j)*fsalx(k,i,j)-dy_u(i-1,j)*fsalx(k,i-1,j) &
                  +dx_v(i,j)*fsaly(k,i,j)-dx_v(i,j-1)*fsaly(k,i,j-1))/surf(i,j)     &
                  +phisal(k,i,j)/dsigu(k)

           ytemp(k)=d2*fwetp(i,j)*temp(k,i,j)-(dy_u(i,j)*ftempx(k,i,j)-dy_u(i-1,j)*ftempx(k,i-1,j) &
                   +dx_v(i,j)*ftempy(k,i,j)-dx_v(i,j-1)*ftempy(k,i,j-1))/surf(i,j)      &
                   +phitemp(k,i,j)/dsigu(k)
#endif

   ! key_offline
#endif

           p0(k)=a(k)*q0(k-1)+b(k)                ! factorisation LU
           q0(k)=-c(k)/p0(k)

#if ! defined key_offline
           u0(k)=(ysal(k)-a(k)*u0(k-1))/p0(k)     ! salinite
           v0(k)=(ytemp(k)-a(k)*v0(k-1))/p0(k)    ! temperature
#endif

!-------------------------
         ENDDO
!-------------------------

!-------------------------
         k=kmax
!-------------------------
         a(k)=-dtm*(d3*0.5_rlg*wz(k-1,i,j)+kz(k-1,i,j)*unsd3/dsigw(k-1))/dsigu(k)
         b(k)=d3*fwet(i,j)+dtm*(-d3*0.5_rlg*wz(k-1,i,j)+kz(k-1,i,j)*unsd3/dsigw(k-1))/dsigu(k)

#if ! defined key_offline

#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
         ysal(k)=d2*fwetp(i,j)*sal(k,i,j)-(fsalx(k,i,j)-fsalx(k,i-1,j)            &
                +fsaly(k,i,j)-fsaly(k,i,j-1)+fdifsal(k,i,j))/surf(i,j) &
                +phisal(k,i,j)/dsigu(k)

         ytemp(k)=d2*fwetp(i,j)*temp(k,i,j)-(ftempx(k,i,j)-ftempx(k,i-1,j)            &
                 +ftempy(k,i,j)-ftempy(k,i,j-1)+fdiftemp(k,i,j))/surf(i,j) &
                 +phitemp(k,i,j)/dsigu(k)
#else
         ysal(k)=d2*fwetp(i,j)*sal(k,i,j)-(dy_u(i,j)*fsalx(k,i,j)-dy_u(i-1,j)*fsalx(k,i-1,j) &
                +dx_v(i,j)*fsaly(k,i,j)-dx_v(i,j-1)*fsaly(k,i,j-1))/surf(i,j)     &
                +phisal(k,i,j)/dsigu(k)

         ytemp(k)=d2*fwetp(i,j)*temp(k,i,j)-(dy_u(i,j)*ftempx(k,i,j)-dy_u(i-1,j)*ftempx(k,i-1,j) &
                 +dx_v(i,j)*ftempy(k,i,j)-dx_v(i,j-1)*ftempy(k,i,j-1))/surf(i,j)      &
                 +phitemp(k,i,j)/dsigu(k)
#endif

   ! key_offline
#endif

         p0(k)=a(k)*q0(k-1)+b(k)                ! factorisation LU

#if ! defined key_offline

         u0(k)=(ysal(k)-a(k)*u0(k-1))/p0(k)     ! salinite
         v0(k)=(ytemp(k)-a(k)*v0(k-1))/p0(k)    ! temperature

! estimation de la salinite et temperature
!-----------------------------------------

         xk=u0(kmax)
         sal(kmax,i,j)=xk
         yk=v0(kmax)
         temp(kmax,i,j)=yk

         DO k=kmax-1,1,-1
           xk=q0(k)*xk+u0(k)
           sal(k,i,j)=xk
           yk=q0(k)*yk+v0(k)
           temp(k,i,j)=yk
         ENDDO

   ! key_offline
#endif

! estimation des traceurs
!------------------------

#ifdef key_substance
         DO l=nvp+1,nv_adv         ! pas vitesse de chute, var dissoute et fixee
           DO k=1,kmax
             y(k,l)=d2*fwetp(i,j)*cv_wat(l,k,i,j)-(dy_u(i,j)*fx(l,k,i,j)-dy_u(i-1,j)*fx(l,k,i-1,j) &
                                       +dx_v(i,j)*fy(l,k,i,j)-dx_v(i,j-1)*fy(l,k,i,j-1) &
                                       -phicon(l,k,i,j)/dsigu(k))/surf(i,j)
#if defined key_biolo || defined key_contaminant || defined key_eco3m
             y(k,l)=y(k,l)+dcdt(l,k,i,j)*dtm*d2*fwetp(i,j)
#endif
           ENDDO
#if defined key_sedim
             y(1,l)=y(1,l)+dtm*flx_s2w(l,i,j)/dsigu(1)
#endif
           u0(1)=y(1,l)/b(1)
           DO k=2,kmax
             u0(k)=(y(k,l)-a(k)*u0(k-1))/p0(k)
           ENDDO
           xk=u0(kmax)
           cv_wat(l,kmax,i,j)=f90(l)*(xk)
           DO k=kmax-1,1,-1
             xk=q0(k)*xk+u0(k)
             cv_wat(l,k,i,j)=f90(l)*(xk)
           ENDDO
         ENDDO
#endif

       ENDIF
     ENDDO
   ENDDO
!$OMP END DO

   IF (l_tssubadv_over_smalldepth) CALL tssub_eq_hm(xe2,xe3,sal,temp &
#ifdef key_substance
                                     ,cv_wat           &
#endif
                                     )

!$OMP END PARALLEL

#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
   CALL dealloc_fdifadsal   !deallocating fdifsal fdifsal
   CALL dealloc_fdifadtemp  !deallocating fdiftemp fadtemp
#endif

  END SUBROUTINE tssub_eq
