  SUBROUTINE wave_qeuler

   !&E-------------------------------------------------------------------------
   !&E                 ***  ROUTINE wave_qeuler  ***
   !&E
   !&E ** Purpose : Wave-current coupling
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E      - Walstra et al (2000) => pk1, pk2, peps
   !&E      - Soulsby et al (1995) => FWST, DF
   !&E      - Uchiyama et al (2000) => taustr, pk_u, pk_v
   !&E      - Battjes (1975) => nub
   !&E
   !&E ** History :
   !&E       !  2013-08  (A-C. Bennis, F. Dumas) creation - postdoc EPIGRAM
   !&E       !  2014-02  (A-C. Bennis, F. Dumas, V. Garnier) Review of implementation of the
   !&E       !           the wave-current quasi-eulerian formalism
   !&E
   !&E-------------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
 
   USE parameters
   USE comvars2d
   USE comvarp2d
   USE comvars3d
   USE comvarp3d,   ONLY : uz,vz,psi,ect
   USE comcolig2d,  ONLY : igdu,igdv
#ifdef key_wave_qeuler_dynamics
   USE comwave,     ONLY : wave_hs
#if defined key_wave_soulsby_bottom_stress
   USE comwave,     ONLY : uorb,FWST,nuz,nvz
#endif
   USE wave,        ONLY : wave_tauwx,wave_tauwy,wave_tauox,wave_tauoy,wave_phioc, &
                           wave_uba,wave_t0m1,wave_bhd,wave_wlm,wave_thm
   USE comwave,     ONLY : wave_number,hrms,THETA,USX,USY,USZ,wave_freq,USX_u,USY_v, &
                            IUSX_u,IUSY_v, &
                            dsigvz_v,dsigx_u, &
                            dsigy_v, &
                            dsiguz_u,dsigvz_v, &
                            peps,taustr,pk,pk_u,pk_v,tausurf,c1eps
   USE cometdyn,    ONLY : tosx,tosy
   USE obccombine,  ONLY : l_obc_south, l_obc_north,l_obc_east,l_obc_west
   USE_MPI toolmpi, ONLY : ex_i_rsh,ex_j_rsh,ex_peri_x_rsh,ex_peri_y_rsh

   IMPLICIT NONE

   !! * Arguments

   !! * Local declarations
! THETA!direction de propagation des vagues, wave_number nombre d onde, mdptc profondeur moyenne de la colonne d eau
! wave_hs  !Hs calcule par WW3 wave_t0m1  !periode calculee par WW3 wave_bhd  !JSMITH calcule par WW3
   REAL(KIND=rsh), dimension (liminm2:limaxp2,ljminm2:ljmaxp2) :: mdptc
   REAL(KIND=rsh) :: ETQE   ! wave energy

!derivee / a sig de la pression induite par les vagues
!-dJ/dx avec sig comme coordonnee verticale
!-dJ/dy avec sig comme coordonnee verticale
   REAL(KIND=rsh) :: USI
   INTEGER :: i,j,k  !indices des boucles
   REAL(KIND=rsh) :: nivzzx,inivzzx,nub,pk1,pk2
   REAL(KIND=rsh) :: cldelta,DF
   REAL(KIND=rlg) :: divt , hz3_w, d, hrmsm, div

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter WAVE_QEULER routine'

#if ! defined key_palm_ww3

   ! start forcing terms initialization
   ! ----------------------------------

   ! Preliminary calculations
   ! ------------------------

   !profondeur moyenne au centre de la maille
   mdptc(liminm2:limaxp2,ljminm2:ljmaxp2)=h0(liminm2:limaxp2,ljminm2:ljmaxp2)+ssh(liminm2:limaxp2,ljminm2:ljmaxp2)

   ! wave number estimate straight without wave dispersion relation
   ! wave direction (to the MARS reference : 0 from the North) : THETA
   ! root mean square of the significative height : hrms
   ! ---------------------------------------------------------

   DO j=ljminp1,ljmax
     DO i=MAX(limin,ig(j)),MIN(limax,id(j))
      IF (j >= jb(i) .AND. j <= jh(i)) THEN
       wave_number(i,j)=0.0_rsh
       THETA(i,j)=0.0_rsh
       hrms(i,j)=0.0_rsh
       IF (h0(i,j)/=-valmanq) THEN
         ! calcul du nombre d onde a partir de la longueur d onde
         IF (wave_wlm(i,j) /= 0.0_rsh) wave_number(i,j)=2.0_rsh*pi/wave_wlm(i,j)
         ! wave number estimate with wave dispersion relation
         ! ---------------------------------------------------------------
         ! IF ((wave_t0m1(i,j).ne.0).and.(mdptc(i,j).ne.0)) CALL wavenumb(wave_t0m1,mdptc)
#if ! defined key_oasis_mars_ww3 && ! defined key_palm_ww3
   ! netCDF WW3 outputs : 0 from the West
   ! 2*pi to make sure it is positive
   ! pi/2 due to the reference change between the calculation and outputs in WW3
         IF (wave_thm(i,j) < 0.0_rsh) wave_thm(i,j)=(wave_thm(i,j)/180.0_rsh+2.5_rsh)*pi
         THETA(i,j) =(wave_thm(i,j)/180.0_rsh+0.5_rsh)*pi
#else
   ! WW3 calculations : 0 from the North : nothing to do
         THETA(i,j)=wave_thm(i,j)
#endif
         hrms(i,j)=wave_hs(i,j)/sqrt(2.0_rsh) !moyenne quadratique de la hauteur des vagues
       END IF
      END IF
     END DO
   END DO
   CALL_MPI ex_i_rsh(0,1,1,limin,limaxp1,ljmin,ljmaxp1,THETA(limin:limaxp1,ljmin:ljmaxp1))
   CALL_MPI ex_j_rsh(0,1,1,limin,limaxp1,ljmin,ljmaxp1,THETA(limin:limaxp1,ljmin:ljmaxp1))
   CALL_MPI ex_i_rsh(0,1,1,limin,limaxp1,ljmin,ljmaxp1,hrms(limin:limaxp1,ljmin:ljmaxp1))
   CALL_MPI ex_j_rsh(0,1,1,limin,limaxp1,ljmin,ljmaxp1,hrms(limin:limaxp1,ljmin:ljmaxp1))


   ! Estimate of the Stokes drift
   ! Estimate of the wave induced pressure gradients
   ! ---------------------------------

   DO j=ljminp1,ljmax
     DO i=MAX(limin,ig(j)),MIN(limax,id(j))
       IF (j >= jb(i) .AND. j <= jh(i)) THEN
       USX(:,i,j)=0.0_rsh
       USY(:,i,j)=0.0_rsh
       IF (h0(i,j)/=-valmanq) THEN
         ETQE=wave_hs(i,j)**2/16.0_rsh     ! wave energy
         !WNX=wave_number(i,j)*COS(THETA(i,j))  ! x-component of the wave number
         !WNY=wave_number(i,j)*SIN(THETA(i,j))  ! y-component of the wave number
         DO k=1,kmax
           IF ( wave_number(i,j)*mdptc(i,j) /= 0.0_rsh .and. wave_t0m1(i,j) >= 1.0e-5_rsh  ) THEN
             USI = 2.0_rsh*pi*ETQE/wave_t0m1(i,j)*COSH(2.0_rsh*wave_number(i,j)*mdptc(i,j)*(1.0_rsh+sig(k))) &
                          /(SINH(wave_number(i,j)*mdptc(i,j)))**2
             USX(k,i,j)=USI*wave_number(i,j)*COS(THETA(i,j))
             USY(k,i,j)=USI*wave_number(i,j)*SIN(THETA(i,j))
           END IF
         END DO
       END IF 
      END IF
     END DO
   END DO
   CALL_MPI ex_i_rsh(0,1,kmax,limin,limaxp1,ljmin,ljmaxp1,USX(:,limin:limaxp1,ljmin:ljmaxp1))
   CALL_MPI ex_j_rsh(0,1,kmax,limin,limaxp1,ljmin,ljmaxp1,USX(:,limin:limaxp1,ljmin:ljmaxp1))
   CALL_MPI ex_i_rsh(0,1,kmax,limin,limaxp1,ljmin,ljmaxp1,USY(:,limin:limaxp1,ljmin:ljmaxp1))
   CALL_MPI ex_j_rsh(0,1,kmax,limin,limaxp1,ljmin,ljmaxp1,USY(:,limin:limaxp1,ljmin:ljmaxp1))


   ! calcul de la composante verticale de la derive de stokes
   ! utilisation du fait que la derive de stokes soit a divergence nulle
   ! =========================================================

   DO j=ljminp1,ljmaxm1
     DO i=MAX(limin,ig(j)),MIN(limaxm1,id(j))
       USX_u(:,i,j)=0.0_rsh
       USY_v(:,i,j)=0.0_rsh
       IUSX_u(i,j)=0.0_rsh
       IUSY_v(i,j)=0.0_rsh
!cval l introduction de idgu/v (au lieu de test sur h0) a modifie les resultats
       IF (igdu(i,j)/=4) THEN
         USX_u(:,i,j)=0.5_rsh*(USX(:,i,j)+USX(:,i+1,j))
         IUSX_u(i,j)=sum(USX_u(:,i,j)*dsigu(:))
       END IF
       IF (igdv(i,j)/=4) THEN
         USY_v(:,i,j)=0.5_rsh*(USY(:,i,j)+USY(:,i,j+1))
         IUSY_v(i,j)=sum(USY_v(:,i,j)*dsigu(:))
       END IF
     END DO
   END DO
   CALL_MPI ex_i_rsh(-1,0,kmax,liminm1,limax  ,ljmin  ,ljmax  ,USX_u(:,liminm1:limax  ,ljmin  :ljmax))
   CALL_MPI ex_j_rsh(-1,0,kmax,limin  ,limax  ,ljminm1,ljmax  ,USY_v(:,limin  :limax  ,ljminm1:ljmax  ))
   CALL_MPI ex_i_rsh(-1,0,1,liminm1,limax,ljmin,ljmax,IUSX_u(liminm1:limax,ljmin:ljmax))
   CALL_MPI ex_j_rsh(-1,0,1,limin,limax,ljminm1,ljmax,IUSY_v(limin:limax,ljminm1:ljmax))


   ! calcul du Ws par la divergence 
   ! =========================================================

!cval utile ? banc decouvrant sans doute
   USZ(:,:,:)=0.0_rsh
   DO j=ljminp2,ljmaxm1
     DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)
      IF (j >= jb(i) .AND. j <= jh(i)) THEN
       USZ(:,i,j)=0.0_rsh
       d=h0(i,j)+ssh(i,j)
       IF (d > 0.0_rsh) THEN
         divt=0.0_rsh
         DO  k=1,kmax-1
#if defined key_siggen || defined key_gencoord
           hz3_w=f_hzw(h0(i,j),ssh(i,j),k,i,j)
           IF (h0(i,j)+ssh(i,j) <= hm) hz3_w=hm/REAL(kmax,rsh)
           div=(USX_u(k,i,j)*hzex(k,i,j)*dy_u(i,j)-USX_u(k,i-1,j)*hzex(k,i-1,j)*dy_u(i-1,j))/surf(i,j) &
              +(USY_v(k,i,j)*hzey(k,i,j)*dx_v(i,j)-USY_v(k,i,j-1)*hzey(k,i,j-1)*dx_v(i,j-1))/surf(i,j)
           divt=divt+div*dsigu(k)
           USZ(k,i,j)=-divt/hz3_w
#else
           div=(USX_u(k,i,j)*hex(i,j)*dy_u(i,j)-USX_u(k,i-1,j)*hex(i-1,j)*dy_u(i-1,j))/surf(i,j) &
              +(USY_v(k,i,j)*hey(i,j)*dx_v(i,j)-USY_v(k,i,j-1)*hey(i,j-1)*dx_v(i,j-1))/surf(i,j)
           divt=divt+div*dsigu(k)
           USZ(k,i,j)=-(divt)/d
#endif
        END DO   ! ends loop on k
       END IF   ! ends test on water depth
      ELSE
       USZ(:,i,j)=0.0_rsh
      END IF
     END DO   ! ends loop on i
   END DO   ! ends loop on j

!$OMP SINGLE
   IF (l_obc_cycl_x) THEN

#ifdef key_MPI_2D
     CALL_MPI ex_peri_x_rsh(2,kmax,limin,limaxp1,ljmin,ljmaxp1,USZ(:,limin:limaxp1,ljmin:ljmaxp1))
#else
     DO j=ljminp2,ljmax
       usz(:,ig(j)-1,j)=usz(:,id(j)-2,j)
       usz(:,ig(j)  ,j)=usz(:,id(j)-1,j)
       usz(:,id(j)  ,j)=usz(:,ig(j)+1,j)
     END DO
#endif

   ELSE IF (.NOT. l_obc_cycl_x .AND. l_obc_west) THEN

     IF (limin==imin) THEN
       DO j=ljminp2,ljmax
         usz(:,ig(j),j)=usz(:,ig(j)+1,j)
       ENDDO
     END IF

   ELSE IF (.NOT. l_obc_cycl_x .AND. l_obc_east) THEN

     IF (limax==imax) THEN
       DO j=ljminp2,ljmax
         usz(:,id(j),j)=usz(:,id(j)-1,j)
       ENDDO
     END IF
   END IF

   IF (l_obc_cycl_y) THEN
#ifdef key_MPI_2D
     CALL_MPI ex_peri_y_rsh(2,kmax,limin,limaxp1,ljmin,ljmaxp1,USZ(:,limin:limaxp1,ljmin:ljmaxp1))
#else
     DO i=liminp2,limaxm1
       usz(:,i,jb(i)-1)=usz(:,i,jh(i)-2)
       usz(:,i,jb(i))=usz(:,i,jh(i)-1)
       usz(:,i,jh(i))=usz(:,i,jb(i)+1)
    END DO
#endif

   ELSE IF (.NOT. l_obc_cycl_y .AND. l_obc_south) THEN

     IF (ljmin.EQ.jmin) THEN
       DO i=liminp2,limaxm1
           usz(:,i,jb(i))=usz(:,i,jb(i)+1)
       END DO
     END IF

   ELSE IF (.NOT. l_obc_cycl_y .AND. l_obc_north) THEN

     IF (ljmax.EQ.jmax) THEN
       DO i=liminp2,limaxm1
         usz(:,i,jh(i))=usz(:,i,jh(i)-1)
       END DO
     END IF

   END IF
!$OMP END SINGLE
! cval voir ce qu il faut mettre en vrai. O en fonction de la bathy ?

   CALL_MPI ex_i_rsh(0,1,kmax,limin,limaxp1,ljmin,ljmaxp1,USZ(:,limin:limaxp1,ljmin:ljmaxp1))
   CALL_MPI ex_j_rsh(0,1,kmax,limin,limaxp1,ljmin,ljmaxp1,USZ(:,limin:limaxp1,ljmin:ljmaxp1))

   ! end forcing terms initialization
   ! --------------------------------

#endif


   ! start quasi-eulerian ocean-wave dynamics
   ! ----------------------------------------

#ifdef key_palm_ww3
   IF ( t >= tdeb .and. mod(nbouc,2) /= 1 ) THEN
#endif

   ! derivee / a sigma de uz et vz
   ! -----------------------------

!cval ATTENTION siggen
   DO j=ljminp1,ljmax
     DO i=liminp1,limax
       dsiguz_u(:,i,j)=0.0_rsh
       IF (igdu(i,j) /= 4) THEN
         k=1 
         dsiguz_u(k,i,j)=(-3.0_rsh*uz(k,i,j)+4.0_rsh*uz(k+1,i,j)-uz(k+2,i,j))/(2.0_rsh*dsigu(k))
         DO k=2,kmax-1
           dsiguz_u(k,i,j)=(uz(k+1,i,j)-uz(k-1,i,j))/(2*dsigu(k))
         END DO
         k=kmax
         dsiguz_u(k,i,j)=(3.0_rsh*uz(k,i,j)-4.0_rsh*uz(k-1,i,j)+uz(k-2,i,j))/(2.0_rsh*dsigu(k))
       END IF
     END DO
   END DO

   DO j=ljminp1,ljmax
     DO i=liminp1,limax
       dsigvz_v(:,i,j)=0.0_rsh
       IF (igdv(i,j) /= 4) THEN
         k=1 
         dsigvz_v(k,i,j)=(-3.0_rsh*vz(k,i,j)+4.0_rsh*vz(k+1,i,j)-vz(k+2,i,j))/(2.0_rsh*dsigu(k))
         DO k=2,kmax-1
           dsigvz_v(k,i,j)=(vz(k+1,i,j)-vz(k-1,i,j))/(2*dsigu(k))
         END DO
         k=kmax
         dsigvz_v(k,i,j)=(3.0_rsh*vz(k,i,j)-4.0_rsh*vz(k-1,i,j)+vz(k-2,i,j))/(2.0_rsh*dsigu(k))
       END IF
     END DO
   END DO

   CALL_MPI ex_i_rsh(-1,0,kmax,liminm1,limax,ljmin,ljmaxp1,dsiguz_u(:,liminm1:limax,ljmin:ljmaxp1))
   CALL_MPI ex_j_rsh( 0,1,kmax,liminm1,limax,ljmin,ljmaxp1,dsiguz_u(:,liminm1:limax,ljmin:ljmaxp1))
   CALL_MPI ex_i_rsh( 0,1,kmax,limin,limaxp1,ljminm1,ljmax,dsigvz_v(:,limin:limaxp1,ljminm1:ljmax))
   CALL_MPI ex_j_rsh(-1,0,kmax,limin,limaxp1,ljminm1,ljmax,dsigvz_v(:,limin:limaxp1,ljminm1:ljmax))


   !computation of dsigma/dx and disgma/dy for the vortex force and wave induced pressure term (cf annex A2 Bennis et al 2011) 
   ! --------------------------------------------------

   DO j=ljminp1,ljmax
     DO i=MAX(limin,ig(j)),MIN(limax,id(j))
       DO k=1,kmax
         dsigx_u(k,i,j)=0.0_rsh
         IF (igdu(i,j)/=4)  dsigx_u(k,i,j)=-1.0_rsh/hex(i,j)*(ssh(i+1,j)-ssh(i,j))/dx_u(i,j) &
             -sig(k)/hex(i,j)*(mdptc(i+1,j)-mdptc(i,j))/dx_u(i,j)
         dsigy_v(k,i,j)=0.0_rsh
         IF (igdv(i,j)/=4)  dsigy_v(k,i,j)=-1.0_rsh/hey(i,j)*(ssh(i,j+1)-ssh(i,j))/dy_v(i,j) &
           -sig(k)/hey(i,j)*(mdptc(i,j+1)-mdptc(i,j))/dy_v(i,j)
       END DO
     END DO
   END DO
   CALL_MPI ex_i_rsh( 0,1,kmax,limin,limaxp1,ljminm1,ljmax,dsigy_v(:,limin:limaxp1,ljminm1:ljmax))
   CALL_MPI ex_j_rsh(-1,0,kmax,limin,limaxp1,ljminm1,ljmax,dsigy_v(:,limin:limaxp1,ljminm1:ljmax))
   CALL_MPI ex_i_rsh(-1,0,kmax,liminm1,limax,ljmin,ljmaxp1,dsigx_u(:,liminm1:limax,ljmin:ljmaxp1))
   CALL_MPI ex_j_rsh( 0,1,kmax,liminm1,limax,ljmin,ljmaxp1,dsigx_u(:,liminm1:limax,ljmin:ljmaxp1))
     

   ! correction k-epsilon selon Walstra(2000)
   ! longueur de rugosite pres du fond
   ! ----------------------------------------

   cldelta=0.0_rsh
!fd a priori que le teste sur le h0 pour le masque
   DO j=ljminp1,ljmax
     !DO i=liminp1,limax
     DO i=MAX(limin,ig(j)),MIN(limax,id(j))
      IF (j >= jb(i) .AND. j <= jh(i)) THEN

       uorb(i,j)=0.0_rsh
!cval cette initialisation generale de FWST( a change les resultats
       FWST(i,j)=0.0_rsh
       taustr(:,i,j)=0.0_rsh
       wave_freq(i,j)=0.0_rsh

       IF (h0(i,j)/=-valmanq) THEN


   ! computation of the wave intrinsic frequency from the period provided by WWIII
   ! -----------------------------------------------------------------------------

         IF (wave_t0m1(i,j) /= 0.0_rsh) wave_freq(i,j)=2.0_rsh*pi/wave_t0m1(i,j) !frequence intrinseque des vagues

#if defined key_wave_soulsby_bottom_stress
   ! Computation of rms orbital bottom velocity
   ! ------------------------------------------
!cval pb la, je prefererais lire le fichier !
!!!#if defined key_wave_readww3txt || defined key_oasis_mars_ww3 || defined key_palm_ww3
#if defined key_oasis_mars_ww3 || defined key_palm_ww3
         uorb(i,j)=wave_uba(i,j)                    !norme de la vitesse orbitale des vagues
!!!cval#else
#elif defined key_wave_readww3txt
!cvalquestion et ce test pourquoi ? masque terre plutot ?
         IF ( wave_number(i,j) * ( mdptc(i,j) ) /= 0.0_rsh ) &
         uorb(i,j)=wave_freq(i,j)*hrms(i,j)/(2*SINH(wave_number(i,j)*mdptc(i,j)))
#endif


   ! Computation of FWST for the formulation of the Soulsby drag
   ! -----------------------------------------------------------
!         IF (uorb(i,j) /= 0.0_rsh)  FWST(i,j)=1.39_rsh*(wave_freq(i,j)*z0b(i,j)/uorb(i,j))**(20/38)
         IF (uorb(i,j) /= 0.0_rsh)  FWST(i,j)=1.39_rsh*(wave_freq(i,j)*z0b(i,j)/uorb(i,j))**0.52_rsh
#endif

   ! Computation of wave boundary layer thickness cdelta
   ! -----------------------------------------------------------
         IF (wave_freq(i,j) /= 0.0_rsh) THEN
           cldelta = min( (0.8_rsh/wave_freq(i,j))*(uorb(i,j)*sqrt(FWST(i,j)/2.0_rsh)), mdptc(i,j))
         ELSE
           cldelta = min( 1.0e-5, mdptc(i,j) )
!cval oh la variable en dur!!
         END IF

         DO k=1,kmax
           !profondeur a partir de la surface
           nivzzx=mdptc(i,j)*(sig(kmax)-sig(k))
           inivzzx=mdptc(i,j)-cldelta

   ! Computation of wave boundary layer thickness cdelta
   ! -----------------------------------------------------------
           pk1=0.0_rsh
#if defined key_wave_soulsby_bottom_stress
           IF ( nivzzx >= inivzzx .and. nivzzx <= mdptc(i,j) ) THEN
             IF (cldelta/=0.0_rsh) THEN
               DF=FWST(i,j)*uorb(i,j)**3.0_rsh/(2.0_rsh*sqrt(pi))
               ! pk1: source de tke due au frottement sur le fond
               pk1=((2.0_rsh*DF)/cldelta)*(1.0_rsh-(mdptc(i,j)-nivzzx)/cldelta)
               ! taustr: norme du streaming
               IF (wave_freq(i,j)/=0.0_rsh) taustr(k,i,j)=((DF*wave_number(i,j))/wave_freq(i,j)) &
                                                          *(1.0_rsh-(mdptc(i,j)-nivzzx)/cldelta)
             END IF
           END IF
#endif
           ! pk2: source de tke due au deferlement
           IF ( nivzzx <= 0.5_rsh*hrms(i,j) .and. hrms(i,j) /= 0.0_rsh ) THEN
             pk2=((4.0_rsh*wave_phioc(i,j))/hrms(i,j))*(1.0_rsh-(2.0_rsh*nivzzx)/hrms(i,j))
           ELSE
             pk2=0.0_rsh
           END IF

           !pk: source totale de tke due aux vagues
           pk(k,i,j)=pk1+pk2
           !peps: source totale de dissipation due aux vagues
           peps(k,i,j)=c1eps*pk(k,i,j)*psi(k,i,j)/ect(k,i,j)
         END DO
       END IF
      END IF
     END DO
   END DO

   CALL_MPI ex_i_rsh(0,1,1,limin,limaxp1,ljmin,ljmaxp1,uorb(limin:limaxp1,ljmin:ljmaxp1))
   CALL_MPI ex_j_rsh(0,1,1,limin,limaxp1,ljmin,ljmaxp1,uorb(limin:limaxp1,ljmin:ljmaxp1))
   CALL_MPI ex_i_rsh(0,1,1,limin,limaxp1,ljmin,ljmaxp1,FWST(limin:limaxp1,ljmin:ljmaxp1))
   CALL_MPI ex_j_rsh(0,1,1,limin,limaxp1,ljmin,ljmaxp1,FWST(limin:limaxp1,ljmin:ljmaxp1))
   CALL_MPI ex_i_rsh(0,1,kmax,limin,limaxp1,ljmin,ljmaxp1,taustr(:,limin:limaxp1,ljmin:ljmaxp1))
   CALL_MPI ex_j_rsh(0,1,kmax,limin,limaxp1,ljmin,ljmaxp1,taustr(:,limin:limaxp1,ljmin:ljmaxp1))

   ! pk_u et pk_v: contribution to the quasi eulerian momentum due to wave loss of momentum.
   ! for k=1, it includes the momentum source due to wave loss of momentum relative to bottom friction when the wave bottom boundary layer is resolved 
!fd en detail pour le masque
   DO j=ljminp1,ljmaxm1
!     DO i=liminp1,limaxm1
     DO i=MAX(liminp1,ig(j)),MIN(limaxm1,id(j)-1)
      IF (j >= jb(i) .AND. j <= jh(i)-1) THEN
       pk_u(:,i,j)=0.5_rsh*(taustr(:,i+1,j)*cos(THETA(i+1,j))+taustr(:,i,j)*cos(THETA(i,j)))
       pk_v(:,i,j)=0.5_rsh*(taustr(:,i,j+1)*sin(THETA(i,j+1))+taustr(:,i,j)*sin(THETA(i,j)))
       ! for k=1, parameterization of the momentum source due to wave loss of momentum relative to bottom friction when the wave bottom boundary layer is not resolved 
       nuz(i,j)=uz(1,i,j)
       nvz(i,j)=vz(1,i,j)
       IF (pk_u(1,i,j) == 0.0_rsh) nuz(i,j) = nuz(i,j)-1.5_rsh*USX_u(1,i,j)
       IF (pk_v(1,i,j) == 0.0_rsh) nvz(i,j) = nvz(i,j)-1.5_rsh*USY_v(1,i,j)
      END IF
     END DO
   END DO
   CALL_MPI ex_i_rsh(-1,0,1,liminm1,limax,ljmin  ,ljmax,nuz(liminm1:limax,ljmin  :ljmax))
   CALL_MPI ex_j_rsh(-1,0,1,limin  ,limax,ljminm1,ljmax,nvz(limin  :limax,ljminm1:ljmax))


   ! condition limite pour k-epsilon en presence de vagues: Burchard(2001)
   ! ---------------------------------------------------------------------

! est-ce que c est bien propre de calcul le tausurf ainsi ? ne faudrait-il pas faire intervenir les tosx(i-1),tosy(j-1) ?
! formulation battjes + z0s + tension de surface
   DO j=ljminp1,ljmax
     DO i=MAX(limin,ig(j)),MIN(limax,id(j))
      IF (j >= jb(i) .AND. j <= jh(i)) THEN
       IF (h0(i,j)/=-valmanq) THEN
         ! au centre de la maille
         tausurf(i,j)=sqrt((tosx(i,j)+wave_tauwx(i,j)+wave_tauox(i,j))**2 &
                          +(tosy(i,j)+wave_tauwy(i,j)+wave_tauoy(i,j))**2)
         z0s(i,j)=0.6_rsh*sqrt(2.0_rsh)*hrms(i,j)
         nub=0.08_rsh*mdptc(i,j)*((wave_phioc(i,j)/rhoref)**(1/3))
!cval ATTENTION ne pas mettre dans code de base
! de plus attention aux igdu (voir turbsmago)
! test sepcifique Anne Claire
         IF (i >= imax-20 .AND. (l_testcase .AND. num_testcase==35)) THEN
           vish_xe(:,i,j) = (10.0_rsh+1.0*(real(i)-real(imax-20)))*0.01_rsh*dx(i,j)**1.15_rsh
           vish_phi(:,i,j)= (10.0_rsh+1.0*(real(i)-real(imax-20)))*0.01_rsh*dx(i,j)**1.15_rsh
         ELSE
           vish_xe(:,i,j) = 10.0_rsh*0.01_rsh*dx(i,j)**1.15_rsh+nub
           vish_phi(:,i,j)= 10.0_rsh*0.01_rsh*dx(i,j)**1.15_rsh+nub
         END IF
       END IF
      END IF
     END DO
   END DO
OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-1,0,kmax,liminm1,limax  ,ljminm1,ljmax  ,vish_phi(:,liminm1:limax  ,ljminm1:ljmax  ))
   CALL_MPI ex_j_rsh(-1,0,kmax,liminm1,limax  ,ljminm1,ljmax  ,vish_phi(:,liminm1:limax  ,ljminm1:ljmax  ))
   CALL_MPI ex_i_rsh( 0,1,kmax,limin  ,limaxp1,ljmin  ,ljmaxp1,vish_xe (:,limin  :limaxp1,ljmin  :ljmaxp1))
   CALL_MPI ex_j_rsh( 0,1,kmax,limin  ,limaxp1,ljmin  ,ljmaxp1,vish_xe (:,limin  :limaxp1,ljmin  :ljmaxp1))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(vish_phi,vish_xe)

#ifdef key_palm_ww3
   END IF
#endif

   ! end quasi-eulerian ocean-wave dynamics
   ! --------------------------------------

   PRINT_DBG*, 'exit WAVE_QEULER routine'

#else

   PRINT_DBG*, 'WAVE_QEULER routine : you should not see this line. ERROR !'

#endif

  END SUBROUTINE wave_qeuler
